function [x,y,v]=calculatetabprofile(tabwidth,tabspace,toptab,drawbounds)
%Calculates x and y coordinates of tabs that can be plotted on a normalized
%axes. 
%
%[x,y]=calculatetabshape(tabwidth,tabspace,toptab)
%
%TABWIDTH is a vector of length=number of tabs, in which each element is a
%normalized width of that tab relative to other tabs. For example, make
%elements here approximately equal to the number of characters in the text 
%you're entering into each tab.
%
%TABSPACE is the width of the region between tabs, where diagonal tab
%borders are drawn. Normalized to same scale as TABWIDTH.
%
%TOPTAB is an index into TABWIDTH of which tab to draw as being "on top".
%
%DRAWBOUNDS is logical 0 or 1. If 1, will draw a line along inside edge of
%tabs, to show page border of top tab. Use 0 if you dont want this.
%
%Default output is for vertical tabs. For horizontal tabs, interchange x
%and y.
%
%v is a vector of the lower y position of each tab, ie the value you should
%use as the y position for any object you want to place inside the tag.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


%normalize all dimensions into range 0-1

numtabs=numel(tabwidth);

totallength=sum(tabwidth)+tabspace*(numtabs+1);

tabwidth=tabwidth/totallength;

tabspace=tabspace/totallength;

tabwidth=[0 cumsum(tabwidth)];

%initialize position vectors... 4 points per tab.

x=zeros(1,4*numtabs);
y=x;
v=zeros(1,numtabs);

for i=1:toptab
    t=(i-1)*4+1;
    x(t:t+3)=[1 .05 .05 .5];
    y(t:t+3)=[1-tabwidth(i)-tabspace*(i-1) 1-tabwidth(i)-tabspace*(i) 1-tabwidth(i+1)-tabspace*(i) 1-tabwidth(i+1)-tabspace*(i+.5)];
    v(i)=y(t+2);
end

x(t+3)=1;
y(t+3)=y(t+3)-tabspace/2;

if toptab<numtabs
    for i=toptab+1:numtabs
        t=(i-1)*4+1;
        x(t:t+3)=[.5 .05 .05 1];
        y(t:t+3)=[1-tabwidth(i)-tabspace*(i-.5) 1-tabwidth(i)-tabspace*(i) 1-tabwidth(i+1)-tabspace*(i) 1-tabwidth(i+1)-tabspace*(i+1)];
        v(i)=y(t+2);
    end
end

if drawbounds==1
   x=[1 x 1];
   y=[1-tabwidth(toptab)-tabspace*(toptab-1) y 1-(tabwidth(toptab+1)+tabspace*(toptab+1))];
end








