function outs=characterwidths(varargin)
% Calculates the number of pixels for each character of a string with the
% properties in pvpairs. Valid properties to include in pvpairs include:
% fontname, fontsize, fontweight, fontangle, 
%

if nargin==0
    pvpairs={};
else
    pvpairs=varargin;
end

if nargin==1;
    t=pvpairs;
    if ~ishandle(t) || ~strcmp(get(t,'type'),'text')
        error('Inputs must be a text object handle or a list of text object property-value pairs')
    end
else
    t=[];
end

str={char(33:64);char(65:90);char(91:126)}; %string of standard ascii characters

 for s=1:3 %intersperse spaces
     str{s}=[str{s};char(32*ones(size(str{s})))];
     str{s}=str{s}(:)';
 end

f=figure('units','normal','position',[0 0.1 1 1],'toolbar','none','menubar','none','visible','off');
a=axes('units','normal','position',[0.01 0.01 1 1],'color',[0 1 1],'ytick',[],'xtick',[],'visible','off');

set([f a],'units','pixels')
pos=get(a,'pos');

if isempty(t)
    t=text(0,0,str,'hor','left','ver','bottom',pvpairs{:},'color',[0 0 0],'backgroundcolor',[1 1 1],'units','pixels','interpreter','none');
else
    t=copyobj(t,a);
    set(t,'position',[0 0],'hor','left','ver','bottom','color',[0 0 0],'backgroundcolor',[1 1 1],'string',str,'units','pixels','interpreter','none');
end


ext=get(t,'extent');
set(f,'position',[100 100 ext(3)+pos(1)+5 ext(4)+pos(2)+5])


rez=get(0,'screenpixelsperinch');
figpos=getpixelposition(f); %dont need to change anything here
resolution=get(0,'ScreenPixelsPerInch'); %dont need to change anything here
set(f,'paperunits','inches','papersize',figpos(3:4)/resolution,'paperposition',[0 0 figpos(3:4)/resolution]); %dont need to change anything here
path=pwd; %the folder where you want to put the file
name='fonttemp.png'; %what you want the file to be called
print(f,fullfile(path,name),'-dpng',['-r',int2str(rez)],'-painters') %save file (see notes below)

img=imread('fonttemp.png');
delete('fonttemp.png');
delete(f);
iii=img~=255;
iii=any(iii,3);

%find rowbreaks:
fullrows=any(iii,2);
rowstarts=find(diff(fullrows)==1)+1;
rowends=find(diff(fullrows)==-1);

outs=[];

for R=1:numel(rowstarts)
    
    jjj=iii(rowstarts(R):rowends(R),:);
    
    %find occupied columns:
    fullcolumns=any(jjj,1);
    
    letterstarts=find(diff(fullcolumns)==1)+1;
    letterends=find(diff(fullcolumns)==-1);
    
    letterwidths=letterends-letterstarts+1;
    
    %need to correct the " which may be treated as two separate letters:
    if R==1 && numel(letterwidths)==33
        letterwidths(2)=letterends(3)-letterstarts(2)+1;
        letterwidths(3)=[];
    end
    
    %correct the numerals... assume all fonts use fixedwidth for numbers:
    if R==1 
        letterwidths(16:25)=max(letterwidths(16:25))+2;
    end
    
    outs=[outs letterwidths];
end

outs=[zeros(1,32) outs zeros(1,129)];



