function [ax,vscroll]=drawlistbox(parent,pos,string,max,drawempty,value,listboxtop,border,bordercolor,rowheight,callback)

% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%

handles=guidata(parent);
udata=get(handles.fakeax(handles.activeindex),'userdata');

string=string(:);
value=value(:);

colwidths=pos(3);
data=string;
isselected=false(numel(string),1);
isselected(value)=true;



pos=pos-[0 0 18 0]; %make room for scroll bar

p=uipanel('parent',parent,'units','pixels','position',pos,'bordertype',border,'foregroundcolor',[0 0 0]); %the panel

if strcmp(border,'line')
    set(p,'highlightcolor',bordercolor)
end

numrows=size(data,1);
numcols=1;
%note, numrows/numcols refer to the main data table only, headers may have different number of
%elements.

%% make main table image:
%get colors:
axescolors=handles.colors.axes{udata.format.bg};
unselectedcolor=axescolors(2,:);
selectedcolor=udata.format.selectedcolor;

if ~isempty(data)
    t=ones(numrows*rowheight,1,3);
    t(:,:,1)=unselectedcolor(1);
    t(:,:,2)=unselectedcolor(1);
    t(:,:,3)=unselectedcolor(1);
    
    %color according to row state:
    selectedrows=find(isselected);
    adders=repmat(0:rowheight-1,size(selectedrows,1),1);
    pixes=selectedrows(:).*rowheight-rowheight;
    pixes=bsxfun(@plus,pixes,adders);
    pixes=pixes(:)+1;
    t(pixes,:,1)=selectedcolor(1);
    t(pixes,:,2)=selectedcolor(2);
    t(pixes,:,3)=selectedcolor(3);
    %display:
    headerwidth=0;
    headerheight=0;
    tax=axes('units','pixels','position',[headerwidth pos(4)-size(t,1)-headerheight sum(colwidths) size(t,1)],'parent',p);
    image(t,'parent',tax,'tag','mainimage','buttondownfcn',{@selectlistbox,handles.activeindex,callback},'hittest','on','userdata',isselected);
    set(tax,'visible','off','tag','tax');
    tpos=get(tax,'position');
    %set(tax,'buttondownfcn',@selecttableallpoints,'hittest','on')
    
    
    %% make text for table:
    texthandles=zeros(size(data));

    ttax=axes('units','pixels','parent',p);
    
    collines=[1 cumsum(colwidths)];
    rowlines=rowheight:rowheight:(rowheight*numrows);
    rowcoords=rowlines-rowheight/2+1;
    for r=1:numrows
        for c=1:numcols
            texthandles(r,c)=text(collines(c)+3,-rowcoords(r),data{r,c},'parent',ttax,'erasemode','normal',...
                'fontname',get(0,'defaultuicontrolfontname'),'fontsize',8,'color',[0 0 0],'hittest','off','interpreter','none');
        end
    end
    
    set(ttax,'position',get(tax,'position')+[0 0 0 1],'color','none','hittest','off','tag','ttax');
    ylim=get(tax,'ylim');
    set(ttax,'ylim',[-ylim(2) -ylim(1)+1],'visible','off','xlim',[.5 tpos(3)+.5]);
    
    %hide invisible text:
    lastvisiblerow=ceil((pos(4)-headerheight)/rowheight);
    lastvisiblecol=find([0 cumsum(colwidths)]>pos(3)-headerwidth,1);
    
    set(texthandles(lastvisiblerow+3:end,:),'visible','off');
    set(texthandles(:,lastvisiblecol+1:end),'visible','off');
else
    texthandles=[];
end

%% make scrollbars:

%image command resets userdata:
vstep=pos(4);
set(ttax,'userdata',[tax p]);
set(tax,'userdata',texthandles);

%tax = table background axes
%ttax = table text axes

ax=p;

if ~isempty(data)
    vscroll=drawvscroll(parent,ttax,[pos(1)+pos(3)-1 pos(2)+1 17 pos(4)-1],0,tpos(4),vstep,tpos(4),@vertlistboxscroll,1);
else
    vscroll=[];
end


function selectlistbox(src,ev,ai,callback)
%this function selects data in a drawlistbox when you click on it.
%src is the handle of the background image of the table. 
handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');


axescolors=handles.colors.axes{udata.format.bg};
bgcolor=axescolors(2,:);
txt=axescolors(5,:);
unselectedcolor=bgcolor;
selectedcolor=udata.format.selectedcolor;

tax=get(src,'parent');
p=get(tax,'parent');

thisax=get(src,'parent');
clicked=get(thisax,'currentpoint');

%ai=find(handles.fakeax==thisax);
%udata=get(handles.fakeax(ai),'userdata');

originalselection=get(src,'userdata');
clicked=clicked(1,2); %only need y coordinate
rowheight=16;

thisrow=ceil(clicked./rowheight);
%allrows=udata.view.allrows;

newselection=false(size(originalselection));

if strcmp(get(gcf,'currentmodifier'),'control')
        newselection=originalselection;
        newselection(thisrow)=true;
elseif strcmp(get(gcf,'currentmodifier'),'alt')
        newselection=originalselection;
        newselection(thisrow)=false;
else
        newselection(thisrow)=true;
end

set(src,'userdata',newselection);
numrows=size(newselection,1);

t=ones(numrows*rowheight,1,3);
t(:,:,1)=unselectedcolor(1);
t(:,:,2)=unselectedcolor(1);
t(:,:,3)=unselectedcolor(1);

%color according to row state:
selectedrows=find(newselection);
adders=repmat(0:rowheight-1,size(selectedrows,1),1);
pixes=selectedrows(:).*rowheight-rowheight;
pixes=bsxfun(@plus,pixes,adders);
pixes=pixes(:)+1;
t(pixes,:,1)=selectedcolor(1);
t(pixes,:,2)=selectedcolor(2);
t(pixes,:,3)=selectedcolor(3);


%color text:
txts=get(udata.view.tax,'userdata');
set(txts,'color',txt);

a=[0.299 0.587 0.114]'*selectedcolor;

if a<0.5
    set(txts(selectedrows),'color',[1 1 1])
else
    set(txts(selectedrows),'color',[0 0 0])
end

set(src,'cdata',t);

fig=ancestor(src,'figure');
set(fig,'windowbuttonmotionfcn',{@listboxdrag,src,thisrow,originalselection,ai,callback})
set(fig,'windowbuttonupfcn',{@listboxdragdone,selectedrows,ai,callback})

function listboxdrag(src,ev,img,startrow,wasselected,ai,callback)
handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');

axescolors=handles.colors.axes{udata.format.bg};
bgcolor=axescolors(2,:);
txt=axescolors(5,:);
unselectedcolor=bgcolor;
selectedcolor=udata.format.selectedcolor;

thisax=get(img,'parent');
clicked=get(thisax,'currentpoint');
%udata=get(handles.fakeax(ai),'userdata');

clicked=clicked(1,2); %only need y coordinate
rowheight=16;
endrow=ceil(clicked./rowheight);

if endrow<startrow
    a=endrow;
    endrow=startrow;
    startrow=a;
end

originalselection=get(img,'userdata');
%set(src,'userdata',newselection);
numrows=size(originalselection,1);

if startrow<1;startrow=1;end
if endrow>numrows;endrow=numrows;end

selectedrows=false(numrows,1);


if strcmp(get(gcf,'currentmodifier'),'control')
    selectedrows=originalselection;
    selectedrows(startrow:endrow)=true; 
elseif strcmp(get(gcf,'currentmodifier'),'alt')
    selectedrows=originalselection;
    selectedrows(startrow:endrow)=false;
else
    selectedrows(startrow:endrow)=true; 
end

t=ones(numrows*rowheight,1,3);
t(:,:,1)=unselectedcolor(1);
t(:,:,2)=unselectedcolor(1);
t(:,:,3)=unselectedcolor(1);

set(img,'userdata',selectedrows);

%color according to row state:
selectedrows=find(selectedrows);
adders=repmat(0:rowheight-1,size(selectedrows,1),1);
pixes=selectedrows(:).*rowheight-rowheight;
pixes=bsxfun(@plus,pixes,adders);
pixes=pixes(:)+1;
t(pixes,:,1)=selectedcolor(1);
t(pixes,:,2)=selectedcolor(2);
t(pixes,:,3)=selectedcolor(3);

%color text:
txts=get(udata.view.tax,'userdata');
set(txts,'color',txt);

a=[0.299 0.587 0.114]'*selectedcolor;

if a<0.5
    set(txts(selectedrows),'color',[1 1 1])
else
    set(txts(selectedrows),'color',[0 0 0])
end


set(img,'cdata',t);

set(src,'windowbuttonupfcn',{@listboxdragdone,selectedrows,ai,callback})

function listboxdragdone(src,ev,selectedrows,ai,callback)
fig=ancestor(src,'figure');
set(fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(fig,'windowbuttonupfcn','')

callback(src,ai,selectedrows);






