function [axhandle,javahandle]=drawpopupmenu(parent,pos,string,value,callback,backgroundcolor,jobj)
% %DRAWPOPUPMENU draw a popupmenu using non-java objects.
%
% ALL INPUTS ARE REQUIRED (except the last one, background color, is optional)
% parent is the figure or uipanel on which to place the button.
% pos is the position of the button in pixels 
% string is a cell array of strings, containing the options for the popup
%     menu
% value is an integer value indexing into string, indicating the initial
%     entry to be selected in this popup menu
% callback is a function (handle) that runs whenever the pushbutton is clicked.
%     This function must take 2 inputs: the first is the handle of the
%     figure containing the pushbutton, the second is the new value of the popup menu. 
% backgroundcolor is the color of the next visible object behind this,
%     used to recolor the border (if empty, it will be white). 
% jobj is an optional jComboBox (or equivalent) to use as the java popup.
%     If empty, a new one will be created.
% The first output, axhandles, is the axes handle that comprises the
%     popupmenu.
% The second output, javahandle, is an invisible JComboBox that is used to
%     display the popup portion of the menu when it is clicked.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%this function achieves the popupness by using an invisible JComboBox that
%is co-located with the fake popupmenu. Clicking the fake popupmenu makes
%the editor of the JComboBox appear. This seems to achieve the
%desired effect with no nasty side effects...

pos=ceil(pos); %ensure integers

if numel(value)>1;
    axhandle=[];
    javahandle=[];
    error('drawpopupmenu can only be used with a scalar value. use a listbox for multiple values')
    return
end


%make java box:

% mh=uicontrol('units','pixels','parent',parent,'position',[pos(1:3) 1],'visible','on','style','popupmenu','string',{''});
% m=findjobj(mh);
% 
% m.setPreferredSize(java.awt.Dimension(pos(3),1))
% m.setSize(java.awt.Dimension(pos(3),1))
% 
% p=m.getParent;
% p.setPreferredSize(java.awt.Dimension(pos(3),1))
% p.setSize(java.awt.Dimension(pos(3),1))
% 

%I guess we'll do this the hard way:
%peer = get(ancestor(parent,'figure'),'JavaFrame');
j=javax.swing.JComboBox(string);
% javaObjectEDT(j);
% m=handle(j,'callbackProperties');
% hgp = handle(peer.addchild(j));
% mh=hgjavacomponent('parent',parent,'units','pixels');
% hgp.setUIContainer(double(mh));
% set(mh,'javapeer',m)
% 

j.setSelectedIndex(value-1);

[m,mh]=javacomponent(j,[pos(1:3) 0.1],parent);

%make main axes:
a=axes('parent',parent,'units','pixels','position',pos,'visible','off');

axhandle=a;
javahandle=mh;
%set(javahandle,'deletefcn',''); %remove deletefcns that are used in newer matlab versions and dont work in older versions.

if pos(3)<26 || pos(4) < 10
    warning('Size of drawpopupmenu must be at least 26x10 pixels. Not rendering menu.')
    set(a,'visible','off')
    return
end

if ~exist('backgroundcolor','var')
    backgroundcolor=[1 1 1];
end


bg=[1 1 1];
bgimage=ones(pos(4),pos(3),3);
bgimage(:,:,1)=bg(1)*bgimage(:,:,1);
bgimage(:,:,2)=bg(2)*bgimage(:,:,2);
bgimage(:,:,3)=bg(3)*bgimage(:,:,3);

bgimage(:,end-16:end,1)=213/255;
bgimage(:,end-16:end,2)=223/255;
bgimage(:,end-16:end,3)=233/255;


%add button border:
bgimage(:,end-16,1)=118/255;
bgimage(:,end-16,2)=138/255;
bgimage(:,end-16,3)=158/255;

bgimage(1,:,:)=152/255;
bgimage(end,3:end-2,:)=188/255;
bgimage(end-1,:,:)=91/255;
bgimage(end,end-1:end,:)=1;

bgimage(1,end-16:end,1)=99/255;
bgimage(1,end-16:end,2)=119/255;
bgimage(1,end-16:end,3)=139/255;

bgimage(end-1,end-16:end,1)=38/255;
bgimage(end-1,end-16:end,2)=58/255;
bgimage(end-1,end-16:end,3)=78/255;





%make corners:
tlcorner=[255 255 189 167;
    255 173 194 255;
    186 197 255 255;
    160 237 255 255]./255;

blcorner = [122 212 255 255;
    152 157 255 255;
    204 129 155 210;
    255 198 145 114]./255;

trcorner=[141 172 216 255;
    220 171 149 216;
    223 223 169 169;
    223 223 219 132]./255;

brcorner=[223 223 208 093;
    223 223 140 135;
    207 138 104 201;
    085 129 194 255]./255;

tlcorner=repmat(tlcorner,[1,1,3]);
blcorner=repmat(blcorner,[1,1,3]);
trcorner=repmat(trcorner,[1,1,3]);
brcorner=repmat(brcorner,[1,1,3]);

trcorner(:,:,1)=trcorner(:,:,1)-10/255;
trcorner(:,:,3)=trcorner(:,:,3)+10/255;
brcorner(:,:,1)=brcorner(:,:,1)-10/255;
brcorner(:,:,3)=brcorner(:,:,3)+10/255;

trcorner(trcorner>1)=1;
brcorner(brcorner>1)=1;


bgimage(1:4,1:4,:)=tlcorner;
bgimage(end-4:end-1,1:4,:)=blcorner;
bgimage(1:4,end-3:end,:)=trcorner;
bgimage(end-4:end-1,end-3:end,:)=brcorner;
bgimage(5:end-5,1,:)=repmat(linspace(160,122,pos(4)-9)'./255,[1,1,3]);
bgimage(5:end-5,end,:)=repmat(linspace(110,74,pos(4)-9)'./255,[1,1,3]);

arrow=double(imread('downarrow.bmp'))./255;
bgimage(floor(pos(4)/2)-2:floor(pos(4)/2)+3,end-11:end-5,:)=arrow;


adata=ones(size(bgimage,1),size(bgimage,2));

%blend with background:
atlcorner=[0 0 0.8;
    0 1 1
    0.8 1 1];
adata(1:3,1:3)=atlcorner;

ablcorner=[0.8 1 1;
    0 0.8 1;
    0 0 0.5];
adata(end-2:end,1:3)=ablcorner;

atrcorner=[1 0.8 0;
    1 1 0.8;
    1 1 1];
adata(1:3,end-2:end)=atrcorner;

abrcorner=[1 1 0.8;
    1 1 0;
    0.8 0 0];
adata(end-2:end,end-2:end)=abrcorner;


mixdata=ones(size(bgimage));
mixdata(:,:,1)=backgroundcolor(1)*mixdata(:,:,1);
mixdata(:,:,2)=backgroundcolor(2)*mixdata(:,:,2);
mixdata(:,:,3)=backgroundcolor(3)*mixdata(:,:,3);

bgimage=bsxfun(@times,adata,bgimage)+bsxfun(@times,1-adata,mixdata);


%make text:
imdata=maketextimage(string{value},pos);
bgimage(2:end-2,6:end-17,:)=imdata;

asfd=1;
%
%t=text(0,pos(4)/2,[' ',string{value}],'parent',a,'verticalalignment','middle');
i=image('cdata',bgimage,'parent',a,'hittest','on','buttondownfcn',{@popupmenu_callback,j},'userdata',callback);
axis(a,'image')
%uistack(t,'top');
set(a,'visible','off','ydir','reverse')



set(m,'ActionPerformedCallback',{@popupactivation,i});



function popupmenu_callback(src,ev,j)
j.setPopupVisible(true);


function popupactivation(jsrc,jev,i)
%i is image. update its cdata...
string=jsrc.getSelectedItem;
value=jsrc.getSelectedIndex+1;
cdata=get(i,'cdata');
pos=[0 0 size(cdata,2),size(cdata,1)];
imdata=maketextimage(string,pos);
cdata(2:end-2,6:end-17,:)=imdata;
set(i,'cdata',cdata);

callback=get(i,'userdata');

handles=guidata(i);

if iscell(callback)
    c=callback{1};
    c(handles.fig,value,callback{2:end});
else
    callback(handles.fig,value);
end
    
    
    
    



function imdata=maketextimage(string,pos)
%make imagedata of text in string limited to position pos
pos=pos-[0 0 22 3];
f=figure('visible','off','position',[100 100 pos(3) pos(4)],'toolbar','none','menubar','none','color',[1 1 1]);
a=axes('units','pixels','position',[0 0 pos(3:4)],'ylimmode','manual','xlimmode','manual','visible','off');
text(0,0.5,string,'fontsize',9,'interpreter','none')
resolution=get(0,'ScreenPixelsPerInch');
set(f,'papersize',pos(3:4)/resolution,'paperposition',[0 0 pos(3:4)/resolution])
imdata=hardcopy(f,'-dzbuffer','-r96');
imdata=double(imdata)./255;
delete(f);


