function controls=makeuicontrolsderivativefield(src,ev)
%Derivative Field
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');

if handles.isfunction{udata.view.dataset}(udata.view.x) && handles.isfunction{udata.view.dataset}(udata.view.y)
    %do stuff
    
    %filtering uicontrols:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    settingsheader=text(.15-.10317,.85*1.0317-.0317,'Settings','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,...
        'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    
    resolutiontext=text(.25-.10317,.80*1.0317-.0317,'Grid Resolution:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    resolutionedit=uicontrol('position',[origin(1)+190 origin(2)+488 50 22],...
        'style','edit','string','30','callback',[],...
        'backgroundcolor',[1 1 1],'tag','resolutionedit');
    
    poptext=text(.25-.10317,.755*1.0317-.0317,'Population Size:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    gentext=text(.25-.10317,.707*1.0317-.0317,'Generations:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    popedit=uicontrol('position',[origin(1)+190 origin(2)+458 50 22],...
        'style','edit','string','300','callback',[],...
        'backgroundcolor',[1 1 1],'tag','popedit');
    
    genedit=uicontrol('position',[origin(1)+135 origin(2)+428 105 22],...
        'style','edit','string','20 20 20 80','callback',[],...
        'backgroundcolor',[1 1 1],'tag','genedit');
    
    gobutton=uicontrol('position',[origin(1)+65 origin(2)+380 165 30],...
        'style','pushbutton','string','Generate Grid','backgroundcolor',handles.colors.buttonbg,...
        'callback',@royal_sampler,'enable','on','tag','gobutton');
    
    vectorheader=text(.15-.10317,.55*1.0317-.0317,'Vector Field','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,...
        'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    
    derivtext=text(.25-.10317,.52*1.0317-.0317,'Choose Derivatives to View','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none',...
        'tag','derivtext','visible','on');
    
    %get variables for these two functions:
    funvars=union(handles.funvars{udata.view.dataset}{udata.view.x},handles.funvars{udata.view.dataset}{udata.view.y});
    varnames=handles.allnames{udata.view.dataset}(funvars);
    
    derivlist=uicontrol('position',[origin(1)+60 origin(2)+160 180 145],...
        'style','listbox','string',[varnames,{'<none>'}],...
        'callback',{@makederivativefield,handles.fig},...
        'backgroundcolor',[1 1 1],'tag','derivlist');
    
    statusheader=text(.15-.10317,.15*1.0317-.0317,'Status','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,...
        'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    
    statustext=text(.25-.10317,.1*1.0317-.0317,'Idle...','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none',...
        'tag','statustext','visible','on');
    
    timetext=text(.25-.10317,.05*1.0317-.0317,'Elapsed Time: 0 sec','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none',...
        'tag','timetext','visible','on');
    
    scaletext=text(.25-.10317,.25*1.0317-.0317,'Arrow Size:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none',...
        'tag','scale','visible','on');
    
    %arrow size slider:
    [jslider hslider]=javacomponent('javax.swing.JSlider',[origin(1)+110 origin(2)+118 140 40]);
    jslider.setPaintTicks(1)
    
    jslider.setSnapToTicks(1)
    jslider.setMinimum(1)
    jslider.setMaximum(10)
    jslider.setMajorTickSpacing(1);
    jslider.setValue(10);
    set(jslider,'background',java.awt.Color.white)
    
    set(jslider,'StateChangedCallback',{@derivative_jslider_Callback,handles.fig});
    
%     handles.javacomps=jslider;
    handles.temp.finalgrid=2^.5;
    
    handles.tabobjects(end+1:end+16)=[resolutiontext resolutionedit settingsheader    ...
        gobutton poptext gentext popedit genedit statusheader statustext timetext ...
        vectorheader derivtext derivlist scaletext hslider];
    
else
    %explain that they can't do that.
    nonotext=text(.25-.10317,.820*1.0317-.0317,{'This feature is only available';'if both variables are modeled';'by regressions.'},...
        'horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],...
        'fontweight','bold','backgroundcolor','none');
    handles.tabobjects(end+1)=nonotext;
    
end



guidata(src,handles);


function derivative_jslider_Callback(src,eventdata,fighandle) %#ok<INUSL>
%convert java handle to figure handle:
moving = get(src,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end

makederivativefield(fighandle,[],fighandle);



function makederivativefield(src,ev,fighandle)
handles=guidata(fighandle);
src=findobj(handles.tabobjects,'tag','derivlist');

udata=get(handles.fakeax(handles.activeindex),'userdata');
delete(findobj(handles.activegraph,'tag','derivativefield'))

%calculate derivatives at these points for these variables:

%get points:

thegrid=findobj(handles.activegraph,'tag','royalgrid');
if isempty(thegrid);return;end

xdata=get(thegrid,'userdata');

%xdata=handles.temp.royalinputs;

%get index of selected vars:

val=get(src,'value');
string=get(src,'string');
namez=string(val);

indexx=find(ismember(handles.allnames{udata.view.dataset},namez));

if ~isempty(indexx)
    set(thegrid,'visible','off')
else
    set(thegrid,'visible','on')
end


%determine points to evaluate derivative:
del=mean(xdata(:,indexx))*.001;

if isempty(del)
    del=0;
end

lowerpoints=xdata;
lowerpoints(:,indexx)=xdata(:,indexx)-del;
upperpoints=xdata;
upperpoints(:,indexx)=xdata(:,indexx)+del;

basepointsout=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],xdata);
lowerpointsout=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],lowerpoints);
upperpointsout=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],upperpoints);

basepointsx=basepointsout(:,1);
lowerpointsx=lowerpointsout(:,1);
upperpointsx=upperpointsout(:,1);

basepointsy=basepointsout(:,2);
lowerpointsy=lowerpointsout(:,2);
upperpointsy=upperpointsout(:,2);

xdiff=upperpointsx-lowerpointsx;
ydiff=upperpointsy-lowerpointsy;

cax=handles.activegraph;
xlim=get(cax,'xlim');
ylim=get(cax,'ylim');
pos=get(cax,'pos');
xpixperunit=pos(3)/diff(xlim);
ypixperunit=pos(4)/diff(ylim);

x=(basepointsx-xlim(1))*xpixperunit;
y=(basepointsy-ylim(1))*ypixperunit;
u=xdiff*xpixperunit;
v=ydiff*ypixperunit;


alpha = 0.33; % length of arrow head


mslider=findobj(handles.tabobjects,'type','hgjavacomponent');
jslider=get(mslider,'javapeer');

autoscale=jslider.getValue;
autoscale=autoscale/10;

if autoscale  %#ok<BDLGI>
    % Base autoscale value on average spacing in the x and y
    % directions.  Estimate number of points in each direction as
    % either the size of the input arrays or the effective square
    % spacing if x and y are vectors.
    if min(size(x))==1, n=sqrt(numel(x)); m=n; else [m,n]=size(x); end
    delx = diff([min(x(:)) max(x(:))])/n;
    dely = diff([min(y(:)) max(y(:))])/m;
    del = delx.^2 + dely.^2;
    if del>0
        len = sqrt((u.^2 + v.^2)/del);
        maxlen = max(len(:));
    else
        maxlen = 0;
    end
    
    if maxlen>0
        autoscale = autoscale*0.9 / maxlen;
    else
        autoscale = autoscale*0.9;
    end
    u = u*autoscale; v = v*autoscale;
end

% Make velocity vectors
x = x(:).'; 
y = y(:).';
u = u(:).'; 
v = v(:).';
uu = [x;x+u;repmat(NaN,size(u))];
vv = [y;y+v;repmat(NaN,size(u))];

x1=x+u*(1-alpha);
y1=y+v*(1-alpha);

x2=x+u;
y2=y+v;

phi=20*pi/180;

x3=x2+(x1-x2)*cos(phi) + (y1-y2)*sin(phi);
y3=y2-(x1-x2)*sin(phi) + (y1-y2)*cos(phi);

x4=x2+(x1-x2)*cos(-phi) + (y1-y2)*sin(-phi);
y4=y2-(x1-x2)*sin(-phi) + (y1-y2)*cos(-phi);

%convert back to graph units:
x2=x2/xpixperunit+xlim(1);
x3=x3/xpixperunit+xlim(1);
x4=x4/xpixperunit+xlim(1);
uu=uu/xpixperunit+xlim(1);

y2=y2/ypixperunit+ylim(1);
y3=y3/ypixperunit+ylim(1);
y4=y4/ypixperunit+ylim(1);

vv=vv/ypixperunit+ylim(1);

%draw center line
h1=line(uu(:),vv(:),'parent',cax,'buttondownfcn',@lineobject_callback,'hittest','on','tag','derivativefield');


hu=[x3;x2;x4;repmat(NaN,size(u))];
hv=[y3;y2;y4;repmat(NaN,size(u))];

%draw head
h2=line(hu(:),hv(:),'parent',cax,'tag','derivativefield','buttondownfcn',@lineobject_callback);

udat.handles=[h1 h2];
udat.delete=1;

set([h1 h2],'userdata',udat);

uistack([h1 h2],'bottom'); %seems like more often than not you'd want this below the data.


function writederivative(src,ev) %display the derivative of the selected arrow in the infobar:
%note: actually calculates derivative at clicked point and displays that. Can't directly get derivs
%from arrows cause they all share the same xdata/ydata.

delete(findobj('tag','derivativefieldnote'));

handles=guidata(src);

clicked=get(get(src,'parent'),'currentpoint');

%calculate distance from this point to all points in x/y:
xc=clicked(1,1);
yc=clicked(1,2);

xdata=get(src,'xdata');
ydata=get(src,'ydata');

x=xdata-xc;
y=ydata-yc;

y=y./mean(y(~isnan(y)))*mean(x(~isnan(x))); %scale data.

distt=hypot(x,y);

index=find(distt==min(distt));

line(xdata(index),ydata(index),'marker','o','markersize',6,'markeredgecolor',[1 0 0],'tag','derivativefieldnote','parent',get(src,'parent'));

%look 1 in either direction to get bounds:
if index==1
    index=[index 2];
elseif index==length(x);
    index=[index-1 index];
else
    index=[index-1 index index+1];
end

x=xdata(index);
y=ydata(index);

xd=diff(x);
yd=diff(y);

xd=xd(~isnan(xd));
yd=yd(~isnan(yd));

deriv=yd/xd;
set(handles.infobar,'string',['Selected Derivative= ',num2str(deriv)]);



