function controls=makeuicontrolsoverlaycontours(src,ev)

%Overlay Contours (UNDER CONSTRUCTION)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');



if handles.isfunction{udata.view.dataset}(udata.view.x) && handles.isfunction{udata.view.dataset}(udata.view.y)
    %do stuff
    
    settingsheader=text(.15-.10317,.85*1.0317-.0317,'Settings','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,...
        'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    
    typetext=text(.25-.10317,.82*1.0317-.0317,'Choose objective function:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    oemmenu=uicontrol('position',[origin(1)+70 origin(2)+480 160 22],'style','popupmenu',...
        'string',{'Simple Additive Weighting','OEC'},'value',1,'max',1);
    
    weightstext=text(.25-.10317,.72*1.0317-.0317,'Weights:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    weightscheck=uicontrol('position',[origin(1)+70 origin(2)+422 160 18],'style','checkbox',...
        'string','Normalize Weights','value',1,'backgroundcolor',handles.colors.tabbg,'tag','weightscheck');
    
    %make magic box:
    %% this text copied straight from magicbox function:
    %format position:
    pos=[20 300 230 120]; %250
    
    d=udata.view.dataset;
    xvar=udata.view.x;
    yvar=udata.view.y;
    
    allnames=handles.allnames{d};
    
    varnames=allnames([xvar yvar]);
    
    nvars=numel(varnames);
    
    ranges=repmat([0 1],[max([xvar yvar]) 1]);
    handles.pprof.ranges=ranges;
    
    ndisp=floor(pos(4)/14.7); %number of rows of text that will fit in the box
    
    axpos=[origin(1)+pos(1) origin(2)+pos(2) pos(3)-18 pos(4)];
    slpos=[origin(1)+pos(1)+pos(3)-17 origin(2)+pos(2)-1 17 pos(4)+1];
    
    %create the axes:
    handles.pprof.ax=axes('units',get(handles.fig,'units'),'position',axpos,...
        'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],...
        'xtickmode','manual','ytickmode','manual','xtick',[],'ytick',[],'box','on',...
        'xcolor',[.7 .7 .7 ],'ycolor',[.7 .7 .7]);
    
    %create the slidebar:
    handles.pprof.slider=uicontrol('units',get(handles.fig,'units'),'position',slpos,...
        'style','slider','callback',@pprof_scroll);
    
    cmap=handles.variablecolors{udata.view.dataset}(union(xvar,yvar),:); %#ok<NASGU>
    
    %initialize variables: this must be done to avoid errors.
    L=zeros(1,nvars);
    handles.pprof.blobs=L;
    handles.pprof.valtexts=L;
    
    %make box:
    if nvars>ndisp
        for t=1:nvars
            thisvar=find(strcmp(allnames,varnames{end-t+1}));
            text(0.38,t/ndisp,varnames{end-t+1},'horizontalalignment','right','clipping','on','verticalalignment','top');
            
            set(handles.pprof.slider,'max',nvars/ndisp-1,'value',nvars/ndisp-1);
            set(handles.pprof.ax,'ylim',[nvars/ndisp-1 nvars/ndisp]);
            range=get(handles.pprof.slider,'max')-get(handles.pprof.slider,'min');
            set(handles.pprof.slider,'sliderstep',[1/(ndisp*range)-.0001 (1-1/ndisp)/range])
            
            L(t)=line([0.43 .505 .580 .655 .730],repmat(t/ndisp-.08,[1 5]),'linewidth',1,'color',[0 0 0],'marker','+');
            
            handles.pprof.blobs(thisvar)=line(0.58,t/ndisp-.08,'linestyle','none','markerfacecolor',handles.colors.text,'marker','o',...
                'markeredgecolor',[0 0 0],'buttondownfcn',@pprofclick,'userdata',thisvar,'tag','valuegrabber');
            
            handles.pprof.valtexts(thisvar)=text(0.78,t/ndisp,num2str(mean(ranges(thisvar,:))),...
                'horizontalalignment','left','clipping','on','verticalalignment','top','userdata',thisvar);
        end
        
    else %small amount of variables... don't need slider
        for t=1:nvars
            thisvar=find(strcmp(allnames,varnames{end-t+1}));
            text(0.38,(t+ndisp-nvars)/ndisp,varnames{end-t+1},...
                'horizontalalignment','right','clipping','off','verticalalignment','top');
            
            set(handles.pprof.slider,'enable','off','visible','off')
            set(handles.pprof.ax,'visible','off')
            
            L(t)=line([0.430 0.505 0.580 0.655 .730],repmat((t+ndisp-nvars)/ndisp-.07,[1 5]),...
                'linewidth',1,'color',[0 0 0],'marker','+');
            
            handles.pprof.blobs(thisvar)=line(0.58,(t+ndisp-nvars)/ndisp-.07,'linestyle','none',...
                'markerfacecolor',handles.colors.text,'marker','o',...
                'markeredgecolor',[0 0 0],'buttondownfcn',@valuecontour_click,'userdata',thisvar,'tag','valuegrabber');
            
            handles.pprof.valtexts(thisvar)=text(0.78,(t+ndisp-nvars)/ndisp,num2str(mean(ranges(thisvar,:))),...
                'horizontalalignment','left','clipping','off','verticalalignment','top','userdata',thisvar);
        end
        
    end
    
    
    controls=[handles.pprof.ax handles.pprof.slider handles.pprof.valtexts(handles.pprof.valtexts>0),...
        settingsheader typetext oemmenu weightstext weightscheck];
    
    
    
    
else
    %explain that they can't do that.
    nonotext=text(.25-.10317,.820*1.0317-.0317,{'This feature is only available';'if both variables are modeled';'by regressions.'},...
        'horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],...
        'fontweight','bold','backgroundcolor','none');
    
    controls=nonotext;
end


guidata(src,handles);

function pprof_scroll(src,eventdata)
%this is the callback for scrolling the magic box in pprof.
handles=guidata(src);

set(handles.pprof.ax,'ylim',[get(handles.pprof.slider,'value') get(handles.pprof.slider,'value')+1]);


function valuecontour_click(hObject,ev)

handles=guidata(hObject);
set(handles.fig,'windowbuttonmotionfcn',{@valueslide,handles.fig})
set(handles.fig,'windowbuttonupfcn',{@valuedone,handles.fig})

function valueslide(hObject,eventdata,fig)
handles=guidata(fig);
udata=get(handles.fakeax(handles.activeindex),'userdata');

weightscheck=findobj(handles.tabobjects,'tag','weightscheck');


clicked=get(handles.pprof.ax,'currentpoint');
val=clicked(1,1);

%move marker within bounds
if val>0.73
    val=0.73;
elseif val<0.43
    val=0.43;
end


set(gco,'xdata',val)

%update text values
thisvar=get(gco,'userdata');
range=handles.pprof.ranges(thisvar,:);
if range(1)-range(2)==0;return;end %avoids errors with constant variables.

newval=range(1)+((val-0.43)/0.3)*(range(2)-range(1));

set(handles.pprof.valtexts(thisvar),'string',num2str(newval));


%% NOTE: this only works for 2 var case. need to figure out what to do for higher dims.
if get(weightscheck,'value') %also move other slider to add up to 1.
    grabberz=findobj('tag','valuegrabber');
    varz=[udata.view.x udata.view.y];
    othervar=setdiff(varz,thisvar);
    othergrabber=setdiff(grabberz,gco);
    range=handles.pprof.ranges(othervar,:);
    otherval=1-newval;
    val=0.43+((otherval-range(1))/(range(2)-range(1)))*.3;
    set(handles.pprof.valtexts(othervar),'string',num2str(otherval));
    set(othergrabber,'xdata',val);
end


function valuedone(hObject,eventdata,fig)
handles=guidata(fig);
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonupfcn','')



