function controls=makeuicontrolsplaneprofiler(src,ev)
%Response Profiler
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');

%delete exising profiler:
if isfield(udata.explore,'pprof')
    allhandles=[udata.explore.pprof.linepaths udata.explore.pprof.lineplus udata.explore.pprof.tickpaths udata.explore.pprof.pspot];
    delete(allhandles(ishandle(allhandles))); 
end


xcol=udata.view.x;
ycol=udata.view.y;

d=udata.view.dataset;
a=udata.view.analysis;

vars=union(handles.funvars{d}{xcol},handles.funvars{d}{ycol});
allnames=handles.allnames{d};
varnames=allnames(vars);
nvars=numel(varnames);

ranges=[handles.minlimit{d}{a}' handles.maxlimit{d}{a}'];
xdials=handles.xdials{d}{a};
datatypes=handles.datatype{d};

raveresizeaxes(src,[],handles.activeindex)
clear ravearrangeobjects

pos=[20 300 230 220]; %250
axpos=[origin(1)+pos(1) origin(2)+pos(2) pos(3)-18 pos(4)];
slpos=[origin(1)+pos(1)+pos(3)-17 origin(2)+pos(2)-1 17 pos(4)+1];

rpix=14.7; %pixels per row of text.
tpix=rpix*(nvars+1); %total number of pixels needed to display all vars.
dpix = axpos(4); %number of pixels visible at any time.

ndisp=floor(axpos(4)/rpix);

%create the axes:
handles.pprof.ax=axes('units',get(handles.fig,'units'),'position',axpos,...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[1 ndisp*rpix],...
    'xtickmode','manual','ytickmode','manual','xtick',[],'ytick',[],'box','on',...
    'xcolor',[.7 .7 .7 ],'ycolor',[.7 .7 .7]);

cmap=handles.variablecolors{udata.view.dataset};

%initialize variables: this must be done to avoid errors.
L=zeros(1,nvars);
handles.pprof.blobs=L;
handles.pprof.valtexts=L;

%normalize xdials to range used in magicbox:
startvalues=(xdials-ranges(:,1)')./(ranges(:,2)'-ranges(:,1)')*.3+.43;
startvalues(isinf(startvalues))=0.58; %mid range.
startvalues(isnan(startvalues))=0.58;

%make box:
downer=rpix/2;

%% this text copied straight from magicbox function:

%make box:
for t=1:nvars
    thisvar=vars(end-t+1);
    thistype=datatypes{thisvar};
    val=xdials(thisvar);
    
    %make current value texts:
    if ~any(strcmp(thistype,{'string','logical'}))
        valtext=strtrim(sprintf('%5.5g',val));
    elseif strcmp(thistype,'string')
        valtext=handles.discretevalues{d}{a}{thisvar}{val};
    else %logical
        if val
            valtext=handles.prefs.logicaltrue;
        else
            valtext=handles.prefs.logicalfalse;
        end
    end
    
    oor=false; %oor=out of range
    if startvalues(thisvar)<.43
        startvalues(thisvar)=.43;
        oor=true;
    elseif startvalues(thisvar)>.73
        startvalues(thisvar)=.73;
        oor=true;
    end
    
    if oor
        startcolor=[1 0 0];
    else
        startcolor=[0 0 0];
    end
    
    text(0.38,t/ndisp*dpix,allnames{thisvar},'horizontalalignment','right','clipping','on','verticalalignment','top','parent',handles.pprof.ax);
    
    L(t)=line([0.43 .505 .580 .655 .730],repmat(t/ndisp*dpix-downer,[1 5]),'linewidth',1,'color',[0 0 0],'marker','+','parent',handles.pprof.ax);
    
    handles.pprof.blobs(end-t+1)=line(startvalues(thisvar),t/ndisp*dpix-downer,'linestyle','none','markerfacecolor',cmap(thisvar,:),'marker','o',...
        'markeredgecolor',[0 0 0],'buttondownfcn',@pprofclick,'userdata',thisvar,'parent',handles.pprof.ax);
    
    handles.pprof.valtexts(end-t+1)=text(0.78,t/ndisp*dpix,valtext,...
        'horizontalalignment','left','clipping','on','verticalalignment','top','userdata',thisvar,...
        'buttondownfcn',@sliderschangevaltexts,'hittest','on','parent',handles.pprof.ax,'color',startcolor);
end

handles.pprof.slider=drawvscroll(handles.wspanel,handles.pprof.ax,slpos,0,tpix,dpix,tpix,@sliderscroll,1);


%store ranges for future use.
udata.explore.pprof.ranges=ranges;
% 
% %on/off switches:
% handles.pprof.onoff=false(numel(handles.allnames{d}),1);

%IDEA: save last slide bar position/onoff and use this as default the next time user chooses
%this option.
%% OTHER UI CONTROLS FOR THIS METHOD:

methodtext=text(.15-.10317,.85*1.0317-.0317,'Variable Values','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

optionstext=text(.15-.10317,.40*1.0317-.0317,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

boldcheck=uicontrol('position',[origin(1)+30 origin(2)+182 120 20],...
    'style','checkbox','string','Use Bold Lines','value',0,'backgroundcolor',handles.colors.tabbg,...
    'horizontalalignment','left','callback',@boldcheck_callback,...
    'fontweight','normal');
tickcheck=uicontrol('position',[origin(1)+30 origin(2)+165 180 20],...
    'style','checkbox','string','Show Move Increment Ticks','value',0,'backgroundcolor',handles.colors.tabbg,...
    'horizontalalignment','left','callback',@tickcheck_callback,...
    'fontweight','normal','tag','tickcheck');

helptext=uicontrol('position',[origin(1)+30 origin(2)+202 200 20],...
    'style','text','string','Click a point to jump to its location.','backgroundcolor',handles.colors.tabbg,...
    'horizontalalignment','left','fontweight','normal');


deletepprof=uicontrol('style','pushbutton','position',[origin(1)+70 origin(2)+30 120 24],'string','Delete Plane Profiler',...
    'tag','deletepprof','callback',@deletepprof_callback);

%get variable values from slide bars:
values=str2num(char(get(handles.pprof.valtexts(handles.pprof.valtexts>0),'string')));
varorder=cell2mat(get(handles.pprof.valtexts(handles.pprof.valtexts>0),'userdata'));

varz=handles.alldata{udata.view.dataset}(1,:);
varz(varorder)=values;

outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],varz);

xvalues=outputs(:,1);
yvalues=outputs(:,2);

%make the white dot at the center:
udata.explore.pprof.pspot=line(xvalues,yvalues,'marker','o','markeredgecolor',[0 0 0],...
    'markerfacecolor',[1 1 1],'markersize',6,'parent',handles.allgraphs(handles.activeindex),'tag','pprof','linestyle','none');
userdata.handles=udata.explore.pprof.pspot;
userdata.delete=0;
set(udata.explore.pprof.pspot,'userdata',userdata,'buttondownfcn',@lineobject_callback)

pos=get(handles.pprof.ax,'position');

%display response values:
handles.pprof.responsevalues=axes('units','pixels','position',[pos(1) origin(2)+266 pos(3) 50],...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],...
    'xtickmode','manual','ytickmode','manual','xtick',[],'ytick',[],'box','off',...
    'xcolor',[.7 .7 .7 ],'ycolor',[.7 .7 .7],'visible','off','tag','junk');
handles.pprof.yvaltext=text(.78,.3,num2str(yvalues,4),...
    'horizontalalignment','left','clipping','off','verticalalignment','top','tag','junk');
handles.pprof.xvaltext=text(.78,.60,num2str(xvalues,4),...
    'horizontalalignment','left','clipping','off','verticalalignment','top','tag','junk');
handles.pprof.yvartext=text(.38,.3,handles.allnames{d}{udata.view.y},...
    'horizontalalignment','right','clipping','on','verticalalignment','top','tag','junk');
handles.pprof.xvartext=text(.38,.60,handles.allnames{d}{udata.view.x},...
    'horizontalalignment','right','clipping','on','verticalalignment','top','tag','junk');


%% MAKE PATHS:
npoints=13; %path resolution

udata.explore.pprof.linepaths=zeros(1,nvars);

udata.explore.pprof.linepaths=zeros(1,nvars);
udata.explore.pprof.lineplus=zeros(1,nvars);
udata.explore.pprof.tickpaths=zeros(1,nvars);


for g=1:nvars
    thisvar=varorder(g);
    thiscolor=cmap(thisvar,:);
    
    startpoint=varz;
    
    range=udata.explore.pprof.ranges(thisvar,:);
    
    steps=interp1([1 npoints],range,1:npoints);
    
    xdata=repmat(startpoint,[npoints 1]);
    xdata(:,thisvar)=steps;
    
    outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],xdata);
    
    xvalues=outputs(:,1);
    yvalues=outputs(:,2);
    udata.explore.pprof.linepaths(g)=line(xvalues,yvalues,'linewidth',1,'marker','none','color',thiscolor,...
        'parent',handles.allgraphs(handles.activeindex),'hittest','on','tag','pprof','buttondownfcn',@lineobject_callback);
    udata.explore.pprof.lineplus(g)=line(xvalues(end),yvalues(end),'marker','+','color',thiscolor,...
        'parent',handles.allgraphs(handles.activeindex),'hittest','on','tag','pprof','buttondownfcn',@lineobject_callback); %#ok<COLND,COLND>
    userdata.handles=[handles.pprof.blobs(g) udata.explore.pprof.linepaths(g) udata.explore.pprof.lineplus(g)];
    userdata.delete=0;
    userdata.thisvar=thisvar;
    set(userdata.handles,'userdata',userdata);
    
end

%% MAKE TICK MARKS:
npoints=5; %path resolution

udata.explore.pprof.tickpaths=zeros(1,nvars);

for p=1:nvars
    thisvar=varorder(p);
    thiscolor=cmap(thisvar,:);
    startpoint=varz;
    range=udata.explore.pprof.ranges(thisvar,:);
    steps=interp1([1 npoints],range,1:npoints);
    
    xdata=repmat(startpoint,[npoints 1]);
    xdata(:,thisvar)=steps;
    
    outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],varz);
    
    xvalues=outputs(:,1);
    yvalues=outputs(:,2);
    
    %make paths:
    udata.explore.pprof.tickpaths(p)=line(xvalues,yvalues,'linestyle','none','marker','.',...
        'markerfacecolor',thiscolor,'markeredgecolor',thiscolor,...
        'parent',handles.allgraphs(handles.activeindex),'hittest','off','visible','off');
end

uistack(udata.explore.pprof.pspot,'top');



%record presence of profiler:
udata.explore.method=union(udata.explore.method,'responseprofiler');
set(handles.fakeax(handles.activeindex),'userdata',udata);


controls=[handles.pprof.ax handles.pprof.slider handles.pprof.valtexts(handles.pprof.valtexts>0),...
    methodtext optionstext boldcheck helptext tickcheck deletepprof handles.pprof.responsevalues];

guidata(src,handles);



function pprofslide(hObject,eventdata,fig) %#ok<INUSL>
handles=guidata(fig);
udata=get(handles.fakeax(handles.activeindex),'userdata');
d=udata.view.dataset;

if ~isempty(hObject) %if this function is called as a callback: (get mouse location and slide bars)
    clicked=get(handles.pprof.ax,'currentpoint');
    val=clicked(1,1);
    
    %move marker within bounds
    if val>0.73
        val=0.73;
    elseif val<0.43
        val=0.43;
    end
    set(gco,'xdata',val)
    
    %update text values
    linedata=get(gco,'userdata');
    thisvar=linedata.thisvar;
    range=udata.explore.pprof.ranges(thisvar,:);
    if range(1)-range(2)==0;return;end %avoids errors with constant variables.
    
    newval=range(1)+((val-0.43)/0.3)*(range(2)-range(1));
    
    %set(handles.pprof.valtexts(thisvar),'string',num2str(newval));
    thisvaltext=findobj(handles.pprof.valtexts,'userdata',thisvar);
    set(thisvaltext,'string',sprintf('%-12.5g',newval));
    
    handles.xdials{udata.view.dataset}{udata.view.analysis}(thisvar)=newval;
    
    
end


guidata(hObject,handles);

raveupdatepprof(hObject,eventdata,fig)

%{

%% UPDATE GRAPH:
%get variable values from slide bars:
values=str2num(char(get(handles.pprof.valtexts(handles.pprof.valtexts>0),'string')));
varorder=cell2mat(get(handles.pprof.valtexts(handles.pprof.valtexts>0),'userdata'));
varz=handles.alldata{udata.view.dataset}(1,:);
varz(varorder)=values;
nvars=numel(values);

outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],varz);

xvalues=outputs(:,1);
yvalues=outputs(:,2);


set(udata.explore.pprof.pspot,'xdata',xvalues,'ydata',yvalues);
set(handles.pprof.xvaltext,'string',sprintf('%-12.8g',xvalues));
set(handles.pprof.yvaltext,'string',sprintf('%-12.8g',yvalues));



npoints=13;
%update paths:
for g=1:nvars
    thisvar=varorder(g);
    %thiscolor=cmap(g,:);
    
    startpoint=varz;
    
    range=handles.pprof.ranges(thisvar,:);
    
    steps=interp1([1 npoints],range,1:npoints);
    
    xdata=repmat(startpoint,[npoints 1]);
    xdata(:,thisvar)=steps;
    
    outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],xdata);
    
    xvalues=outputs(:,1);
    yvalues=outputs(:,2);
    
    set(udata.explore.pprof.linepaths(g),'xdata',xvalues,'ydata',yvalues);
    set(udata.explore.pprof.lineplus(g),'xdata',xvalues(end),'ydata',yvalues(end)); 
    
end

%UPDATE TICK MARKS:
tickcheck=findobj(handles.tabobjects,'tag','tickcheck');
if get(tickcheck,'value') %ticks are on..... do work:
    %% MAKE TICK MARKS:
    npoints=5; %path resolution
    for p=1:nvars
        thisvar=varorder(p);
        %thiscolor=cmap(p,:);
        startpoint=varz;
        range=handles.pprof.ranges(thisvar,:);
        steps=interp1([1 npoints],range,1:npoints);
        
        xdata=repmat(startpoint,[npoints 1]);
        xdata(:,thisvar)=steps;

        outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],xdata);
        xvalues=outputs(:,1);
        yvalues=outputs(:,2);

        %make paths:
        set(udata.explore.pprof.tickpaths(p),'xdata',xvalues,'ydata',yvalues);
        
        
    end
end

%fix axes limits:
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
    'xlim',get(handles.activegraph,'xlim'),'ylim',get(handles.activegraph,'ylim'))

guidata(handles.fig,handles);

%}
function pprofdone(hObject,eventdata,fig) %#ok<INUSL>
handles=guidata(fig);
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonupfcn','')

raveupdateanalysis(handles.fig,[],'xdials',[]);

function boldcheck_callback(hObject,eventdata)
handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
linez=findobj(get(handles.allgraphs(handles.activeindex),'children'),'type','line','-not','marker','o');
if get(hObject,'value') %make bold
    set(linez,'linewidth',2)
    set(udata.explore.pprof.tickpaths,'markersize',12)
    
else %make not bold
    set(linez,'linewidth',1);
    set(udata.explore.pprof.tickpaths','markersize','default')
    
end

function tickcheck_callback(hObject,eventdata)
handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
if get(hObject,'value')
    set(udata.explore.pprof.tickpaths,'visible','on');
else
    set(udata.explore.pprof.tickpaths,'visible','off');
end
pprofslide([],[],handles.fig); %note, first argumnt must remain empty.


function pprofclick(hObject,eventdata)
handles=guidata(hObject);

%change color on right click.
if strcmp(get(handles.fig,'selectiontype'),'alt')
    lineobject_callback(hObject,[]);
    return
end

set(handles.fig,'windowbuttonmotionfcn',{@pprofslide,handles.fig})
set(handles.fig,'windowbuttonupfcn',{@pprofdone,handles.fig})


function deletepprof_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

stuff2delete=findobj(handles.activegraph,'tag','pprof');
delete(stuff2delete);

udata.explore.option=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);

methodmenu=findobj(handles.tabobjects,'tag','methodmenu');
set(methodmenu,'value',1);
drawnow;

exploretab(handles.fig,[]); %refresh tab

% function pprof_scroll(src,eventdata)
% %this is the callback for scrolling the magic box in pprof.
% handles=guidata(src);
% 
% set(handles.pprof.ax,'ylim',[get(handles.pprof.slider,'value') get(handles.pprof.slider,'value')+1]);

