function controls=makeuicontrolsquickmontecarlo(src,ev)
%Quick Monte Carlo
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');




 %Find uniform distributions in output space.
        if handles.isfunction{udata.view.dataset}(udata.view.x) && handles.isfunction{udata.view.dataset}(udata.view.y)
            %do stuff

            mcheader=text(.15-.10317,.85*1.0317-.0317,'Monte Carlo','horizontalalignment','left',...
                'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,...
                'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

            mctext=text(.25-.10317,.8*1.0317-.0317,'Number of Samples','horizontalalignment','left',...
                'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
                'fontname','arial','color',[0 0 0],'fontweight','normal','backgroundcolor','none');

            mcedit=uicontrol('position',[origin(1)+180 origin(2)+488 60 22],...
                'style','edit','string','1000','callback',[],...
                'backgroundcolor',[1 1 1],'tag','mcedit');
            mcbutton=uicontrol('position',[origin(1)+60 origin(2)+440 165 30],...
                'style','pushbutton','string','Add MC Runs','backgroundcolor',handles.colors.buttonbg,...
                'callback',@explore_mc,'enable','on','tag','mcbutton');

            controls=[mcheader mctext mcedit mcbutton ];

        else
            %explain that they can't do that.
            nonotext=text(.25-.10317,.820*1.0317-.0317,{'This feature is only available';'if both variables are modeled';'by regressions.'},...
                'horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
                'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],...
                'fontweight','bold','backgroundcolor','none');
            controls=nonotext;
        end


guidata(src,handles);

function explore_mc(hObject,eventdata)
%quick monte carlo bidness
handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');

x=udata.view.x;
y=udata.view.y;
d=udata.view.dataset;

%get number of points to add:
mcedit=findobj('tag','mcedit');

numpts=str2double(get(mcedit,'string'));

%get col numbers of input variables that will be changed (sorted)
indexx=union(handles.funvars{d}{x},handles.funvars{d}{y});

nvars=numel(indexx);%(number of input vars) not the same as size(d,2)

ranges=[min(handles.alldata{d}); max(handles.alldata{d})];

baseline=ranges(1,:);
multiplier=ranges(2,:)-ranges(1,:);


%make mc:
mc=rand(numpts,nvars);

MC=zeros(numpts,size(handles.alldata{d},2));
MC(:,indexx)=mc;
MC=bsxfun(@times,MC,multiplier);
MC=bsxfun(@plus,MC,baseline);

%evaluate these points:

objective=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],MC);


%plot results:
line(objective(:,1),objective(:,2),'marker','+','linestyle','none','parent',handles.activegraph,...
    'tag','junk','linewidth',1,'markerfacecolor',[0 1 1],'markersize',2,'visible','on','markeredgecolor',[0 1 1]);

