function controls=makeuicontrolsroyalsampler(src,ev)
%Inverse Monte Carlo
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');


if handles.isfunction{udata.view.dataset}(udata.view.x) && handles.isfunction{udata.view.dataset}(udata.view.y)
    %do stuff
    
    %filtering uicontrols:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    settingsheader=text(.15-.10317,.85*1.0317-.0317,'Settings','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,...
        'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    
    resolutiontext=text(.30-.10317,.80*1.0317-.0317,'Grid Resolution','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    resolutionedit=uicontrol('position',[origin(1)+190 origin(2)+488 50 22],...
        'style','edit','string','50','callback',[],...
        'backgroundcolor',[1 1 1],'tag','resolutionedit');
    
    poptext=text(.30-.10317,.755*1.0317-.0317,'Population Size','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    gentext=text(.30-.10317,.707*1.0317-.0317,'Generations','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    popedit=uicontrol('position',[origin(1)+190 origin(2)+458 50 22],...
        'style','edit','string','300','callback',[],...
        'backgroundcolor',[1 1 1],'tag','popedit');
    
    genedit=uicontrol('position',[origin(1)+135 origin(2)+428 105 22],...
        'style','edit','string','20 20 20 80','callback',[],...
        'backgroundcolor',[1 1 1],'tag','genedit');
    
    gobutton=uicontrol('position',[origin(1)+65 origin(2)+380 165 30],...
        'style','pushbutton','string','Start Sampling','backgroundcolor',handles.colors.buttonbg,...
        'callback',@royalsampler_helper,'enable','on','tag','gobutton');
    
    statusheader=text(.15-.10317,.55*1.0317-.0317,'Status','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,...
        'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    
    statustext=text(.30-.10317,.50*1.0317-.0317,'Idle...','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none',...
        'tag','statustext','visible','on');
    
    timetext=text(.30-.10317,.45*1.0317-.0317,'Elapsed Time: 0 sec','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none',...
        'tag','timetext','visible','on');
    
    exportbutton=uicontrol('position',[origin(1)+65 origin(2)+200 165 30],...
        'style','pushbutton','string','Export Results as .txt','backgroundcolor',handles.colors.buttonbg,...
        'callback',@royalsampler_export,'enable','on','tag','exportbutton','enable','off');
    
    controls=[resolutiontext resolutionedit settingsheader    ...
        gobutton poptext gentext popedit genedit statusheader statustext timetext exportbutton];
    
else
    %explain that they can't do that.
    nonotext=text(.25-.10317,.820*1.0317-.0317,{'This feature is only available';'if both variables are modeled';'by regressions.'},...
        'horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],...
        'fontweight','bold','backgroundcolor','none');
    controls=nonotext;
end


guidata(src,handles);


function royalsampler_export(hObject,eventdata)
handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');

[name,path,index]=uiputfile({'*.txt';'*.*'},'Save data as:'); %#ok<NASGU>
if name~=0
    
    
    indata=handles.temp.royalinputs;
    
    
    
    funnames=handles.funnames{udata.view.dataset};
    
    outdata=zeros(size(indata,1),size(funnames,2));
    
    nargs=handles.nargout{udata.view.dataset};
    thisonedone=false(size(funnames));
    for g=1:numel(funnames)
        if isempty(funnames{g}) || thisonedone(g)
            thisonedone(g)=true;
            continue
        end
        
        thisname=funnames{g};
        samenames=strcmp(thisname,funnames);
        thisonedone(samenames)=true;
        thisargs=nargs(samenames);
        
        y=raveevaluatefunction(thisname,indata,thisargs);
        
        outdata(:,samenames)=y;
        
        
    end
    
    outdata(:,~handles.isfunction{udata.view.dataset})=indata(:,~handles.isfunction{udata.view.dataset});
    
    varnames=handles.allnames{udata.view.dataset};
    
    printnames=varnames{1};
    
    for j=2:length(varnames)
        printnames=[printnames,'\t',varnames{j}]; %#ok<AGROW>
    end
    printnames=[printnames,'\n'];
    
    
    fid=fopen([path name],'w');
    fprintf(fid,printnames);
    fclose(fid);
    dlmwrite([path name],outdata, '-append','delimiter','\t','precision',15);
end

