function exportappenddatagui(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
handles.temp=[];
guidata(handles.fig,handles);

%make figure:
f=figure('units','pixels','position',[0 0 651 300],'name',...
    'Save/Export Optimization Results','numbertitle','off','menubar','none','resize','off',...
    'color',handles.colors.frame,'windowstyle','modal');
origin=get(handles.fig,'position');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

%make tabs:
titleaxes=axes('units','pixels','position',[11 699 485 41],'parent',f,'visible','off','tag','titleaxes');

closebutton=uicontrol('style','pushbutton','units','pixels','position',[465 10 180 30],'tag','cancelbutton',...
    'fontsize',16,'string','Close','callback',@closebutton_Callback);

% %make titles
% text(0,.2,'Re-Sample Data Set','parent',titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
%     'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');

u=uipanel('parent',f,'units','pixels','position',[50 50 550 240],'backgroundcolor',handles.colors.tabbg);

%%

a1=axes('parent',u,'units','pixels','position',[5 203 200 40],'visible','off');
text(0,0.5,'1. Choose Optimization Results','parent',a1,'fontname',handles.prefs.fonts.tab,...
    'color',handles.colors.text,'fontsize',16*handles.prefs.fonts.tabscale);

names={};
idx=false(numel(handles.optimizerresults),1);
for n=numel(handles.optimizerresults):-1:1
    if isfield(handles.optimizerresults{n},'name')
        names{end+1}=handles.optimizerresults{n}.name;
    idx(n)=true;
    end
end

names=fliplr(names);



uicontrol('parent',u,'position',[35 187 500 22],'style','text',...
    'string','(Most recent optimization results are at the bottom.)',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',8,'hor','left');

resultsmenu=uicontrol('parent',u,'position',[35 168 500 22],'style','popupmenu',...
    'string',names,'value',numel(names),...
    'tag','resultsmenu','backgroundcolor',[1 1 1],'userdata',find(idx));


%%
a3=axes('parent',u,'units','pixels','position',[5 105+46-20 400 40],'visible','off');
text(0,0.5,'2. Choose Destination','parent',a3,'fontname',handles.prefs.fonts.tab,...
    'color',handles.colors.text,'fontsize',16*handles.prefs.fonts.tabscale);
destmenu=uicontrol('parent',u,'position',[35 85+50-20 250 22],'style','popupmenu',...
    'string',{'Create as a new data set';'Export to a file';'Append to current data set';'Replace current data set'},...
    'tag','destinationmenu','backgroundcolor',[1 1 1],'callback',@destmenu_callback,'value',2);
donebutton=uicontrol('style','pushbutton','units','pixels','position',[200 10 180 30],...
    'fontsize',16,'string','Create DoE','callback',@donebutton_Callback,'tag','donebutton','parent',u);

%%
handles.temp.tabobjects=[];
handles.temp.destobjects=[];
handles.temp.u=u;
handles.temp.f=f;

guidata(f,handles);
guidata(handles.fig,handles);

destmenu_callback(destmenu,[]);

function closebutton_Callback(src,ev)
close(gcf);

function donebutton_Callback(src,ev)

result=processdest(src,ev); %check that destination form was filled out correctly
if ~result;return;end

processexport(src,ev);

function destmenu_callback(src,ev)
handles=guidata(src);
delete(handles.temp.destobjects);
handles.temp.destobjects=[];
u=handles.temp.u;
donebutton=findobj(handles.temp.f,'tag','donebutton');

val=get(src,'value');

if val==1 % new data set
    txt=uicontrol('position',[35 60+50-20 400 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Data set name:',...
        'hor','left','fontsize',10,'fontweight','bold');
    editbox=uicontrol('position',[50 40+50-20 200 22],'parent',handles.temp.u,...
        'style','edit','backgroundcolor',[1 1 1],'hor','left','tag','namebox');
    andfunctions=uicontrol('position',[50 14+50-20 400 22],'parent',handles.temp.u,...
        'style','checkbox','string','New data set inherits function variables from current data set',...
        'tag','andfunctions','backgroundcolor',[1 1 1],'value',1);
    handles.temp.destobjects=[txt editbox andfunctions];
    set(donebutton,'string','Create Data Set','visible','on');
    
elseif val==2 % export to file
    
    editbox=uicontrol('position',[50 40+50-20 400 22],'parent',handles.temp.u,...
         'style','edit','backgroundcolor',[1 1 1],'tag','browsebox','hor','left','visible','off');
    browsebutton=uicontrol('position',[200 10 180 30],'parent',handles.temp.u,...
        'style','push','callback',@saveasbutton_callback,'string','Save As...','fontsize',16);
    andfunctions=uicontrol('position',[50 14+50+20 400 22],'parent',handles.temp.u,...
        'style','checkbox','string','Evaluate function variables and include them as columns in exported file',...
        'tag','andfunctions','backgroundcolor',[1 1 1],'value',1);
    
    handles.temp.destobjects=[editbox browsebutton andfunctions];
    set(donebutton,'visible','off');
    
else %append or replace
    colortestbutton=zeros(1,10);
    for c=1:10
        colortestbutton(c)=uicontrol('position',[35+22*c 35+50-20 18 18],'parent',u,...
            'style','togglebutton','string','','value',0,'backgroundcolor',handles.prefs.rowcolors(c,:),...
            'fontname','arial narrow','fontsize',6,'horizontalalignment','left',...
            'tag','rowcolorblob','userdata',c,'callback',@colorbutton_callback);
    end
    set(colortestbutton(handles.prefs.defaultrowcolor),'value',1);
    colortext=uicontrol('units','pixel','position',[35 55+50-20 300 22],'parent',u,'style','text',...
        'string','Row color of newly added rows:','backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');
    selectexisting=uicontrol('units','pixel','position',[35 10+50-20 200 22],'parent',u,'style','checkbox',...
        'string','Select new rows','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','selectexisting');
    handles.temp.destobjects=[colortestbutton colortext selectexisting];
    set(donebutton,'string','Save Data','visible','on')
    
end

guidata(handles.fig,handles)
guidata(handles.temp.f,handles)

function colorbutton_callback(src,ev)
handles=guidata(src);

cbuttons=findobj(handles.temp.f,'tag','rowcolorblob');
set(cbuttons,'value',0);
set(src,'value',1);

function saveasbutton_callback(src,ev)
handles=guidata(src);

resultsmenu=findobj(handles.temp.f,'tag','resultsmenu');
str=get(resultsmenu,'string');
defname=[str{get(resultsmenu,'value')},'.txt'];

[name,path]=uiputfile('*.txt','Save file name',defname);

if ~ischar(name);return;end

fullpath=fullfile(path,name);
browsebox=findobj(handles.temp.u,'tag','browsebox');
set(browsebox,'string',fullpath);

donebutton_Callback(src,ev)


function result=processdest(src,ev)
result=true;

handles=guidata(src);

destmenu=findobj(handles.temp.f,'tag','destinationmenu');

if get(destmenu,'value')==1
    namebox=findobj('tag','namebox');
    namestr=get(namebox,'string');
    if isempty(namestr)
        result=false;
        errordlg('You must enter a name for the new data set.');
    end
elseif get(destmenu,'value')==2
    browsebox=findobj('tag','browsebox');
    browsestr=get(browsebox,'string');
    if isempty(browsestr)
        result=false;
        errordlg('You must enter a file name.');
    end
end


function processexport(src,ev)
handles=guidata(src);
destmenu=findobj(handles.temp.f,'tag','destinationmenu');
resultsmenu=findobj(handles.temp.f,'tag','resultsmenu');

iidx=get(resultsmenu,'userdata');
idx=get(resultsmenu,'value');
idx=iidx(idx);

d=handles.optimizerresults{idx}.settings.d;
a=handles.optimizerresults{idx}.settings.a;

val=get(destmenu,'value');

allnames=handles.allnames{d};
indeps=~handles.isfunction{d};

optdata=handles.optimizerresults{idx}.finalinputs;

%deal newdata columns to dataset columns:

newdata=repmat(handles.xdials{d}{a},size(optdata,1),1);
%newdata=zeros(size(optdata,1),max(handles.optimizerresults{idx}.settings.funvars));
newdata(:,handles.optimizerresults{idx}.settings.funvars)=optdata;

nrows=size(newdata,1);

if val==1; %data set
    andfunctions=findobj(handles.temp.f,'tag','andfunctions');
    andfunctions=get(andfunctions,'value');
    
    namebox=findobj(handles.temp.f,'tag','namebox');
    dest=get(namebox,'string');
    
    %first, copy existing data set:
    raveduplicatedataset(handles.fig,[],d,dest)
    handles=guidata(handles.fig);
    
    handles.ispointofinterest{end}=false(nrows,1);
    for a=1:numel(handles.analysisnames)
        handles.rowcolors{end}{a}=handles.prefs.defaultrowcolor*ones(nrows,1);
        handles.isvisible{end}{a}=true(nrows,1);
        handles.isselected{end}{a}=false(nrows,1);
        handles.isfiltered{end}{a}=false(nrows,1);
    end
    
    guidata(handles.fig,handles);
    
    newd=numel(handles.datasetnames);
    
    if ~andfunctions
        %now remove extra columns if not keeping functions:
        ravedeletevars(handles.fig,[],newd,handles.isfunction{newd});
        handles=guidata(handles.fig);
        newdata=points;
    else
        %evaluate functions
        index=find(handles.isfunction{d});
        outs=raveevaluatefunction(handles.fig,d,index,newdata);
        newdata(:,index)=outs;
    end
    
    %now replace data with DoE and call raveupdatedataset.
    handles.alldata{end}=newdata;
    handles.alltext{end}=cell(size(newdata));
    handles.isvalid{end}=true(size(newdata));
    
    guidata(handles.fig,handles);
    raveupdatedataset(handles.fig,[],'replaceall',[],numel(handles.datasetnames));
    
elseif val==2; %file
    andfunctions=findobj(handles.temp.f,'tag','andfunctions');
    andfunctions=get(andfunctions,'value');
    
    if ~andfunctions;
        allnames=allnames(indeps);
        newdata=newdata(:,indeps);
%        newdata=points;
    else
        %evaluate functions
        index=find(handles.isfunction{d});
        outs=raveevaluatefunction(handles.fig,d,index,newdata);
        newdata(:,index)=outs;
    end
    
    browsebox=findobj(handles.temp.f,'tag','browsebox');
    dest=get(browsebox,'string');
    
    fid=fopen(dest,'wt');
    for t=1:numel(allnames)-1
        fprintf(fid,'%s\t',allnames{t});
    end
    fprintf(fid,'%s\n',allnames{end});
    fclose(fid);
    dlmwrite(dest,newdata,'-append','delimiter','\t','newline','pc','precision',8);
    
elseif val==3; %append
    %evaluate functions
    index=find(handles.isfunction{d});
    outs=raveevaluatefunction(handles.fig,d,index,newdata);
    newdata(:,index)=outs;
    
    cblob=findobj(handles.temp.f,'tag','rowcolorblob','value',1);
    rowcolor=get(cblob,'userdata');
    
    selectexisting=findobj(handles.temp.f,'tag','selectexisting');
    doselect=get(selectexisting,'value');
    
    raveappendrows(handles.fig,[],d,newdata,rowcolor,doselect,true,false)
    
elseif val==4; %replace
        %evaluate functions
    index=find(handles.isfunction{d});
    if ~isempty(index)
        outs=raveevaluatefunction(handles.fig,d,index,newdata);
        newdata(:,index)=outs;
    end
    
    cblob=findobj(handles.temp.f,'tag','rowcolorblob','value',1);
    rowcolor=get(cblob,'userdata');
    
    selectexisting=findobj(handles.temp.f,'tag','selectexisting');
    doselect=get(selectexisting,'value');
    
    handles.alldata{d}=newdata;
    handles.alltext{d}=cell(size(newdata));
    handles.isvalid{d}=true(size(newdata));
    
    handles.ispointofinterest{d}=false(nrows,1);
    
    for a=1:numel(handles.analysisnames)
        handles.rowcolors{d}{a}=rowcolor*ones(nrows,1);
        handles.isvisible{d}{a}=true(nrows,1);
        handles.isfiltered{d}{a}=false(nrows,1);

        if doselect
            newselect=true(size(newdata,1),1);
        else
            newselect=false(size(newdata,1),1);
        end
        
        handles.isselected{d}{a}=newselect;
    end
    guidata(handles.fig,handles);
    raveupdatedataset(handles.fig,[],'replaceall',[],d);
end

ravefinish







