function f=exportappenddatagui(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%Gui for appending or exporting optimizer results.

%make figure:
handles=guidata(src);

f=figure('units','pixels','position',[0 0 500 530],'name',...
    'Save/Export Optimizer Results','numbertitle','off','menubar','none','resize','off','integerhandle','off','handlevisibility','callback','windowstyle','normal');
guidata(f, handles);

origin=get(0,'userdata');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

%make tabs:

%get theme
%load raveprefs
%handles.prefs=raveprefs;
set(f,'color',handles.colors.frame)

%make initial uicontrols:

background2=uipanel('units','pixels','position',[50 365 406 100],'visible','on','backgroundcolor',[1 1 1]);

titleaxes2=axes('units','pixels','position',[11 280+40+300 485 41],'parent',f,'visible','on','tag','titleaxes');

savebutton=uicontrol('style','pushbutton','units','pixels','position',[100 11+50+300 156 35],'tag','savebutton',...
    'fontname',handles.prefs.fonts.tab,'fontsize',14*handles.prefs.fonts.tabscale,'string','Save','callback',@savebutton_callback);

background=uipanel('units','pixels','position',[50 55 406 220+40],'visible','on','backgroundcolor',[1 1 1]);
titleaxes=axes('units','pixels','position',[11 280+40 485 41],'parent',f,'visible','off','tag','titleaxes');

appendbutton=uicontrol('style','pushbutton','units','pixels','position',[100 11+50 156 35],'tag','appendbutton',...
    'fontname',handles.prefs.fonts.tab,'fontsize',14*handles.prefs.fonts.tabscale,'string','Append','callback',@appendbutton_callback);
donebutton=uicontrol('style','pushbutton','units','pixels','position',[438-100-40 11 156 35],'tag','donebutton',...
    'fontname',handles.prefs.fonts.tab,'fontsize',14*handles.prefs.fonts.tabscale,'string','Done','callback',@donebutton_callback);

text(0,.2,'Add Rows to Data Set','parent',titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
    'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');

selectexisting=uicontrol('units','pixel','position',[50 20+40 200 22],'parent',background,'style','checkbox',...
    'string','Select existing data','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','selectexisting');

selectnew=uicontrol('units','pixel','position',[50 40+40 200 22],'parent',background,'style','checkbox',...
    'string','Select newly added data','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','selectnew');

optionstext=uicontrol('units','pixel','position',[35 60+40 200 22],'parent',background,'style','text',...
    'string','Options','backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');

colortestbutton=zeros(1,10);

for c=1:10
    colortestbutton(c)=uicontrol('position',[50+22*c 90+40 18 18],'parent',background,...
    'style','togglebutton','string','','value',0,'backgroundcolor',handles.prefs.rowcolors(c,:),...
    'fontname','arial narrow','fontsize',6,'horizontalalignment','left',...
    'tag','rowcolorblob','userdata',c,'callback',@colorbutton_callback);
end
colortestbutton=colortestbutton;

set(colortestbutton(handles.prefs.defaultrowcolor),'value',1);

colortext=uicontrol('units','pixel','position',[35 110+40 300 22],'parent',background,'style','text',...
    'string','Row color of newly added rows:','backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');

appendcheck=uicontrol('units','pixel','position',[50 40+120+40 350 22],'parent',background,'style','radiobutton','value',1,...
    'string','Append new rows to existing data set','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','append','callback',@appendcheck_callback);

replacebutton=uicontrol('units','pixel','position',[50 20+120+40 350 22],'parent',background,'style','radiobutton',...
    'string','Replace data set with newly added rows (delete old data)','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','replace','callback',@appendcheck_callback);

optionstext=uicontrol('units','pixel','position',[35 60+120+40 200 22],'parent',background,'style','text',...
    'string','Append or Replace:','backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');

% Update handles structure

set(0,'userdata',[]);


function colorbutton_callback(src,ev)
handles=guidata(src);
set(handles.colortestbutton,'value',0)
set(src,'value',1)

function appendbutton_callback(src,ev)
handles=guidata(src);

thiscolor=findobj(handles.colortestbutton,'value',1);
outputs.color=get(thiscolor,'userdata');

appendbutton=findobj(handles.gui,'tag','append');

if get(appendbutton,'value')
    outputs.method='append';
else
    outputs.method='replace';
end

selectnew=findobj(handles.gui,'tag','selectnew');
selectexisting=findobj(handles.gui,'tag','selectexisting');

outputs.selectnew=get(selectnew,'value');
outputs.selectexisting=get(selectexisting,'value');

set(0,'userdata',outputs);

%close(handles.gui);

function donebutton_callback(src,ev)
thisfig=get(src,'parent');
close(thisfig);


function appendcheck_callback(src,ev)
handles=guidata(src);
appendbutton=findobj(handles.gui,'tag','append');
replacebutton=findobj(handles.gui,'tag','replace');

set([appendbutton replacebutton],'value',0)
set(src,'value',1)




