function buttondownfcnderivativeprofiler(src,eventdata)
%this is the buttondownfcn for derivative profiler axes, to update the x
%value of the selected graph and redraw all graphs in the new position.
%Also includes buttonmotionfcn version.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

thistype=get(src,'tag');
if strcmp(thistype,'points');
    points=src;
    src=get(src,'parent');
else
    points=[];
end

ai=get(src,'userdata');
udata=get(handles.fakeax(ai),'userdata');

%On first click, just select this graph to enable mouseovers etc. 
if handles.activesubgraph~=src
    handles.activesubgraph=src;
    guidata(handles.fig,handles);
    return
end

%On subsequent clicks, update graph...

if strcmp(get(handles.fig,'selectiontype'),'alt') && udata.format.showpoints && ~udata.view.singleyaxis %ctrl click or right click = select point closest to clicked point.
    %need to check if this graph is active. if not , return and do nothing.
    thisindex=get(src,'userdata');
    if thisindex==handles.activeindex
        if strcmp(handles.activetab,'format')
            raveselectsubgraph(src,eventdata)
            return
        end
    end
    %
    %if format tab is active, instead of doing this, make this graph the active subgraph to allow axes
    %changes and such:
    
    cc=get(src,'currentpoint');
    xclick=cc(1,1);
    yclick=cc(1,2);
    
    try %gives error if you didnt click on a point (eg clicked on background)
        index=ravefindclosestpoint2D(xclick,yclick,points);
    catch
        return
    end
    
    %figure out
    
    oldselects=handles.isselected{udata.view.dataset}{udata.view.analysis};
    oldselects=false(size(oldselects));
    oldselects(index)=true;
    handles.isselected{udata.view.dataset}{udata.view.analysis}=oldselects;
    handles.xdials{udata.view.dataset}{udata.view.analysis}=handles.alldata{udata.view.dataset}(index,:);
    guidata(src,handles);
    raveupdateanalysis(src,[],'isselected',[]);
    raveupdateanalysis(src,[],'xdials',[]);
    return
else
    %need to check if this graph is active. if not , return and do nothing.

    %if format tab is active, instead of doing this, make this graph the active subgraph to allow axes
    %changes and such:
    thisindex=get(src,'userdata');
    if thisindex==handles.activeindex
        if strcmp(handles.activetab,'format')
            raveselectsubgraph(src,eventdata)
            return
        end
    end
    raveslideprofiler(handles.fig,[],ai);
    %make sliding along graph update continuously:
    set(handles.fig,'windowbuttonupfcn',{@raveprofdone,ai})
    set(handles.fig,'windowbuttonmotionfcn',{@raveslideprofiler,ai})
    set(handles.fig,'interruptible','off')
end


function raveslideprofiler(src,ev,ai)
%NOTE: SRC FOR THIS FUNCTION IS EQUAL TO HANDLES.FIG REGARDLESS OF WHAT WAS
%CLICKED.
%persistent handles
handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

xcol=udata.view.x;
ycol=udata.view.y;

numy=numel(ycol);

%find this plot's location in the matrix:
[junk,index]=find(handles.subax{ai}==gca);

if isempty(index);
    return
end

%if ~isempty(src) %if this function is called as a callback: (get mouse location and slide bars)
clicked=get(gca,'currentpoint');
val=clicked(1,1);

%move marker within bounds
if val>xmax(xcol(index))
    val=xmax(xcol(index));
elseif val<xmin(xcol(index))
    val=xmin(xcol(index));
end

%if this variable is non-continuous, round val to nearest allowable value:
thistype=handles.datatype{d}{xcol(index)};

if  strcmp(thistype,'constant')
    val=handles.minlimit{d}{a}{xcol(index)};
elseif ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{xcol(index)};
    val=allowed(abs(allowed-val)==min(abs(allowed-val)));
end

%make text label:
if ~any(strcmp(thistype,{'string','logical'}))
    valtext=strtrim(sprintf('%5.5g',val));
elseif strcmp(thistype,'string')
    valtext=handles.discretevalues{d}{a}{xcol(index)}{val};
else %logical
    if val
        valtext=handles.prefs.logicaltrue;
    else
        valtext=handles.prefs.logicalfalse;
    end
end

%set xlabel:
set(udata.view.xhair(:,index),'xdata',[val val])
if udata.view.showcurrentpoint && udata.format.xlabelbelow
    set(get(handles.subfakeax{ai}(end,index),'xlabel'),'string',{' ';' ';['\color[rgb]{',udata.format.linecolorString,'} ',valtext]},'verticalalignment','top','interpreter','tex');
end

%if new value is the same as old value (because of discrete vars) don't
%waste time evaluating funcitons:
xdials=handles.xdials{d}{a};

if xdials(xcol(index))==val;
    return
end

xdials(xcol(index))=val;
handles.xdials{d}{a}=xdials;
guidata(handles.fig,handles);

%calculate profiler data for each graph:
if udata.view.singleyaxis
    rows=1;
else
    rows=length(udata.view.y);
end

cols=length(udata.view.x);
numpts=udata.view.numpts;

%get type of each variable:
datatypes=handles.datatype{d}(xcol);

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

[xsteps totalrows]=ravecalculatexsteps(src,d,a,xcol,numpts,1);

rowz=sum(totalrows);
xfixed=repmat(xdials,[rowz,1]);

%convert to cell if strings are present:
if any(strcmp('string',datatypes))
    xfixed=num2cell(xfixed);
    for k=1:cols
        [xfixed{sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k)}]=deal(xsteps{k});
    end
else
    for k=1:cols
        xfixed(sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k))=xsteps{k};
    end
end

%evaluate the functions:
yvalues=raveevaluatefunction(src,d,ycol,xfixed);

doupdate=0;
for g=1:numy
    if min(yvalues(:,g))<handles.minlimit{d}{a}(ycol(g));handles.minlimit{d}{a}(ycol(g))=min(yvalues(:,g));doupdate=1;end
    if max(yvalues(:,g))>handles.maxlimit{d}{a}(ycol(g));handles.maxlimit{d}{a}(ycol(g))=max(yvalues(:,g));doupdate=1;end
end

if doupdate;
    ax=handles.subax{ai};
    fakeax=handles.subfakeax{ai};
    fakerax=handles.subfakerax{ai};
    ylimmin=handles.minlimit{d}{a}(ycol);
    ylimmax=handles.maxlimit{d}{a}(ycol);
    inset = .06;
    
    %fix y limits
    for r=1:rows,
        if ~udata.view.singleyaxis
            set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[ylimmin(r) ylimmax(r)])
            dy = diff(get(ax(r,1),'ylim'))*inset;
            set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[ylimmin(r)-dy ylimmax(r)+dy])
            set(udata.view.xhair(r,:),'ydata',[ylimmin(r)-dy ylimmax(r)+dy])
        else
            set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[min(ylimmin) max(ylimmax)])
            dy = diff(get(ax(r,1),'ylim'))*inset;
            set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[min(ylimmin)-dy max(ylimmax)+dy])
            set(udata.view.xhair(r,:),'ydata',[min(ylimmin)-dy max(ylimmax)+dy])
        end

    end
    
    for c=1:cols
        ylimits=get(ax(end,c),'Ylim');
        ypos=ylimits(1)-.02*(ylimits(2)-ylimits(1));
        ticks=findall(ax(end,c),'tag','xticklabel');
        
        posses=get(ticks,'position');
        if iscell(posses);posses=cell2mat(posses);end
        posses(:,2)=ypos;
        posses=mat2cell(posses,ones(size(posses,1),1),3);
        set(ticks,{'position'},posses);
    end
    
    guidata(src,handles)
end

ycurrent=raveevaluatefunction(src,d,ycol,xdials);

%put results of each response in a seperate cell:
yvalues=mat2cell(yvalues,size(yvalues,1),ones(1,numy));

ycurrent=mat2cell(ycurrent,size(ycurrent,1),ones(1,numy));

%and put each x for each resposne in its own cell too:
for zz=1:numy
    yvalues{zz}=mat2cell(yvalues{zz},totalrows,1);
end

%update current-value texts for responses:
for tt=1:rows
    %set y label:
    g=udata.view.ylabels(tt);
    if udata.view.showcurrentpoint && ~udata.view.singleyaxis
        set(g,'string',{['\color[rgb]{',udata.format.linecolorString,'} ',strtrim(sprintf('%5.5g',ycurrent{tt}))];'';''});
    end
    
    for cc=1:cols
        if  ~any(strcmp(datatypes{cc},{'continuous';'constant'})) %if this variable is discrete
            allowed=handles.discretevalues{d}{a}{xcol(cc)};
            endind=numel(allowed);
            
            intsides=repmat(2:(endind-1),3,1);
            ymap=[1 1 intsides(:)' endind endind];
            
            xvals=xsteps{cc}(:); %force to be column.
            xintermediates=xvals+[diff(xvals)./2;0];
            
            xvals=sort([xvals;xintermediates(1:end-1)]);
            
            xmap=repmat(2:2:2*(endind-1),3,1);
            xmap(end,:)=xmap(end,:)+ones(size(xmap(end,:)));
            
            xmap=[1 xmap(:)'];
            
            set(udata.view.yhair(tt,cc),'ydata',[ycurrent{tt} ycurrent{tt}])
            set(udata.view.mainhandles(tt,cc),'ydata',yvalues{tt}{cc}(ymap),'xdata',xvals(xmap))
            if udata.explore.mouseover && ~udata.view.singleyaxis
                set(udata.explore.mouseoverhandles(tt,cc),'ydata',yvalues{tt}{cc}(ymap),'xdata',xvals(xmap))
            end
        else
            if ~udata.view.singleyaxis
                set(udata.view.yhair(tt,cc),'ydata',[ycurrent{tt} ycurrent{tt}])
                set(udata.view.mainhandles(tt,cc),'ydata',yvalues{tt}{cc},'xdata',xsteps{cc})
                if udata.explore.mouseover && ~udata.view.singleyaxis
                    set(udata.explore.mouseoverhandles(tt,cc),'ydata',yvalues{tt}{cc},'xdata',xsteps{cc})
                end
                
            else
                for R=1:numy
                    set(udata.view.yhair(R,cc),'ydata',[ycurrent{R} ycurrent{R}])
                    set(udata.view.mainhandles(R,cc),'ydata',yvalues{R}{cc},'xdata',xsteps{cc})
                end
            end
        end
    end
end

if any(udata.view.constraints~=0)
    ravedrawconstraint(handles.allgraphs(ai),ai,udata.view.constraints);
end

%update on slide if desired:
if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    raveupdateanalysis(src,[],'xdials',ai)
end



function raveprofdone(src,ev,ai) %#ok<INUSD>

handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonupfcn','')
set(handles.fig,'interruptible','on')

%update other analyses to reflect new values:
raveupdateanalysis(handles.fig,[],'xdials',ai)
