function viewtabderivativeprofiler(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%move front tabs to front if necessary
% if handles.fronttabsfront==0 %move swap fronttabs and backtabs
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y);
% end
% handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
%view=udata.view; %get view tab structure.
d=udata.view.dataset;
a=udata.view.analysis;

%% text headers

projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

dataseriestext=text(.15-.10317,.76,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

optionstext=text(.15-.10317,.21,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels

analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');

datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

yaxistext=text(.25-.10317,.73,'Choose Responses:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

xaxistext=text(.25-.10317,.56,'Choose Variables:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');


%% variable lists
xvars=handles.funvars{d}{udata.view.y(1)};
for yy=1:length(udata.view.y)
    xvars=union(xvars,handles.funvars{d}{udata.view.y(yy)});
end

allnames=handles.allnames{d};

xvarnames=allnames(xvars);

analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','callback',@analysismenu_callback,'value',a);

datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','callback',@datasetmenu_callback,'value',d);

yaxismenu=uicontrol('position',[origin(1)+55 origin(2)+368 180 80],...
    'style','listbox','string',handles.allnames{d}(logical(handles.isfunction{d})),'value',find(ismember(find(handles.isfunction{d}),udata.view.y)),...
    'backgroundcolor',[1 1 1],'tag','yaxismenu','max',50,'callback',@yaxismenu_callback);

xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+150 180 190],...
    'style','listbox','string',xvarnames,'value',find(ismember(xvarnames,allnames(udata.view.x))),...
    'backgroundcolor',[1 1 1],'tag','xaxismenu','max',50,'callback',@xaxismenu_callback,'listboxtop',1);

%% buttons
currentpointcheck=uicontrol('position',[origin(1)+55 origin(2)+100 200 25],...
    'style','checkbox','string','Show current point on graphs',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.showcurrentpoint,'callback',@currentpointcheck_callback,'tag','currentpointcheck');

singleyaxischeck=uicontrol('position',[origin(1)+55 origin(2)+75 200 25],...
    'style','checkbox','string','Draw all graphs on same Y axis',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.singleyaxis,'callback',@singleyaxischeck_callback,'tag','singleyaxischeck');

usevariablecolorscheck=uicontrol('position',[origin(1)+70 origin(2)+50 150 25],...
    'style','checkbox','string','Use variable colors',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.usevariablecolors,'callback',@usevariablecolorscheck_callback,'tag','singleyaxischeck');


excludecheck=uicontrol('position',[origin(1)+55 origin(2)+50 200 25],...
    'style','checkbox','string','Hide constant columns in menus',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.exclude,'callback',@excludecheck_callback,'tag','excludecheck','visible','off');

scalebutton=uicontrol('position',[origin(1)+55 origin(2)+15 180 30],...
    'style','pushbutton','string','Scale/Transform Variables','backgroundcolor',handles.colors.buttonbg,'visible','off');

%% group items and finish up
handles.tabobjects=[datasettext xaxistext datasetmenu xaxismenu,...
    scalebutton analysistext projecttext analysismenu dataseriestext optionstext excludecheck ...
    yaxistext yaxismenu singleyaxischeck currentpointcheck usevariablecolorscheck];

handles.activetab='view';

%if selected isconstant, turn off filter constant button
if any(handles.isconstant{get(datasetmenu,'value')}(udata.view.x))
    set(excludecheck,'enable','off')
end

guidata(src,handles);

% update menus to hide excluded vars
viewtabmenufilter(src,[])


%% Exclude hidden variables from menus:

function viewtabmenufilter(src,eventdata)

handles=guidata(src);

%get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');

d=get(datasetmenu,'value');

%get full variable list:
xvars=handles.funvars{d}{udata.view.y(1)};
for yy=1:length(udata.view.y)
    xvars=union(xvars,handles.funvars{d}{udata.view.y(yy)});
end

allnames=handles.allnames{d};
xvarnames=allnames(xvars);

%figure out which of these are constant:
constvarnames=handles.allnames{d}(logical(handles.isconstant{d}));
xfilterednames=xvarnames(~ismember(xvarnames,constvarnames));

%if get(useresponsedata,'value') && get(excludecheck,'value') && ~all(handles.isresponse{d}==0)

if view.exclude %hide constant cols
    %get new strings/values
    newxstring=xfilterednames; %list of variable names with the constant ones removed.
    
    newxcols=udata.view.x(~ismember(udata.view.x,find(handles.isconstant{d})));
    newxvalue=find(ismember(xfilterednames,allnames(newxcols)));
    
    if isempty(newxvalue)
        newxvalue=1;
    end
    
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isconstant{d}));
    
    %reset menu strings/values to match displayed data.
    set(xaxismenu,'string',newxstring,'value',newxvalue,'listboxtop',1);
    
    
else %show all variables
    
    %get new strings/values
    newxstring=xvarnames; %list of variable names with the constant ones removed.
    
    newxcols=udata.view.x;
    newxvalue=find(ismember(xvarnames,allnames(newxcols)));
    
    if isempty(newxvalue)
        newxvalue=1;
    end
    
    
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=1:1:length(handles.allnames{d});
    
    %reset menu strings/values to match displayed data.
    set(xaxismenu,'string',newxstring,'value',newxvalue,'listboxtop',1);
    
    
end

guidata(src,handles)

%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata)

handles=guidata(src);
%update other menus to show correct variables
udata=get(handles.fakeax(handles.activeindex),'userdata');

yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');

d=get(datasetmenu,'value');

if d==udata.view.dataset
    return %same data set, do nothing to avoid resetting graph.
end


%get initial columns for this data set:
if all(handles.isfunction{d}==0)
    %no functions in this data set... can't use it.
    set(datasetmenu,'value',udata.view.dataset)
    set(handles.infobar,'string','Selected data set has no variables modeled by functions.')
    return
else
    %data set is valid, make y = first function, x = variables of that
    %function.
    set(handles.infobar,'string','');
    y=find(handles.isfunction{d},1);
    udata.view.y=y;
    udata.view.dataset=d;
    udata.view.x=handles.funvars{d}{y}; %note, these are names, need to convert them to numbers
    
    udata.view.isvalid=1;
    %    calculate starting x values:
    xmax=max(handles.alldata{d}(:,:));
    xmin=min(handles.alldata{d}(:,:));
    udata.view.xdials=mean([xmax;xmin]);
end

%set fakeax userdata

udata.view.dataset=d;
set(handles.fakeax(handles.activeindex),'userdata',udata);
handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);

viewtabmenufilter(src,[])

%update y menu:
set(yaxismenu,'string',handles.allnames{d}(logical(handles.isfunction{d})),'value',1)

ravecreatederivativeprofiler(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata)
handles=guidata(src);
handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatederivativeprofiler(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function yaxismenu_callback(src,eventdata)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
d=udata.view.dataset;

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');

%convert value into
yvalue=find(logical(handles.isfunction{d}));
udata.view.y=yvalue(get(src,'value'));

%change x axis menu string to show all variables for these responses:
xvars=handles.funvars{d}{udata.view.y(1)};
for yy=1:length(udata.view.y)
    xvars=union(xvars,handles.funvars{d}{udata.view.y(yy)});
end

allnames=handles.allnames{d};
xvarnames=allnames(xvars);

%deselect
xvars=intersect(xvars,udata.view.x);
udata.view.x=xvars;

xvalue=find(ismember(xvarnames,allnames(udata.view.x)));

%change x string and value.
set(xaxismenu,'string',xvarnames,'value',xvalue);

%save udata
set(handles.fakeax(handles.activeindex),'userdata',udata);


%update plots
xmax=max(handles.alldata{d}(:,:));
xmin=min(handles.alldata{d}(:,:));
udata.view.xdials=mean([xmax;xmin]);

ravecreatederivativeprofiler(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%if selected isconstant, turn off filter constant button
if any(handles.isconstant{udata.view.dataset}(handles.visiblexcols(get(src,'value'))))
    set(excludecheck,'enable','off')
else
    set(excludecheck,'enable','on')
end

%convert value into
xvarnames=get(src,'string');
xvarnames=xvarnames(get(src,'value'));

allnames=handles.allnames{udata.view.dataset};

xvalue=find(ismember(allnames,xvarnames));
udata.view.x=xvalue;

guidata(src,handles);
%set fakeax userdata
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatederivativeprofiler(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function singleyaxischeck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.singleyaxis=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatederivativeprofiler(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function currentpointcheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.showcurrentpoint=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatederivativeprofiler(src,[]);


function usevariablecolorscheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.usevariablecolors=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatederivativeprofiler(src,[]);


