function formattabcontour(src,eventdata)
%rave format tab for a scatterplot matrix graph.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format; %get format tab structure.

%move front tabs to front if necessary
% if handles.fronttabsfront==0 %move swap fronttabs and backtabs
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y);
% end
% handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% highlight the selected subplot... this only gets highlighted on the format tab, and goes away when you click another tab.
%set(handles.activesubgraph,'xcolor',[1 .5 0],'ycolor',[1 .5 0]);


%% clear existing tab
delete(handles.tabobjects)

%% text headers
graphtext=text(.15-.10317,.97*1.0317-.0317,'Graph','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
axestext=text(.15-.10317,.85*1.0317-.0317,'Axes','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.515*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
legendtext=text(.15-.10317,.16*1.0317-.0317,'Legends','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% GRAPH UICONTROLS

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
gridtext=uicontrol('position',[origin(1)+25 origin(2)+550 180 20],...
    'style','text','string','Grid Lines:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

bgidx=udata.format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{1}(2,:),'value',bgidx==1,...
    'callback',@bgblob_callback,'userdata',1,'tag','bgblob');
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{2}(2,:),'value',bgidx==2,...
    'callback',@bgblob_callback,'userdata',2,'tag','bgblob');
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{3}(2,:),'value',bgidx==3,...
    'callback',@bgblob_callback,'userdata',3,'tag','bgblob');
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{4}(2,:),'value',bgidx==4,...
    'callback',@bgblob_callback,'userdata',4,'tag','bgblob');

gridindex=eye(3);
gridin=format.grid;


%grid radios
nogrid=uicontrol('position',[origin(1)+100 origin(2)+551 60 20],...
    'style','radiobutton','string','None','backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(1,gridin),'callback',@nogrid_callback);
magrid=uicontrol('position',[origin(1)+152 origin(2)+551 60 20],...
    'style','radiobutton','string','Major','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(2,gridin),'callback',@magrid_callback);
migrid=uicontrol('position',[origin(1)+204 origin(2)+551 55 20],...
    'style','radiobutton','string','Minor','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(3,gridin),'callback',@migrid_callback);

handles.gridchecks=[nogrid magrid migrid];

%% Axes UICONTROLS

xlim=get(handles.activegraph,'xlim'); %for scatterplot matrix, can select individual subplots and change their properities from here.
ylim=get(handles.activegraph,'ylim');

xlabeltext=uicontrol('position',[origin(1)+25 origin(2)+498 180 20],...
    'style','text','string','X Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlimtext=uicontrol('position',[origin(1)+35 origin(2)+474 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xto=uicontrol('position',[origin(1)+158 origin(2)+474 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlabelbox=uicontrol('position',[origin(1)+85 origin(2)+500 160 20],...
    'style','edit','string',get(get(handles.activegraph,'xlabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xlabel_callback,'tag','xlabelbox');
xminbox=uicontrol('position',[origin(1)+85 origin(2)+476 70 20],...
    'style','edit','string',xlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmin_callback,...
    'tag','xminbox');
xmaxbox=uicontrol('position',[origin(1)+175 origin(2)+476 70 20],...
    'style','edit','string',xlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmax_callback,...
    'tag','xmaxbox');
xresetbutton=uicontrol('position',[origin(1)+85 origin(2)+452 160 20],...
    'style','pushbutton','string','Reset X Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@xresetbutton_callback);

ylabeltext=uicontrol('position',[origin(1)+25 origin(2)+410 180 20],...
    'style','text','string','Y Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylimtext=uicontrol('position',[origin(1)+35 origin(2)+386 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
yto=uicontrol('position',[origin(1)+158 origin(2)+386 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylabelbox=uicontrol('position',[origin(1)+85 origin(2)+412 160 20],...
    'style','edit','string',get(get(handles.activegraph,'ylabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@ylabel_callback,'tag','ylabelbox');
yminbox=uicontrol('position',[origin(1)+85 origin(2)+388 70 20],...
    'style','edit','string',ylim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymin_callback,...
    'tag','yminbox');
ymaxbox=uicontrol('position',[origin(1)+175 origin(2)+388 70 20],...
    'style','edit','string',ylim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymax_callback,...
    'tag','ymaxbox');
yresetbutton=uicontrol('position',[origin(1)+85 origin(2)+364 160 20],...
    'style','pushbutton','string','Reset Y Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@yresetbutton_callback);


titletext=uicontrol('position',[origin(1)+25 origin(2)+332 180 20],...
    'style','text','string','Title:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');


titel=findobj(handles.activegraph,'tag','title');

titlebox=uicontrol('position',[origin(1)+65 origin(2)+333 180 20],...
    'style','edit','string',get(titel,'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@title_callback);

%% DATA SERIES UICONTROLS

markercolortext=uicontrol('position',[origin(1)+25 origin(2)+280 180 20],...
    'style','text','string','Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

cdata=ravemakecdata(handles.fig,[],handles.cmapnames{udata.format.mcmap});


markercolor=uicontrol('position',[origin(1)+75 origin(2)+282 120 20],...
    'style','pushbutton','string','','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','cdata',cdata,'callback',@mcolor_callback);

linestyletext=uicontrol('position',[origin(1)+25 origin(2)+195 55 20],...
    'style','text','string','Tracers:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
linesizetext=uicontrol('position',[origin(1)+25 origin(2)+240 44 20],...
    'style','text','string','Width:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

linestyleaxes=axes('units','pixels','position',[origin(1)+95 origin(2)+195 100 22],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);

offcolor=[.5 .5 .5];
oncolor=handles.colors.text;
line([0 1],[.9 .9],'linestyle','-','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')
line([0 1],[.62 .62],'linestyle','--','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')
line([0 1],[.38 .38],'linestyle',':','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')
line([0 1],[.1 .1],'linestyle','-.','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')

hhh=findobj(linestyleaxes,'tag','lineblob','linestyle',udata.format.linestyle);
set(hhh,'color',oncolor)

%width slider:
[jslider hslider]=javacomponent('javax.swing.JSlider',[origin(1)+75 origin(2)+247 110 27]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(1)
jslider.setMinimum(1)
jslider.setMaximum(10)
jslider.setMajorTickSpacing(1);
jslider.setValue(udata.format.linesize*2);
set(jslider,'background',java.awt.Color.white)
set(jslider,'StateChangedCallback',{@linewidth_callback,handles.fig});

% handles.javacomps=jslider;

widthaxes=axes('units','pixels','position',[origin(1)+75 origin(2)+237 110 5],'visible','off');
gauge=imread('widthgauge.bmp');
image(gauge,'parent',widthaxes)
set(widthaxes,'visible','off');

facecheck=uicontrol('position',[origin(1)+94 origin(2)+105 150 20],'style','radiobutton',...
    'string','','value',~udata.format.usevariablecolors,'backgroundcolor',handles.colors.tabbg,...
    'callback',@facecheck_callback,'tag','facecheck');
facetext=uicontrol('position',[origin(1)+111 origin(2)+102 150 20],...
    'style','text','string','Line Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

facecolor=uicontrol('position',[origin(1)+195 origin(2)+102 25 25],...
    'style','pushbutton','string','','backgroundcolor',udata.format.tracercolor,'value',0,...
    'callback',@faceblob_callback);


% 
% 
%width slider:
[jslider2 hslider2]=javacomponent('javax.swing.JSlider',[origin(1)+80 origin(2)+147+5 128 27]);
jslider2.setPaintTicks(1)
jslider2.setSnapToTicks(1)
jslider2.setMinimum(0)
jslider2.setMaximum(10)
jslider2.setMajorTickSpacing(1);
jslider2.setValue(udata.format.tracersize*2);
set(jslider2,'background',java.awt.Color.white)
set(jslider2,'StateChangedCallback',{@tracerwidth_callback,handles.fig});

% handles.javacomps=[jslider jslider2];

widthaxes2=axes('units','pixels','position',[origin(1)+125-30 origin(2)+135+5 110 5],'visible','off');
gauge=imread('widthgauge.bmp');
image(gauge,'parent',widthaxes2)
set(widthaxes2,'visible','off');


%% legend uicontrols
legendbutton=uicontrol('position',[origin(1)+85 origin(2)+48 160 20],...
    'style','pushbutton','string','Format Legend','fontname','arial','fontsize',10,...
    'backgroundcolor',handles.colors.buttonbg,'callback',@formatlegend_callback,...
    'fontweight','bold');

colorbarcheck=uicontrol('position',[origin(1)+85 origin(2)+26 160 20],...
    'style','checkbox','string','Show Colorbar','fontname','arial narrow','fontsize',10,...
    'backgroundcolor',handles.colors.tabbg,'callback',@colorbarcheck_callback,...
    'fontweight','normal','horizontalalignment','left','value',udata.format.colorbar);


%% group items and finish up
handles.tabobjects=[graphtext axestext dataseriestext bgtext gridtext bgwhite...
    bglight bgdark bgblack nogrid magrid migrid xlabeltext ylabeltext...
    xlimtext ylimtext xlabelbox ylabelbox xminbox yminbox xto xmaxbox...
    xresetbutton yto ymaxbox yresetbutton markercolortext...
    markercolor titletext titlebox colorbarcheck...
    legendbutton linestyletext linesizetext hslider linestyleaxes widthaxes...
    legendtext facecheck facetext facecolor hslider2 widthaxes2];

handles.activetab='format';

set(handles.infobar,'string','');

guidata(src,handles);

%% HERE BE CALLBACKS
%
%
%
%
%
%
%
%
%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function magrid_callback(src, eventdata)
handles=guidata(src);
%inputs=get(handles.fakerax{handles.activeindex},'userdata');

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')

function migrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);


set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')

function nogrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);


set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function xmin_callback(src, eventdata)

handles=guidata(src);
newval=eval(get(src,'string'));
xlim=get(handles.activegraph,'xlim');
xlim(1)=newval;
set(handles.activegraph,'xlim',xlim)
set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))

function xmax_callback(src, eventdata)

handles=guidata(src);
newval=eval(get(src,'string'));
xlim=get(handles.activegraph,'xlim');
xlim(2)=newval;
set(handles.activegraph,'xlim',xlim)
set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))


function ymin_callback(src, eventdata)

handles=guidata(src);
newval=eval(get(src,'string'));
ylim=get(handles.activegraph,'ylim');
ylim(1)=newval;
set(handles.activegraph,'ylim',ylim)
set(handles.fakeax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))
set(handles.fakerax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))


function ymax_callback(src, eventdata)

handles=guidata(src);
newval=eval(get(src,'string'));
ylim=get(handles.activegraph,'ylim');
ylim(2)=newval;
set(handles.activegraph,'ylim',ylim)
set(handles.fakeax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))
set(handles.fakerax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))


function xresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activegraph,'xlimmode','auto')
xlim=get(handles.activegraph,'xlim');

set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))

xminbox=findobj(handles.tabobjects,'tag','xminbox');
xmaxbox=findobj(handles.tabobjects,'tag','xmaxbox');

set(xminbox,'string',num2str(xlim(1)))
set(xmaxbox,'string',num2str(xlim(2)))


function yresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activegraph,'ylimmode','auto')
ylim=get(handles.activegraph,'ylim');

set(handles.fakeax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))
set(handles.fakerax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))

yminbox=findobj(handles.tabobjects,'tag','yminbox');
ymaxbox=findobj(handles.tabobjects,'tag','ymaxbox');

set(yminbox,'string',num2str(ylim(1)))
set(ymaxbox,'string',num2str(ylim(2)))


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xlabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'xlabel'),'string',get(src,'string'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ylabel_callback(src,ev)

handles=guidata(src);
set(get(handles.activegraph,'ylabel'),'string',get(src,'string'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mcolor_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

newcmap=ravechoosecolormap(src,[],udata.format.mcmap);
drawnow;
if newcmap==udata.format.mcmap;return;end
udata.format.mcmap=newcmap;

colorz=handles.cmapnames;
newcolormap=colorz{newcmap};

cdata=ravemakecdata(handles.fig,[],newcolormap);
set(src,'cdata',cdata);

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatecontour(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function lineblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

lineblobs=findobj(handles.tabobjects,'tag','lineblob');

offcolor=[.5 .5 .5];
set(lineblobs,'color',offcolor)
set(src,'color',handles.colors.text)

udata.format.linestyle=get(src,'linestyle');
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(udata.view.xhair(:),'linestyle',get(src,'linestyle'));
set(udata.view.yhair(:),'linestyle',get(src,'linestyle'));

function linewidth_callback(jsrc,ev,src)

handles=guidata(src);

%update inputs

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.linesize=jsrc.getValue/2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%get target data series
targets=udata.view.mainhandles(:)';

%update graph
set(targets,'linewidth',udata.format.linesize);


function tracerwidth_callback(jsrc,ev,src)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
w=jsrc.getValue/2;
udata.format.tracersize=w;
set(handles.fakeax(handles.activeindex),'userdata',udata);
if w>0
    set(udata.view.xhair(:),'linewidth',udata.format.tracersize,'visible','on')
    set(udata.view.yhair(:),'linewidth',udata.format.tracersize,'visible','on')
else
    set(udata.view.xhair(:),'visible','off')
    set(udata.view.yhair(:),'visible','off')
end


function facecheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
varcolors=findobj(handles.tabobjects,'tag','varcolors');
set(varcolors,'value',0);
udata.format.usevariablecolors=0;
set(handles.fakeax(handles.activeindex),'userdata',udata);


function faceblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
newcolor=ravecolorchooser(handles.fig,udata.format.tracercolor,'Choose a marker fill color');
set(src,'backgroundcolor',newcolor);
udata.format.tracercolor=newcolor;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update:
if ~udata.format.usevariablecolors
    set(udata.view.xhair(:),'color',newcolor);
    set(udata.view.yhair(:),'color',newcolor);
end


function colorbarcheck_callback(src,ev)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
legdata=get(handles.fakerax(handles.activeindex),'userdata');

udata.format.colorbar=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata);
set(handles.fakerax(handles.activeindex),'userdata',legdata);

ravecreatecontour(src,[]);



