function handles=raveupdateplaneprofilerxdials(src,ev,handles)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%handles=guidata(src);
%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
targetaxes=handles.activegraph; %=handles.activegraph

xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;

a=udata.view.analysis;
d=udata.view.dataset; %data set index

%prepare data for gridding:
if ~udata.view.isvalid
    return %no functions loaded.
end


doupdate=0;

datatypes=handles.datatype{d}(zcol);
numpts=udata.view.npoints;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};
nvars=numel(zcol);

[xsteps totalrows]=ravecalculatexsteps(src,d,a,zcol,numpts,1);
% 
% for j=1:numel(datatypes);
%     switch datatypes{j}
%         case 'constant'
%             xsteps{j}=xdials(zcol(j));
%         case 'continuous'
%             xsteps{j}=sort([xmin(zcol(j)):step(zcol(j)):xmax(zcol(j)) xdials(zcol(j))]);
%         case 'integer'
%             if numel(handles.discretevalues{d}{a}{zcol(j)})<20
%                 xsteps{j}=handles.discretevalues{d}{a}{zcol(j)};
%             else
%                 xsteps{j}=sort([xmin(zcol(j)):step(zcol(j)):xmax(zcol(j)) xdials(zcol(j))]);
%             end
%         case 'discrete'
%             xsteps{j}=handles.discretevalues{d}{a}{zcol(j)};
%         case 'string'
%             xsteps{j}=handles.discretevalues{d}{a}{zcol(j)};
%         case 'logical'
%             xsteps{j}=[0 1];
%     end
% end

for j=1:nvars
    thisvar=zcol(j);
    thiscolor=handles.variablecolors{d}(thisvar,:);
    
    startpoint=xdials;

    npoints=numel(xsteps{j});
    xdata=repmat(startpoint,[npoints 1]);
    xdata(:,thisvar)=xsteps{j};
    
    outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],xdata);
    
    xvalues=outputs(:,1);
    yvalues=outputs(:,2);
    set(udata.view.linepaths(j),'xdata',xvalues,'ydata',yvalues);
    set(udata.view.lineplus(j),'xdata',xvalues(end),'ydata',yvalues(end));
    
    if min(xvalues)<handles.minlimit{d}{a}(udata.view.x);handles.minlimit{d}{a}(udata.view.x)=min(xvalues);doupdate=1;end
    if max(xvalues)>handles.maxlimit{d}{a}(udata.view.x);handles.maxlimit{d}{a}(udata.view.x)=max(xvalues);doupdate=1;end
    if min(yvalues)<handles.minlimit{d}{a}(udata.view.y);handles.minlimit{d}{a}(udata.view.y)=min(yvalues);doupdate=1;end
    if max(yvalues)>handles.maxlimit{d}{a}(udata.view.y);handles.maxlimit{d}{a}(udata.view.y)=max(yvalues);doupdate=1;end   

end


%% MAKE TICK MARKS:
numpts=min(5,udata.view.npoints);
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};
nvars=numel(zcol);

[xsteps totalrows]=ravecalculatexsteps(src,d,a,zcol,numpts,1);
% 
% for j=1:numel(datatypes);
%     switch datatypes{j}
%         case 'constant'
%             xsteps{j}=xdials(zcol(j));
%         case 'continuous'
%             xsteps{j}=xmin(zcol(j)):step(zcol(j)):xmax(zcol(j));
%         case 'integer'
%             if numel(handles.discretevalues{d}{a}{zcol(j)})<10
%                 xsteps{j}=handles.discretevalues{d}{a}{zcol(j)};
%             else
%                 xsteps{j}=xmin(zcol(j)):step(zcol(j)):xmax(zcol(j));
%             end
%         case 'discrete'
%             xsteps{j}=handles.discretevalues{d}{a}{zcol(j)};
%         case 'string'
%             xsteps{j}=handles.discretevalues{d}{a}{zcol(j)};
%         case 'logical'
%             xsteps{j}=[0 1];
%     end
% end

for j=1:nvars
    thisvar=zcol(j);
    thiscolor=handles.variablecolors{d}(thisvar,:);
    
    startpoint=xdials;

    npoints=numel(xsteps{j});
    xdata=repmat(startpoint,[npoints 1]);
    xdata(:,thisvar)=xsteps{j};
    
    outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],xdata);
    
    xvalues=outputs(:,1);
    yvalues=outputs(:,2);
    %make paths:
    set(udata.view.tickpaths(j),'xdata',xvalues,'ydata',yvalues);
end

if ~udata.format.showticks
    set(udata.view.tickpaths,'visible','off')
end


if doupdate
    guidata(handles.fig,handles);
    xrange=handles.maxlimit{d}{a}(xcol)-handles.minlimit{d}{a}(xcol);
    yrange=handles.maxlimit{d}{a}(ycol)-handles.minlimit{d}{a}(ycol);
    set(targetaxes,'xlim',[handles.minlimit{d}{a}(xcol)-0.02*xrange,handles.maxlimit{d}{a}(xcol)+0.02*xrange],...
        'ylim',[handles.minlimit{d}{a}(ycol)-0.02*yrange,handles.maxlimit{d}{a}(ycol)+0.02*yrange])
    set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'));
    set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'));
    
end


outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],xdials);

xvalues=outputs(:,1);
yvalues=outputs(:,2);

%make the white dot at the center:
set(udata.view.pspot,'xdata',xvalues,'ydata',yvalues);

%draw constraints:
constraints=udata.view.constraints(udata.view.constraints~=0);
for i=1:numel(constraints)
    thisconstraint=constraints(i);   
    h=ravedrawconstraint(targetaxes,handles.activeindex,thisconstraint);
end


