function raveupdatehyperslice(src,eventdata)
%this is the buttondownfcn for hyperslice axes, to update the x
%value of the selected graph and redraw all graphs in the new position.
%Also includes buttonmotionfcn version.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%need to check if this graph is active. if not , return and do nothing.
thisindex=get(src,'userdata');
% if thisindex~=handles.activeindex
%     return
% end

%if format tab is active, instead of doing this, make this graph the active subgraph to allow axes
%changes and such:

if strcmp(handles.activetab,'format')
    raveselectsubgraph(src,eventdata)
    return
end

raveslidehyperslice(handles.fig,[],thisindex);
%make sliding along graph update continuously:
set(handles.fig,'windowbuttonupfcn',{@raveslidedone,thisindex})
set(handles.fig,'windowbuttonmotionfcn',{@raveslidehyperslice,thisindex})



function raveslidehyperslice(src,ev,ai)
%NOTE: SRC FOR THIS FUNCTION IS EQUAL TO HANDLES.FIG REGARDLESS OF WHAT WAS
%CLICKED.
%persistent handles
handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

mins=handles.minlimit{d}{a};
maxs=handles.maxlimit{d}{a};


xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;

numy=numel(ycol);


%find this plot's location in the matrix:
[yindex,xindex]=find(handles.subax{ai}==gca);
if isempty(xindex);return;end

%if ~isempty(src) %if this function is called as a callback: (get mouse location and slide bars)
clicked=get(gca,'currentpoint');
xval=clicked(1,1);
yval=clicked(1,2);

%move marker within bounds
if xval>maxs(xcol(xindex))
    xval=maxs(xcol(xindex));
elseif xval<mins(xcol(xindex))
    xval=mins(xcol(xindex));
end

if yval>maxs(ycol(yindex))
    yval=maxs(ycol(yindex));
elseif yval<mins(ycol(yindex))
    yval=mins(ycol(yindex));
end


%if this variable is non-continuous, round val to nearest allowable value:
thistype=handles.datatype{d}{xcol(xindex)};
if  strcmp(thistype,'constant')
    xval=handles.minlimit{d}{a}{xcol(xindex)};
elseif ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{xcol(xindex)};
    xval=allowed(abs(allowed-xval)==min(abs(allowed-xval)));
end

thistype=handles.datatype{d}{ycol(yindex)};
if  strcmp(thistype,'constant')
    yval=handles.minlimit{d}{a}{ycol(yindex)};
elseif ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{ycol(yindex)};
    yval=allowed(abs(allowed-yval)==min(abs(allowed-yval)));
end

xval=xval(1);
yval=yval(1);

set(cell2mat(udata.view.xhair(:,xindex)),'xdata',[xval xval])
set(cell2mat(udata.view.yhair(yindex,:)),'ydata',[yval yval])
set(cell2mat(udata.view.yhair(xindex,:)),'ydata',[xval xval])
set(cell2mat(udata.view.xhair(:,yindex)),'xdata',[yval yval])

%if new value is the same as old value (because of discrete vars) don't
%waste time evaluating funcitons:
xdials=handles.xdials{d}{a};

if xdials(xcol(xindex))==xval && xdials(ycol(yindex))==yval
    return
end

xdials(xcol(xindex))=xval;
xdials(ycol(yindex))=yval;

handles.xdials{d}{a}=xdials;
guidata(handles.fig,handles);

%prepare data for gridding:

numpts=udata.view.smooth;
xdials=handles.xdials{d}{a};

[xsteps totalrows]=ravecalculatexsteps(handles.fig,d,a,xcol,numpts,1);

%precompute data so contour levels can be shared among all plots:
allthedata=cell(numel(xcol),numel(xcol),3);
for r=numel(ycol):-1:1,
    for c=numel(xcol):-1:1,
        if r==c;continue;end

        thisxdata=xsteps{c};
        thisydata=xsteps{r};

        [xgrid,ygrid]=meshgrid(thisxdata,thisydata);

        xcalldata=xgrid(:);
        ycalldata=ygrid(:);

        constants=repmat(xdials,[length(xcalldata),1]);
        constants(:,xcol(c))=xcalldata;
        constants(:,xcol(r))=ycalldata;

        %evaluate the functions:

        zvalues=raveevaluatefunction(src,d,udata.view.z,constants,handles);

        allthedata{r,c,1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,3}=reshape(zvalues,numel(thisydata),numel(thisxdata));
    end
end

zdataz=allthedata(:,:,3);
zdataz=zdataz(:);
zdataz=zdataz(~cellfun(@isempty,zdataz));

lowestvalue=min(zdataz{1}(:));
highestvalue=max(zdataz{1}(:));
for b=2:numel(zdataz)
    lowestvalue=min(lowestvalue,min(zdataz{b}(:)));
    highestvalue=max(highestvalue,max(zdataz{b}(:)));
end

%check against existing limits:
doupdate=false;
if lowestvalue<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=lowestvalue;doupdate=true;end
if highestvalue>handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=highestvalue;doupdate=true;end
if doupdate
    guidata(src,handles);

end

lowestvalue=handles.minlimit{d}{a}(udata.view.z);
highestvalue=handles.maxlimit{d}{a}(udata.view.z);

levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);
udata.view.levels=levelz;

for r=numel(ycol):-1:1,
    for c=numel(xcol):-1:1,
        if r==c;continue;end
        zplotdata=allthedata{r,c,3};
            if ~udata.format.reversecolors
                zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            else
                zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            end

            % contourf(ax(r,c),single(allthedata{r,c,1}),single(allthedata{r,c,2}),single(zplotdata),single(levelz),'linecolor','none');        
            C=contourc(allthedata{r,c,1}(1,:),allthedata{r,c,2}(:,1),zplotdata,levelz);
            h=udata.view.mainhandles{r,c};
            contourplot(C,h,'update');
     
            %% draw constraints:
            oldconstraints=findobj(handles.subax{ai}(r,c),'tag','constraint');
            for i=1:numel(oldconstraints)
                ldata=get(oldconstraints(i),'userdata');
                delete(ldata.handles)
            end
    end
end

constraints=udata.view.constraints(udata.view.constraints~=0);
ravedrawconstrainthyperslice(handles.allgraphs(ai),ai,constraints);

%handles.profilers.main{ai}=mainhandles;
guidata(src,handles);
ca=gca;
%update colorbar:
if udata.format.colorbar
    legdata=get(handles.fakerax(ai),'userdata');
    cbar=findobj(legdata,'tag','Colorbar');
    oldlim=udata.view.clim;
    newlim=[lowestvalue highestvalue];
    udata.view.clim=newlim;
    ylim=get(cbar,'ylim');
    
    if doupdate
        ca=gca;
        ravemakecolorbar(handles.fig,[],ai);
        axes(ca); %need to re-set this as the current axes since ravemakecolorbar changes it by making a new axes. 
        legdata=get(handles.fakerax(ai),'userdata');
        cbar=findobj(legdata,'tag','Colorbar');
    end

    %determine current c value:
    thisvalue=raveevaluatefunction(src,udata.view.dataset,udata.view.c,handles.xdials{udata.view.dataset}{udata.view.analysis},handles);
    thiscoord=interp1(newlim,ylim,thisvalue);
    
    %make triangle patch at this value:
    cvpatch=findobj(cbar,'tag','currentvalue');
    set(cvpatch,'ydata',[thiscoord-1.5 thiscoord+1.5 thiscoord]);
    cvtext=findobj(cbar,'tag','currentvaluetxt');
    pos=get(cvtext,'pos');
    set(cvtext,'pos',[pos(1) thiscoord],'string',sprintf('%5.5g',thisvalue));
end

%update on slide if desired:
if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    raveupdateanalysis(src,[],'xdials',ai)
end

set(handles.fig,'currentaxes',ca);
%axes(ca);


function raveslidedone(src,ev,ai) %#ok<INUSD>

handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonupfcn','')

%update other analyses to reflect new values:
raveupdateanalysis(handles.fig,[],'xdials',ai)

clear contourplot %clear persistent vars

