function ravecreatescattergrad3D(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'vis','off');

%inputs=varargin{3};
targetaxes=handles.activegraph;
xcol=udata.view.x(1); %ones are used to take first var in case 2 vars have same name.
ycol=udata.view.y(1);
zcol=udata.view.z(1);
ccol=udata.view.c(1);
scol=udata.view.s(1);
%pscales=udata.view.pscale;
%gscales=udata.view.gscale;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

set(targetaxes,'drawmode','fast');

%clear existing graph
cla(targetaxes);
legend(targetaxes,'off');

set(targetaxes,'xlimmode','auto','ylimmode','auto','zlimmode','auto');
if udata.format.boxon
    set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'box','on')
else
    set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'box','off')
end

%font formatting
fweight='normal';
labelsize=handles.prefs.fonts.graphsize;

%get data
x=handles.alldata{d}(:,xcol);
y=handles.alldata{d}(:,ycol);
z=handles.alldata{d}(:,zcol);

isselected=handles.isselected{d}{a};
nothidden=handles.isvisible{d}{a};
isfiltered=handles.isfiltered{d}{a};


if ccol~=0
    c=handles.alldata{d}(:,ccol);
    %normalize cdata to desired colormap:
    if ~udata.format.reversecolors
        c=(c-min(c))/(max(c)-min(c))*127+1+128*(udata.format.mcmap-1);
    else
        c=-(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap);
    end
else
    %c=zeros(length(x),1);
    if udata.format.ncolor
        c=1920+(handles.rowcolors{d}{a})+ rand -.5; %noisy colors
    else
        c=1920+(handles.rowcolors{d}{a}); %non-noisy colors cdata=0.5
    end
end


set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);

%size points:
s=ones(length(x),1)*udata.format.minmarkersize;
s=sqrt(s); %for consistency with other matlab functions


%% MAKE FILTERED DATA SERIES
usdata=unique(s);

%determine which patch each row belongs to (based on s)
npatches=numel(usdata);
patchidx=zeros(size(s,1),1);

%assign points to patches
for g=1:size(s,1)
    patchidx(g)=find(usdata==s(g),1,'first');
end
%make the patches:

filtpatchhandles=zeros(npatches,1);
mainpatchhandles=zeros(npatches,1);
selpatchhandles=zeros(npatches,1);
%filtered handles
for j=1:npatches
    filtidx=patchidx==j & nothidden;
    if any(filtidx)
        filtpatchhandles(j)=line(x(filtidx),y(filtidx),z(filtidx),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
            'markeredgecolor','none','hittest','off','linestyle','none');
    else
        filtpatchhandles(j)=line(x(1),y(1),z(1),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
            'markeredgecolor','none','visible','off','hittest','off','linestyle','none');
    end
end

%main handles

for j=1:npatches
    mainidx=patchidx==j & ~isfiltered & nothidden;
    if any(mainidx)
        mainpatchhandles(j)=patch(x(mainidx),y(mainidx),z(mainidx),c(mainidx),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor','flat','parent',targetaxes,'cdatamapping','direct','tag','main',...
            'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','hittest','off',...
            'buttondownfcn',@pointselectscattergrad3D);
    else
        mainpatchhandles(j)=patch(x(1),y(1),z(1),c(1),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor','flat','parent',targetaxes,'cdatamapping','direct','tag','main',...
            'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','visible','off','hittest','off',...
            'buttondownfcn',@pointselectscattergrad3D);
    end
end

if strcmp(handles.selectionmode,'point')
    set(mainpatchhandles,'hittest','on');
end

%selected handles
for j=1:npatches
    selidx=patchidx==j & isselected & nothidden;
    if any(selidx)
        selpatchhandles(j)=line(x(selidx),y(selidx),z(selidx),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
            'markeredgecolor',handles.colors.selectionhighlight,...
            'visible','on','tag','selected','linewidth',2,'hittest','off');
    else
        selpatchhandles(j)=line(x(1),y(1),z(1),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
            'markeredgecolor',handles.colors.selectionhighlight,...
            'visible','off','tag','selected','linewidth',2,'hittest','off');
    end
end

udata.view.filthandles=filtpatchhandles;
udata.view.mainhandles=mainpatchhandles;
udata.view.selhandles=selpatchhandles;
udata.view.patchidx=patchidx;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%format for fillstyle:
fillstyle=udata.format.markerfill;
if fillstyle==1
    set(udata.view.mainhandles,'markerfacecolor','none','markeredgecolor','flat');
    set(udata.view.filthandles,'markerfacecolor','none','markeredgecolor',handles.colors.filtered);
elseif fillstyle==2
    set(udata.view.mainhandles,'markerfacecolor','flat','markeredgecolor','none');
    set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
else %fillstyle=3
    set(udata.view.mainhandles,'markerfacecolor','flat','markeredgecolor',[0 0 0]);
    set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
end

%% make arrows:

%determine points to evaluate derivative:
s=udata.view.s;

if s~=0;
    
    del=mean(handles.alldata{d}(:,s))*.0001;
    
    if isempty(del)
        del=0;
    end
    
    indata=handles.alldata{d};
    
    lowerpoints=indata;
    lowerpoints(:,s)=indata(:,s)-del;
    upperpoints=indata;
    upperpoints(:,s)=indata(:,s)+del;
    
    basepointsout=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y udata.view.z],indata);
    lowerpointsout=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y udata.view.z],lowerpoints);
    upperpointsout=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y udata.view.z],upperpoints);
    
    basepointsx=basepointsout(:,1);
    lowerpointsx=lowerpointsout(:,1);
    upperpointsx=upperpointsout(:,1);
    
    basepointsy=basepointsout(:,2);
    lowerpointsy=lowerpointsout(:,2);
    upperpointsy=upperpointsout(:,2);
    
    basepointsz=basepointsout(:,3);
    lowerpointsz=lowerpointsout(:,3);
    upperpointsz=upperpointsout(:,3);
    
    xdiff=upperpointsx-lowerpointsx;
    ydiff=upperpointsy-lowerpointsy;
    zdiff=upperpointsz-lowerpointsz;
    
    xlim=get(targetaxes,'xlim');
    ylim=get(targetaxes,'ylim');
    zlim=get(targetaxes,'zlim');
    
    pos=get(targetaxes,'pos');
    xpixperunit=pos(3)/diff(xlim);
    ypixperunit=pos(3)/diff(ylim);
    zpixperunit=pos(4)/diff(zlim);
    
    x=(basepointsx-xlim(1))*xpixperunit;
    y=(basepointsy-ylim(1))*ypixperunit;
    z=(basepointsz-zlim(1))*zpixperunit;
    
    u=xdiff*xpixperunit;
    v=ydiff*ypixperunit;
    w=zdiff*zpixperunit;
    
    alpha = 0.33; % length of arrow head
    
    autoscale=10;
    autoscale=autoscale/10;
    
    if autoscale>0
        % Base autoscale value on average spacing in the x and y
        % directions.  Estimate number of points in each direction as
        % either the size of the input arrays or the effective square
        % spacing if x and y are vectors.
        
        n=sqrt(numel(x));
        
        
        delx = diff([min(x(:)) max(x(:))])/n;
        dely = diff([min(y(:)) max(y(:))])/n;
        delz = diff([min(z(:)) max(z(:))])/n;
        
        del = delx.^2 + dely.^2 + delz.^2;
        if del>0
            len = sqrt((u.^2 + v.^2 + w.^2)/del);
            maxlen = max(len(:));
        else
            maxlen = 0;
        end
        
        if maxlen>0
            autoscale = autoscale*0.9 / maxlen;
        else
            autoscale = autoscale*0.9;
        end
        u = u*autoscale;
        v = v*autoscale;
        w = w*autoscale;
    end
    
    % Make vectors
    x = x(:).';
    y = y(:).';
    z = z(:).';
    u = u(:).';
    v = v(:).';
    w = w(:).';
    
    uu = [x;x+u;repmat(NaN,size(u))];
    vv = [y;y+v;repmat(NaN,size(u))];
    ww = [z;z+w;repmat(NaN,size(u))];
    
    x1=x+u*(1-alpha);
    y1=y+v*(1-alpha);
    z1=z+w*(1-alpha);
    
    x2=x+u;
    y2=y+v;
    z2=z+w;
    
    phi=20*pi/180;
    
    x3=x2+(x1-x2)*cos(phi) + (y1-y2)*sin(phi);
    y3=y2-(x1-x2)*sin(phi) + (y1-y2)*cos(phi);
    z3=z2;
    
    x4=x2+(x1-x2)*cos(-phi) + (y1-y2)*sin(-phi);
    y4=y2-(x1-x2)*sin(-phi) + (y1-y2)*cos(-phi);
    z4=z2;
    
    x3=x2+(x1-x2)*cos(phi) + (y1-y2)*sin(phi) - (z1-z2)*sin(phi);
    y3=y2-(x1-x2)*sin(phi) + (y1-y2)*cos(phi) + (z1-z2)*sin(phi);
    z3=z2-(x1-x2)*cos(phi) + (y1-y2)*sin(+phi) + (z1-z2)*sin(phi);
    
    x4=x2+(x1-x2)*cos(-phi) + (y1-y2)*sin(-phi) + (z1-z2)*sin(phi);
    y4=y2-(x1-x2)*sin(-phi) + (y1-y2)*cos(-phi) - (z1-z2)*sin(phi);
    z4=z2+(x1-x2)*cos(+phi) + (y1-y2)*sin(+phi) + (z1-z2)*sin(phi); 
    
    
    
    
    
    %convert back to graph units:
    x2=x2/xpixperunit+xlim(1);
    x3=x3/xpixperunit+xlim(1);
    x4=x4/xpixperunit+xlim(1);
    uu=uu/xpixperunit+xlim(1);
    
    y2=y2/ypixperunit+ylim(1);
    y3=y3/ypixperunit+ylim(1);
    y4=y4/ypixperunit+ylim(1);
    vv=vv/ypixperunit+ylim(1);
    
    z2=z2/zpixperunit+zlim(1);
    z3=z3/zpixperunit+zlim(1);
    z4=z4/zpixperunit+zlim(1);
    ww=ww/zpixperunit+zlim(1);
    
    
    %draw center line
    h1=line(uu(:),vv(:),ww(:),'parent',targetaxes,'buttondownfcn',[],'hittest','off','tag','derivativefield',...
        'color',[0 0 0]);

    
    hu=[x3;x2;x4;NaN(size(u))];
    hv=[y3;y2;y4;NaN(size(u))];
    hw=[z3;z2;z4;NaN(size(u))];
    
%     beta=0.333;
%     alpha=0.333;
%     
%        beta = beta * sqrt(u.*u + v.*v + w.*w) ./ (sqrt(u.*u + v.*v) + eps);
%     
%     % Make arrow heads and plot them
%     hu = [x+u-alpha*(u+beta.*(v+eps));x+u; ...
%         x+u-alpha*(u-beta.*(v+eps));repmat(NaN,size(u))];
%     hv = [y+v-alpha*(v-beta.*(u+eps));y+v; ...
%         y+v-alpha*(v+beta.*(u+eps));repmat(NaN,size(v))];
%     hw = [z+w-alpha*w;z+w;z+w-alpha*w;repmat(NaN,size(w))];
%     
%     
    %draw head
  %  h2=line(hu(:),hv(:),hw(:),'parent',targetaxes,'tag','derivativefield','buttondownfcn',[]);
    
  
%   
%     udat.handles=[h1];
%     udat.delete=1;
%     
%     set([h1],'userdata',udat);
%     
    uistack([h1],'bottom'); %seems like more often than not you'd want this below the data.
    
    udata.view.arrowhandles=[h1];
    set(handles.fakeax(handles.activeindex),'userdata',udata);
    
    
else
    udata.view.arrowhandles=[];
end

%% axes stuff:

set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))


%link axes so subsequent limit changes affect all axes:
% linkaxes([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xy');
set(targetaxes,'xlimmode','manual','ylimmode','manual','zlimmode','manual');

%set axes labels

set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
set(get(targetaxes,'ZLabel'),'String',handles.allnames{d}{zcol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)

set(get(handles.fakerax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
set(get(handles.fakerax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
set(get(handles.fakerax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)

set(get(handles.fakeax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakeax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
set(get(handles.fakeax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on','zminorgrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
end

%reposition camera for iso view:

% set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
%     'cameraposition',[1 1 1]);
%
%  view(targetaxes,3);
%  view(handles.fakeax(handles.activeindex),3);
%  view(handles.fakerax(handles.activeindex),3);


%make all axes have same limits
Q=[handles.fakerax(handles.activeindex) handles.fakeax(handles.activeindex)];

set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'),...
    'xtickmode','manual','xtick',get(targetaxes,'xtick'),...
    'ytickmode','manual','ytick',get(targetaxes,'ytick'),...
    'ztickmode','manual','ztick',get(targetaxes,'ztick'));
set(Q,...
    'xticklabel',get(targetaxes,'xticklabel'),...
    'yticklabel',get(targetaxes,'yticklabel'),...
    'zticklabel',get(targetaxes,'zticklabel'));
set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'))
set(targetaxes,'nextplot','add')
set(Q,'ticklength',[0 0]);

%do mouseovers
if udata.explore.mouseover
    l1=line(nan,nan,nan,'marker',udata.format.marker,'markerfacecolor','none','markeredgecolor',handles.colors.mouseoverhighlight,'linewidth',1,...
        'visible','off','parent',targetaxes,'hittest','off','linestyle','none');
    t1=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal');
    t2=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal');
    t3=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes);
    udata.explore.mouseoverhandles=[l1 t1 t2 t3];
    udata.explore.cbarmouseovervalue=[]; %initialize these in case of no colorbar
    udata.explore.cbarmouseovertext=[];
    set(handles.fakeax(handles.activeindex),'userdata',udata);
    handles.mouseoverhandles=[handles.mouseoverhandles;targetaxes;filtpatchhandles(:)];
    handles.mouseoverfunctions=[handles.mouseoverfunctions;{{@ravemouseoverscattergrad3Daxes,handles.activeindex}};repmat({{@ravemouseoverscattergrad3Ddata,handles.activeindex}},numel(filtpatchhandles),1)];
    handles.mouseoverdata=[handles.mouseoverdata;{[]};cell(numel(filtpatchhandles),1)];
    clear ravedefaultwindowbuttonmotionfcn ravemouseoverscattergrad3Daxes ravemouseoverscattergrad3Ddata
end

%update da handles
guidata(handles.fig,handles)


%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.


raveresizeaxes(src,[],targetindex) %call this before and after maketexticks to avoid a problem with zaxis not updating.
ravemaketextticks(targetaxes,[],xcol,ycol,zcol,[])
raveresizeaxes(src,[],targetindex)
ravemakecolorbar(handles.fig,[]);

set(Q,...
    'xticklabel','',...
    'yticklabel','',...
    'zticklabel','');

set(get(handles.fakerax(handles.activeindex),'XLabel'),'String','')
set(get(handles.fakerax(handles.activeindex),'YLabel'),'String','')
set(get(handles.fakerax(handles.activeindex),'ZLabel'),'String','')

set(get(handles.fakeax(handles.activeindex),'XLabel'),'String','')
set(get(handles.fakeax(handles.activeindex),'YLabel'),'String','')
set(get(handles.fakeax(handles.activeindex),'ZLabel'),'String','')


%fix rotation if necessary:
view(handles.activegraph,udata.view.az,udata.view.el)
view(handles.fakeax(handles.activeindex),udata.view.az,udata.view.el)
view(handles.fakerax(handles.activeindex),udata.view.az,udata.view.el)

xxx=get(handles.activegraph,'xlabel');
set(xxx,'buttondownfcn',@ravemove3dlabels,'hittest','on');
yyy=get(handles.activegraph,'ylabel');
set(yyy,'buttondownfcn',@ravemove3dlabels,'hittest','on');

set(targetaxes,'buttondownfcn',@buttondownfcnscattergrad3D,'hittest','on')

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects




