function ravecreatediscretecontrols(src,ev)
%create discrete graph controls on workspace.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');


targetaxes=handles.activegraph; %=handles.activegraph
xcol=udata.view.x;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

set(targetaxes,'visible','off')
set(handles.fakeax(handles.activeindex),'visible','off')
set(handles.fakerax(handles.activeindex),'visible','off')

origin=get(targetaxes,'position');

%make pixel coordinates:
set(targetaxes,'xlim',[1 origin(3)],'ylim',[1 origin(4)],'xticklabel','','yticklabel','');
text(0,88,'Select Data:','parent',targetaxes,'fontweight','bold');

switch handles.selectionmode
    case 'point'
        vals=[1 0 0];
    case 'lasso'
        vals=[0 0 1];
    case 'box'
        vals=[0 1 0];
end


pointtoggle=drawtogglebutton(handles.wspanel,[origin(1)+10 origin(2)+50 26 26],handles.colors.togglebuttonbg,imread('singlepoint.bmp'),'',@ravesetpointselectionmode,vals(1));
boxtoggle=drawtogglebutton(handles.wspanel,[origin(1)+40 origin(2)+50 26 26],handles.colors.togglebuttonbg,imread('box.bmp'),'',@ravesetboxselectionmode,vals(2));
lassotoggle=drawtogglebutton(handles.wspanel,[origin(1)+70 origin(2)+50 26 26],handles.colors.togglebuttonbg,imread('lasso.bmp'),'',@ravesetlassoselectionmode,vals(3));

set(pointtoggle,'tag','pointtoggle')
set(boxtoggle,'tag','boxtoggle')
set(lassotoggle,'tag','lassotoggle')

text(120,88,'Hide Selected Data:','parent',targetaxes,'fontweight','bold');
hidebutton=drawpushbutton(handles.wspanel,[origin(1)+130 origin(2)+50 26 26],handles.colors.togglebuttonbg,imread('hide.bmp'),'',@hidebutton_callback);


text(0,30,'Recolor Selected Data:','parent',targetaxes,'fontweight','bold');
%make color buttons:
colortestbutton=zeros(1,10);
thecolors=handles.prefs.rowcolors;

for c=1:10
    colortestbutton(c)=drawpushbutton(handles.wspanel,[origin(1)+10+22*(c-1) origin(2)+0 18 18],...
        thecolors(c,:),[],'',@rowcolorblob_callback);
    set(colortestbutton(c),'userdata',c);
end

udata.view.allbuttons=[colortestbutton pointtoggle boxtoggle lassotoggle hidebutton];
udata.view.colorbuttons=colortestbutton;
udata.view.pointbutton=pointtoggle;
udata.view.boxbutton=boxtoggle;
udata.view.lassobutton=lassotoggle;
udata.view.hidebutton=hidebutton;


set(handles.fakeax(handles.activeindex),'userdata',udata);

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects




function rowcolorblob_callback(src,ev)
handles=guidata(src);

d=handles.lastd;
a=handles.lasta;

thesepoints=handles.isselected{d}{a};
handles.rowcolors{d}{a}(thesepoints)=get(src,'userdata');

guidata(handles.fig,handles);
raveupdateanalysis(handles.fig,[],'rowcolors',[]);

%{

function pointbutton_callback(src,ev,value)
handles=guidata(src);

lassobutton=findobj(handles.fig,'tag','lassobutton','type','uicontrol'); %real toggle buttons
set(lassobutton,'value',0); 
lassotoggle=findobj(handles.fig,'tag','lassotoggle','type','axes'); %drawtogglebutton
unpushtogglebutton(lassotoggle);

boxbutton=findobj(handles.fig,'tag','boxbutton','type','uicontrol'); %real toggle buttons
set(boxbutton,'value',0); 
boxtoggle=findobj(handles.fig,'tag','boxtoggle','type','axes'); %drawtogglebutton
unpushtogglebutton(boxtoggle);

if strcmp(handles.activetab,'row')
    otherbuttons=findobj(handles.tabobjects,'tag','boxbutton','-or','tag','lassobutton');
    set(otherbuttons,'value',0);
    thisbutton=findobj(handles.tabobjects,'tag','pointbutton');
    set(thisbutton,'value',1);
end

if value
    handles.selectionmode='point';
else
    handles.selectionmode='none';
end
guidata(src,handles);


function boxbutton_callback(src,ev,value)
handles=guidata(src);

lassobutton=findobj(handles.fig,'tag','lassobutton','type','uicontrol'); %real toggle buttons
set(lassobutton,'value',0); 
lassotoggle=findobj(handles.fig,'tag','lassotoggle','type','axes'); %drawtogglebutton
unpushtogglebutton(lassotoggle);

pointbutton=findobj(handles.fig,'tag','pointbutton','type','uicontrol'); %real toggle buttons
set(pointbutton,'value',0); 
pointtoggle=findobj(handles.fig,'tag','pointtoggle','type','axes'); %drawtogglebutton
unpushtogglebutton(pointtoggle);

if strcmp(handles.activetab,'row')
    otherbuttons=findobj(handles.tabobjects,'tag','pointbutton','-or','tag','lassobutton');
    set(otherbuttons,'value',0);
    thisbutton=findobj(handles.tabobjects,'tag','boxbutton');
    set(thisbutton,'value',1);
end

if value
    handles.selectionmode='box';
else
    handles.selectionmode='none';
end
guidata(src,handles);

function lassobutton_callback(src,ev,value)
handles=guidata(src);

pointbutton=findobj(handles.fig,'tag','pointbutton','type','uicontrol'); %real toggle buttons
set(pointbutton,'value',0); 
pointtoggle=findobj(handles.fig,'tag','pointtoggle','type','axes'); %drawtogglebutton
unpushtogglebutton(pointtoggle);

boxbutton=findobj(handles.fig,'tag','boxbutton','type','uicontrol'); %real toggle buttons
set(boxbutton,'value',0); 
boxtoggle=findobj(handles.fig,'tag','boxtoggle','type','axes'); %drawtogglebutton
unpushtogglebutton(boxtoggle);

if strcmp(handles.activetab,'row')
    otherbuttons=findobj(handles.tabobjects,'tag','boxbutton','-or','tag','pointbutton');
    set(otherbuttons,'value',0);
    thisbutton=findobj(handles.tabobjects,'tag','lassobutton');
    set(thisbutton,'value',1);
end

if value
    handles.selectionmode='lasso';
else
    handles.selectionmode='none';
end
guidata(src,handles);

%}

function hidebutton_callback(src,ev,value)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
handles.isvisible{udata.view.dataset}{udata.view.analysis}(handles.isselected{udata.view.dataset}{udata.view.analysis})=false;
guidata(src,handles);

raveupdateanalysis(handles.activegraph,[],'visiblerows',[])




