function viewtabline(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%RAVE: creates uicontrols on the view tab for line graphs.

handles=guidata(src);

%move front tabs to front if necessary
% if handles.fronttabsfront==0 %swap fronttabs and backtabs
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y);
% end
% handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

set(handles.infobar,'string','');

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:

%inputs=get(handles.fakeax(handles.activeindex),'userdata');
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.

%% text headers
projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.77*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
optionstext=text(.15-.10317,.29*1.0317-.0317,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels
analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
xaxistext=text(.25-.10317,.74*1.0317-.0317,'X-Axis Variable:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','tag','xaxistext');
yaxistext=text(.25-.10317,.67*1.0317-.0317,'Y-Axis Variable(s):','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','tag','yaxistext');
consttext=text(.25-.10317,.47,'Draw Lines of Constant:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','tag','caxistext');
trajtext=text(.25-.10317,.39,'Connect points in order of:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','tag','saxistext');


%% variable lists
analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,'callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',view.analysis);
datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',view.dataset);
xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+420 180 30],...
    'style','popupmenu','string',[{'<Data Point Index>'} handles.allnames{view.dataset}],'callback',@xaxismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','xaxismenu','value',udata.view.x+1);
yaxismenu=uicontrol('position',[origin(1)+55 origin(2)+309 180 96],...
    'style','listbox','string',handles.allnames{view.dataset},'callback',@yaxismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','yaxismenu','max',1000,'value',udata.view.y);
if udata.view.c>0;set(yaxismenu,'max',1);end

constmenu=uicontrol('position',[origin(1)+55 origin(2)+175 180 110],...
    'style','popupmenu','string',[{'<Draw Single Line>'},handles.allnames{view.dataset}(1:end)],'callback',@constmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','caxismenu','value',udata.view.c+1);
if numel(udata.view.y)>1;set(constmenu,'enable','off');end


trajmenu=uicontrol('position',[origin(1)+55 origin(2)+125 180 110],...
    'style','popupmenu','string',[{'<X-Axis Variable>'},{'<Row Order in Data Set>'},handles.allnames{view.dataset}(1:end)],'callback',@trajmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','saxismenu','value',udata.view.s+2);

% lefttext=text(.26-.10317,.57*1.0317-.0317,'Left Axis','horizontalalignment','left',...
%     'backgroundcolor','none',...
%     'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','normal','rotation',90,'visible','off');
% righttext=text(.26-.10317,.42*1.0317-.0317,'Right','horizontalalignment','left',...
%     'backgroundcolor',handles.colors.tabbg,...
%     'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','normal','rotation',90,'visible','off');

%% buttons

scalebutton=uicontrol('position',[origin(1)+55 origin(2)+30 180 24],...
    'style','pushbutton','string','Scale/Transform Variables','backgroundcolor',handles.colors.buttonbg,'visible','off');
changeunitsbutton=uicontrol('position',[origin(1)+55 origin(2)+2 180 24],...
    'style','pushbutton','string','Change Units','backgroundcolor',handles.colors.buttonbg,'visible','off');

usevariablecolorscheck=uicontrol('position',[origin(1)+55 origin(2)+135 200 26],...
    'style','checkbox','string','Use fixed variable colors','value',view.usevariablecolors,...
    'backgroundcolor',handles.colors.tabbg,'tag','usevariablecolorscheck','callback',@usevariablecolorscheck_callback);

if udata.view.c~=0;set(usevariablecolorscheck,'enable','off');end


if udata.view.autoscale==0
    valz=[0 0];
elseif udata.view.autoscale==1;
    valz=[1 0];
else
    valz=[0 1];
end

autoscalecheck=uicontrol('position',[origin(1)+55 origin(2)+115 200 26],...
    'style','checkbox','string','Autoscale Y-axis data',...
    'backgroundcolor',handles.colors.tabbg,'value',valz(1),'callback',@autoscalecheck_callback,'tag','autoscalecheck');

normalizecheck=uicontrol('position',[origin(1)+55 origin(2)+85 200 26],...
    'style','checkbox','string','Normalize Y-axis data from 0-1',...
    'backgroundcolor',handles.colors.tabbg,'value',valz(2),'callback',@autoscalecheck_callback,'tag','normalizecheck');

% useresponsedata=uicontrol('position',[origin(1)+55 origin(2)+95 200 26],...
%     'style','checkbox','string','Limit Y-axis to Response Variables',...
%     'backgroundcolor',handles.colors.tabbg,'value',view.response,'callback',@useresponsedata_callback,'tag','useresponsedata');
% sortdatacheck=uicontrol('position',[origin(1)+55 origin(2)+75 200 26],...
%     'style','checkbox','string','Sort Data by X-Axis Variable',...
%     'backgroundcolor',handles.colors.tabbg,'value',0,'tag','sortdatacheck');
% excludecheck=uicontrol('position',[origin(1)+55 origin(2)+55 200 26],...
%     'style','checkbox','string','Hide constant columns in menus',...
%     'backgroundcolor',handles.colors.tabbg,'value',view.exclude,'callback',@excludecheck_callback,'tag','excludecheck');
%



%% group items and finish up
handles.tabobjects=[datasettext xaxistext yaxistext consttext datasetmenu xaxismenu yaxismenu constmenu,...
    scalebutton analysistext projecttext analysismenu dataseriestext optionstext,...
    changeunitsbutton autoscalecheck usevariablecolorscheck trajtext trajmenu normalizecheck];

handles.activetab='view';

guidata(src,handles);
% update menus to hide excluded vars
%viewtabmenufilter(src,[])


%% Exclude hidden variables from menus:
%
% function viewtabmenufilter(src,eventdata)
%
% handles=guidata(src);
%
% %get input info from graph:
% udata=get(handles.fakeax(handles.activeindex),'userdata');
% %view=udata.view; %get view tab structure.
%
% %datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
% xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
% yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
% constmenu=findobj(handles.tabobjects,'tag','constmenu');
% %useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
% %excludecheck=findobj(handles.tabobjects,'tag','excludecheck');
%
% %get old strings/values
% % oldxvalue=view.x;
% % oldyvalue=view.y;
% % oldcvalue=view.c;
% %
%
% %d=udata.view.dataset;
%
% newxvalue=udata.view.x;
% newyvalue=udata.view.y;
% newcvalue=udata.view.c+1;
%
% set(yaxismenu,'value',newyvalue);
% set(xaxismenu,'value',newxvalue);
% set(constmenu,'value',newcvalue);

%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%update other menus to show correct variables
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

d=get(datasetmenu,'value');

if d==udata.view.dataset;
    return
end


%get initial columns for this data set:
if ~all(handles.isresponse{d}==0)
    x=find((~handles.isconstant{d} & ~handles.isresponse{d}),1);
    y=find((~handles.isconstant{d} & handles.isresponse{d}),1);
    c=0;
else
    x=find((~handles.isconstant{d}),1);
    y=find((~handles.isconstant{d}),1);
    c=0;
end

udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.x=x;
udata.view.y=y;
udata.view.c=c;
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);


handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);

%viewtabmenufilter(src,[])

ravecreateline(src,[]);

viewtab(handles.fig,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata)
handles=guidata(src);
handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata)
handles=guidata(src);
guidata(src,handles);

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.x=get(src,'value')-1;

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function yaxismenu_callback(src,eventdata)
handles=guidata(src);

guidata(src,handles);

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.y=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

if numel(udata.view.y)>1
    set(findobj(handles.tabobjects,'tag','caxismenu'),'enable','off')
else
    set(findobj(handles.tabobjects,'tag','caxismenu'),'enable','on')
end

ravecreateline(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function zaxismenu_callback(src,eventdata)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function constmenu_callback(src,eventdata)
handles=guidata(src);
guidata(src,handles);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.c=get(src,'value')-1;
set(handles.fakeax(handles.activeindex),'userdata',udata);

if udata.view.c>0
    set(findobj(handles.tabobjects,'tag','yaxismenu'),'max',1)
else
    set(findobj(handles.tabobjects,'tag','yaxismenu'),'max',1000)
end


ravecreateline(src,[]);


function trajmenu_callback(src,eventdata)
handles=guidata(src);
guidata(src,handles);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.s=get(src,'value')-2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);



function zaxischeck_callback(src,ev)
handles=guidata(src);
zaxismenu=findobj(handles.tabobjects,'tag','zaxismenu');

if get(src,'value')
    set(zaxismenu,'enable','on','backgroundcolor',[1 1 1])
else
    set(zaxismenu,'enable','off','backgroundcolor',[.75 .75 .75])
end

function usevariablecolorscheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.usevariablecolors=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);



function autoscalecheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.autoscale=0;


ascheck=findobj(handles.tabobjects,'tag','autoscalecheck');
normcheck=findobj(handles.tabobjects,'tag','normalizecheck');

if src==ascheck
    set(normcheck,'value',0);
    if get(src,'value')
        udata.view.autoscale=1;
    end
else
    set(ascheck,'value',0);
    if get(src,'value')
        udata.view.autoscale=2;
    end
end

set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreateline(src,[]);




