function parallelcoordsfilter(src,ev)
%filter a parallelcoords graph by dragging the top or bottomarrows.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
ai=find(handles.allgraphs==gca | handles.fakeax==gca);
udata=get(handles.fakeax(ai),'userdata');
%clicked=get(handles.allgraphs(ai),'currentpoint');

%grow filter vectors if new columns have been added since this graph was
%last updated:
if numel(udata.view.highfilter) < size(handles.alldata{udata.view.dataset},2)
    n=numel(udata.view.highfilter);
    newhighs=handles.maxfilter{udata.view.dataset}{udata.view.analysis}((n+1):end);
    newlows=handles.minfilter{udata.view.dataset}{udata.view.analysis}((n+1):end);
    udata.view.highfilter=[udata.view.highfilter, newhighs];
    udata.view.lowfilter=[udata.view.lowfilter, newlows];
    udata.view.varmins=handles.minlimit{udata.view.dataset}{udata.view.analysis};
    udata.view.varranges=handles.maxlimit{udata.view.dataset}{udata.view.analysis}-handles.minlimit{udata.view.dataset}{udata.view.analysis};
    set(handles.fakeax(handles.activeindex),'userdata',udata);
end

uistack(gco,'top') %this makes sure the dragger doesnt get stuck underneath the opposite dragger.

index=get(src,'userdata');

series=get(gco,'tag');
switch series
    case 'maxdragger'
        texter=findobj(handles.fakeax(ai),'tag','maxtexter');
        texter=texter(index);
        set(texter,'visible','on');
        set(handles.fig,'windowbuttonmotionfcn',{@maxfilter,index,src,texter,ai,handles})
        [idx,junk]=find(udata.view.maxdraggers==gco);
        uistack(udata.view.maxdraggers(idx,1),'top');
    case 'mindragger'
        texter=findobj(handles.fakeax(ai),'tag','mintexter');
        texter=texter(index);
        set(texter,'visible','on');
        set(handles.fig,'windowbuttonmotionfcn',{@minfilter,index,src,texter,ai,handles})
        [idx,junk]=find(udata.view.mindraggers==gco);
        uistack(udata.view.mindraggers(idx,1),'top');
end

set(handles.fig,'windowbuttonupfcn',{@filterdone,ai})



function maxfilter(src,ev,index,handy,texter,ai,handles)
%handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');
thisax=get(handy,'parent');
newvalue=get(thisax,'currentpoint');
yval=newvalue(1,2,1);

if yval<udata.view.minfiltercoord(udata.view.x(index));yval=udata.view.minfiltercoord(udata.view.x(index));end
if yval>1.00001;yval=1.00001;end

ydata=yval;

[idx,junk]=find(udata.view.maxdraggers==handy);
xdata=get(handy,'xdata');

set(udata.view.maxdraggers(idx,:),'ydata',ydata,'xdata',xdata);

textpos=get(texter,'position');

set(texter,'pos',[textpos(1) ydata],...
    'string',['',raveformatnumber(ydata*udata.view.varranges(udata.view.x(index))+udata.view.varmins(udata.view.x(index)))]);

udata.view.highfilter(udata.view.x(index))=ydata*udata.view.varranges(udata.view.x(index))+udata.view.varmins(udata.view.x(index));
udata.view.maxfiltercoord(udata.view.x(index))=ydata;
set(handles.fakeax(ai),'userdata',udata);

%% do filtering
handles=parallelcoordsapplyfilter(handles.fig,[],ai,handles);

%update on slide if desired:
if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    handles.maxfilter{udata.view.dataset}{udata.view.analysis}=udata.view.highfilter;
    %note: for speed, do not need to call guidata here for change in
    %handles.maxfilter because raveupdateanalysis will call guidata at the
    %end.
    raveupdateanalysis(src,handles,'isfiltered',ai,udata.view.dataset,udata.view.analysis)
    if ~handles.prefs.filteredselectable
        raveupdateanalysis(src,handles,'isselected',ai,udata.view.dataset,udata.view.analysis) %do not include handles as 2nd input here in case it changed because of the previous raveupdateanalysis
    end
end

function minfilter(src,ev,index,handy,texter,ai,handles)
%handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');
thisax=get(handy,'parent');
newvalue=get(thisax,'currentpoint');
yval=newvalue(1,2,1);

if yval<-0.00001;yval=-0.00001;end
if yval>udata.view.maxfiltercoord(udata.view.x(index));yval=udata.view.maxfiltercoord(udata.view.x(index));end

%ydata=get(handy,'ydata');
ydata=yval;

[idx,junk]=find(udata.view.mindraggers==handy);
xdata=get(handy,'xdata');

set(udata.view.mindraggers(idx,:),'ydata',ydata,'xdata',xdata);

textpos=get(texter,'position');

set(texter,'pos',[textpos(1) ydata],...
    'string',['',raveformatnumber(ydata*udata.view.varranges(udata.view.x(index))+udata.view.varmins(udata.view.x(index)))]);

udata.view.lowfilter(udata.view.x(index))=ydata*udata.view.varranges(udata.view.x(index))+udata.view.varmins(udata.view.x(index));
udata.view.minfiltercoord(udata.view.x(index))=ydata;
set(handles.fakeax(ai),'userdata',udata);

handles=parallelcoordsapplyfilter(handles.fig,[],ai,handles);


%update on slide if desired:
if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    handles.minfilter{udata.view.dataset}{udata.view.analysis}=udata.view.lowfilter;
    %note: for speed, do not need to call guidata here for change in
    %handles.maxfilter because raveupdateanalysis will call guidata at the
    %end.
    raveupdateanalysis(src,handles,'isfiltered',ai,udata.view.dataset,udata.view.analysis)
    if ~handles.prefs.filteredselectable
        raveupdateanalysis(src,handles,'isselected',ai,udata.view.dataset,udata.view.analysis) %do not include handles as 2nd input here in case it changed because of the previous raveupdateanalysis
    end
end


function filterdone(src,ev,ai)
handles=guidata(src);

udata=get(handles.fakeax(ai),'userdata');

set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonup','')

if ~udata.format.showlimits
    texter=findobj(handles.allgraphs(ai),'tag','maxtexter','-or','tag','mintexter');
    set(texter,'visible','off');
end

%one last filter....
allkidz=sort(findobj(handles.allgraphs(ai),'tag','main'));
allydata=cell2mat(get(allkidz,'ydata'));

maxoutlier=bsxfun(@gt,allydata,udata.view.maxfiltercoord(udata.view.x));
minoutlier=bsxfun(@lt,allydata,udata.view.minfiltercoord(udata.view.x));

outliers=minoutlier | maxoutlier;
outliers=any(outliers,2);

udata.view.filtered=outliers;
set(handles.fakeax(ai),'userdata',udata);

%update other graphs:
handles.minfilter{udata.view.dataset}{udata.view.analysis}=udata.view.lowfilter;
handles.maxfilter{udata.view.dataset}{udata.view.analysis}=udata.view.highfilter;
guidata(handles.fig,handles);
raveupdateanalysis(src,[],'isfiltered',ai,udata.view.dataset,udata.view.analysis)
if ~handles.prefs.filteredselectable
    raveupdateanalysis(src,[],'isselected',ai,udata.view.dataset,udata.view.analysis)    
end


