function ravecreatehistogram3D(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');

%inputs=varargin{3};
targetaxes=handles.activegraph;
xcol=udata.view.x(1); %ones are used to take first var in case 2 vars have same name.
ycol=udata.view.y(1);

a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);
legend(targetaxes,'off');

set(targetaxes,'xlimmode','auto','ylimmode','auto','zlimmode','auto');
set(targetaxes,'xtickmode','auto','ytickmode','auto','ztickmode','auto');
set(targetaxes,'xticklabelmode','auto','yticklabelmode','auto','zticklabelmode','auto');


if udata.format.boxon
    set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'box','on')
else
    set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'box','off')
end



%font formatting
fweight='normal';

%get data
x=handles.alldata{d}(:,xcol);
y=handles.alldata{d}(:,ycol);

%maintain hidden points:
nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};
x=x(nothidden);
y=y(nothidden);

[mainrows,selectedrows,filteredrows]=ravemakeplotdataseries(handles,udata);
[xcount,xselectedcount,xfiltcount,nbarsx,xedges,xbins]=ravehistogramcount(handles,d,a,xcol,[],mainrows,selectedrows,filteredrows,udata.view.nbars,udata.view.discreteascategory);

%restore hidden points in xbins
xbinz=zeros(size(handles.alldata{d},1),1);
xbinz(nothidden)=xbins;

if ~isempty(x)
    for j=nbarsx:-1:1
        [ymaincount,yselectedcount,yfiltcount,nbarsy,yedges,ybins]=ravehistogramcount(handles,d,a,ycol,xbinz==j,mainrows,selectedrows,filteredrows,udata.view.nbars,udata.view.discreteascategory);
        ycount(j,:)=histc(y(xbins==j),yedges)';
    end
end

if udata.view.normalize %normalize counts (make pdfs)
    ycount=ycount./length(x);
end

set(targetaxes,'nextplot','add'); %cuz im sure bar3 is doing something stupid...
%main data series:
if ~isempty(x)
    mainhandle=bar3(targetaxes,yedges,ycount',(udata.view.barwidth-0.005)); %subtract a little from barwidth to avoid renderering error when barwitdh=1;
else
    mainhandle=[];
end

if udata.format.drawedge
    edgecolor=udata.format.edgecolor;
else
    edgecolor=udata.format.facecolor;
end

set(mainhandle,'facecolor',udata.format.facecolor,'hittest','off','tag','main','edgecolor',edgecolor);
set(targetaxes,'plotboxaspectratiomode','auto');%this gets screwed up by bar3...
set(targetaxes,'hittest','on','buttondownfcn',@buttondownfcnhistogram3D)


%fix xdata to match actual x coordinates:
%xdata of bars is normalized from 1 to nbars, with barwidth calculated
%according to udata.view.barwidth. need to stretch these values out to
%fill proper range of x data.

set(targetaxes,'xlimmode','auto','ylimmode','auto','zlimmode','auto','userdata','histogram3D');

%xrange=max(xedges)-min(xedges);
%xstep=xrange./(nbarsx);
%xwidth=xstep*(udata.view.barwidth-0.005)/2;

if ~isempty(x)
    for g=1:nbarsx;
         xdata=get(mainhandle(g),'xdata');
         ydata=get(mainhandle(g),'ydata');

        set(mainhandle(g),'facecolor','flat')
        %cdata=get(mainhandle(g),'cdata');

        c=ycount(g,:)';
        maxc=max(max(ycount));
        minc=min(min(ycount));
        
        if udata.format.hideempty
            for h=1:nbarsy+1
                if ycount(g,h)==0
                    xdata((h-1)*6+1:h*6,:)=nan;
                    ydata((h-1)*6+1:h*6,:)=nan;
                end
            end
            set(mainhandle(g),'xdata',xdata,'ydata',ydata);
        else
            %make empty grid surface:
            surface(xdata,ydata,zeros(size(ydata)),[1 1 1],'facecolor',[1 1 1],'edgecolor',[0 0 0],'tag','emptygrid');
        end
        
        if udata.format.usecmap
            c=(c-minc)/(maxc-minc)*127+128*(udata.format.mcmap-1)+1;
            e=repmat(1:numel(c),[6 1]);
            e=e(:);
            f=repmat(c(e),[1 4]);
            set(mainhandle(g),'cdata',f)
        else
            set(mainhandle(g),'facecolor',udata.format.facecolor);
        end
        
    end
    set(targetaxes,'xtickmode','auto','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize)
    set(handles.fakeax(handles.activeindex),'xtickmode','auto','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize)
    set(handles.fakerax(handles.activeindex),'xtickmode','auto','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize)
end

empties=findobj(targetaxes,'tag','emptygrid');
uistack(empties,'bottom');

axis(targetaxes,'tight');
set(targetaxes,'zlimmode','auto');


set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);
set(mainhandle,'tag','main');
set(handles.activegraph,'xgrid','off','ygrid','off','zgrid','off','color','none')

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',handles.prefs.fonts.graphsize,...
    'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graph)
set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
    'fontsize',handles.prefs.fonts.graphsize,'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graph)
set(get(targetaxes,'ZLabel'),'String','Count',...
    'fontsize',handles.prefs.fonts.graphsize,'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graph)

set(get(handles.fakerax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',handles.prefs.fonts.graphsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
set(get(handles.fakerax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
    'fontsize',handles.prefs.fonts.graphsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
set(get(handles.fakerax(handles.activeindex),'ZLabel'),'String','Count',...
    'fontsize',handles.prefs.fonts.graphsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)

set(get(handles.fakeax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',handles.prefs.fonts.graphsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakeax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
    'fontsize',handles.prefs.fonts.graphsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
set(get(handles.fakeax(handles.activeindex),'ZLabel'),'String','Count',...
    'fontsize',handles.prefs.fonts.graphsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on','zminorgrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
end

%make all axes have same limits
Q=[handles.fakerax(handles.activeindex) handles.fakeax(handles.activeindex)];

set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'),...
    'xtickmode','manual','xtick',get(targetaxes,'xtick'),...
    'ytickmode','manual','ytick',get(targetaxes,'ytick'),...
    'ztickmode','manual','ztick',get(targetaxes,'ztick'));

set(Q,...
    'xticklabel',get(targetaxes,'xticklabel'),...
    'yticklabel',get(targetaxes,'yticklabel'),...
    'zticklabel',get(targetaxes,'zticklabel'));
set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'))
set(targetaxes,'nextplot','add')
set(Q,'ticklength',[0 0]);

%update handles
guidata(handles.fig,handles)

ravemakecolorbar(handles.fig,[]);

%fix rotation if necessary:
view(handles.activegraph,udata.view.az,udata.view.el)
view(handles.fakeax(handles.activeindex),udata.view.az,udata.view.el)
view(handles.fakerax(handles.activeindex),udata.view.az,udata.view.el)


%fix rotation if necessary:
view(handles.activegraph,udata.view.az,udata.view.el)
view(handles.fakeax(handles.activeindex),udata.view.az,udata.view.el)
view(handles.fakerax(handles.activeindex),udata.view.az,udata.view.el)

set([handles.activegraph handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ticklength',[0.02 0.0025]);

%note: for 3d graphs, set view before making text ticks, because zticks
%dont get made in a 2d view.

zticks=get(targetaxes,'zticklabel');
if ~iscell(zticks);
    zticks=mat2cell(zticks,ones(size(zticks,1),1),size(zticks,2));
    zticks=cellfun(@strtrim,zticks,'uniformoutput',0);
end

ravemaketextticks(targetaxes,[],udata.view.x,udata.view.y,0,[],[],[],zticks);

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects
