function ravemakecolorbarhistogram3D(src,ev,ai) 
%make a colorbar for a density graph, which can color by data density.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

if nargin<3;
    ai=handles.activeindex;
end

udata=get(handles.fakeax(ai),'userdata');
legdata=get(handles.fakerax(ai),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

%remove old cbar from legdata (for rasterizing/exporting);
oldcbar=findobj(handles.fig,'tag','Colorbar','userdata',ai);

if ~isempty(oldcbar)
    legdata=setdiff(legdata,oldcbar);
    delete(oldcbar)
    set(handles.fakerax(ai),'userdata',legdata);
end

%make a new colorbar if needed
if ~udata.format.colorbar;return;end
if udata.view.c==0 %color by density
    zlim=get(handles.allgraphs(ai),'zlim');
    lims=[0 zlim(2)];
else %color by variable as usual
    if strcmp(handles.datatype{d}{udata.view.c},'constant');return;end %trying to make constant colorbar = errors.
    %global or local scaling?
    if isfield(udata.format,'clim') && ~isempty(udata.format.clim)
        lims=udata.format.clim;
    else
        lims=[handles.minlimit{d}{a}(udata.view.c) handles.maxlimit{d}{a}(udata.view.c)];
    end
end


axpos=get(handles.allgraphs(ai),'position');
if strcmpi(handles.graphinfo.(get(handles.allgraphs(ai),'userdata')).axesformat,'3d')
    offset3d=20;
else
    offset3d=0;
end

aaa=axes('parent',handles.wspanel,'visible','on','units','pixels','position',[axpos(1)+axpos(3)+30+offset3d axpos(2) 27 axpos(4)],...
    'fontname',handles.prefs.fonts.tick,'fontsize',handles.prefs.fonts.ticksize,'hittest','off','box','on','layer','top');

cdata=get(handles.fig,'colormap');
cdata=permute(cdata,[1,3,2]);
image('cdata',cdata,'parent',aaa);

set(aaa,'xtick',[],'xticklabel','','yaxislocation','right','ylim',lims);

newyticks=get(aaa,'ytick');
newyticklabels=get(aaa,'yticklabel');

if strcmp(handles.activetype,'parallelcoords')
    properYrange=[128*(udata.format.lcmap-1)+1 128*udata.format.lcmap];
else
    properYrange=[128*(udata.format.mcmap-1)+1 128*udata.format.mcmap];
end

set(aaa,'ylim',properYrange,'ylimmode','manual');

if udata.format.reversecolors
    set(aaa,'ydir','reverse')
    newyticklabels=flipud(newyticklabels);
else
    set(aaa,'ydir','normal')
end

newscaledticks=interp1(lims,properYrange,newyticks);

set(aaa,'ytick',newscaledticks,'yticklabel',newyticklabels);

set(aaa,'userdata',ai,'tag','Colorbar');
udata.format.colorbar=1;

legdata=union(legdata,aaa);

set(handles.fakerax(ai),'userdata',legdata);

ylab=get(aaa,'ylabel');

set(ylab,'string','Data Points','interpreter',handles.prefs.interpreter,...
        'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph);

xlim=get(aaa,'xlim');
ylim=get(aaa,'ylim');
pos=get(aaa,'pos');

%display current value:
if ~isfield(udata.view,'clim');return;end

if isfield(udata.view,'showcurrentpoint') && ~udata.view.showcurrentpoint
    return
end

clim=udata.view.clim;
set(aaa,'xlimmode','manual')

%determine current c value:
% thisvalue=raveevaluatefunction(src,udata.view.dataset,udata.view.c,handles.xdials{d}{a});
% thiscoord=interp1(clim,ylim,thisvalue);
% pixels=pos(3)./diff(xlim); %pixels per unit
% 
% try
%     facecolor=udata.format.tracercolor;
% catch
%     facecolor=handles.colors.text;
% end

% %make triangle patch at this value:
% p= patch([xlim(1)-7/pixels xlim(1)-7/pixels xlim(1)-1/pixels],[thiscoord-1.5 thiscoord+1.5 thiscoord],facecolor,'parent',aaa,'tag','currentvalue');
% t= text(xlim(2)+3/pixels,thiscoord,sprintf('%5.5g',thisvalue),'color',facecolor,'parent',aaa,'tag','currentvaluetxt','fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph);

