function ravecreateboxplot(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

%inputs=varargin{3};

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;
ccol=udata.view.c;


a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph

labels=findall(handles.activegraph,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);

leg=get(handles.fakerax(handles.activeindex),'userdata');
if ~isempty(leg)
    delete(leg(ishandle(leg)));
end

cla(targetaxes);
legend(targetaxes,'off');

set(targetaxes,'xtickmode','auto','ytickmode','auto')

if xcol==0
    text(0.5,0.5,{'Data set must contain at least';'one numerical variable';'in order to create box plot.'},'hor','center','ver','mid','parent',handles.activegraph);
    return
end


%get data
x=handles.alldata{d}(:,xcol);

%calculate statistics:

asdf=1;


meanz=mean(x);
qs=[0 0.25 0.5 0.75 1];
Qs=qs*(size(x,1)-1)+1;


for i=1:numel(xcol)
    s=sort(x(:,i));
    for j=1:numel(Qs)
        quantz(j,i)=interp1(1:size(x,1),s,Qs(j));
    end
end


%determine box widths:


centerz=linspace(0,1,numel(xcol)+2);

widthz=(centerz(2)-centerz(1))*udata.format.boxwidth/2;



set(targetaxes,'xlim',[0 1]);

ls={'linewidth',udata.format.linewidth,'color',udata.format.linecolor};
ps={'linewidth',udata.format.linewidth,'edgecolor',udata.format.linecolor};


for c=1:numel(xcol)
    w=centerz(c+1)+[-widthz widthz];
    
    
    %plot mean line;
    line(w,[quantz(3,c) quantz(3,c)],ls{:},'parent',targetaxes);
    %plot quartile box:
    patch([w fliplr(w)],[quantz(2,c) quantz(2,c) quantz(4,c) quantz(4,c)],[1 1 1],ps{:},'parent',targetaxes,'facecolor','none');
    %plot outer lines:
    line([centerz(c+1) centerz(c+1)],[quantz(1,c) quantz(2,c)],ls{:},'parent',targetaxes);
    line([centerz(c+1) centerz(c+1)],[quantz(4,c) quantz(5,c)],ls{:},'parent',targetaxes);
end



set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))


%% make tick marks:

%set axes labels
set(get(targetaxes,'XLabel'),'String','')
set(targetaxes,'xtick',centerz(2:end-1),'xticklabel','');
%ravemakecustomticks(targetaxes,'x',centerz(2:end-1),handles.allnames{d}(xcol));


yticks=get(targetaxes,'ytick');
yticklabels=num2cell(yticks);
yticklabels=cellfun(@num2str,yticklabels,'UniformOutput',false);

xticklabels=handles.allnames{d}(xcol);

ravemaketextticks(targetaxes,[],0,0,[],[],xticklabels,yticklabels)

raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.
clear ravearrangeobjects

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end

%update da handles
guidata(src,handles)



