function ravemaketablecore(src,ev,ai,data,rownames,rowheight,colnames,colwidths,headerwidth,headerheight,rowcolors,rhcolors)
% This function makes a table-type graph from a cell array of string data (named
% "data') that must already be in the workspace. THis is to avoid having this code
% appear in multiple files. See "ravecreatetableallpoints" for an example of how
% to use this script and what needs to be done before/after using this script.
%
% NOTE: To make a table with no row or column headers, set the
% corresponding headerwidth/headerheight = 0.1. This avoids an error with
% making 0 width/height axes. 

handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

%determine where to put graphs:
if handles.sheet(ai)==handles.activesheet
    parent=handles.wspanel;
else
    parent=handles.boneyard;
end


%format=udata.format;
targetaxes=handles.allgraphs(ai);
%activetype=get(targetaxes,'userdata');

maxpos=udata.view.maxpos;

rowheight=ceil(rowheight);

p=handles.fakerax(ai); %the panel
pos=get(p,'position');

numrows=size(data,1);
numcols=size(data,2);
%note, numrows/numcols refer to the main data table only, headers may have different number of
%elements.

nudge=3; %number of pixels occupied by border of uipanel.. avoid drawing into these.

%clear existing table
delete(get(targetaxes,'children'));
delete(get(handles.fakeax(ai),'children'));
pkidz=setdiff(get(handles.fakerax(ai),'children'),[targetaxes handles.fakeax(ai)]);
delete(pkidz);
delete(udata.view.vscroll);
delete(udata.view.hscroll);
delete(udata.format.titleaxes);

%make title
udata.format.titleaxes=axes('units','pixels','parent',parent,'position',[pos(1) pos(2)+pos(4)+5 pos(3) 30],'xlim',[0 1],'ylim',[0 1],'visible','off');
text(0,0,udata.format.title,'parent',udata.format.titleaxes,'fontname',handles.prefs.fonts.graphtitle,...
    'fontsize',handles.prefs.fonts.graphtitlesize,'ver','baseline');

%make column header image:
if ~isempty(colnames)
    ch=ones(1,sum(colwidths),3);
    ch(:,:,1)=udata.format.colheadercolor(1);
    ch(:,:,2)=udata.format.colheadercolor(2);
    ch(:,:,3)=udata.format.colheadercolor(3);
    
    %color column gridlines:
    collines=[1 cumsum(colwidths)];
    ch(:,collines,1)=udata.format.gridlinecolor(1);
    ch(:,collines,2)=udata.format.gridlinecolor(2);
    ch(:,collines,3)=udata.format.gridlinecolor(3);
    
    chax=axes('parent',p,'units','pixels','position',[headerwidth pos(4)-headerheight sum(colwidths) headerheight],'tag','colheader',...
        'ydir','reverse','xlim',[0.5 size(ch,2)+0.5],'ylim',[.5 size(ch,1)+0.5],'ydir','reverse','visible','off');
    image('cdata',ch,'parent',chax)
    for c=1:numel(colnames)
        text(collines(c)+3,1,colnames{c},'parent',chax,'fontname',udata.format.fontname,...
            'fontsize',udata.format.fontsize,'color',udata.format.textcolor,...
            'color',udata.format.textcolor,'interpreter','none');
    end
    line([0.5 sum(colwidths)+.5],[1.5 1.5],'color',udata.format.gridlinecolor,'parent',chax);
else
    ch=[];
    chax=[];
end

%make row header image:
if ~isempty(rownames) && ~isempty(data)
    rh=ones(rowheight*numrows,1,3);
    rhtxtcolors=repmat(udata.format.textcolor,numrows,1);
    
    if size(rhcolors,1)==1;
        rh(:,:,1)=rhcolors(1);
        rh(:,:,2)=rhcolors(2);
        rh(:,:,3)=rhcolors(3);
        
        %make white txt on dark background:
        shade =[0.299 0.587 0.114]*rhcolors';
        if shade<0.5,rhtxtcolors(:,:)=1;end

        
    else
        rh=repmat(rhcolors,[1 1 rowheight]);
        rh=permute(rh,[3 1 2]);
        rh=reshape(rh,[rowheight*numrows,3]);
        rh=permute(rh,[1 3 2]);

        shade = bsxfun(@times,[0.299 0.587 0.114],rhcolors);
        shade=sum(shade,2);
        rhtxtcolors(shade>=0.5,:)=0;
        rhtxtcolors(shade<0.5,:)=1;
    end
    
    %color row gridlines:
    rowlines=rowheight:rowheight:numrows*rowheight;
    rh(rowlines,:,1)=udata.format.gridlinecolor(1);
    rh(rowlines,:,2)=udata.format.gridlinecolor(2);
    rh(rowlines,:,3)=udata.format.gridlinecolor(3);
    
    rhax=axes('parent',p,'units','pixels','position',[0 pos(4)-size(rh,1)-headerheight headerwidth numrows*rowheight],...
        'tag','rowheader','xlim',[.5 1.5],'ylim',[0.5 size(rh,1)+0.5],'ydir','reverse','visible','off');
    image('cdata',rh,'parent',rhax,'tag','rowheaderimage')
    for r=1:numel(rownames)
        text(.5+3/headerwidth,r*rowheight,rownames{r},'parent',rhax,'verticalalignment','bottom',...
            'fontname',udata.format.fontname,'fontsize',udata.format.fontsize,...
            'color',rhtxtcolors(r,:),'interpreter','none')
    end
    
    line([1.5 1.5],[.5 rowheight*numrows+.5],'parent',rhax,'color',udata.format.gridlinecolor);
    
else
    rh=[];
    rhax=[];
end


handles.subax{ai}=[chax rhax];

%if ~isempty(colnames) &&  ~isempty(rownames) && ~isempty(data)
%make a little block in upper corner to scroll both at once:
cornerax=axes('parent',p,'units','pixels','position',[0 pos(4)-headerheight headerwidth headerheight],'tag','cornerblocker',...
    'ylim',[0.5 1.5],'xlim',[0.5 1.5],'visible','on','xcolor',udata.format.gridlinecolor,'xtick',[],'ycolor',udata.format.gridlinecolor,'box','on');


cb=ones(1,1,3);
cb(:,:,1)=udata.format.colheadercolor(1);
cb(:,:,2)=udata.format.colheadercolor(2);
cb(:,:,3)=udata.format.colheadercolor(3);

image('cdata',cb,'parent',cornerax);
line([.5 1.5],[1.5 1.5],'parent',cornerax,'color',udata.format.gridlinecolor);
line([.5 .5],[.5 1.5],'parent',cornerax,'color',udata.format.gridlinecolor);


if headerheight<1 || headerwidth<1
    set(cornerax,'visible','off');
    set(get(cornerax,'children'),'visible','off')
end



%% make main table image:
if ~isempty(data)
    t=ones(numrows*rowheight,1,3);
    if size(rowcolors,1)==1
        t(:,:,1)=rowcolors(1);
        t(:,:,2)=rowcolors(2);
        t(:,:,3)=rowcolors(3);
    else
        t=repmat(rowcolors,[1 1 rowheight]);
        t=permute(t,[3 1 2]);
        t=reshape(t,[rowheight*numrows,3]);
        t=permute(t,[1 3 2]);
    end
    %color horizontal grid lines:
    rowlines=rowheight:rowheight:(rowheight*numrows);
    t(rowlines,:,1)=udata.format.gridlinecolor(1);
    t(rowlines,:,2)=udata.format.gridlinecolor(2);
    t(rowlines,:,3)=udata.format.gridlinecolor(3);
    
    %display:
    tax=handles.fakeax(ai);
    set(tax,'position',[headerwidth pos(4)-size(t,1)-headerheight sum(colwidths) size(t,1)],'color',[1 0 1],...
        'visible','off','xlim',[0.5 size(t,2)+0.5],'ylim',[0.5 size(t,1)+0.5],'ydir','reverse',...
        'buttondownfcn',[],'hittest','on');
    
    image('cdata',t,'parent',tax,'tag','mainimage','buttondownfcn','','hittest','off');
    tpos=get(tax,'position');
    
    %% make text for table:
    texthandles=zeros(size(data));
    
    collines=[1 cumsum(colwidths)];
    rowlines=rowheight:rowheight:(rowheight*numrows);
    %
    rowcoords=rowlines-rowheight/2+1;
    for r=1:numrows
        for c=1:numcols
            texthandles(r,c)=text(collines(c)+3,-rowcoords(r),data{r,c},'parent',targetaxes,'erasemode','normal',...
                'fontname',udata.format.fontname,'fontsize',udata.format.fontsize,'color',udata.format.textcolor,'hittest','off','interpreter','none');
        end
    end
    
    set(targetaxes,'position',get(handles.fakeax(ai),'position')+[0 0 0 1],'color','none','hittest','off');
    ylim=get(handles.fakeax(ai),'ylim');
    set(targetaxes,'ylim',[-ylim(2) -ylim(1)+1],'visible','off','xlim',[.5 sum(colwidths)+.5]);
    
    %make vertical gridlines:
    for cc=1:numel(collines)
        line([collines(cc) collines(cc)],[-ylim(2) -ylim(1)+1],'color',udata.format.gridlinecolor,'tag','gridlines','parent',targetaxes,'hittest','off');
    end
    
    dax=handles.activegraph;
    
    %hide invisible text:
    lastvisiblerow=ceil((pos(4)-headerheight)/rowheight);
    lastvisiblecol=find([0 cumsum(colwidths)]>pos(3)-headerwidth,1);

    set(texthandles(lastvisiblerow+3:end,:),'visible','off');
    set(texthandles(:,lastvisiblecol+1:end),'visible','off');
    
    if udata.view.tight && tpos(1)+tpos(3)<maxpos(3) && tpos(4)+headerheight<maxpos(4)
        %table is smaller than the uipanel, so shrink uipanel to just hold table.
        set(p,'position',[pos(1) pos(2) tpos(1)+tpos(3) tpos(4)]);
        pos=get(p,'position');
        
        set(chax,'position',[0 pos(4)-headerheight sum(colwidths) headerheight]);
        set(rhax,'position',[0 pos(4)-size(rh,1)-nudge headerwidth numrows*rowheight]);
        set(tax,'position',[headerwidth pos(4)-size(t,1)-nudge tpos(3) tpos(4)]);
        set(dax,'position',[headerwidth pos(4)-size(t,1)-nudge tpos(3) tpos(4)]);
        
    end
    
    udata.view.texthandles=texthandles;
else
    udata.view.texthandles=[];
end

%store handles:
%set(handles.activegraph,'userdata',activetype);
set(rhax,'tag','rowheader');
set(chax,'tag','colheader');

udata.view.rowheader=rhax;
udata.view.colheader=chax;
udata.view.vscroll=[];
udata.view.hscroll=[];

set(handles.fakeax(ai),'userdata',udata);
