function lineobject_callback(src,ev) %#ok<INUSD>
%this is a one-size-fits-all callback for line objects that have been added
%to graphs, that causes them to be deleted by double clicking, or allows
%changing colors by right clicking.

%objects using this callback have to have a userdata that is a structure
%with the following fields:

%userdata.handles= handles of all objects to apply this change to
%userdata.delete=1 if object can be deleted = 0 if object cant be deleted
%userdata.ai= activeindex of the graph containing this line. (required to use the optional things below)

%optional: userdata.constraintnumber = number of this constraint (for constraints only)
%          userdata.variable = number of this variable, within its respective data set (for variables only).
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu



handles=guidata(src);

linedata=get(src,'userdata');

switch get(handles.fig,'selectiontype')
    
    case 'normal' %bring this line to top
        uistack(src,'top');
    case 'extend' %delete this line (if allowed)
        if linedata.delete
            
            delete(linedata.handles);
            set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xlim',get(handles.activegraph,'xlim'),'ylim',...
                get(handles.activegraph,'ylim'));
        end
    case 'alt' %change color of this variable/constraint permanently
        pos=get(handles.fig,'position');
        newcolor=ravecolorchooser([pos(1)+pos(3)/2-300 pos(2)+pos(4)/2-300],get(src,'color'),'Choose a new color for this constraint');
        set(linedata.handles,'color',newcolor,'markerfacecolor',newcolor)
        
        if isfield(linedata,'variable') %permanently change color
            udata=get(handles.fakeax(linedata.ai),'userdata');
            handles.variablecolors{udata.view.dataset}(linedata.variable,:)=newcolor;
            guidata(src,handles)
            
            thistype=get(handles.allgraphs(linedata.ai),'userdata');
            if strcmp(thistype,'line')
               thesemarkers=findobj(handles.allgraphs(linedata.ai),'tag','marker','userdata',linedata.variable);
               set(thesemarkers,'markerfacecolor',newcolor)
            end
            
            
        end
        
        
        %if this is a constraint, update its constraintcolor:
        if isfield(linedata,'constraintnumber')
            thisconstraint=linedata.constraintnumber;
            ai=linedata.ai;
            udata=get(handles.fakeax(ai),'userdata'); 
            handles.constraintcolors{udata.view.dataset}{udata.view.analysis}{thisconstraint}=newcolor;
            guidata(src,handles);
            
            raveupdateanalysis(src,[],'constraints',ai,udata.view.dataset,udata.view.analysis)
            
            
        end
        
    case 'open' %change color/style (not permanent)
        inspect(linedata.handles);
        set(linedata.handles,'hittest','on','buttondownfcn',@lineobject_callback,'userdata',linedata,'handlevisibility','on') % in case they screwed this up.
 
        
end