function termmatrix=ravechoosetermsgui(src,ev,d,x,varnames,terms)
%gui for choosing polynomial terms for making an RSE (or for anything
%else...)
%inputs: 
% src only used to get handles.fig
% ev unused
% d data set index
% x indices of variables to show in list
% varnames names of variables to show in list
% terms is a matrix of terms to include

handles=guidata(src);

termmatrix=terms';
load raveprefs

origin=get(handles.fig,'position');

hhh=figure('menubar','none','units','pixels','position',[origin(1)+origin(3)/2-500 origin(2)+origin(4)/2-300 1000 600],'color',raveprefs.colors.frame,...
    'name','Choose Model Terms','numbertitle','off','resize','off','windowstyle','modal');

titleaxes=axes('units','pixels','position',[20 550 400 40],'parent',hhh,'visible','off');
text(0,.4,'Select Terms','parent',titleaxes,'fontname',raveprefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
    'color',raveprefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');

%instructions:
uicontrol('style','text','pos',[30 505 940 50],'background',raveprefs.colors.frame,'hor','left',...
    'string',{'Each column in the table below represents one term in the model. Enter exponents corresponding to each variable in each term. For example, if you have a variable named "x" and you would like a term b*x^2, find the row corresponding to "x", enter a 2 in its cell and leave all other cells in that column blank. Empty cells are interpreted as a 0 exponent. You can fill the table manually and/or use the controls at the right to quickly add terms to the model.'});

%controls:
uicontrol('style','text','pos',[760 464 200 40],'string','Select variables to include in generated terms:','backgroundcolor',raveprefs.colors.frame,'hor','left','fontsize',10,'fontweight','bold')


uicontrol('style','listbox','pos',[770 200 210 260],'string',varnames,'backgroundcolor',[1 1 1],'tag','xmenu','max',100)
uicontrol('style','text','pos',[760 170 260 22],'string','Term generation macro:    Order:','backgroundcolor',raveprefs.colors.frame,'hor','left','fontsize',10,'fontweight','bold')
uicontrol('style','popupmenu','pos',[770 150 160 22],'string',{'Homogenous terms up to';'Homogeneous terms';'Full factorial up to';'Full factorial';'Cross terms up to';'Cross terms'},'backgroundcolor',[1 1 1],'tag','macromenu')
uicontrol('style','edit','pos',[940 150 40 22],'string','1','backgroundcolor',[1 1 1],'tag','orderedit')
uicontrol('style','pushbutton','pos',[770 110 210 26],'string','Add these terms to model','callback',@macrobutton_callback)
uicontrol('style','pushbutton','pos',[30 20 160 26],'string','Delete Selected Terms','callback',@deletebutton_callback)

uicontrol('style','pushbutton','pos',[880 10 100 30],'string','Done','fontsize',16,'callback',@donebutton_callback)

uicontrol('style','pushbutton','pos',[770 10 100 30],'string','Cancel','fontsize',12,'callback',@cancelbutton_callback)

%make table:
nvars=numel(varnames);



if isempty(termmatrix)
dblArray=javaArray('java.lang.Integer',nvars,50);
else
dblArray=javaArray('java.lang.Integer',nvars,size(termmatrix,2));

for r=1:size(termmatrix,1)
    for c=1:size(termmatrix,2)
        if termmatrix(r,c)~=0;
            dblArray(r,c)=java.lang.Integer(termmatrix(r,c));
        end
    end
end



end


colnames=num2cell(32*ones(1,50));
colnames=cellfun(@char,colnames,'UniformOutput',false);

imodel = RaveDataTypeTableModel(dblArray,colnames);
jtable=javax.swing.JTable(imodel);
jtable.setAutoResizeMode(jtable.AUTO_RESIZE_OFF);
jscroll=javax.swing.JScrollPane(jtable);

dblRows=javaArray('java.lang.Object',nvars,1); %row header numbers
for g=1:nvars
    dblRows(g,1)=java.lang.String(varnames{g});
end
rh=javax.swing.table.DefaultTableModel(dblRows,{'Variable Name'});
rowHeader=javax.swing.JTable(rh);

[jhandle mhandle]=javacomponent(jscroll,[30 60 700 440],hhh);

%make the header the appropriate width:
headerWidth=150;
rhHeight=rowHeader.getPreferredSize.height;
rowHeader.setPreferredSize(java.awt.Dimension(headerWidth,rhHeight))
rowHeader.setSize(java.awt.Dimension(headerWidth,rhHeight))

%put the row header in a viewport.
jv=javax.swing.JViewport;
jv.setView(rowHeader);
jv.setPreferredSize(java.awt.Dimension(headerWidth,rhHeight));

jscroll.setRowHeader(jv);
jscroll.setCorner(javax.swing.ScrollPaneConstants.UPPER_LEFT_CORNER, rowHeader.getTableHeader());

cols=jtable.getColumnModel().getColumns;
colcount=jtable.getColumnModel().getColumnCount;

for c=0:(colcount-1)
    thisCol=cols.nextElement;
    thisCol.setMinWidth(16);
    thisCol.setMaxWidth(16);   
    thisCol.setPreferredWidth(16);
end

jtable.setRowSelectionAllowed(false);
jtable.setColumnSelectionAllowed(true);

jtable.getTableHeader().setReorderingAllowed(false);
jtable.getTableHeader().setResizingAllowed(false);


uiwait(hhh);

if strcmp(get(0,'userdata'),'cancel')
    termmatrix=[];
    return
end

%get data from table:
tabledata=nan(jtable.getRowCount,jtable.getColumnCount);

for r=1:jtable.getRowCount
    for c=1:jtable.getColumnCount
        if ~isempty(jtable.getValueAt(r-1,c-1))
            try %error if non numeric data
                tabledata(r,c)=jtable.getValueAt(r-1,c-1);
            end
        end
    end
end


tabledata=tabledata';
tabledata(tabledata==0)=nan;
tabledata=tabledata(~all(isnan(tabledata),2),:);
tabledata(isnan(tabledata))=0;

tabledata=[tabledata;zeros(50,size(tabledata,2))];
termmatrix=tabledata;





function donebutton_callback(src,ev)
set(0,'userdata','');
close(gcf);

function cancelbutton_callback(src,ev)
set(0,'userdata','cancel')
close(gcf);

function deletebutton_callback(src,ev)
jscroll=findobj(gcf,'type','hgjavacomponent');
jscroll=get(jscroll,'javapeer');
jtable=jscroll.getComponent(0).getComponent(0);

tabledata=nan(jtable.getRowCount,jtable.getColumnCount);

for r=1:jtable.getRowCount
    for c=1:jtable.getColumnCount
        if ~isempty(jtable.getValueAt(r-1,c-1))
            try %error if non numeric data
                tabledata(r,c)=jtable.getValueAt(r-1,c-1);
            end
        end
    end
end

%get column selection:
dels=jtable.getSelectedColumns+1;

tabledata(:,dels)=[];

tabledata=tabledata';
tabledata(tabledata==0)=nan;
tabledata=tabledata(~all(isnan(tabledata),2),:);
tabledata(isnan(tabledata))=0;

tabledata=[tabledata;zeros(50,size(tabledata,2))];
tabledata=tabledata';

setTableData(jtable,tabledata);







function macrobutton_callback(src,ev)

xmenu=findobj(gcf,'tag','xmenu');
macromenu=findobj(gcf,'tag','macromenu');
orderedit=findobj(gcf,'tag','orderedit');
jscroll=findobj(gcf,'type','hgjavacomponent');
jscroll=get(jscroll,'javapeer');
jtable=jscroll.getComponent(0).getComponent(0);

order=str2double(get(orderedit,'string'));

if isnan(order) || isempty(order) || order<1;
    errordlg('You must enter a positive integer in the "Order" text box.')
    return
else
    order=floor(order);
end

if order>6
    errordlg('Order must be 6 or less. If you''re sure you want to do that, edit "ravechoosetermsgui" and delete this safety check on lines 213-216.')
    return
end

xvars=get(xmenu,'value');
nx=numel(xvars);
nall=numel(get(xmenu,'string'));

%get existing terms from the table:

tabledata=nan(jtable.getRowCount,jtable.getColumnCount);
pause(0.1) %this avoids a java error... not sure why. 
for r=1:jtable.getRowCount
    for c=1:jtable.getColumnCount
        if ~isempty(jtable.getValueAt(r-1,c-1))
            try %error if non numeric data
                tabledata(r,c)=jtable.getValueAt(r-1,c-1);
            end
        end
%TO DO: defaulttablemodel ALWAYS returns columnclass as object, even when using a double javaarray. to avoid use of str2double here, make a custom table model that returns numeric types for getColumnClass.
    end
end
%pause(0.1);
tabledata=tabledata';
tabledata(tabledata==0)=nan;
tabledata=tabledata(~all(isnan(tabledata),2),:);
tabledata(isnan(tabledata))=0;
% 
% if all(tabledata(1,:)==0)
%     tabledata=tabledata(2:end,:);
% end

switch get(macromenu,'value')
    case 1 %homogeneous terms up to
        nterms=order*nx;
        terms=zeros(nterms,nall);
        for i=1:order
            for j=1:nx
                terms((i-1)*nx+j,xvars(j))=i;            
            end
        end 

    case 2 %homogenous terms
        nterms=order*nx;
        terms=zeros(nterms,nall);
        for i=order
            for j=1:nx
                terms((i-1)*nx+j,xvars(j))=i;            
            end
        end 
        
    case 3 %full factorial up to
        levels=(order+1).*ones(1,nx);
        allcombos=[];
        
        for N=1:order
            combos=uint8(generatesums(N));
            %remove combos with more columns than nx:
            ncols=sum(combos>0,2);
            combos=combos(ncols<=nx,:);
            if order>nx && size(combos,2)>nx
                combos=combos(:,1:nx);
            end

            combos=[combos zeros(size(combos,1),nx-N)];
            allcombos=[allcombos;combos];
        end
        
        allcombos=unique(allcombos,'rows');
        
        points=[];
        
        for g=1:size(allcombos,1)
        [nPerms pInds Perms] = uperms(allcombos(g,:), inf);
        points=[points;Perms];
        end
        
        points=unique(points,'rows');
        allpoints=zeros(size(points,1),nall);
        allpoints(:,xvars)=points;
        %rowsum=sum(allpoints,2);
        terms=allpoints;
        %terms=allpoints(rowsum<=order,:);
        
    case 4 %full factorial
        levels=(order+1).*ones(1,nx);
        points=fullfactbuild(levels);
        points=points-1;
        allpoints=zeros(size(points,1),nall);
        allpoints(:,xvars)=points;
        rowsum=sum(allpoints,2);
        terms=allpoints(rowsum==order,:);
        
    case 5 %cross terms up to
        levels=(order+1).*ones(1,nx);
        points=fullfactbuild(levels);
        points=points-1;
        allpoints=zeros(size(points,1),nall);
        allpoints(:,xvars)=points;
        rowsum=sum(allpoints,2);
        maxval=max(allpoints,[],2);        
        terms=allpoints(rowsum<=order & maxval<order,:);
        
    case 6 %cross terms
        levels=(order+1).*ones(1,nx);
        points=fullfactbuild(levels);
        points=points-1;
        allpoints=zeros(size(points,1),nall);
        allpoints(:,xvars)=points;
        rowsum=sum(allpoints,2);
        maxval=max(allpoints,[],2);        
        terms=allpoints(rowsum==order & maxval<order,:);
end

%remove any rows from terms that are already in the table:
[junk,i]=setdiff(terms,tabledata,'rows');
terms=terms(sort(i),:);
if isempty(terms);return;end


allzero=sum(terms,2)==0;
terms=terms(~allzero,:);
% 
% if all(terms(1,:)==0);
%     terms(1,:)=[];
% end

if isempty(terms);return;end

tabledata=[tabledata;terms];
tabledata=[tabledata;zeros(50,nall)];
tabledata=tabledata';

setTableData(jtable,tabledata);

function setTableData(jtable,tabledata)
dblRows=javaArray('java.lang.Integer',size(tabledata,1),size(tabledata,2)); 
for r=1:size(tabledata,1);
    for c=1:size(tabledata,2);
        if tabledata(r,c)~=0
            dblRows(r,c)=java.lang.Integer(tabledata(r,c));
        end
    end
end
colnames=num2cell(32*ones(1,size(tabledata,2)));
colnames=cellfun(@char,colnames,'UniformOutput',false);

imodel=RaveDataTypeTableModel(dblRows,colnames);
jtable.setModel(imodel);

cols=jtable.getColumnModel().getColumns;
colcount=jtable.getColumnModel().getColumnCount;

for c=0:(colcount-1)
    thisCol=cols.nextElement;
    thisCol.setMinWidth(16);
    thisCol.setMaxWidth(16);   
    thisCol.setPreferredWidth(16);
end


