function settings=ravemakemodelrse(src,ev,settings)
%creates an RSE based on the settings matrix.
%must do three things:
%1) Fit the model.
%2) Assess the model against all rows and calculate predicted values. Store
%   these in settings.predictedvalues.
%3) Make a cell array of strings to be printed in a text file to make a
%   valid matlab function to calculate this model. (this is the hard part).

handles=guidata(src);


xcol=settings.inputs;
d=settings.dataset;
termmatrix=settings.termmatrix;

fitdata=handles.alldata{d}(settings.fitrows,xcol);
fittodata=handles.alldata{d}(settings.fitrows,settings.output);

switch settings.transformby
    case 'exp(y)'
        fittodata=exp(fittodata);
    case 'log(y)'
        fittodata=log(fittodata);
    case '1/y'
        fittodata=1./fittodata;
end
        




%multiply fit data by termmatrix:
nrows=size(fitdata,1);
nterms=size(termmatrix,1);
terms=zeros(nrows,nterms);
for c=1:nterms
    terms(:,c)=prod(bsxfun(@power,fitdata,termmatrix(c,:)),2);
end

%remove constant cols to avoid rank deficiency:
termtest=~all(bsxfun(@minus,terms,terms(1,:))==0,1);
if settings.includeintercept
    termtest(1)=true; %it gets set to false by the check above
end


fitterms=terms(:,termtest);

        



%do least squares fit:
B=fitterms\fittodata;

%replace constant terms:
b=zeros(numel(termtest),1);
b(termtest')=B;

%calculate predictions:
alldata=handles.alldata{d}(:,xcol);
nrows=size(alldata,1);
nterms=size(termmatrix,1);
predterms=zeros(nrows,nterms);
for c=1:nterms
    predterms(:,c)=prod(bsxfun(@power,alldata,termmatrix(c,:)),2);
end

predictedvalues=(b'*predterms')';



switch settings.transformby
    case 'exp(y)'
        predictedvalues=real(log(predictedvalues));
    case 'log(y)'
        predictedvalues=exp(predictedvalues);
    case '1/y'
        predictedvalues=1./predictedvalues;
end
     
settings.predictedvalues=predictedvalues;


%make model string:
varnames=handles.allnames{d}(xcol);
varnames=ravemakevariablenames(varnames);

line1='xdata = [';
for v=1:numel(varnames);
    line1=[line1,varnames{v},' ']; %#ok<AGROW>
end

str={};
str{end+1} = [line1(1:end-1),'];'];
str{end+1} = ['pwrs = ',mat2str(termmatrix),';'];
str{end+1} = ['coeffs = ',mat2str(b),';'];
str{end+1} = 'terms=zeros(size(xdata,1),size(pwrs,1));';
str{end+1} = 'for q=1:size(pwrs,1)';
str{end+1} = '    terms(:,q)=prod(bsxfun(@power,xdata,pwrs(q,:)),2);';
str{end+1} = 'end';
str{end+1} = 'y = (coeffs''*terms'')'';';


switch settings.transformby
    case 'exp(y)'
        str{end+1} = 'y = real(log(y));';
        %fittodata=exp(fittodata);
    case 'log(y)'
        %fittodata=log(fittodata);
        str{end+1} = 'y = exp(y);';
    case '1/y'
        %fittodata=1/fittodata;
        str{end+1} = 'y = 1./y;';
end
        


settings.modelstring=str';

