function s=updateffneuralnetsettingstable(trainalg)
%fill the settings table for ff neural networks based on the particular
%training algorithm that has been selected.

%cell array s has one row for each line of the table. each row of s has
%three columns: 1st column: string of property name as you want it to
%appear in the table, 2nd column: string of actual property name to be set
%below "net." (may include more dots or indices etc). 3rd column: default
%value to appear in the table.

%make rows that appear in all algorithms:
S={'Epochs','epochs',1000;...
    'Max Time','time',60;...}
    'Goal','goal',0;...
    'Display Epochs','show',10};

switch trainalg
    case 'trainb'
        s={'Max Fail','max_fail',10};
    case 'trainbfg'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Search Function','searchFcn','srchcha';...
            'Scal Tol','scal_tol',20;...
            'Alpha','alpha',0.001;...
            'Beta','beta',0.1;...
            'Delta','delta',0.01;...
            'Gamma','gama',0.1;...
            'Lower Limit','low_lim',0.1;...
            'Upper Limit','up_lim',0.5;...
            'Max Step','maxstep',100;...
            'Min Step','minstep',1e-6;...
            'B max','bmax',26;...
            'Batch Frag','batch_frag',0};
        
    case 'trainbfgc'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Search Function','searchFcn','srchchacxc';...
            'Scal Tol','scal_tol',20;...
            'Alpha','alpha',0.001;...
            'Beta','beta',0.1;...
            'Delta','delta',0.01;...
            'Gamma','gama',0.1;...
            'Lower Limit','low_lim',0.1;...
            'Upper Limit','up_lim',0.5;...
            'Max Step','maxstep',100;...
            'Min Step','minstep',1e-6;...
            'B max','bmax',26};
        
    case 'trainbr'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Mu','mu',0.005;...
            'Mu Decrement','mu_dec',0.1;...
            'Mu Increment','mu_inc',10;...
            'Mu Max','mu_max',10e10;...
            'Mem Reduction','mem_reduc',1};
        
    case 'trainbuwb'
        s={};
    case 'trainc'
        s={'Max Fail','max_fail',10};
    case 'traincgb'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Search Function','searchFcn','srchcha';...
            'Scal Tol','scal_tol',20;...
            'Alpha','alpha',0.001;...
            'Beta','beta',0.1;...
            'Delta','delta',0.01;...
            'Gama','gama',0.1;...
            'Lower Limit','low_lim',0.1;...
            'Upper Limit','up_lim',0.5;...
            'Max Step','maxstep',100;...
            'Min Step','minstep',1e-6;...
            'B max','bmax',26};
    case 'traincgf'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Search Function','searchFcn','srchcha';...
            'Scal Tol','scal_tol',20;...
            'Alpha','alpha',0.001;...
            'Beta','beta',0.1;...
            'Delta','delta',0.01;...
            'Gama','gama',0.1;...
            'Lower Limit','low_lim',0.1;...
            'Upper Limit','up_lim',0.5;...
            'Max Step','maxstep',100;...
            'Min Step','minstep',1e-6;...
            'B max','bmax',26};
    case 'traincgp'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Search Function','searchFcn','srchcha';...
            'Scal Tol','scal_tol',20;...
            'Alpha','alpha',0.001;...
            'Beta','beta',0.1;...
            'Delta','delta',0.01;...
            'Gama','gama',0.1;...
            'Lower Limit','low_lim',0.1;...
            'Upper Limit','up_lim',0.5;...
            'Max Step','maxstep',100;...
            'Min Step','minstep',1e-6;...
            'B max','bmax',26};
    case 'traingd'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Learning Rate','lr',0.01};
    case 'traingda'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Learning Rate','lr',0.01;...
            'LR Increment','lr_inc',10.5;...
            'LR Decrement','lr_dec',0.7;...
            'Max Perf Increase','max_perf_inc',1.04};
    case 'traingdm'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Learning Rate','lr',0.01;...
            'Momentum Const','mc',0.9};
    case 'traingdx'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Learning Rate','lr',0.01;...
            'LR Increment','lr_inc',10.5;...
            'LR Decrement','lr_dec',0.7;...
            'Max Perf Increase','max_perf_inc',1.04;...
            'Momentum Const','mc',0.9};
    case 'trainlm'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Mu','mu',1;...
            'Mu Decrement','mu_dec',0.8;...
            'Mu Increment','mu_inc',1.5;...
            'Mu Max','mu_max',10e10;...
            'Mem Reduction','mem_reduc',1};
    case 'trainoss'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Search Function','searchFcn','srchcha';...
            'Scal Tol','scal_tol',20;...
            'Alpha','alpha',0.001;...
            'Beta','beta',0.1;...
            'Delta','delta',0.01;...
            'Gama','gama',0.1;...
            'Lower Limit','low_lim',0.1;...
            'Upper Limit','up_lim',0.5;...
            'Max Step','maxstep',100;...
            'Min Step','minstep',1e-6;...
            'B max','bmax',26};
    case 'trainr'
        s={};
    case 'trainrp'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Learning Rate','lr',0.01;...
            'Delta Increment','delt_inc',1.2;...
            'Delta Decrement','delt_dec',0.5;...
            'Delta Initial','delta0',0.07;...
            'Delta Max','deltamax',50};
    case 'trains'
        S={};
        s={'Passes','passes',1};
    case 'trainscg'
        s={'Min Grad','min_grad',1e-10;...
            'Max Fail','max_fail',10;...
            'Sigma','sigma',5e-5;...
            'Lambda','lambda',5e-7};
    otherwise
        s={};
end

s=[S;s];

