function P=make_new_population(P,crowding,rank,pcross,pmutation,cviolation,handles,settings)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%% TOURNAMENT SELECTION:

N=size(P,1);
ngenes=size(P,2); %=nvars*nbits

matches = N;
toursize=2;

isfeasible=~cviolation;

%select combatants:
gladiators=ceil(N*rand(matches,toursize));
winner=false(matches,1);

%check which is more fit:

feas=0;
for g=1:matches

    %WINNER==0 MEANS 1ST GLADIATOR WINS. WINNER==1 MEANS 2ND GLADIATOR WINS.
    if isfeasible(gladiators(g,1)) && isfeasible(gladiators(g,2)) %BOTH FEASIBLE

        if rank(gladiators(g,1))<rank(gladiators(g,2))
            %winner(g)=0;
        elseif rank(gladiators(g,1))>rank(gladiators(g,2))
            winner(g)=1;
        elseif crowding(gladiators(g,1))>crowding(gladiators(g,2))
            winner(g)=0;
        else
            winner(g)=1;
        end
        feas=feas+1;

    elseif isfeasible(gladiators(g,1)) && ~isfeasible(gladiators(g,2)) %one is feasible... choose the feasible one.
        %winner(g)=0;
        winner(g)=0;
        feas=feas+1;
        
    elseif ~isfeasible(gladiators(g,1)) && isfeasible(gladiators(g,2))
        winner(g)=1;
        feas=feas+1;

    elseif ~isfeasible(gladiators(g,1)) && ~isfeasible(gladiators(g,2))
        
        %neither is feasible
        if cviolation(gladiators(g,1)) >= cviolation(gladiators(g,2))
            winner(g)=1;
        end
        
    else
        error('something happened')
    end
end

matingpool=gladiators(:,1);
matingpool(winner)=gladiators(winner,2);


%% CROSSOVER:
%P=double(P);
if strcmpi(settings.usegray,'on')
    dogray=true;
else
    dogray=false;
end

if dogray %covert each variable's columns independently.
    for v=1:(size(P,2)/settings.nbits)
        idx=(v-1)*settings.nbits+1:v*settings.nbits;
        P(:,idx)=bi2gray(P(:,idx));
    end
end 

for j=1:2:matches-1
    if rand(1)<pcross;
        bounds=ceil(rand(1,2)*ngenes);
        if bounds(1)<=bounds(2)
            splice=P([matingpool(j) matingpool(j+1)],bounds(1):bounds(2));
            %perform crossover:
            P(matingpool(j),bounds(1):bounds(2))=splice(2,:);
            P(matingpool(j+1),bounds(1):bounds(2))=splice(1,:);
        else
            splice1=P([matingpool(j) matingpool(j+1)],1:bounds(1));
            splice2=P([matingpool(j) matingpool(j+1)],bounds(2):end);
            %perform crossover
            P(matingpool(j),1:bounds(1))=splice1(2,:);
            P(matingpool(j+1),1:bounds(1))=splice1(1,:);
            P(matingpool(j),bounds(2):end)=splice2(2,:);
            P(matingpool(j+1),bounds(2):end)=splice2(1,:);
        end
    end
end

%% MUTATION
for j=1:matches
    if rand(1)<pmutation
        mutant=ceil(rand(1)*ngenes);
        if P(j,mutant)==0
            P(j,mutant)=1;
        else
            P(j,mutant)=0;
        end
    end
end


if dogray
    for v=1:(size(P,2)/settings.nbits)
        idx=(v-1)*settings.nbits+1:v*settings.nbits;
        P(:,idx)=gray2bi(P(:,idx));
    end
end 


if handles.prefs.gpgpu
P=gsingle(P);
end




