function results=raveoptweightedpnorm(src,ev,settings)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

%need two settings structures, one for nbi and one for the single objective
%optimizer:
pnormsettings=settings;
d=pnormsettings.d;
a=pnormsettings.a;
P=pnormsettings.p;

%start animation:
raveinitializeoptimizeranimation;

%prepare single obj opt:
alg=pnormsettings.subalgorithm;

%this is going to overwrite the settings structure:
if ~isempty(which(['optimizersettings',alg]))
    eval(['optimizersettings',alg]);
else
    errordlg(['No optimizer named "',alg,'" was found in your Rave directory'])
end

clear settings; %clear settings to avoid using it later since it's name is now ambiguous.

%load settings for single objective optimizer:
[tabsettingnames,tabsettingvalues]=raveparseoptimizerobjectivetab(src,ev);

if strcmp(pnormsettings.settingsfile,'default')
    [displaynames,settingnames,settingvalues,isastring]=raveparseoptimizersettingsdefault(src,ev,alg); %#ok<NASGU,ASGLU>
else
    [displaynames,settingnames,settingvalues,isastring]=raveparseoptimizersettingsdefault(src,ev,alg); %#ok<NASGU,ASGLU>
    [fdisplaynames,fsettingvalues]=raveparseoptimizersettingsfile(dest,headerindex);
    fileidx=[];
    for f=1:numel(fdisplaynames)
        idx=find(strcmp(fdisplaynames{t},displaynames));
        if ~isempty(idx)
            fileidx(end+1)=idx; %#ok<AGROW>
        end
    end
    settingnames=displaynames(fileidx);
    settingvalues=fsettingvalues;
end

settingnames=[tabsettingnames(:);settingnames(:)];
settingvalues=[tabsettingvalues(:);settingvalues(:)];

%turn into a structure:
namevaluepairs=[settingnames settingvalues]';
namevaluepairs=namevaluepairs(:)';
subsettings=struct(namevaluepairs{:});

%fix things that don't get autofilled correctly:
subsettings.method=alg;
subsettings.stopbutton=pnormsettings.stopbutton;
subsettings.showinfo=0;

%% solve for univariate optima. these are used to get normalization factors
% for objectives:
optfun=str2func(['raveopt',alg]);
numobj=numel(pnormsettings.y);
L=zeros(numobj,4);

infotext=findobj(pnormsettings.plotaxes,'tag','optinfotext');
for i=1:numobj
    subsettings.y=pnormsettings.y(i);
    subsettings.targets=pnormsettings.targets(i);
    
    %get variables for this functions:
    indexx=handles.funvars{d}{subsettings.y};
    
    %also variables for any constraints:
    activecons=subsettings.activeconstraints(subsettings.activeconstraints>0);
    for ac=1:numel(activecons)
        indexx=union(indexx,handles.constraintvars{d}{a}{ac});
    end
    
    %get lowest level inputs:
    while any(handles.isfunction{d}(indexx))
        arefuns=find(handles.isfunction{d}(indexx),1,'last');
        thisfun=indexx(arefuns);
        newvars=handles.funvars{d}{thisfun};
        indexx=union(indexx,newvars);
        indexx=setdiff(indexx,thisfun);
    end
    
    isindexx=ismember(pnormsettings.funvars,indexx);
    
    %keep only settings for these variables:
    subsettings.funvars=pnormsettings.funvars(isindexx);
    subsettings.x0=pnormsettings.x0(isindexx);
    subsettings.normalizationfactors=pnormsettings.normalizationfactors(:,isindexx);
    overload.alreadyanimating=1;
    
    if pnormsettings.showinfo
        set(infotext,'string',{['Finding univariate optima: ',num2str(i),'/',num2str(numobj)]});
    end
    
    %do optimizer:
    subresults{i}=optfun(src,[],subsettings,overload);
    subresults{i}.independentvars=indexx;
    
    %prevent animations from being deleted:
    if pnormsettings.plotprogress
        L(i,1)=findobj(pnormsettings.plotaxes,'tag','optinfeasible');
        L(i,2)=findobj(pnormsettings.plotaxes,'tag','optfeasible');
        L(i,3)=findobj(pnormsettings.plotaxes,'tag','opthistory');
        L(i,4)=findobj(pnormsettings.plotaxes,'tag','optcurrent');
        set(L,'tag','nbiopthandles');
    end
end

%form normalization factors:
univariateoptima=zeros(numobj);
for i=1:numobj
    xidx=subresults{i}.independentvars;
    %thesevars=ismember(nbisettings.funvars,xidx) ;
    
    currentpoint(pnormsettings.funvars)=pnormsettings.x0;
    currentpoint(xidx)=subresults{i}.finalinputs;
    
    mins(pnormsettings.funvars)=pnormsettings.normalizationfactors(1,:);
    ranges(pnormsettings.funvars)=pnormsettings.normalizationfactors(2,:);
    
    %denormalize:
    currentpoint=bsxfun(@times,currentpoint,ranges);
    currentpoint=bsxfun(@plus,currentpoint,mins);
    
    yout=raveevaluatefunction(src,d,pnormsettings.y,currentpoint,handles);
    univariateoptima(:,i)=yout(:);
end

yminz=min(univariateoptima,[],2);
ymaxz=max(univariateoptima,[],2);

pnormsettings.ymins=yminz;
pnormsettings.ymaxs=ymaxz;


%% form weight grid:
weights=ravetrigridn(pnormsettings.ndivs,numobj);

%make starting point of independent variables:
lastbest=zeros(1,numel(pnormsettings.funvars));
isfinished=false(size(weights,1),1);

targs=-sign(pnormsettings.targets(:));

normycalc= @(x) pnormobjective(x,pnormsettings,handles);

delete(findobj(pnormsettings.plotaxes,'tag','nbiopthandles'));

subsettings.TolFun=10e-14;

for j=size(weights,1):-1:1
    
    %make objective fcn:
    thisweight=weights(j,:);
    
    obj=@(xdata) thisweight*((targs.*normycalc(xdata)').^P);
    overload.objectivefcn=obj;
    overload.alreadyanimating=1;
    
    %start searching at final point of last iteration:
    subsettings.x0=lastbest; 
    
    if pnormsettings.showinfo
        set(infotext,'string',{['Solving weight vector: ',num2str(size(weights,1)-j+1),'/',num2str(size(weights,1))]});
    end
    
    %do optimizer:
    optresults{j}=optfun(src,[],subsettings,overload);
    
    %prevent animations from being deleted:
    L(1)=findobj(pnormsettings.plotaxes,'tag','optinfeasible');
    L(2)=findobj(pnormsettings.plotaxes,'tag','optfeasible');
    L(3)=findobj(pnormsettings.plotaxes,'tag','opthistory');
    L(4)=findobj(pnormsettings.plotaxes,'tag','optcurrent');
    
    delete(L(1:3))
    set(L(4),'tag','optplots','hittest','off');

    isfinished(j)=true;
    
    %start next iteration at final point that is nearest to the next iteration's weight vector:
    if j>1
       nextweight=weights(j-1,:);
       finishedweights=weights(isfinished,:);
       
       dists=ipdm(nextweight,finishedweights);
       idx=find(dists==min(dists),1);
       
       finishidx=find(isfinished);

       lastbest=optresults{finishidx(idx)}.finalinputs;
    end
end

%finish animation:
settings=pnormsettings; %<-- ravefinishanimation needs this to be called settings.
ravefinishoptimizeranimation;

%export results:
results.finalinputs=zeros(size(weights,1),numel(pnormsettings.funvars));
results.finalobjective=zeros(size(weights,1),numel(pnormsettings.y));
results.settings=settings;
results.name=['PNorm, P=',num2str(P),', ',num2str(pnormsettings.ndivs),' pt grid, '];
for r=1:numel(pnormsettings.y)
    results.name=[results.name,handles.allnames{d}{pnormsettings.y(r)}];
    if r<numel(pnormsettings.y)
        results.name=[results.name,' '];
    end
end

for b=size(weights,1):-1:1;
    results.finalinputs(b,:)=optresults{b}.finalinputs;
end

%denormalize:
results.finalinputs=bsxfun(@times,results.finalinputs,pnormsettings.normalizationfactors(2,:));
results.finalinputs=bsxfun(@plus,results.finalinputs,pnormsettings.normalizationfactors(1,:));

%evaluate objectives:
xdata=zeros(size(weights,1),size(handles.alldata{d},2));
xdata(:,pnormsettings.funvars)=results.finalinputs;
results.finalobjectives=raveevaluatefunction(src,d,pnormsettings.y,xdata,handles);


function y=pnormobjective(x,settings,handles)

%denormalize x:
mins=settings.normalizationfactors(1,:);
ranges=settings.normalizationfactors(2,:);

%denormalize x:
x=bsxfun(@times,x,ranges);
x=bsxfun(@plus,x,mins);

%deal columns of X to correct data set columns:
xdata=zeros(1,size(handles.alldata{settings.d},2));
xdata(settings.funvars)=x; %remove the t from the end


y=raveevaluatefunction(handles.fig,settings.d,settings.y,xdata,handles);

%normalize y:

ymin=settings.ymins';
ymax=settings.ymaxs';

y = (y-ymin)./(ymax-ymin);





