function results=raveoptparetosort(src,ev,settings)
%this is not really an optimizer, but finds the pareto efficient members of your existing dataset
%and selects these rows in the current analysis.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%settings is a structure with the following fields:
% .method ='paretosort';
% .targets = best value for each objective
% .d = dataset to use
% .a = analysis to use
% .x = list of variables to use as objectives
% .activeconstraints = index of constraints to use 
% .startbutton = handle of startbutton

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

set(settings.startbutton,'enable','off')
set(handles.infobar,'string','Finding Pareto efficient points. Please wait...')
drawnow;

paretotarget=settings.targets;%[str2double(get(targ1edit,'string')) str2double(get(targ2edit,'string'))];

handles.temp.activeconstraints=settings.activeconstraints; %get(constraintlist,'value')-1;
handles.temp.activeconstraints=handles.temp.activeconstraints(logical(handles.temp.activeconstraints)); %remove 0 from list.

d=settings.d;
a=settings.a;

M=numel(settings.y); %number of objectives

objective=handles.alldata{d}(:,settings.y);


%% Reformat data such that a larger number is always best.
for j=1:M
    if paretotarget(j)==inf
        objective(:,j)=objective(:,j); %max is best
    elseif paretotarget(j)==-inf
        objective(:,j)=-objective(:,j); %min is best
    elseif ~isnan(paretotarget(j))
        objective(:,j)=-abs(objective(:,j)-paretotarget(j)); %nominal is best
    end
end

activeconstraints=setdiff(settings.activeconstraints,0);

%% evaluate constraints:
cviolation=zeros(length(objective(:,1)),numel(activeconstraints));
if isempty(handles.temp.activeconstraints)
    cviolation=zeros(length(objective(:,1)),1);
else
    for c=1:numel(handles.temp.activeconstraints)
        thisconstraint=handles.temp.activeconstraints(c);
        ignore=0;
        cviolation(:,c)=raveevaluateconstraint(handles,d,a,thisconstraint,Preal,ignore);
    end

    cviolation(cviolation<0)=0; %set feasible points to 0 violation
    scalers=handles.constraintscalers{settings.d}{settings.a}(activeconstraints);
    cviolation=bsxfun(@rdivide,cviolation,scalers);
    cviolation=sum(cviolation,2);
end

isfeasible=~cviolation;

%do pareto:
if any(isfeasible)
    feasibleobj=objective(isfeasible,:);
    isnondominated=raveparetofront(feasibleobj);
else
    isnondominated=false(size(objective,1),1);
end

%store results:
handles.isselected{d}{a}=logical(isnondominated);
ravefinish;
guidata(handles.fig,handles);
set(handles.infobar,'string','')
%update graphs:
raveupdateanalysis(handles.fig,[],'isselected',[],settings.d,settings.a);

results.settings=settings;



