function state = raveanimategads(options, state, flag)
%animation functions for the optimizers in the global optimization toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
settings=get(0,'userdata');
src=settings.src;
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%delete pop up window that appears:
handles.temp.crapwindow=gcf;
set(gcf,'visible','off')
guidata(src,handles);

switch settings.method
    case {'ga','gamultiobj'}
        pop=state.Population;
    case {'simulatedannealing','simulannealbnd'}
        pop=[state.x;state.bestx];
    case 'patternsearch'
        state=options;
        pop=state.x;
        
end

%denormalize population:
%denormalize x:
pop=bsxfun(@times,pop,settings.normalizationfactors(2,:));
pop=bsxfun(@plus,pop,settings.normalizationfactors(1,:));

plotvars=settings.plotvars;
N=size(pop,1); %population size

%distribute variables to correct columns:
indexx=settings.funvars;
%indata=zeros(N,max(indexx));

indata=repmat(handles.xdials{udata.view.dataset}{udata.view.analysis},N,1); %need this in case objective fun and plot fun's dont have same input variable set.
indata(:,indexx)=pop;
indata(:,indexx)=pop;

objective=raveevaluatefunction(src,settings.d,plotvars,indata);


cviolation=zeros(N,1);

if strcmp(settings.method,'gamultiobj')
    rank=raveparetofront(-state.Score,true);
end




switch settings.method
    case 'ga' %MULTIOBJECTIVE METHODS, USE RANK
trash=findobj(settings.plotaxes,'tag','optinfeasible','-or','tag','optfeasible','-or','tag','optcurrent','-or','tag','opthistory');
delete(trash);


        timestr=sprintf('%6.0f',cputime-state.StartTime);
        lastgoodgen=sprintf('%4.0f',state.Generation-state.LastImprovement);
        
        set(handles.infobar,'string',['Gens: \fontname{monospace} ',sprintf('%4.0f',state.Generation),'\fontname{',handles.prefs.fonts.tab,'}   Fcn Evals: ',sprintf('%6.0f',state.FunEval),'\fontname{',handles.prefs.fonts.tab,'}   Elapsed Time: ' timestr 's     Gens since improvement:' lastgoodgen])
        
        if settings.showinfo
            txt=findobj(settings.plotaxes,'tag','optinfotext');
            if ~isempty(state.Best)
                bestval=state.Best(end);
            else
                bestval=inf;
            end
            set(txt,'string',{['Generation: ',sprintf('%4.0f',state.Generation)];['Fcn Evals: ',sprintf('%6.0f',state.FunEval)];['Best Obj: ',sprintf('%6.6f',bestval)]})
        end

        switch settings.plottype
            case 'single'
                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives.
                plotdata=objective(cviolation==0,:);
                if ~isempty(plotdata)
                    L=line(plotdata(:,1),plotdata(:,2),'marker','none','marker','o','parent',settings.plotaxes,...
                        'tag','optfeasible','linewidth',1,'markeredgecolor',handles.colors.optfeasible(2,:),'markerfacecolor',handles.colors.optfeasible(1,:),...
                        'markersize',handles.prefs.optmarkersize,'buttondownfcn',@lineobject_callback,'linestyle','none');
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata); %for use in lineobjectcallback
                end
                
                plotdata=objective(cviolation~=0,:);
                if ~isempty(plotdata)
                    L=line(plotdata(:,1),plotdata(:,2),'marker','none','marker','o','parent',settings.plotaxes,...
                        'tag','optinfeasible','linewidth',1,'markeredgecolor',handles.colors.optinfeasible(2,:),...
                        'markerfacecolor',handles.colors.optinfeasible(1,:),...
                        'markersize',handles.prefs.optmarkersize,'buttondownfcn',@lineobject_callback,'linestyle','none');
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata); %for use in lineobjectcallback
                end
                
                %try %#ok<TRYNC>
                if ~isempty(state.Best)
                    plotdata=objective;
                bestidx=state.Score==state.Best(end);
                L=line(plotdata(bestidx,1),plotdata(bestidx,2),'marker','o','parent',settings.plotaxes,...
                    'tag','optcurrent','markeredgecolor',handles.colors.optoptimal(1,:),...
                    'markerfacecolor',handles.colors.optoptimal(2,:),'markersize',handles.prefs.optmarkersize,'linestyle','none');
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata);
                end
                %end
                
            case '3D'
                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives.
                plotdata=sortrows(objective(cviolation==0,:));
                if ~isempty(plotdata)
                    L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'marker','none','marker','o','parent',settings.plotaxes,...
                        'tag','optfeasible','linewidth',1,'markeredgecolor',handles.colors.optfeasible(2,:),'markerfacecolor',handles.colors.optfeasible(1,:),...
                        'markersize',6,'buttondownfcn',@lineobject_callback,'linestyle','none');
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata); %for use in lineobjectcallback
                end
                
                plotdata=sortrows(objective(cviolation~=0,:));
                if ~isempty(plotdata)
                    L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'marker','none','marker','o','parent',settings.plotaxes,...
                        'tag','optinfeasible','linewidth',1,'markeredgecolor',handles.colors.optinfeasible(2,:),...
                        'markerfacecolor',handles.colors.optinfeasible(1,:),...
                        'markersize',handles.prefs.optmarkersize,'buttondownfcn',@lineobject_callback,'linestyle','none');
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata); %for use in lineobjectcallback
                end
                
                %try %#ok<TRYNC>
                bestidx=state.Score==state.Best(end);
                if ~isempty(bestidx)
                    L=line(plotdata(bestidx,1),plotdata(bestidx,2),plotdata(bestidx,3),'marker','o','parent',settings.plotaxes,...
                        'tag','optcurrent','markeredgecolor',handles.colors.optoptimal(1,:),...
                        'markerfacecolor',handles.colors.optoptimal(2,:),'markersize',handles.prefs.optmarkersize,'linestyle','none');
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata);
                end
                %end
        end
        
    case 'gamultiobj' %MULTIOBJECTIVE METHODS, USE RANK
trash=findobj(settings.plotaxes,'tag','optinfeasible','-or','tag','optfeasible','-or','tag','optcurrent','-or','tag','opthistory');
delete(trash);
        
        timestr=sprintf('%6.0f',cputime-state.StartTime);
        set(handles.infobar,'string',['Gens: \fontname{monospace} ',sprintf('%4.0f',state.Generation),'\fontname{',handles.prefs.fonts.tab,'}   Elapsed Time: ' timestr 's'])
        
        if settings.showinfo
            txt=findobj(settings.plotaxes,'tag','optinfotext');
            set(txt,'string',{['Generation: ',sprintf('%4.0f',state.Generation)];['Fcn Evals: ',sprintf('%6.0f',state.FunEval)]})
        end
        
        
        switch settings.plottype
            case 'single'
                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives. 
                %for q=max(rank):-1:2
                    plotdata=sortrows(objective(rank>1 & cviolation==0,:));
                    L=line(plotdata(:,1),plotdata(:,2),'marker','o','linestyle','none','parent',handles.activegraph,...
                        'tag','optfeasible','markerfacecolor',handles.colors.optfeasible(1,:),...
                        'markeredgecolor',handles.colors.optfeasible(2,:),'markersize',handles.prefs.optmarkersize);
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata); %for use in lineobjectcallback
                %end
                
                if numel(settings.y)==2
                    linestyle='-';
                else
                    linestyle='none';
                end
                
                plotdata=sortrows(objective(rank==1 & cviolation==0,:));
                L=line(plotdata(:,1),plotdata(:,2),'marker','o','linestyle',linestyle,'parent',handles.activegraph,...
                    'tag','optcurrent','markerfacecolor',handles.colors.optoptimal(1,:),...
                    'markeredgecolor',handles.colors.optoptimal(2,:),'markersize',handles.prefs.optmarkersize);
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback
                
                %plot infeasible points:
                plotdata=objective(cviolation~=0,:);
                L=line(plotdata(:,1),plotdata(:,2),'marker','o','linestyle','none','parent',handles.activegraph,...
                    'tag','optinfeasible','markerfacecolor',handles.colors.optinfeasible(1,:),...
                    'markeredgecolor',handles.colors.optinfeasible(2,:),'markersize',handles.prefs.optmarkersize);
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback

            case '3D'                
                for q=max(rank):-1:1
                    %plotdata=objective(rank==q & cviolation==0,:);
                    
                    if numel(settings.y)==2 %draw line instead of points;
                       plotdata=sortrows(objective(rank==q & cviolation==0,:));
                    else
                       plotdata=objective(rank==q & cviolation==0,:);
                    end
                   
                    L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'parent',handles.activegraph,'marker','o',...
                        'tag','optfeasible','markerfacecolor',handles.colors.optfeasible(1,:),...
                        'markeredgecolor',handles.colors.optfeasible(2,:),'markersize',handles.prefs.optmarkersize,'linestyle','none');
                    
                    if q==1
                       set(L,'markerfacecolor',handles.colors.optoptimal(1,:),'markeredgecolor',handles.colors.optoptimal(2,:)); 
                    end
                end

                %plot infeasible points:
                plotdata=objective(cviolation~=0,:);
                L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'marker','o','linestyle','none','parent',handles.activegraph,...
                    'tag','optinfeasible','markerfacecolor',handles.colors.optinfeasible(1,:),...
                    'markeredgecolor',handles.colors.optinfeasible(2,:),'markersize',handles.prefs.optmarkersize,'buttondownfcn',@lineobject_callback);
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback
        end
    case {'simulatedannealing','simulannealbnd'}
        trash=findobj(settings.plotaxes,'tag','optinfeasible','-or','tag','optfeasible','-or','tag','optcurrent','-or','tag','opthistory');
delete(trash);
        
        
        set(handles.infobar,'string',['Iteration: ',sprintf('%6.0f',state.iteration),'  Fcn Evals: ',sprintf('%6.0f',state.funccount),'  Avg Temp: ',sprintf('%6.6f',mean(state.temperature)),'  Best Obj: ',sprintf('%6.6f',state.bestfval)])
        
        if settings.showinfo
            txt=findobj(settings.plotaxes,'tag','optinfotext');

            set(txt,'string',{['Iteration: ',sprintf('%6.0f',state.iteration)];['Fcn Evals: ',sprintf('%6.0f',state.funccount)];['Avg Temp: ',sprintf('%6.6f',mean(state.temperature))];['Best Obj: ',sprintf('%6.6f',state.bestfval)]})
        end
        
        switch settings.plottype
            case 'single'
                sah=findobj('tag','opthistory');
                if isempty(sah)
                    sah=line(nan,nan,'parent',settings.plotaxes,'tag','opthistory','color',handles.colors.opthistory(1,:));
                end
                
                sahx=get(sah,'xdata');
                sahy=get(sah,'ydata');
                sahx=[sahx,objective(1,1)];
                sahy=[sahy,objective(1,2)];
                
                set(sah,'xdata',sahx,'ydata',sahy);

                %PLOT current point:
                L=line(objective(1,1),objective(1,2),'marker','o','parent',settings.plotaxes,...
                    'tag','optfeasible','linewidth',1,'markerfacecolor',handles.colors.optfeasible(1,:),'markeredgecolor',handles.colors.optfeasible(2,:),...
                    'markersize',handles.prefs.optmarkersize,'linestyle','none');

                %Plot best ever point:
                L=line(objective(2,1),objective(2,2),'marker','o','parent',settings.plotaxes,...
                    'tag','optcurrent','linewidth',1,'markerfacecolor',handles.colors.optoptimal(1,:),'markeredgecolor',handles.colors.optoptimal(2,:),...
                    'markersize',handles.prefs.optmarkersize,'linestyle','none');  
        end
        
    case 'patternsearch'
%         trash=findobj(settings.plotaxes,'tag','optinfeasible','-or','tag','optfeasible');
% delete(trash);
        
        set(handles.infobar,'string',['Iteration: ',sprintf('%6.0f',state.iteration),'  Fcn Evals: ',sprintf('%6.0f',state.funccount),'  Best Obj: ',sprintf('%6.6f',state.fval)])
        
        if settings.showinfo
            txt=findobj(settings.plotaxes,'tag','optinfotext');
            set(txt,'string',{['Iteration: ',sprintf('%6.0f',state.iteration)];['Fcn Evals: ',sprintf('%6.0f',state.funccount)];['Best Obj: ',sprintf('%6.6f',state.fval)]})
        end

        set(handles.infobar,'string',['Iteration: ',num2str(state.iteration),'  Fcn Evals: ',num2str(state.funccount)])
        
        switch settings.plottype
            case 'single'
                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives.

                ph=findobj('tag','opthistory');
                if isempty(ph)
                    ph=line(nan,nan,'parent',settings.plotaxes,'tag','opthistory','color',handles.colors.opthistory(1,:));
                end
                
                phx=get(ph,'xdata');
                phy=get(ph,'ydata');
                phx=[phx,objective(1,1)];
                phy=[phy,objective(1,2)];
                
                set(ph,'xdata',phx,'ydata',phy);
                
                oldplots=findobj(settings.plotaxes,'tag','optcurrent','type','line');
                set(oldplots,'markerfacecolor',handles.colors.optfeasible(1,:),'markeredgecolor',handles.colors.optfeasible(2,:),'tag','optfeasible');

%                 %PLOT current point:
%                 L=line(objective(1,1),objective(1,2),'marker','o','parent',settings.plotaxes,...
%                     'tag','optplots','linewidth',1,'markerfacecolor',handles.colors.optfeasible(1,:),'markeredgecolor',handles.colors.optfeasible(2,:),...
%                     'markersize',handles.prefs.optmarkersize,'linestyle','none');

                %Plot best ever point:
                L=line(objective(1,1),objective(1,2),'marker','o','parent',settings.plotaxes,...
                    'tag','optcurrent','linewidth',1,'markerfacecolor',handles.colors.optoptimal(1,:),'markeredgecolor',handles.colors.optoptimal(2,:),...
                    'markersize',handles.prefs.optmarkersize,'linestyle','none');  

                
            case '3D'
        end
        
end

%fix axes limits:
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
    'xlim',get(settings.plotaxes,'xlim'),'ylim',get(settings.plotaxes,'ylim'),'zlim',get(settings.plotaxes,'zlim'))
drawnow;

%export animation if necessary:
if settings.animation
    Framez=get(handles.fig,'userdata');
    Framez(end+1) = getscreen(handles.fig,settings.framebounds);
    set(handles.fig,'userdata',Framez);
end

%if strcmp(settings.method,'simulatedannealing') || strcmp(settings.method,'simulannealbnd') || strcmp(settings.method,'patternsearch')
if ismember(settings.method,{'simulatedannealing','simulannealbnd','patternsearch'})
    state=false; %set this to true to stop at the current iteration.
    if get(settings.stopbutton,'value');state=true;end
elseif ismember(settings.method,{'ga','gamultiobj'})
    if get(settings.stopbutton,'value')
        state.StopFlag='Stopped by user.';
    end
end

if settings.pause
    waitforbuttonpress;
    if uint8(get(gcf,'currentcharacter'))== 27
        settings.pause=0;
        set(0,'userdata',settings)
    end
    figure(handles.temp.crapwindow);
end





