function results=raveoptpatternsearch(src,ev,settings)
%optimize in rave using the functions from the gads toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
%starttime=0;

raveinitializeoptimizeranimation;

d=settings.d;
a=settings.a;

guidata(handles.fig,handles);

%% make objective/constraint functions:
[functionname,index,mins,maxs]=ravemakeminimizefunction(src,ev,settings);
FITNESSFCN=str2func(functionname);

%% make options structure

options=psoptimset;
options.Cache=settings.cache;
options.CacheSize=settings.cachesize;
options.CacheTol=settings.cachetol;
options.CompletePoll=settings.completepoll;
options.CompleteSearch=settings.completesearch;
options.Display='final';
options.InitialMeshSize=settings.initmeshsize;
options.InitialPenalty=10;
options.MaxFunEvals=settings.maxfun;
options.MaxIter=settings.maxiter;
options.MaxMeshSize=settings.maxmeshsize;
options.MeshAccelerator=settings.meshaccel;
options.MeshContraction=settings.meshcontract;
options.MeshExpansion=settings.meshexpand;
options.MeshRotate=settings.meshrotate;
options.OutputFcn=[];
options.PenaltyFactor=settings.penalty;
options.PollingOrder=settings.pollorder;
options.PollMethod=settings.pollmeth;
options.ScaleMesh=settings.scalemesh;
options.SearchMethod=settings.searchmeth;
options.TimeLimit=settings.timelimit;
options.TolBind=settings.bindtol;
options.TolCon=settings.consttol;
options.TolFun=settings.objtol;
options.TolMesh=settings.meshtol;
options.TolX=settings.inputvartol;


if settings.plotprogress
    options.PlotFcns=str2func(settings.plotfunction);
    settings.src=src;
    set(0,'userdata',settings);
            trash=findobj(settings.plotaxes,'tag','optinfeasible','-or','tag','optfeasible','-or','tag','optcurrent','-or','tag','opthistory');
delete(trash);
        
else
    set(0,'userdata',settings);
    options.PlotFcns=[];
end

options.Vectorized='on';
options.UseParallel='never';



%% do optimizer:
%tic;

startpt=handles.xdials{d}{a}(index);
startpt=bsxfun(@minus,startpt,mins);
startpt=bsxfun(@rdivide,startpt,maxs-mins);
startpt(isnan(startpt))=0;

[X,FVAL,EXITFLAG,OUTPUT] = patternsearch(FITNESSFCN,startpt,[],[],[],[],zeros(size(mins)),ones(size(mins)),options); %#ok<NASGU>

%denormalize x:
X=bsxfun(@times,X,settings.normalizationfactors(2,:));
X=bsxfun(@plus,X,settings.normalizationfactors(1,:));


ph=findobj(settings.plotaxes,'tag','patternhist');
set(ph,'tag','optplots');

handles=guidata(src);

if settings.plotprogress && ishandle(handles.temp.crapwindow)
    delete(handles.temp.crapwindow);
end

%distribute population to correct columns:
indexx=settings.funvars;

population=X;


finalpop=repmat(handles.minlimit{d}{a},size(population,1),1);
finalpop(:,indexx)=population;

%evaluate this for ALL variables:
respvars=find(handles.isfunction{settings.d});
objective=raveevaluatefunction(src,settings.d,respvars,finalpop);
finalpop(:,respvars)=objective;

%elapsed=toc/(24*3600);
%timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.

%set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',settings.gens),'\fontname{comic sans ms} Generations Completed.    Elapsed Time: ' timestr])

ravefinishoptimizeranimation;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
objs=handles.allnames{settings.d}{settings.y};

results.name=['Pattern Search ',objs,'   (',handles.datasetnames{settings.d},')'];

[sortedpop,index]=sortrows(finalpop,settings.y);
results.finalinputs=X;
results.finalobjective=FVAL;
results.settings=settings;
