function varargout=rave
% Welcome to the Rave.
% For help with Rave you should read the
% documentation at http://www.rave.gatech.edu/help/
%
% Quick help: type "rave" at the MATLAB command prompt to start the program.
% If you haven't used Rave before, an installer will run. 
% Then click Manage Data Sets to load a data set (must be "flat file" format).
% Then click "Create New..." to make a graph, or go the Model tab to load a
% model or make a surrogate model. Once you have loaded a model, you can
% run optimizers from the Optimize tab.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% If you would like to obtain RAVE under a different license, contact
% Georgia Tech Reserach Corporation http://otl.gtrc.gatech.edu/
%
% Rave was created by Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu
% Please contact me with bug reports, feature requests, collaboration
% ideas, or feedback on how you are using RAVE.

warning('off','MATLAB:HandleGraphics:ObsoletedProperty:JavaFrame');
clear functions %attempt to avoid errors due to old stuff already being in memory

if ~usejava('swing') || ~usejava('awt')
    errordlg('Rave requires Java to run.');
    return
end

set(0,'showhiddenhandles','off')

if isempty(which('raveprefs.mat'));
    raveinstall;
    return;
end

load raveprefs

raveprefs=ravevalidateprefs(raveprefs); %#ok<NODEF>
thisver=ravedatenum;
if thisver>raveprefs.currentversion;raveprefs.currentversion=thisver;end
save(which('raveprefs.mat'),'raveprefs');

raveroot=which('rave');
raveroot=raveroot(1:end-6);
addpath(genpath(raveroot))
if raveprefs.isportable
    raveprefs.defaultdir=fullfile(raveroot,'ravefiles');
end

if isempty(which('RaveTableSorter'))
    javaaddpath(which('RaveTable.jar'));
end

%reset default font sizes to ensure things fit on the screen:
set(0,'defaulttextfontsize',10)
set(0,'defaultuicontrolfontsize',8);

if ispc
    set(0,'defaultuicontrolfontname','ms sans serif')
elseif ismac
    set(0,'defaultuicontrolfontname','geneva')    
end

set(0,'defaultuicontrolhittest','off'); %maybe speeds up the hittest function?

%check if monitor position has changed:
mpos=get(0,'MonitorPositions');
if numel(mpos(:))~=numel(raveprefs.monitorpositions(:)) || ~all(mpos(:)==raveprefs.monitorpositions(:))
    answer=questdlg('Your screen resolution has changed since you last used Rave. Do you want to resize the Rave window to better fit your new resolution?','Resize Rave Window');
    drawnow;
    if strcmpi(answer,'yes')
        hhh=raveresizewindow([],[]);
        uiwait(hhh);
        load raveprefs
        raveprefs.monitorpositions=get(0,'MonitorPositions');
    end
end
drawnow;
raveprefs.monitorpositions=get(0,'MonitorPositions');
save(which('raveprefs.mat'),'raveprefs');

handles.prefs=raveprefs;

%make tray icon:
sysTray = java.awt.SystemTray.getSystemTray;
if (sysTray.isSupported)
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);
    myIcon = fullfile(raveroot,'images','tinyrave.png');
    iconImage = java.awt.Toolkit.getDefaultToolkit.createImage(myIcon);
    handles.trayIcon = java.awt.TrayIcon(iconImage, 'initial tooltip');
    handles.trayIcon.setToolTip('Rave is running.');
    sysTray.add(handles.trayIcon)   
else
    handles.trayIcon=[];
end

fff=figure('visible','on','toolbar','none','name','Rave','menubar','none','numbertitle','off',...
    'resize','off','color',raveprefs.colors.frame,'units','pixels','renderermode','manual','renderer','painters',...
    'handlevisibility','callback','integerhandle','off','doublebuffer','on');
handles.fig=fff;

%opengl renderer settings:
if ispc
    if ~isempty(raveprefs.wvisual);set(fff,'wvisual',raveprefs.wvisual);end
elseif isunix
    if ~isempty(raveprefs.xvisual);set(fff,'xvisual',raveprefs.xvisual);end   
end

if nargout==1
    varargout={fff};
else
    varargout=cell(0);
end

outerwidth=raveprefs.preferredposition(3); %new
outerheight=raveprefs.preferredposition(4); %new

%set up colors
handles.colors=raveprefs.colors;

%set work dir to default dir to start.
handles.defaultdir=raveprefs.defaultdir;
if ~strcmp(handles.defaultdir(end),char(92)) %if it doesnt end in a \, put one there.
    handles.defaultdir(end+1)=char(92);
end

%default colormap:
guidata(handles.fig,handles);
[cmap,cmapnames]=ravemakecolormap(handles.fig);

set(handles.fig,'colormap',cmap);
handles.cmapnames=cmapnames;

set(handles.fig,'position',raveprefs.preferredposition)

%set up data structure for passing info between windows:
setappdata(handles.fig,'backupdata',{});

% handles.alldata=cell(0);
% handles.datasetnames=cell(0);
% handles.datasetfiles=cell(0);
% handles.datasettypes=cell(0);
% handles.datasetlinks=cell(0);
% handles.datasetlinkinfo=cell(0);
% handles.analysisnames={'Default Analysis'};
% handles.allnames=cell(0);
% handles.isvalid=cell(0);
% handles.isresponse=cell(0);
% handles.nargout=cell(0);
% handles.allunits=cell(0);
% handles.isincluded=cell(0);
% handles.isdecided=cell(0);
% handles.alltext=cell(0);
% handles.discretevalues=cell(0);
% %analysis stuff:
% handles.targetvalues=cell(0);
% handles.xdials=cell(0);
% handles.minlimit=cell(0);
% handles.maxlimit=cell(0);
% handles.minquery=cell(0);
% handles.maxquery=cell(0);
% handles.discretequery=cell(0);
% handles.rowcolors=cell(0);
% handles.ineqconstraints=cell(0);
% handles.isselected=cell(0);
% handles.isvisible=cell(0);
% handles.ispointofinterest=cell(0);
% handles.variablecolors=cell(0);
% handles.preferences=cell(0);
% handles.normalizepreferences=cell(0);
% handles.isobjective=cell(0); % a special type of function that uses preferences/targets
% handles.projectname='';
% handles.rankingcolumn=cell(0);
% handles.rowlabelcolumn=cell(0);
% handles.roi=[]; %print region of interest;
% handles.selectionmode='lasso'; %current selection mode: none, box, lasso, or point.
% handles.zoommode='selection';
% handles.axespositions=cell(0); %unused
% handles.isnoise=cell(0);
% handles.distname=cell(0);
% handles.distparams=cell(0);
% handles.modelqueue=cell(0); %surrogate models waiting to be fit.
% handles.models=cell(0); %surrogate models that have finished fitting.
% handles.axeslisteners={};
% handles.lastsheet=-1;
% handles.optimizerresults=struct([]);
% 
% 
% raveroot = which('rave');
% handles.raveroot=raveroot(1:end-6);
% 
% %initialize variables
% handles.newgraphtype='blank';
% handles.currentgraph=[];
% handles.allgraphs=[];
% handles.graphanalysis=[]; %same length as allgraphs, contains number of analysis for indicated graph.
% handles.graphdataset=[]; %ditto for data sets.
% handles.fakeax=[];
% handles.fakerax=[]; %allgraphs, fakeax, and fakerax contain the handles to the outermost (biggest) axes for a graph. They are always dim=(1,1), so they dont need to be cells.
% handles.allblockers=[];
% handles.expblockers=[];
% handles.expAblockers=[];
% handles.activeindex=[];
% handles.activegraph=[];
% handles.activetype='';
% handles.activetab='';
% handles.activedir=[]; %the directory this project resides in.
% handles.activesubgraph=[]; %for use with plot matrices. contains the index of a SINGLE selected sub-axes. To get all subaxes for a graph, use handles.subax{activeindex}(:,:);
% handles.subax=cell(0);
% handles.subfakeax=cell(0);
% handles.subfakerax=cell(0);
% handles.tempplots=[];
% %handles.tabflexgrid=[];
% handles.rotater=0;
% handles.rasters=cell(0);
% handles.grabbers=cell(0);
% %handles.javacomps=[];
% handles.textboxj=cell(0);
% handles.textboxh=cell(0);
% handles.Agrabbers=cell(0);
% handles.putaway=[]; %index of graphs that are currently put away.
% handles.graphnames=cell(0);
% handles.projectname='';
% handles.activesheet=1;
% handles.sheet=[];
% handles.optimizetabobjects=[]; %an annoying special case.
% handles.tabsvisible=1;
% handles.mouseoverhandles=[];
% handles.mouseoverfunctions=[];
% handles.mouseoverdata={[]};
% handles.isjavaworkspaceobject=false(0);
% 
% handles.lasta=1; %analysis of the last graph that was clicked
% handles.lastd=1; %dataset of the last graph that was clicked
% 
% %intitialize "udata" for draw tab:
% handles.draw.textbg=[1 1 .8]; %post-it yellow
% handles.draw.edittextmode=0;%if 1, process drawtab callbacks on current text box.
% handles.draw.fontname=handles.prefs.fonts.textbox;
% handles.draw.fontstyle=4; %plain by default
% handles.draw.fontsize=14;
% handles.draw.showborder=1;
% handles.draw.markercolor=[0 .5 1];
% handles.draw.markersize=2;
% 
% %save last export config for easy exporting:
% handles.lastexport.scale=10;
% handles.lastexport.antialias='none';
% handles.lastexport.filetype='.png';
% 
% %maintable stuff:
% handles.tablemode=0; %true when table is visible.
% handles.maintable.undos=cell(50,1); %history of table changes, used to undo.
% handles.maintable.redos=cell(50,1); %for updates that have been undone.
% handles.maintable.x=cell(0); %visible columns for each data set.
% handles.maintable.colwidths=cell(0);
% handles.maintable.wasdragged=0; %draggings sets this to 1 so mousereleasecallback knows not to select column.
% handles.maintable.colorrows=1; %use row colors?
% handles.maintable.shadehidden=1; %shade hidden rows?
% handles.maintable.allowedits=1;
% handles.maintable.allowselection=1;
% handles.datasetupdatespending=cell(0);
% handles.analysisupdatespending=cell(0);
% %handles.maintableupdatespending=cell(0); %deprecated
% handles.tablemodelupdatespending=cell(0);
% 
% handles.maintable.d=0;
% handles.maintable.a=0;
% handles.maintable.ignoreupdates=false;
% 
% handles.helproot='http://www.rave.gatech.edu/help/index.php?title=';

%Initialize handles fields:
handles=raveinitializehandles(handles);
guidata(handles.fig,handles);

%% Load all plugins:

%Load data for workspace objects:
raveupdategraphtypes(handles.fig,[]);

%Load data for optimizers:
raveupdateoptimizertypes(handles.fig,[])

%Load data for surrogate models:
raveupdatemodeltypes(handles.fig,[])

%Load data for explore methods:
raveupdateexploretypes(handles.fig,[]);
handles=guidata(handles.fig);

try %#ok<TRYNC> %see if system will let you save the path.
    savepath
end

%%
innerposition=get(handles.fig,'position');
handles.width=innerposition(3);
handles.height=innerposition(4);

%% make main data table
guidata(handles.fig,handles);
ravecreatemaintable(handles.fig,[]);
handles=guidata(handles.fig);

set(handles.maintable.handle,'visible','off');

%% set up main (workspace) axes
handles.boneyard=uipanel('position',[0 0 .3 .3],'backgroundcolor',[1 1 1],'units','pixels',...
    'foregroundcolor',handles.colors.frame,'parent',handles.fig,'highlightcolor',handles.colors.frame,'shadowcolor',handles.colors.frame,'visible','off','hittest','off');
handles.wspanel=uipanel('position',[0 0 .3 .3],'backgroundcolor',[1 1 1],'units','pixels',...
    'foregroundcolor',handles.colors.frame,'parent',handles.fig,'highlightcolor',handles.colors.frame,'shadowcolor',handles.colors.frame,'hittest','off');

handles.wscols=handles.prefs.wswidth/handles.prefs.gridsize;
handles.wsrows=handles.prefs.wsheight/handles.prefs.gridsize;


%% set up frame axes
handles.leftmargin=axes('units','pixels','position',[0 -5 30+315*handles.prefs.tabscale handles.height+11],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame);

handles.rightmargin=axes('units','pixels','position',[handles.width-13 -5 40 handles.height+20],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame);

handles.topmargin=axes('units','pixels','position',[-5 handles.height-35 handles.width+5 60],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame,'xlimmode','manual','ylimmode','manual',...
    'xlim',[0 1],'ylim',[0 1]);
handles.bottommargin=axes('units','pixels','position',[-5 1 handles.width+10 17],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame);

leftpos=get(handles.leftmargin,'position');
rightpos=get(handles.rightmargin,'position');
toppos=get(handles.topmargin,'position');
botpos=get(handles.bottommargin,'position');

handles.workspacedims=[rightpos(1)-leftpos(1)-leftpos(3),toppos(2)-botpos(2)-botpos(4)];
handles.workspacepos=[leftpos(1)+leftpos(3) botpos(2)+botpos(4) handles.workspacedims(1) handles.workspacedims(2)];

%make infobar:
infopos=get(handles.topmargin,'position');
infoaxes=axes('units','pixels','position',infopos+[360 0 -360 0],'color','none','visible','off','parent',handles.fig,'tag','infoaxes','hittest','off');
handles.infobar=text(.0,.05,'','horizontalalignment','left',...
    'fontsize',14*raveprefs.fonts.tabscale,'fontname',raveprefs.fonts.tab,'color',[.3 .6 .8],'parent',infoaxes,'verticalalignment','baseline','interpreter','tex','hittest','off');

handles.mainaxes=axes('units','pixels','position',[350 toppos(2)-handles.wsrows*handles.prefs.gridsize-1 handles.wscols*handles.prefs.gridsize handles.wsrows*handles.prefs.gridsize],...
    'parent',handles.fig,'ytickmode','manual','yticklabel','','xtickmode',...
    'manual','xticklabel','','box','off','clipping','off',...
    'xlimmode','manual','ylimmode','manual','color',handles.colors.wsbg,'buttondownfcn',@ravedragview);

handles.foregroundaxes=axes('units','pixels','position',[350 toppos(2)-handles.wsrows*handles.prefs.gridsize-1 handles.wscols*handles.prefs.gridsize handles.wsrows*handles.prefs.gridsize],...
    'parent',handles.fig,'ytickmode','manual','yticklabel','','xtickmode',...
    'manual','xticklabel','','box','off','clipping','off',...
    'xlimmode','manual','ylimmode','manual','color','none','hittest','off','visible','off');

axessize=get(handles.mainaxes,'position');
axessize(3:4)=handles.prefs.gridsize.*floor(axessize(3:4)./handles.prefs.gridsize);
set(handles.mainaxes,'position',axessize);

%draw grids
set(handles.mainaxes,'xlim',[0 axessize(3)],'ylim',[0 axessize(4)])

handles.prefs.wswidth=axessize(3);
handles.prefs.wsheight=axessize(4);

r=0:(handles.prefs.gridsize/4):axessize(4);
xcoords=repmat([0 axessize(3) axessize(3) 0],1,ceil(numel(r)./2));
ycoords=[r;r];
ycoords=ycoords(:);
xcoords=xcoords(1:numel(ycoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','sgrid');

r=0:(handles.prefs.gridsize/2):axessize(4);
xcoords=repmat([0 axessize(3) axessize(3) 0],1,ceil(numel(r)./2));
ycoords=[r;r];
ycoords=ycoords(:);
xcoords=xcoords(1:numel(ycoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','mgrid');

r=0:handles.prefs.gridsize:axessize(4);
xcoords=repmat([0 axessize(3) axessize(3) 0],1,ceil(numel(r)./2));
ycoords=[r;r];
ycoords=ycoords(:);
xcoords=xcoords(1:numel(ycoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','lgrid');

c=0:(handles.prefs.gridsize/4):axessize(3);
ycoords=repmat([0 axessize(4) axessize(4) 0],1,ceil(numel(c)./2));
xcoords=[c;c];
xcoords=xcoords(:);
ycoords=ycoords(1:numel(xcoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','sgrid');

c=0:(handles.prefs.gridsize/2):axessize(3);
ycoords=repmat([0 axessize(4) axessize(4) 0],1,ceil(numel(c)./2));
xcoords=[c;c];
xcoords=xcoords(:);
ycoords=ycoords(1:numel(xcoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','mgrid');

c=0:(handles.prefs.gridsize):axessize(3);
ycoords=repmat([0 axessize(4) axessize(4) 0],1,ceil(numel(c)./2));
xcoords=[c;c];
xcoords=xcoords(:);
ycoords=ycoords(1:numel(xcoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','lgrid');

handles.lgrid=findobj(handles.mainaxes,'tag','lgrid');
handles.mgrid=findobj(handles.mainaxes,'tag','mgrid');
handles.sgrid=findobj(handles.mainaxes,'tag','sgrid');

%fix grid visibility
if handles.prefs.gridmode==1
    set(handles.lgrid,'visible','on')
    set(handles.mgrid,'visible','off')
    set(handles.sgrid,'visible','off')
    handles.snapsize=handles.prefs.gridsize;
elseif handles.prefs.gridmode==2
    set(handles.lgrid,'visible','on')
    set(handles.mgrid,'visible','on')
    set(handles.sgrid,'visible','off')
    handles.snapsize=handles.prefs.gridsize/2;
else
    set(handles.lgrid,'visible','on')
    set(handles.mgrid,'visible','on')
    set(handles.sgrid,'visible','on')
    handles.snapsize=handles.prefs.gridsize/4;
end

if ~handles.prefs.snaptogrid
    handles.snapsize=1;
end

handles.wsrows=axessize(4)/handles.prefs.gridsize;
handles.wscols =axessize(3)/handles.prefs.gridsize;
handles.wsdims=[axessize(3) axessize(4)];

%draw additional box around axes that is always visible
for r=0:axessize(4):axessize(4)
    line([0 axessize(3)],[r r],'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','wspanelborder');
end
for c=0:axessize(3):axessize(3)
    line([c c],[0 axessize(4)],'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','wspanelborder');
end

%move uipanel to encompass mainaxes, and make uipanel mainaxes's parent
putpanelhere=get(handles.mainaxes,'position');
set(handles.wspanel,'position',[putpanelhere(1)-5 putpanelhere(2)-3 putpanelhere(3)+5 putpanelhere(4)+5]);
set(handles.boneyard,'position',[putpanelhere(1)-5 putpanelhere(2)-3 putpanelhere(3)+5 putpanelhere(4)+5]);
set([handles.mainaxes handles.foregroundaxes],'parent',handles.wspanel);
set([handles.mainaxes handles.foregroundaxes],'position',[1 2 putpanelhere(3) putpanelhere(4)])


%% make page controls:
handles.pageedit=uicontrol('style','edit','position',[280+5 handles.height-31 40 20],'parent',handles.fig,...
    'fontname','Arial','fontsize',16,'backgroundcolor',handles.colors.frame,'foregroundcolor',handles.colors.title,...
    'string','1','callback',@raveeditpage,'tag','pageedit');
ppp=findjobj(handles.pageedit);
ppp.setBorder(javax.swing.BorderFactory.createEmptyBorder)

lbutton=imread('leftbuttonup.png','backgroundcolor',handles.colors.frame);
rbutton=imread('rightbuttonup.png','backgroundcolor',handles.colors.frame);

handles.pagecontrols.left=uicontrol('units','pixels','style','pushbutton','position',[263+5 handles.height-35 17 27],'parent',handles.fig,...
    'cdata',lbutton,'buttondownfcn',{@ravechangepage,-1},'tag','pageleftbutton','enable','inactive','hittest','on');
handles.pagecontrols.right=uicontrol('units','pixels','style','pushbutton','position',[320+5 handles.height-35 17 27],'parent',handles.fig,...
    'cdata',rbutton,'buttondownfcn',{@ravechangepage,1},'tag','pagerightbutton','enable','inactive','hittest','on');


%% set up workspace explorer axes
if handles.height>=900
    pushdown=30; %move down a little so its not so cramped
    explheight=150;
else
    pushdown=0;
    explheight=110;
end
if handles.wscols>=240/explheight*handles.wsrows %width constrained
    
    handles.explorer=axes('units','pixels','position',[100 handles.height-155-pushdown 240 handles.wsrows/handles.wscols*240],...
        'parent',handles.fig,...
        'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off','clipping','off',...
        'xlimmode','manual','ylimmode','manual','xlim',get(handles.mainaxes,'xlim'),...
        'ylim',get(handles.mainaxes,'ylim'),'ycolor',[1 1 1],'xcolor',[1 1 1],...
        'buttondownfcn',@ravemoveview,'hittest','on','color',handles.colors.wsbg);
    
else%height constrained
    handles.explorer=axes('units','pixels','position',[100 handles.height-5-explheight-pushdown handles.wscols/handles.wsrows*explheight explheight],...
        'parent',handles.fig,...
        'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off','clipping','off',...
        'xlimmode','manual','ylimmode','manual','xlim',get(handles.mainaxes,'xlim'),'ylim',get(handles.mainaxes,'ylim'),...
        'ycolor',[1 1 1],'xcolor',[1 1 1],...
        'buttondownfcn',@ravemoveview,'hittest','on','color',handles.colors.wsbg);
end


%draw explorer grid
handles.explorergrid=copyobj(handles.lgrid,handles.explorer);
set(handles.explorergrid,'color',(handles.colors.wsgrid+handles.colors.wsbg)/2)

if handles.height<=776
    set(handles.explorer,'position',[-500 -500 100 100]);
end


%% draw current-view frame

%calculate where to put it.
mainaxespos=get(handles.wspanel,'position');

makethistall=min(handles.workspacepos(4),handles.prefs.wsheight);
makethiswide=min(handles.workspacepos(3),handles.prefs.wswidth);
putthisfarup=handles.workspacepos(2)-mainaxespos(2);
putthisfarleft=handles.workspacepos(1)-mainaxespos(1);

%draw it
handles.currentview=rectangle('position',[putthisfarleft putthisfarup makethiswide makethistall],'parent',handles.explorer,...
    'linewidth',2,'clipping','off','edgecolor',handles.colors.text,'hittest','off','tag','currentview');

%get x y coords of rectangle in explorer
cvpos=get(handles.currentview,'position');

%% make explorer title if on large screen
aaa=axes('units','pixels','position',[340-handles.wscols/handles.wsrows*150 handles.height-2-pushdown 200 50],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off','clipping','off',...
    'xlimmode','manual','ylimmode','manual','xlim',get(handles.mainaxes,'xlim'),'ylim',get(handles.mainaxes,'ylim'),'ycolor',handles.colors.frame,'xcolor',handles.colors.frame,...
    'buttondownfcn','','hittest','off','visible','off','color','none');
handles.explorertitle=text(0,.2,'Navigator','fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,'color',handles.colors.title,...
    'parent',aaa,'verticalalignment','baseline');

if handles.height<900
    set(handles.explorertitle,'visible','off')
end


%% make scrollbars
originalLnF = javax.swing.UIManager.getLookAndFeel;  %class
if ~handles.prefs.dontchangemylaf
    try %#ok<TRYNC>
        newLnF = 'com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel';
        javax.swing.UIManager.setLookAndFeel(newLnF);
    end
end

vslidestep=[.05 cvpos(4)/((handles.prefs.gridsize*handles.wsrows)-cvpos(4))];

if any(vslidestep<0)|| any(isinf(vslidestep))
    handles.vscroll=uicontrol('position',[handles.width-16 17 16 handles.height-53],'style','slider',...
        'callback',{@vertscroll,handles.fig},'min',cvpos(4)/2,'max',cvpos(4)/2+eps(cvpos(4)),...
        'sliderstep',[.02 0.1],...
        'value',cvpos(4)/2,'parent',handles.fig,'enable','off');
else
    handles.vscroll=uicontrol('position',[handles.width-16 17 16 handles.height-53],'style','slider',...
        'callback',{@vertscroll,handles.fig},'min',cvpos(4)/2,'max',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2+5,...
        'sliderstep',vslidestep,...
        'value',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2,'parent',handles.fig);
end
%horizontalslider
hslidestep=[.02 cvpos(3)/((handles.prefs.gridsize*handles.wscols)-cvpos(3))];
if any(hslidestep<0) || any(isinf(hslidestep))
    handles.hscroll=uicontrol('position',[345+1 1 handles.width-345-17 16],'style','slider',...
        'callback',{@horizscroll,handles.fig},'min',cvpos(3)/2,'max',cvpos(3)/2+eps(cvpos(3)),...
        'sliderstep',[.02 0.1],...
        'value',cvpos(3)/2,'parent',handles.fig,'enable','off');
else
    handles.hscroll=uicontrol('position',[345+1 1 handles.width-345-17 16],'style','slider',...
        'callback',{@horizscroll,handles.fig},'min',cvpos(3)/2,'max',handles.prefs.gridsize*handles.wscols-cvpos(3)/2+5,...
        'sliderstep',[.02 cvpos(3)/((handles.prefs.gridsize*handles.wscols)-cvpos(3))],...
        'value',cvpos(3)/2,'parent',handles.fig);
end
drawnow;
javax.swing.UIManager.setLookAndFeel(originalLnF);

set([handles.vscroll handles.hscroll],'handlevisibility','off');

%% set up title axes if on a large monitor
aaaa=axes('units','pixels','position',[20 20 320 40],'parent',handles.fig,...
    'visible','off','xlim',[0 1],'ylim',[0 1]);
handles.titleaxes=text(.5,0,'rave','fontname','magneto','parent',aaaa,'fontsize',80,'color',handles.colors.title,...
    'verticalalignment','baseline','backgroundcolor','none','hor','center','hittest','on','buttondownfcn',@ravefixtabs);
if handles.height<900
    set(handles.titleaxes,'visible','off')
    axesoffset=25;
else
    axesoffset=85;
end

%% set up tab axes

tabwidth=1; %pixel width of tab borders

if handles.height>=900
    pushup=15;
else
    pushup=0;
end

handles.backtabposition=[20 pushup-20+axesoffset-650*(handles.prefs.tabscale-1) 30*handles.prefs.tabscale 630*handles.prefs.tabscale];
handles.fronttabposition=[20+30*(handles.prefs.tabscale) 20+pushup-20+axesoffset-650*(handles.prefs.tabscale-1) 30*handles.prefs.tabscale 631*handles.prefs.tabscale];

handles.bgtabaxes=axes('units','pixels','position',[20+30*(handles.prefs.tabscale) pushup+axesoffset-20-650*(handles.prefs.tabscale-1) 261*handles.prefs.tabscale 631*handles.prefs.tabscale],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'clipping','off',...
    'ycolor',handles.colors.frame,'xcolor',handles.colors.frame,'zcolor',handles.colors.tabfg,'color','none','visible','on','hittest','off');
handles.tabaxes=axes('units','pixels','position',[20+60*(handles.prefs.tabscale) 20+pushup-20+axesoffset-650*(handles.prefs.tabscale-1) 261*handles.prefs.tabscale 631*handles.prefs.tabscale],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'clipping','off',...
    'ycolor',handles.colors.frame,'xcolor',handles.colors.frame,'zcolor',handles.colors.tabfg,'color','none','visible','on','hittest','off');
handles.fronttabaxes=axes('units','pixels','position',handles.fronttabposition,'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'color','none',...
    'ycolor',[0 0 0],'xcolor',[0 0 0],'zcolor',handles.colors.tabbg,...
    'visible','off','hittest','off');
handles.backtabaxes=axes('units','pixels','position',handles.backtabposition,'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'color','none',...
    'ycolor',[0 0 0],'xcolor',[0 0 0],'zcolor',handles.colors.frame,...
    'visible','off','hittest','off');

%set up non-tab tab borders
%background
patch([0 1 1 0],[1 1 0 0],handles.colors.tabbg,'parent',handles.tabaxes,'edgecolor','none','tag','tabbg','hittest','off')
patch([0 1 1 0],[1 1 0 0],handles.colors.tabbg,'parent',handles.bgtabaxes,'edgecolor','none','tag','tabbg','hittest','off')

line([0 1],[0 0],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.tabaxes,'hittest','off');%bottom
line([0 1],[1 1],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.tabaxes,'hittest','off');%top
line([1 1],[0 1],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.tabaxes,'hittest','off');%right
line([0 1],[0 0],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.bgtabaxes,'hittest','off');%subbottom
line([1 1],[0 1],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.bgtabaxes,'hittest','off');%subright

handles.controltitle=text(.15,1,'Control Panel','backgroundcolor','none','color',handles.colors.title,'fontsize',20*handles.prefs.fonts.titlescale,...
    'horizontalalignment','left','verticalalignment','baseline','parent',handles.tabaxes,...
    'fontname',handles.prefs.fonts.title,'margin',1,'handlevisibility','off','tag','controltitle','hittest','off');

if handles.height<900
    set(handles.controltitle,'visible','off')
end

%draw tabs:
handles.fronttabprofile=[1.1 .7 0.9 1.1 1];
handles.backtabprofile=[0.85 1.1 1.1 .65 .7 0.8,];

%front row:
[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,1,1);

handles.fronttabbg=patch(x,y,handles.colors.tabbg,'parent',handles.fronttabaxes,'edgecolor','none','tag','tabbg','hittest','off');
handles.fronttablines=line(x,y,'parent',handles.fronttabaxes,'linewidth',tabwidth,'color',handles.colors.tabfg,'hittest','off');

tabfs=18*((handles.prefs.tabscale-1)*2+1);

%put text in tabs:
%front row
text(.15,v(1)+0.03,'Manage','fontname',raveprefs.fonts.tab,'color',handles.colors.text,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@managetab,'interruptible','off','tag','managetab')
text(.17,v(2)+0.02,'View','fontname',raveprefs.fonts.tab,'color',handles.colors.text,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@viewtab,'hittest','off','interruptible','off','tag','viewtab')
text(.17,v(3)+0.01,'Format','fontname',raveprefs.fonts.tab,'color',handles.colors.text,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@formattab,'hittest','off','interruptible','off','tag','formattab')
text(.17,v(4)+0.01,'Specialize','fontname',raveprefs.fonts.tab,'color',handles.colors.text,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@exploretab,'hittest','off','interruptible','off','tag','exploretab')
text(.17,v(5)+0.02,'Export','fontname',raveprefs.fonts.tab,'color',handles.colors.text,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@exporttab,'interruptible','off','tag','exporttab')
%back row:

[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0);

handles.backtabbg=patch(x,y,handles.colors.tabbg,'parent',handles.backtabaxes,'edgecolor','none','tag','tabbg','hittest','off');
handles.backtablines=line(x,y,'parent',handles.backtabaxes,'linewidth',tabwidth,'color',handles.colors.tabfg,'hittest','off');

%put text in tabs:

text(.17,v(1)+0.01,'Model','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@modeltab,'interruptible','off','tag','modeltab')
text(.17,v(2),'Constrain','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@constraintab,'interruptible','off','tag','constraintab')
text(.17,v(3)+0.01,'Optimize','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@optimizetab,'interruptible','off','tag','optimizetab')
text(.17,v(4)+0.01,'Row ','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@rowtab,'interruptible','off','tag','rowtab')
text(.17,v(5)+0.01,'Draw','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@drawtab,'interruptible','off','tag','drawtab')
text(.17,v(6)+0.01,'Setup','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@setuptab,'interruptible','off','tag','setuptab')

handles.alltabs=findobj([handles.fronttabaxes,handles.backtabaxes],'type','text');

%make tabs unclickable until the user loads a data set (managedatagui turns
%them on)
set(handles.alltabs,'hittest','off');

handles.fronttabsfront=1; %1 if fronttabs in front, 0 if backtabs in front


%% set up non-tabbed ui controls:
handles.stopbutton=uicontrol('style','pushbutton','position',[handles.width-105 handles.height-32 100 32],...
    'string','debug','fontname','arial narrow','fontsize',20,'foregroundcolor',handles.colors.title,...
    'callback',@stopbutton,'backgroundcolor',handles.colors.buttonbg,'parent',handles.fig);

%an invisible object that can be used to remove the focus from visible
%uicontrols:
handles.dummyobj=uicontrol('style','pushbutton','position',[-100 -100 50 50],'parent',handles.fig);

if handles.height>=900
    gap=9;
else
    gap=3;
end

handles.tablemodebutton=uicontrol('style','pushbutton','position',[18 handles.height-35-pushdown 73 32],...
    'string','Table','callback',@tablemodebutton_callback,'cdata',[],'tooltipstring','View Data Table','parent',handles.fig,...
    'fontsize',12,'tag','tablemodebutton');
handles.newbutton=uicontrol('style','pushbutton','position',[18 handles.height-66-gap-pushdown 32 32],...
    'string','','callback',@newbutton,'cdata',imread('newgraph.bmp'),'tooltipstring','Create New Plot','parent',handles.fig);
handles.zoombutton=uicontrol('style','togglebutton','position',[59 handles.height-66-gap-pushdown 32 32],...
    'string','','callback','','cdata',imread('zoom.bmp'),'tooltipstring',' Zoom ','parent',handles.fig);
handles.showgridbutton=uicontrol('style','togglebutton','position',[18 handles.height-128-3*gap-pushdown 32 32],...
    'string','','callback',@showgridbutton,'value',1,...
    'tooltipstring','Show/Hide Gridlines','cdata',imread('showgrid.bmp'),'parent',handles.fig,...
    'backgroundcolor',handles.colors.togglebuttonbg,'visible','off');
handles.openglbutton=uicontrol('style','pushbutton','position',[59 handles.height-97-2*gap-pushdown 32 32],...
    'string','','callback',@changerenderer_callback,'tooltipstring','Change Renderer','parent',handles.fig,...
    'cdata',imread('raveclock.bmp'),'backgroundcolor',handles.colors.togglebuttonbg);
handles.hideshowbutton=uicontrol('style','pushbutton','position',[18 handles.height-97-2*gap-pushdown 32 32],...
    'string','','callback',@ravehidetabs,'cdata',imread('hideshow.bmp'),...
    'tooltipstring','Hide/Show Control Panel','parent',handles.fig);

if handles.height<=776
    set(handles.tablemodebutton,'position',[20 660 73 32])
    set(handles.newbutton,'position',[85+20 660 32 32]);
    set(handles.zoombutton,'position',[85+40+20 660 32 32]);
    set(handles.showgridbutton,'position',[85+2*40+20 660 32 32]);
    set(handles.openglbutton,'position',[85+3*40+20 660 32 32]);
    set(handles.hideshowbutton,'position',[85+4*40+20 660 32 32]);
end

%% initialize handles.tabobjects so tab changefcn has something to delete
dummyobj=uicontrol('position',[-200 -200 10 10],'style','pushbutton','parent',handles.fig);
handles.tabobjects=dummyobj;
handles.tabobjects=[];

%% load sound effects:
try %64 bit:
    [Y,Fs]=wavread('ravefinish.wav');
    handles.sounds.finish=audioplayer(Y,Fs,24);
catch %32 bit:
    handles.sounds.finish=wavread('ravefinish.wav');
end


%% set up close request function:
set(handles.fig,'closerequestfcn',@ravequitfunction)

%default window keypress fcn:
set(handles.fig,'windowkeypressfcn',@ravedefaultkeypressfcn)
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)

%mousemotion timer:
handles.mousetimer=timer('TimerFcn',[],'startdelay',handles.prefs.mouseoverdelay,'executionmode','singleshot');


%% show splash screen
drawnow;
pause(0.2); %need to pause because on vista figure window "fades in" and screws up getscreen.

[splash,map,splashalpha]=imread('ravesplashlite.png','png');

splashposition=floor([(outerwidth-size(splash,2))/2+28 (outerheight-size(splash,1))/2 size(splash,2) size(splash,1)]);
splashax=axes('units','pixels','position',splashposition,'parent',handles.fig,'color','none','visible','off');
%get background pixels:
splashback=getscreen(splashax);
splashback=splashback.cdata;

splashback=double(splashback);
splash=double(splash);
splashalpha=double(splashalpha);

splashback=bsxfun(@times,splashback,(255-splashalpha))./255;
splash=bsxfun(@times,splash,(splashalpha))./255;

spah=splash+splashback;

image(uint8(spah),'parent',splashax);
set(splashax,'visible','off')

text(635,190,['Version ',ravever],...
    'parent',splashax,'fontname','times','fontsize',12,'hor','right','fontangle','italic','fontweight','bold')
%Notice to users -- do not modify/remove this line:
text(635,330,{'Created by Matthew Daskilewicz';'Copyright 2012';'Georgia Tech Research Corporation';'All Rights Reserved'},...
    'parent',splashax,'fontname','times','fontsize',12,'hor','right','fontangle','italic')
%Notice to users -- do not modify/remove this line:
text(635,390,{'Distributed under GNU General Public License Ver. 2'},...
    'parent',splashax,'fontname','times','fontsize',11,'hor','right','fontangle','normal')

%% wrap it up...
%Choose default command line output for rave
handles.output = handles.fig;
% Update handles structure
guidata(handles.fig, handles);

%% create manage tab
managetab(handles.fig,[]);
handles=guidata(handles.fig);

%% finish up...
set(handles.fig,'visible','on','windowscrollwheelfcn',@ravewheelscroll,'resizefcn',@raveresizefcn) %note, dont set resizefcn sooner or it will try to run before all objects have been created.
handles.axiscanvas=findjobj(fff,'class','AxisCanvas'); %<-- used to provide a graphics context for determining string wrapping

if handles.prefs.isresizable
    set(handles.fig,'resize','on')
end


% Update handles structure
guidata(handles.fig, handles);

%load context menus:
ravemakecontextmenus(handles.fig,[]);
handles=guidata(handles.fig);

%% do java stuff:
%add rave icon:
pause(2); %wait to make sure java handles are valid.

handles=guidata(handles.fig);
handles.jvscroll=findjobj(handles.vscroll);
handles.jhscroll=findjobj(handles.hscroll);
try %#ok<TRYNC>
    handles.jvscroll.MouseDraggedCallback = {@vertscroll,handles.fig};
    handles.jhscroll.MouseDraggedCallback = {@horizscroll,handles.fig};
end

guidata(handles.fig,handles);
delete(splashax)
if ismember('Magneto',listfonts);
    set(handles.infobar,'string',['Welcome to the \fontsize{18}\fontname{magneto}\color[rgb]{',num2str(handles.colors.text),'}rave\fontsize{14}\fontname{' handles.prefs.fonts.tab '}\color[rgb]{',num2str(handles.colors.text),'}, click "Manage Data Sets" to begin.']);
else
    set(handles.infobar,'string','Welcome to the Rave, click "Manage Data Sets" to begin.');
end

drawnow;

%% check for updates:
if raveprefs.checkforupdates
    ravecheckforupdates;
end



%% callbacks...
function stopbutton(hObject,eventdata)  %#ok<INUSD>
handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
keyboard


function newbutton(hObject,eventdata)
handles=guidata(hObject);
uicontrol(handles.dummyobj);

ravenewgraph(handles.fig,[])

%KEEP THIS FUNCTION HERE FOR BACKWARD COMPATIBILITY (even though the button
%has been moved to the setuptab)
function showgridbutton(hObject,eventdata)
handles=guidata(hObject);
uicontrol(handles.dummyobj);

%do not alter the currently selected graph, or any annotation class graph.
changethese=false(size(handles.allgraphs));

for c=1:numel(changethese)
    if ishandle(handles.allgraphs(c)) && handles.allgraphs(c)~=handles.activegraph && ~strcmp(handles.graphinfo.(get(handles.allgraphs(c),'userdata')).class,'annotation')
        changethese(c)=true;
    end
end

if get(hObject,'value')
    if handles.prefs.gridmode==1
        set(handles.lgrid,'visible','on')
    elseif handles.prefs.gridmode==2
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','on')
    else
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','on')
        set(handles.sgrid,'visible','on')
    end
    
    set(handles.allblockers(changethese),'edgecolor',handles.colors.wsgrid)
    
else
    set(handles.sgrid,'visible','off')
    set(handles.mgrid,'visible','off')
    set(handles.lgrid,'visible','off')
    set(handles.allblockers(changethese),'edgecolor','none')
    
end

function changerenderer_callback(src,ev) %#ok<INUSD>
handles=guidata(src);

rend=get(handles.fig,'renderer');

switch lower(rend)
    case 'painters'
        set(handles.fig,'renderer','zbuffer');
         set(handles.infobar,'string','Renderer set to ZBuffer')
    case 'zbuffer'
        set(handles.fig,'renderer','opengl');
        set(handles.infobar,'string','Renderer set to Open GL')
    case 'opengl'
        set(handles.fig,'renderer','painters');
        set(handles.infobar,'string','Renderer set to Painters')
end
       
drawnow;

%update java graphics stuff:
handles.axiscanvas=findjobj(handles.fig,'class','AxisCanvas'); 
guidata(handles.fig,handles);
clear functions %<--I dont know which function is causing problems, but something inmem is causing nullpointerexceptions 
%in superiorlinewrap after the renderer is changed from opengl to painters. 


function tablemodebutton_callback(src,ev)
handles=guidata(src);

if handles.tablemode %go from table to workspace
    handles.activesheet=handles.lastsheet;
    handles.lastsheet=0;
    guidata(handles.fig,handles);
    set(handles.pageedit,'string',num2str(handles.activesheet))
    showhideworkspace(handles.fig,[]);
else %go from workspace to table:
    handles.lastsheet=handles.activesheet;
    handles.activesheet=0;
    guidata(handles.fig,handles);
    set(handles.pageedit,'string',num2str(handles.activesheet))
    showhideworkspace(handles.fig,[]);
end







