function h=ravedrawconstraintcore(src,ai,C,thisconstraint,xcol,ycol,width)
%A subfunction for ravedrawconstraint that does the actual constraint plotting.
%Inputs: src is the axes in which to plot the constraint, ai is the activeindex of the graph containing src,
%C is the constraint matrix returned by contourc
%thisconstraint is the index of this constraint
%xcol is src's x axis variable
%ycol is src's y axis variable
%width is width to draw line
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


if isempty(C);return;end %this constraint is off the edge of the plot

handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');
d=udata.view.dataset;
a=udata.view.analysis;

h=[]; %initialize

%first column of contourz=(0;npoints), remaining cols= x/y data.
allxdata=C(1,1:end);
allydata=C(2,1:end);

%line(xdata,ydata,'parent',handles.activegraph,'buttondownfcn','delete(gco)','color',[1 0 0],'linewidth',2)
%set(handles.infobar,'string','Double-click on constraints to edit appearance')

%NOTE: contourc may return multiple contours even though we're only plotting one. So
%need to check for that and plot each individually

cumindex=1; %cumulative index
while cumindex<length(allxdata)
    index=allydata(cumindex);
    %the count of how many points are in this contour
    xdata=allxdata(cumindex+1:cumindex+index);
    ydata=allydata(cumindex+1:cumindex+index);
    cumindex=cumindex+index+1;

    %determine if infeasible space is to left or right of this curve. hashes
    %are drawn to the right, so if infeasible space is on the left need to
    %flipud xdata/ydata:

    %take a test point a small distance to the right of the center of the line:

    midpoint=round(length(xdata)/2);

    %if something went wrong, quit:
    if numel(xdata)<=2;continue;end

    %rotate the vector [testx testy] 90 degrees to the right to get a point on
    %the rhs of it.
    xlim=get(src,'xlim');
    ylim=get(src,'ylim');

    dx=xdata(midpoint+1)-xdata(midpoint);
    dy=ydata(midpoint+1)-ydata(midpoint);
    
    if dx==0 && dy==0 %crazy as it seems, this does happen....
        if numel(xdata)==1
            %do nothing
            
        elseif numel(xdata)==2
            dx=xdata(2)-xdata(1);
            dy=ydata(2)-ydata(1);
        else
            midpoint=midpoint+1;
            dx=xdata(midpoint+1)-xdata(midpoint);
            dy=ydata(midpoint+1)-ydata(midpoint); 
        end
    end
    
    testy=ydata(midpoint)-sign(dx)*.01*diff(ylim);
    testx=xdata(midpoint)+sign(dy)*.01*diff(xlim);

    indata=handles.xdials{d}{a};
    indata(xcol)=testx;
    indata(ycol)=testy;

    ignore=false(1,size(handles.alldata{d},2));
    ignore([xcol ycol])=true;

    testz=raveevaluateconstraint(handles,d,a,thisconstraint,indata,ignore);

    if testz<0 %this point is feasible, so draw hashes on other side.
        xdata=fliplr(xdata);
        ydata=fliplr(ydata);
    end

    %plot this contour:
    set(src,'xlimmode','manual','ylimmode','manual');
    xrange=get(src,'xlim');
    yrange=get(src,'ylim');
    %draw the line:
    ar=(yrange(2)-yrange(1))/(xrange(2)-xrange(1));
    spc=0.02*(xrange(2)-xrange(1));
    len=1.4;

    try %#ok<TRYNC> %this will give an error if contour is just a single point, and i dont feel like fixing it.
        h=ravehatchedline(xdata,ydata,45*pi/180,ar,spc,len,...
            'parent',src,'buttondownfcn',@lineobject_callback,'color',...
            handles.constraintcolors{d}{a}{thisconstraint},'linewidth',width,'tag','constrainthatches');

        constraintdata.ai=ai;
        constraintdata.handles=h;
        constraintdata.delete=1;
        constraintdata.string=handles.constrainttext{d}{a}{thisconstraint};
        constraintdata.constraintnumber=thisconstraint;

        set(h,'userdata',constraintdata) %user data for each line of constraint is handles to ALL lines in this constraint.
        set(h(1),'tag','constraint') %only  set tag for one line of this group, so that findobj('tag','constraint') isnt redundant.
    catch %#ok<CTCH>
        set(handles.infobar,'string','Selected constraint can''t be drawn on this graph.')
    end
end
