function askbox=raveeditthemefontsgui(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

if ~isempty(src);
    handles=guidata(src);
    screensize=get(handles.fig,'position');
    load raveprefs

else
    screensize=get(0,'monitorposition');
    screensize=screensize(1,:);
    load raveprefstemp
end

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-580 screensize(2)+screensize(4)/2-225 1160 450],...
    'numbertitle','off','menubar','none','name','Change RAVE fonts','color',raveprefs.colors.frame,...
    'resize','off','windowstyle','normal');

if ~isempty(src);
    uicontrol('style','text','string','For title and tabs, select a font then use slider to scale font so that the text fits (mostly) within the bounds of the rectangles.',...
        'position',[30 400 640 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left');
else
    uicontrol('style','text','string','Select fonts used by RAVE interface (you can change these later from within RAVE). Select a font then use slider to scale font so that the text fits (mostly) within the bounds of the rectangles. Click "Next" when you are finished.',...
        'position',[20 400 660 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left');
end


%guidata(askbox,handles);

R=raveprefs.colors.frame(1);
G=raveprefs.colors.frame(2);
B=raveprefs.colors.frame(3);

fontz=listfonts;

[junk idx]=ismember(lower(raveprefs.fonts.title),lower(fontz));

if idx==0
[junk idx]=ismember('arial',lower(fontz));
end

if idx==0
    idx=1;
end


a1=axes('units','pixels','parent',askbox,'position',[40 120 200 300],'ylim',[0 300],'xlim',[0 200],'visible','off','color','none');
l1=uicontrol('style','popup','pos',[50 90 180 20],'string',fontz,'back',[1 1 1],'callback',@titlefont_Callback,'value',idx,'tag','titlemenu');
[jslider h1]=javacomponent('javax.swing.JSlider',[50 50 180 30]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(0)
jslider.setMinimum(12)
jslider.setMaximum(40)
jslider.setMajorTickSpacing(2);
jslider.setValue(raveprefs.fonts.titlescale*20);
set(jslider,'background',java.awt.Color(R,G,B))
set(jslider,'foreground',java.awt.Color.black)
set(jslider,'StateChangedCallback',{@titlescale_Callback,askbox});
set(h1,'tag','titlescale');

r1(1)=rectangle('position',[0 18 117 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r1(2)=rectangle('position',[0 68 160 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r1(3)=rectangle('position',[0 118 81 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r1(4)=rectangle('position',[0 168 178 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r1(5)=rectangle('position',[0 218 128 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);

t1(1)=text(0,25,'Legends','parent',a1,'fontname',fontz{idx},'fontsize',18*raveprefs.fonts.titlescale,'hor','left','tag','t1');
t1(2)=text(0,75,'Data Series','parent',a1,'fontname',fontz{idx},'fontsize',18*raveprefs.fonts.titlescale,'hor','left','tag','t1');
t1(3)=text(0,125,'Graph','parent',a1,'fontname',fontz{idx},'fontsize',18*raveprefs.fonts.titlescale,'hor','left','tag','t1');
t1(4)=text(0,175,'Control Panel','parent',a1,'fontname',fontz{idx},'fontsize',18*raveprefs.fonts.titlescale,'hor','left','tag','t1');
t1(5)=text(0,225,'Navigator','parent',a1,'fontname',fontz{idx},'fontsize',18*raveprefs.fonts.titlescale,'hor','left','tag','t1');

text(0,265,'Title Font:','parent',a1,'fontsize',12,'hor','left','fontweight','bold');


%tab font:

[junk idx]=ismember(lower(raveprefs.fonts.tab),lower(fontz));

if idx==0
[junk idx]=ismember('arial',lower(fontz));
end

if idx==0
    idx=1;
end

a2=axes('units','pixels','parent',askbox,'position',[220+40 120 200 300],'ylim',[0 300],'xlim',[0 200],'visible','off','color','none');
l1=uicontrol('style','popup','pos',[220+50 90 180 20],'string',fontz,'back',[1 1 1],'callback',@tabfont_Callback,'value',idx,'tag','tabmenu');
[jslider h2]=javacomponent('javax.swing.JSlider',[220+50 50 180 30]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(0)
jslider.setMinimum(12)
jslider.setMaximum(40)
jslider.setMajorTickSpacing(2);
jslider.setValue(raveprefs.fonts.tabscale*20);
set(jslider,'background',java.awt.Color(R,G,B))
set(jslider,'foreground',java.awt.Color.black)
set(jslider,'StateChangedCallback',{@tabscale_Callback,askbox});
set(h2,'tag','tabscale');

r2(1)=rectangle('position',[0 18 85 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r2(2)=rectangle('position',[0 68 50 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r2(3)=rectangle('position',[0 118 110 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r2(4)=rectangle('position',[0 168 80 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r2(5)=rectangle('position',[0 218 98 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);

t2(1)=text(0,27,'Manage','parent',a2,'fontname',fontz{idx},'fontsize',18*raveprefs.fonts.tabscale,'hor','left','tag','t2');
t2(2)=text(0,77,'View','parent',a2,'fontname',fontz{idx},'fontsize',18*raveprefs.fonts.tabscale,'hor','left','tag','t2');
t2(3)=text(0,127,'Constrain','parent',a2,'fontname',fontz{idx},'fontsize',18*raveprefs.fonts.tabscale,'hor','left','tag','t2');
t2(4)=text(0,177,'Explore','parent',a2,'fontname',fontz{idx},'fontsize',18*raveprefs.fonts.tabscale,'hor','left','tag','t2');
t2(5)=text(0,227,'Optimize','parent',a2,'fontname',fontz{idx},'fontsize',18*raveprefs.fonts.tabscale,'hor','left','tag','t2');

text(0,265,'Tab Font:','parent',a2,'fontsize',12,'hor','left','fontweight','bold');

if ~isempty(src)
    cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
    donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Done','fontsize',12,'callback',@donebutton_Callback);
else
    cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton2_Callback);
    donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@donebutton2_Callback);
end


%axes font:
[junk idx]=ismember(lower(raveprefs.fonts.graph),lower(fontz));

if idx==0
[junk idx]=ismember('arial',lower(fontz));
end

if idx==0
    idx=1;
end

a3=axes('units','pixels','parent',askbox,'position',[440+40 120 200 250],'ylim',[0 250],'xlim',[0 200],...
    'visible','on','color',[1 1 1],'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1],'ytick',[],'xtick',[]);
l3=uicontrol('style','popup','pos',[440+50 90 180 20],'string',fontz,'back',[1 1 1],'callback',@graphfont_Callback,'value',idx,'tag','graphmenu');
[jslider h3]=javacomponent('javax.swing.JSlider',[440+50 50 145 30]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(0)
jslider.setMinimum(6)
jslider.setMaximum(32)
jslider.setMajorTickSpacing(2);
jslider.setValue(raveprefs.fonts.graphsize);
set(jslider,'background',java.awt.Color(R,G,B))
set(jslider,'foreground',java.awt.Color.black)
set(jslider,'StateChangedCallback',{@graphsize_Callback,askbox});
set(h3,'tag','graphsize');

e3=uicontrol('style','edit','pos',[440+50+150 60 30 20],'string',num2str(raveprefs.fonts.graphsize),'back',[1 1 1],'callback',@graphsizeedit_Callback,'tag','graphsizeedit');

t3(1)=text(0,27,'NOPQRSTUVWXYZ','parent',a3,'fontname',fontz{idx},'fontsize',raveprefs.fonts.graphsize,'hor','left','tag','t3');
t3(2)=text(0,77,'ABCDEFGHIJKLM','parent',a3,'fontname',fontz{idx},'fontsize',raveprefs.fonts.graphsize,'hor','left','tag','t3');
t3(3)=text(0,127,'nopqrstuvwxyz','parent',a3,'fontname',fontz{idx},'fontsize',raveprefs.fonts.graphsize,'hor','left','tag','t3');
t3(4)=text(0,177,'abcdefghijklm','parent',a3,'fontname',fontz{idx},'fontsize',raveprefs.fonts.graphsize,'hor','left','tag','t3');
t3(5)=text(0,227,'0123456789','parent',a3,'fontname',fontz{idx},'fontsize',raveprefs.fonts.graphsize,'hor','left','tag','t3');

text(0,265,'Graph Font:','parent',a3,'fontsize',12,'hor','left','fontweight','bold');

%tick font:
[junk idx]=ismember(lower(raveprefs.fonts.tick),lower(fontz));

if idx==0
[junk idx]=ismember('arial',lower(fontz));
end

if idx==0
    idx=1;
end

a4=axes('units','pixels','parent',askbox,'position',[660+40 120 200 250],'ylim',[0 250],'xlim',[0 200],...
    'visible','on','color',[1 1 1],'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1],'ytick',[],'xtick',[]);
l4=uicontrol('style','popup','pos',[660+50 90 180 20],'string',fontz,'back',[1 1 1],'callback',@tickfont_Callback,'value',idx,'tag','tickmenu');
[jslider h4]=javacomponent('javax.swing.JSlider',[660+50 50 145 30]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(0)
jslider.setMinimum(6)
jslider.setMaximum(32)
jslider.setMajorTickSpacing(2);
jslider.setValue(raveprefs.fonts.ticksize);
set(jslider,'background',java.awt.Color(R,G,B))
set(jslider,'foreground',java.awt.Color.black)
set(jslider,'StateChangedCallback',{@ticksize_Callback,askbox});
set(h4,'tag','ticksize');

e4=uicontrol('style','edit','pos',[660+50+150 60 30 20],'string',num2str(raveprefs.fonts.ticksize),'back',[1 1 1],'callback',@ticksizeedit_Callback,'tag','ticksizeedit');

t4(1)=text(0,27,'NOPQRSTUVWXYZ','parent',a4,'fontname',fontz{idx},'fontsize',raveprefs.fonts.ticksize,'hor','left','tag','t4');
t4(2)=text(0,77,'ABCDEFGHIJKLM','parent',a4,'fontname',fontz{idx},'fontsize',raveprefs.fonts.ticksize,'hor','left','tag','t4');
t4(3)=text(0,127,'nopqrstuvwxyz','parent',a4,'fontname',fontz{idx},'fontsize',raveprefs.fonts.ticksize,'hor','left','tag','t4');
t4(4)=text(0,177,'abcdefghijklm','parent',a4,'fontname',fontz{idx},'fontsize',raveprefs.fonts.ticksize,'hor','left','tag','t4');
t4(5)=text(0,227,'0123456789','parent',a4,'fontname',fontz{idx},'fontsize',raveprefs.fonts.ticksize,'hor','left','tag','t4');

text(0,265,'Graph Tick Label Font:','parent',a4,'fontsize',12,'hor','left','fontweight','bold');

%tick font:
[junk idx]=ismember(lower(raveprefs.fonts.graphtitle),lower(fontz));

if idx==0
[junk idx]=ismember('arial',lower(fontz));
end

if idx==0
    idx=1;
end

a5=axes('units','pixels','parent',askbox,'position',[880+40 120 200 250],'ylim',[0 250],'xlim',[0 200],...
    'visible','on','color',[1 1 1],'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1],'ytick',[],'xtick',[]);
l5=uicontrol('style','popup','pos',[880+50 90 180 20],'string',fontz,'back',[1 1 1],'callback',@graphtitlefont_Callback,'value',idx,'tag','graphtitlemenu');
[jslider h5]=javacomponent('javax.swing.JSlider',[880+50 50 145 30]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(0)
jslider.setMinimum(6)
jslider.setMaximum(32)
jslider.setMajorTickSpacing(2);
jslider.setValue(raveprefs.fonts.graphtitlesize);
set(jslider,'background',java.awt.Color(R,G,B))
set(jslider,'foreground',java.awt.Color.black)
set(jslider,'StateChangedCallback',{@graphtitlesize_Callback,askbox});
set(h5,'tag','graphtitlesize');

e5=uicontrol('style','edit','pos',[880+50+150 60 30 20],'string',num2str(raveprefs.fonts.graphtitlesize),'back',[1 1 1],'callback',@graphtitlesizeedit_Callback,'tag','graphtitlesizeedit');

t5(1)=text(0,27,'NOPQRSTUVWXYZ','parent',a5,'fontname',fontz{idx},'fontsize',raveprefs.fonts.graphtitlesize,'hor','left','tag','t5');
t5(2)=text(0,77,'ABCDEFGHIJKLM','parent',a5,'fontname',fontz{idx},'fontsize',raveprefs.fonts.graphtitlesize,'hor','left','tag','t5');
t5(3)=text(0,127,'nopqrstuvwxyz','parent',a5,'fontname',fontz{idx},'fontsize',raveprefs.fonts.graphtitlesize,'hor','left','tag','t5');
t5(4)=text(0,177,'abcdefghijklm','parent',a5,'fontname',fontz{idx},'fontsize',raveprefs.fonts.graphtitlesize,'hor','left','tag','t5');
t5(5)=text(0,227,'0123456789','parent',a5,'fontname',fontz{idx},'fontsize',raveprefs.fonts.graphtitlesize,'hor','left','tag','t5');

text(0,265,'Graph Title Font:','parent',a5,'fontsize',12,'hor','left','fontweight','bold');




if ~isempty(src)
    cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
    donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Done','fontsize',12,'callback',@donebutton_Callback);
else
    cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton2_Callback);
    donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@donebutton2_Callback);
end




function titlescale_Callback(jsrc,jev,src)
moving = get(jsrc,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end
newscale=jsrc.getValue./20;
t1=findobj(gcf,'tag','t1');
set(t1,'fontsize',18*newscale);


function titlefont_Callback(src,ev)
fontz=get(src,'string');
newfont=fontz{get(src,'value')};
t1=findobj(gcf,'tag','t1');
set(t1,'fontname',newfont);


function tabscale_Callback(jsrc,jev,src)

moving = get(jsrc,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end
newscale=jsrc.getValue./20;
t2=findobj(gcf,'tag','t2');
set(t2,'fontsize',18*newscale);


function tabfont_Callback(src,ev)
fontz=get(src,'string');
newfont=fontz{get(src,'value')};
t2=findobj(gcf,'tag','t2');
set(t2,'fontname',newfont);


function graphsize_Callback(jsrc,jev,src)
moving = get(jsrc,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end
newsize=jsrc.getValue;
t3=findobj(gcf,'tag','t3');
e3=findobj(gcf,'tag','graphsizeedit');
set(t3,'fontsize',newsize);
set(e3,'string',num2str(newsize));

function graphfont_Callback(src,ev)
fontz=get(src,'string');
newfont=fontz{get(src,'value')};
t3=findobj(gcf,'tag','t3');
set(t3,'fontname',newfont);

function graphsizeedit_Callback(src,ev)
newsize=str2double(get(src,'string'));
h3=findobj(gcf,'tag','graphsize');
j3=get(h3,'javapeer');
j3.setValue(newsize);


function ticksize_Callback(jsrc,jev,src)
moving = get(jsrc,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end
newsize=jsrc.getValue;
t4=findobj(gcf,'tag','t4');
e4=findobj(gcf,'tag','ticksizeedit');
set(t4,'fontsize',newsize);
set(e4,'string',num2str(newsize));

function tickfont_Callback(src,ev)
fontz=get(src,'string');
newfont=fontz{get(src,'value')};
t4=findobj(gcf,'tag','t4');
set(t4,'fontname',newfont);

function ticksizeedit_Callback(src,ev)
newsize=str2double(get(src,'string'));
h4=findobj(gcf,'tag','ticksize');
j4=get(h4,'javapeer');
j4.setValue(newsize);


function graphtitlesize_Callback(jsrc,jev,src)
moving = get(jsrc,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end
newsize=jsrc.getValue;
t5=findobj(gcf,'tag','t5');
e5=findobj(gcf,'tag','graphtitlesizeedit');
set(t5,'fontsize',newsize);
set(e5,'string',num2str(newsize));

function graphtitlefont_Callback(src,ev)
fontz=get(src,'string');
newfont=fontz{get(src,'value')};
t5=findobj(gcf,'tag','t5');
set(t5,'fontname',newfont);

function graphtitlesizeedit_Callback(src,ev)
newsize=str2double(get(src,'string'));
h5=findobj(gcf,'tag','graphtitlesize');
j5=get(h5,'javapeer');
j5.setValue(newsize);



function cancelbutton_Callback(src,ev)
close(gcf);

function donebutton_Callback(src,ev)

tabscale=findobj(gcf,'tag','tabscale');
titlescale=findobj(gcf,'tag','titlescale');
graphsize=findobj(gcf,'tag','graphsize');
ticksize=findobj(gcf,'tag','ticksize');
graphtitlesize=findobj(gcf,'tag','graphtitlesize');


tabscale=get(tabscale,'javapeer');
titlescale=get(titlescale,'javapeer');
graphsize=get(graphsize,'javapeer');
ticksize=get(ticksize,'javapeer');
graphtitlesize=get(graphtitlesize,'javapeer');


tabmenu=findobj(gcf,'tag','tabmenu');
titlemenu=findobj(gcf,'tag','titlemenu');
graphmenu=findobj(gcf,'tag','graphmenu');
tickmenu=findobj(gcf,'tag','tickmenu');
graphtitlemenu=findobj(gcf,'tag','graphtitlemenu');

prefspath=which('raveprefs.mat');
load raveprefs
fontz=listfonts;
raveprefs.fonts.tab=fontz{get(tabmenu,'value')};
raveprefs.fonts.tabscale=tabscale.getValue./20;
raveprefs.fonts.title=fontz{get(titlemenu,'value')};
raveprefs.fonts.titlescale=titlescale.getValue./20;
raveprefs.fonts.graph=fontz{get(graphmenu,'value')};
raveprefs.fonts.graphsize=graphsize.getValue;
raveprefs.fonts.tick=fontz{get(tickmenu,'value')};
raveprefs.fonts.ticksize=ticksize.getValue;
raveprefs.fonts.graphtitle=fontz{get(graphtitlemenu,'value')};
raveprefs.fonts.graphtitlesize=graphtitlesize.getValue;

%calculate characterwidths:
% 
% widths=characterwidths('fontname',raveprefs.fonts.graph,'fontsize',raveprefs.fonts.graphsize);
% widths(32)=5; %space
% widths=widths+0.5;% a little buffer
% 
% if raveprefs.fonts.graphsize<8
%     widths=widths+0.5;
% elseif raveprefs.fonts.graphsize<=10
%     widths=widths+1;
% elseif raveprefs.fonts.graphsize<=12
%     widths=widths+1.5;
% elseif raveprefs.fonts.graphsize<=14
%     widths=widths+2;
% else
%     widths=widths+(raveprefs.fonts.graphsize-10)/2;
% end

%raveprefs.fonts.graphwidths=widths;
save(prefspath,'raveprefs');

close(gcbf);

function cancelbutton2_Callback(src,ev)
close(gcfb);

function donebutton2_Callback(src,ev)

tabscale=findobj(gcbf,'tag','tabscale');
titlescale=findobj(gcbf,'tag','titlescale');
graphsize=findobj(gcbf,'tag','graphsize');

tabscale=get(tabscale,'javapeer');
titlescale=get(titlescale,'javapeer');
graphsize=get(graphsize,'javapeer');

tabmenu=findobj(gcbf,'tag','tabmenu');
titlemenu=findobj(gcbf,'tag','titlemenu');
graphmenu=findobj(gcbf,'tag','graphmenu');

prefspath=which('raveprefstemp.mat');
load raveprefstemp
fontz=listfonts;
raveprefs.fonts.tab=fontz{get(tabmenu,'value')};
raveprefs.fonts.tabscale=tabscale.getValue./20;
raveprefs.fonts.title=fontz{get(titlemenu,'value')};
raveprefs.fonts.titlescale=titlescale.getValue./20;
raveprefs.fonts.graph=fontz{get(graphmenu,'value')};
raveprefs.fonts.graphsize=graphsize.getValue;

widths=characterwidths('fontname',raveprefs.fonts.graph,'fontsize',raveprefs.fonts.graphsize);
%adjust calculated widths for inter-letter spacing:

widths(32)=5; %space
widths=widths+0.5;% a little buffer

if raveprefs.fonts.graphsize<8
    widths=widths+0.5;
elseif raveprefs.fonts.graphsize<=10
    widths=widths+1;
elseif raveprefs.fonts.graphsize<=12
    widths=widths+1.5;
elseif raveprefs.fonts.graphsize<=14
    widths=widths+2;
else
    widths=widths+(raveprefs.fonts.graphsize-10)/2;
end

%raveprefs.fonts.graphwidths=widths;


save(prefspath,'raveprefs');

close(gcbf);




