function raveexportgraph(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

set(handles.infobar,'string','Preparing image for export, please wait...');
drawnow;
set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

linesmooth=findobj(handles.tabobjects,'tag','linesmooth');
mayasmooth=findobj(handles.tabobjects,'tag','mayasmooth');
allinone=findobj(handles.tabobjects,'tag','allinone');

if ~get(allinone,'value') %export single graph
    ai=handles.activeindex;
    position=get(handles.allblockers(ai),'position'); 

else %export all graphs in the roi.
    position=get(handles.roi(1),'position'); 
    if isempty(handles.roi);return;end
    ai=[];
    roipos=get(handles.roi(1),'pos');
    for a=1:numel(handles.allgraphs);
        if ~ishandle(handles.allgraphs(a));continue;end
        if handles.sheet(a)~=handles.activesheet;continue;end
        thispos=get(handles.allgraphs(a),'position');
        if thispos(1)>=roipos(1) && thispos(1) <= roipos(1)+roipos(3) && thispos(2)>=roipos(2) && thispos(2)<=roipos(2)+roipos(4)
            ai=union(ai,a);
        end
    end
end

numgraphs=numel(ai);

for f=1:numgraphs
    if ~isempty(handles.subax{ai(f)})
        numsubgraphs(f)=numel(handles.subax{ai(f)}); %#ok<AGROW>
    else
        numsubgraphs(f)=0; %#ok<AGROW>
    end
end

%make a window for each graph:
savewindow=figure('name','Saving... DO NOT CLOSE THIS WINDOW','units','pixels','position',[1 1 position(3) position(4)],...
    'color',get(handles.mainaxes,'color'),'colormap',get(handles.fig,'colormap'),'visible','on',...
    'renderer','painters','inverthardcopy','off','numbertitle','off');

panel=uipanel('units','pixels','parent',savewindow,'position',get(handles.wspanel,'position'),'backgroundcolor',get(handles.mainaxes,'color'));


graphpos=get(handles.allgraphs(ai),'position');
if numgraphs==1
    graphpos={graphpos}; %make into a cell if it's not already.
end

offset=cell(numgraphs,1);
for g=1:numgraphs
    offset{g}=graphpos{g}-position;
end

%reset graph vectors for newhandles:
newhandles=handles;
newhandles.activeindex=1;
newhandles.activegraph=[];
newhandles.allgraphs=[];
newhandles.fakeax=[];
newhandles.fakerax=[];
newhandles.subax={};
newhandles.subfakeax={};
newhandles.subfakerax={};
newhandles.allblockers=[];
newhandles.expblockers=[];

allorigax=[];
allnewax=[];

for f=1:numgraphs
    udata=get(handles.fakeax(ai(f)),'userdata');
    legdata=get(handles.fakerax(ai(f)),'userdata');
    
    legdata=legdata(ishandle(legdata));
    %compare figure size to the size you told it to be. for large graphs
    %figure may be smaller.
    
    ratio=get(savewindow,'position')./position;
    ratio=min(ratio(3:4));

    %copy graphs to new window:
    SfakerAX=copyobj(handles.fakerax(ai(f)),panel);
    SfakeAX=copyobj(handles.fakeax(ai(f)),panel);
    SAX=copyobj(handles.allgraphs(ai(f)),panel);
    SsubfakerAX=copyobj(handles.subfakerax{ai(f)},panel); %NOTE: need to make ALL graphs expand these arrays or this will give error for reference greater than index
    SsubfakeAX=copyobj(handles.subfakeax{ai(f)},panel);
    SsubAX=copyobj(handles.subax{ai(f)},panel);
    Slegends=copyobj(legdata,panel);
    
    %copy other graph-specific objects:
    type=get(handles.allgraphs(ai(f)),'userdata');
    if isfield(handles.graphinfo.(type),'deletehandles')
        deletehandles=handles.graphinfo.(type).deletehandles;
        for g=1:numel(deletehandles)
            thesehandles=eval(deletehandles{g});
            thesehandles=thesehandles(ishandle(thesehandles));
            copyobj(thesehandles,panel);
        end
    end
    
    %copy explore methods objects:
    if isfield(udata,'explore');
        exps=fieldnames(udata.explore);
        exps=setdiff(exps,'method');
        for i=1:numel(exps)
            if isfield(udata.explore.(exps{i}),'deletehandles')
                thesehandles=udata.explore.(exps{i}).deletehandles;
                copyobj(thesehandles(ishandle(thesehandles)),panel);
            end
        end
    end
    
    %fix axis label positioning: (for some reason copyobj resets axis label
    %positions)
    origax=[handles.fakerax(ai(f));handles.fakeax(ai(f));handles.allgraphs(ai(f));handles.subfakerax{ai(f)}(:);handles.subfakeax{ai(f)}(:);handles.subax{ai(f)}(:);legdata(:)];
    newax=[SfakerAX;SfakeAX;SAX;SsubfakerAX(:);SsubfakeAX(:);SsubAX(:);Slegends(:)];
    
    %exclude any of these that are not axes (e.g. they could be uipanels)
    typez=get(origax,'type');
    idx=strcmp(typez,'axes');

    origxlabelz=get(origax(idx),'xlabel');
    origylabelz=get(origax(idx),'ylabel');
    origzlabelz=get(origax(idx),'zlabel');
    
    newxlabelz=get(newax(idx),'xlabel');
    newylabelz=get(newax(idx),'ylabel');
    newzlabelz=get(newax(idx),'zlabel');
    
    origposses=get(cell2mat([origxlabelz;origylabelz;origzlabelz]),'position');
    set(cell2mat([newxlabelz;newylabelz;newzlabelz]),{'position'},origposses)
    
    %save handles for use below...
    allorigax=[allorigax;origax];
    allnewax=[allnewax;newax];
end

origin=get(handles.allblockers(ai),'position');
if iscell(origin);origin=cell2mat(origin);end
origin=min(origin,[],1);

    
%if necessary, enlarge window to encompass legends:
if ~get(allinone,'value')
    posses=get(allnewax,'position');
    if iscell(posses);posses=cell2mat(posses);end
    posses=[posses;origin];
    
    %reposition origin to bottom left corner:
    origin(1)=min(posses(:,1));
    origin(2)=min(posses(:,2));
    
    %check upper right corner:
    rightmost=max(posses(:,1)+posses(:,3)-origin(1));
    topmost=max(posses(:,2)+posses(:,4)-origin(2));
    
    %add extra margin if allblockers is not the limiting factor:
    if rightmost>origin(3);rightmost=rightmost+5;end
    if topmost>origin(4);topmost=topmost+5;end

    set(savewindow,'position',[50 50 max(rightmost) max(topmost)]);
end

pos=get(panel,'position');
pos=[-origin(1:2) pos(3:4)];
set(panel,'position',pos);

%determine stack order: some graphs may put fakeax in front of real ax
%(e.g. parallelcoords does this). Check how the axes are stacked inthe
%original Rave window and restack them that way in the new window: 
allkidz=get(handles.wspanel,'children');
idx=ismember(allkidz,allorigax);
subkidz=allkidz(idx); %the handles from the main rave window, in the correct stack order

%make newidx vector so that allorigax(newidx)==subkidz
newidx=zeros(size(allorigax));
for q=1:numel(allorigax); %im sure there's a non-loop way to do this, but why bother...
    newidx(q)=find(allorigax==subkidz(q));
end

%now apply newidx order to allnewax:
allnewax=allnewax(newidx);

allkidz=get(panel,'children');
idx=ismember(allkidz,allnewax);
allkidz(idx)=allnewax;
set(panel,'children',allkidz);

%{
    set(SfakerAX,'position',[offset{f}(1) offset{f}(2) graphpos{f}(3) graphpos{f}(4)]*ratio);
    set(SfakeAX,'position',[offset{f}(1) offset{f}(2) graphpos{f}(3) graphpos{f}(4)]*ratio);
    set(SAX,'position',[offset{f}(1) offset{f}(2) graphpos{f}(3) graphpos{f}(4)]*ratio);
    
    %% LEGENDS
    for y=1:numel(Slegends) %for each legend:
        legpos=get(legdata(y),'position');
        legoffset=legpos-position;
        set(Slegends(y),'position',[legoffset(1) legoffset(2) legpos(3) legpos(4)]*ratio)
        
        %if legend is outside bounds of savewindow, expand window to
        %encompass legend
        tttt=get(Slegends(y),'title');
        if ~isempty(get(tttt,'string'));
            set(tttt,'ver','middle');
            legendextent=[legoffset(1) legoffset(2) legoffset(1)+legpos(3) legoffset(2)+legpos(4)+10]*ratio;
        else
            legendextent=[legoffset(1) legoffset(2) legoffset(1)+legpos(3) legoffset(2)+legpos(4)]*ratio;
        end

        figpos=get(savewindow,'position');
        
        legpos=getpixelposition(Slegends(y));
        
        if legpos(1)<=0
            legpos(1)=5;
            figpos(3)=figpos(3)+legpos(3)+15;
            allaxes=setdiff(get(savewindow,'children'),Slegends);
            for t=1:numel(allaxes)
                thispos=get(allaxes(t),'position');
                thispos(1)=thispos(1)+legpos(3)+10;
                set(allaxes(t),'position',thispos)
            end
        end
        
        if legpos(2)<=0
            legpos(2)=5;
            figpos(4)=figpos(4)+legpos(4)+15;
            allaxes=setdiff(get(savewindow,'children'),Slegends);
            for t=1:numel(allaxes)
                thispos=get(allaxes(t),'position');
                thispos(2)=thispos(2)+legpos(4)+10;
                set(allaxes(t),'position',thispos)
            end
        end
        
        set(savewindow,'position',figpos)
        set(Slegends(y),'position',legpos)
        %legpos=getpixelposition(Slegends(y));
        
        if figpos(3)<=legendextent(3)
            figpos(3)=legendextent(3)+5;
        end
        
        if figpos(4)<=legendextent(4)
            figpos(4)=legendextent(4)+5;
        end  
        set(savewindow,'position',figpos);   
    end
        
    %% SUBAXES
    if numsubgraphs(f)~=0
        subgraphpos=get(handles.subax{ai(f)},'position');
        if iscell(subgraphpos);subgraphpos=cell2mat(subgraphpos);end;
        %subgraphpos=cell2mat(get(handles.subax{handles.activeindex(f)},'position'));
        
        if iscell(position)
            suboffset=bsxfun(@minus,subgraphpos,position{:});
        else
            suboffset=bsxfun(@minus,subgraphpos,position);
        end
            
        newpos=[suboffset(:,1) suboffset(:,2) subgraphpos(:,3) subgraphpos(:,4)];
        
        for g=1:numsubgraphs(f)
            
                set(SsubfakerAX(g),'position',newpos(g,:));
                set(SsubfakeAX(g),'position',newpos(g,:));
            
            set(SsubAX(g),'position',newpos(g,:));
        end
    end
    
    
end

%}

switch handles.exportdestination
    case 'print' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        set(savewindow,'paperpositionmode','auto')
        set(savewindow,'inverthardcopy','off')
        printdlg(savewindow)
        close(savewindow)
        
    case 'newfig'
        set(savewindow,'name','default','numbertitle','on','visible','on')

    case 'file' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if get(mayasmooth,'value')
            filetypez={ '*.png','Portable Network Graphics (*.png)';...
                '*.tif','TIFF (*.tif)';...
                '*.bmp','Windows Bitmap (*.bmp)';...
                '*.jpg','JPEG (*.jpg)';...
                '*.*',  'All Files (*.*)'};
        else
            filetypez={'*.rvf','Rave Interactive Figure (*.rvf)';...
                '*.fig','MATLAB Figure (*.fig)';...
                '*.tif','TIFF (*.tif)';...
                '*.emf','Enhanced Metafile (*.emf)';...
                '*.bmp','Windows Bitmap (*.bmp)';...
                '*.png','Portable Network Graphics (*.png)';...
                '*.pdf','Portable Document Format (*.pdf)';...
                '*.jpg','JPEG (*.jpg)';...
                '*.eps','Encapsulated PostScript (*.eps)';...
                '*.ai','Adobe Illustrator (*.ai)';...
                '*.*','All Files (*.*)'};
        end
        
        [name, path, index] = uiputfile(filetypez,'Save as...'); %#ok<NASGU>
        
        set(savewindow,'paperpositionmode','auto')
        set(savewindow,'inverthardcopy','off')
        
        if get(linesmooth,'value') %do smoothing.
            linez=findobj(savewindow,'type','line','marker','none');
            set(linez,'LineSmoothing','on');
        end
        
        if name~=0 %if user didn't cancel:
            if get(mayasmooth,'value') %do MYAA:
                
                if udata.format.bg<=2
                    set(SfakeAX,'xcolor',[0 0 0],'ycolor',[0 0 0],'zcolor',[0 0 0])
                    set(SfakerAX,'xcolor',[0 0 0],'ycolor',[0 0 0],'zcolor',[0 0 0])
                    set(SsubfakeAX,'xcolor',[0 0 0],'ycolor',[0 0 0],'zcolor',[0 0 0])
                    set(SsubfakerAX,'xcolor',[0 0 0],'ycolor',[0 0 0],'zcolor',[0 0 0])
                else
                    set(SfakeAX,'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1])
                    set(SfakerAX,'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1])
                    set(SsubfakeAX,'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1])
                    set(SsubfakerAX,'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1])
                end
                
                set(savewindow,'visible','on');
                
                mslider=findobj(handles.tabobjects,'type','hgjavacomponent');
                jslider=get(mslider,'javapeer');
                scalefactor=jslider.getValue*0.1;
                
                [figyer,drawring]=ravemyaa([3 3./scalefactor],'standard'); %opens a new figure
                set(savewindow,'visible','off'); %we'll close it later.
                close(figyer); %myaa window
                
                imwrite(drawring,fullfile(path,name),name(end-2:end));
                
            else %do non-maya export with print.
                
                opengl=findobj(handles.tabobjects,'tag','opengl');
                if get(opengl,'value')
                    rend='-opengl';
                else
                    rend='-zbuffer';
                end
                
                switch name(end-2:end)
                    case 'bmp'
                        set(savewindow,'visible','on');
                        formatstring='-dbmp';
                        renderer=rend;
                    case 'tif'
                        formatstring='-dtiff';
                        renderer=rend;
                    case 'png'
                        formatstring='-dpng';
                        renderer=rend;
                    case 'jpg'
                        formatstring='-djpeg';
                        renderer=rend;
                    case 'emf'
                        formatstring='-dmeta';
                        renderer='-painters';
                    case 'eps'
                        formatstring='-depsc';
                        renderer='-painters';
                    case '.ai'
                        formatstring='-dill';
                        renderer='-painters';
                    case 'pdf'
                        %use FEX print_pdf function for maximum quality. REQUIRES GHOSTSCRIPT EXE TO BE ON SYSTEM PATH.
                        % see:  http://www.mathworks.com/support/solutions/data/1-15ZLK.html?solution=1-15ZLK
                        %see: http://www.proschematic.com/manual/Printing.html
                        didntwork=raveprint_pdf(fullfile(path,name), savewindow);
                        
                        if didntwork %if no ghostscript, use regular print command.
                            set(handles.infobar,'string','To export PDFs, you must put gswin32c.exe on your system path.')
                            formatstring='-dpdf';
                            renderer='-painters';
                        else
                            close(savewindow)
                            return
                        end
                    case 'rvf'
                        close(savewindow)
                        set(handles.roi,'visible','off');
                        raveexportrvffile(handles.fig,[],fullfile(path,name),ai);
                        set(handles.infobar,'string','Figure file saved successfully.')
                        set(handles.roi,'visible','on');
                        return                        
                    case 'fig'
                        close(savewindow)
                        raveexportfigfile(handles.fig,[],fullfile(path,name),ai);
                        set(handles.infobar,'string','Figure file saved successfully.')
                        return
                        
                end
                
                figpos=get(savewindow,'position');
                resolution=get(0,'ScreenPixelsPerInch')/ratio;
                set(savewindow,'papersize',figpos(3:4)/resolution,'paperposition',[0 0 figpos(3:4)/resolution]);
                
                
                
                mslider=findobj(handles.tabobjects,'type','hgjavacomponent');
                jslider=get(mslider,'javapeer');
                scalefactor=jslider.getValue*0.1;
                
                
                switch name(end-2:end)
                    case  {'emf','eps','.ai','pdf'}
                        rez=2400; %for vector formats, resolution controls number of decimal places used to store data. use a high value to ensure graphs of curved lines look right, expecially things like derivative profiler.
                    otherwise
                        rez=num2str(round(resolution*scalefactor));
                end
                
                if strcmp(formatstring,'-dmeta') || strcmp(formatstring,'-dpdf') %override scaling for vector output. use factor of 2 to fix marker scaling issues.
                    rez=num2str(round(resolution*2));
                end
                print(savewindow,[path name],formatstring,['-r',num2str(rez)],renderer)
            end
        end
        close(savewindow)
           
    case 'ppt' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    case 'word' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
end

ravefinish;
set(handles.infobar,'string','Image exported successfully.')