function [handles,success]=raveloadmetadata(handles,d,path)
%NOTE: THIS funciton takes in handles directly instead of retrieving it
%from guidata (to avoid problems with managedatagui.
%path is path of file to be read.
%results is a structure whose fields depend on the filetype of the file in "path".
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

success=true;

if isempty(path);return;end

%read in file:
fid=fopen(path,'rt');

linez={};
tline=fgetl(fid);
while ischar(tline)
    if ~isempty(strtrim(tline));
        linez{end+1}=strtrim(tline); %#ok<AGROW>
    end
    tline=fgetl(fid);
end
fclose(fid);

%find seperators:

sections=[];
for r=1:numel(linez)
    if length(linez{r})>1 && strcmp(linez{r}(1:2),'%#')
        sections(end+1)=r; %#ok<AGROW>
    end
end

sections(end+1)=numel(linez)+1;


sectiontypes=cell(numel(sections)-1,1);
sectiona=zeros(numel(sections),1);

for s=1:numel(sections)-1
    sectiontypes{s}=upper(linez{sections(s)}(isstrprop(linez{sections(s)},'alpha')));
    if any(linez{sections(s)}==':')
        sectiona(s)=str2double(linez{sections(s)}(isstrprop(linez{sections(s)},'digit')));
    end
end

for s=1:numel(sectiontypes)
    
    contents=linez(sections(s)+1:sections(s+1)-1);
    if isempty(contents);continue;end
    
    switch sectiontypes{s}
        
        case 'FUNCTIONS'
            
            evalthese=[];
            %nvars=size(handles.alldata{d}); %initial number of variables;
            for c=1:numel(contents)
                infoline=contents{c};
                infoz=regexp(infoline,'\t+','split');
                
                thisvarname{c}=infoz{1};
                if thisvarname{c}(1)=='*';
                    thisvarname{c}=thisvarname{c}(2:end);
                    isobjective(c)=true;
                else
                    isobjective(c)=false;
                end
                thisfunpath=infoz{2};
                thisfunclass{c}=infoz{3};
                thisnargout{c}=infoz{4};
                varz=infoz(5:end);
                
                warningz{c}='';
                isok(c)=true;
                
                [path,name,ext]=fileparts(thisfunpath);
                name=[name,ext];
                
                if ~exist(thisfunpath,'file')
                    if isempty(which(name))
                        warningz{c}=['No file named ''',name,''' could be found on your path.'];
                        isok(c)=false;
                    else
                        thisfunpath=which(name);
                        [path,name,ext]=fileparts(thisfunpath);
                        warningz{c}=['The file ''',name,''' was not in the folder listed in the .rvm file, but I found a copy in ',path];
                        isok(c)=true;
                    end
                end
                [path,name,ext]=fileparts(thisfunpath);
                if ~isempty(which(name)) && ~strcmp(which(name),thisfunpath)
                    if isdir(path)
                        addpath(path);
                        warningz{c}=['There was another file named ''',name,''' higher on your search path. I have reordered your path to use the version specified in the .rvm file.'];
                        isok(c)=true;
                    end
                end
                
                funpath{c}=thisfunpath;
                
                if ~strcmp(ext,'.m')
                    addpath(path);
                    warningz{c}=['I couldn''t load the file ''',name,''' because only .m files can be loaded from a .rvm file.'];
                    isok(c)=false;
                end
                
                [thispath,namez{c},junk]=fileparts(funpath{c});
                addpath(thispath);
                
                %thisnargout{c}=contents{c}(tabz(3)+1:tabz(4)-1);
                
                for t=1:numel(varz)
                    invars(t)=find(strcmp(handles.allnames{d},varz{t})); %#ok<AGROW>
                end
                thisinvars{c}=sort(invars);
                %load all this info, but dont evaluate functions yet. first
                %check for rveopt file in case any of these are objectives.
            end
            
            
            if any(~isok)
                errordlg([{'I could not load the functions in your .rvm file because of the following errors. I loaded the data set with no functions, and you can use the model tab to load these functions later.'},{''},unique(warningz)],'Error Loading Metadata')
                continue
            else
                idx=~cellfun(@isempty,warningz);
                if any(idx)
                    warningz=warningz(idx);
                    warningz=unique(warningz);
                    warndlg([{'I made the following changes to the function information in your .rvm file:'},{''},warningz],'Metadata Changes')
                end
            end
            
            
            for c=1:numel(contents)
                handles.allnames{d}{end+1}=thisvarname{c};
                handles.isfunction{d}(end+1)=1;
                handles.isresponse{d}(end+1)=1;
                handles.allunits{d}{end+1}=''; %havent implemented this yet
                handles.isincluded{d}(end+1)=1;
                handles.isdecided{d}(end+1)=0;
                handles.isconstant{d}(end+1)=0; %note, should really use an if statement here to check if it's constant.
                handles.funclass{d}{end+1}=thisfunclass{c};
                handles.funnames{d}{end+1}=namez{c};
                handles.funvars{d}{end+1}=unique(thisinvars{c});
                handles.nargout{d}(end+1)=str2double(thisnargout{c});
                handles.datatype{d}{end+1}='continuous';
                handles.isnoise{d}(end+1)=false;
                handles.distname{d}{end+1}='uniform';
                handles.distparams{d}{end+1}=[0 1]; %NOTE, MUST UPDATE THIS TO KEEP DISTRIBUTION AS PART OF METADATA.
                handles.preferences{d}(end+1)=0;
                handles.targetvalues{d}(end+1)=-inf; %load prefs/targs here in case rveopt file is not found.
                handles.isobjective{d}(end+1)=isobjective(c);
                handles.variablecolors{d}(end+1,:)=rand(1,3);
                handles.maintable.colwidths{d}(end+1)=handles.prefs.colwidth;
                
                for a=1:numel(handles.analysisnames)
                    handles.xdials{d}{a}(end+1)=0;
                    handles.minlimit{d}{a}(end+1)=0;
                    handles.maxlimit{d}{a}(end+1)=0;
                    handles.discretevalues{d}{a}{end+1}=[];
                    handles.minfilter{d}{a}(end+1)=inf;
                    handles.maxfilter{d}{a}(end+1)=-inf;
                end
                evalthese(end+1)=numel(handles.allnames{d}); %#ok<AGROW>
            end
            
            optpath=[path(1:end-3) 'opt'];
            if exist(optpath,'file')
                handles=raveloadmetadata(handles,d,optpath);
            end
            
            %evaluate functions and fill in alldata:
            
            %this code copied from raveevaluatefunction since we can't use
            %guidata here:
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            outnum=evalthese;
            
            if isempty(outnum);continue;end
            
            data=handles.alldata{d};
            prefs=handles.preferences{d};
            targets=handles.targetvalues{d};
            
            %deterine dependency:
            num=sort(outnum,'descend');
            f=numel(num);
            while f<=numel(num)
                thesefunvars=handles.funvars{d}{num(f)};
                num=union(num,thesefunvars(handles.isfunction{d}(thesefunvars)));
                f=f+1;
            end
            name=handles.funnames{d}(num);
            narg=handles.nargout{d}(num);
            
            if ~iscell(name) %one response only
                if handles.isfunction{d}(num)
                    xfun=str2func(name);
                    alloutputs=cell(1,nargout(xfun));
                    
                    switch handles.funclass{d}{num}
                        case 'matlab_generic' %not used anymore but keep here for backward compatibility
                            if nargin(xfun)==3
                                [alloutputs{:}]=xfun(data,prefs,targets);
                            else
                                [alloutputs{:}]=xfun(data);
                            end
                        case 'matlab_singleinput';
                            [alloutputs{:}]=xfun(data);
                        case 'matlab_threeinput';
                            [alloutputs{:}]=xfun(data,prefs,targets);
                        case 'matlab_varnames'; %each variable in its own column
                            udat=mat2cell(data(),size(data,1),ones(1,size(data,2)));
                            [alloutputs{:}]=xfun(udat{:});
                        case 'independent'
                            alloutputs={data(:,num)};
                            
                    end
                    
                    y=alloutputs{narg};
                elseif num<=size(data,2)
                    y=data(:,num);
                end
            else %multiple responses:
                %enlarge (if necessary) data to have as many columns as largest index:
                if size(data,2)<max(num)
                    data=[data zeros(size(data,1),max(num)-size(data,2))];
                end
                
                isfuns=find(handles.isfunction{d}(num));
                if ~isempty(isfuns)
                    
                    [junk,index]=unique(name(isfuns)); %get index of first occurence of unique funs
                    funname=name(isfuns);
                    uniquefuns = funname(sort(index)); %extract unique funs without changing their evaluation order.
                    
                    for g=1:numel(uniquefuns);
                        thisfun=uniquefuns{g};
                        isthisfun=find(strcmp(name,thisfun)); %responses matching current function
                        xfun=str2func(thisfun);
                        alloutputs=cell(1,nargout(xfun));
                        
                        switch handles.funclass{d}{num(isfuns(g))}
                            case 'matlab_generic' %not used anymore but keep here for backward compatibility
                                if nargin(xfun)==3
                                    [alloutputs{:}]=xfun(data,prefs,targets);
                                else
                                    [alloutputs{:}]=xfun(data);
                                end
                            case 'matlab_singleinput';
                                [alloutputs{:}]=xfun(data);
                            case 'matlab_threeinput'; %
                                [alloutputs{:}]=xfun(data,prefs,targets);
                            case 'matlab_varnames'; %each variable in its own column
                                udat=data(:,handles.funvars{d}{num(isfuns(g))});
                                udat=mat2cell(udat,size(udat,1),ones(1,size(udat,2)));
                                [alloutputs{:}]=xfun(udat{:});
                            case 'independent'
                                alloutputs={data(:,num(isfuns(g)))};
                        end
                        
                        alloutputs=cell2mat(alloutputs);
                        data(:,num(isthisfun))=alloutputs(:,narg(isthisfun));
                        
                        %y(:,isthisfun)=alloutputs(:,narg(isthisfun));
                    end
                    
                end
                y=data(:,outnum);
            end
            
            handles.alldata{d}(:,evalthese)=y;
            handles.alltext{d}=[handles.alltext{d},cell(size(y))];
            handles.isvalid{d}(:,evalthese)=true(size(y));
            
            for a=1:numel(handles.analysisnames)
                
                limitz=[min(handles.alldata{d}(:,evalthese),[],1);max(handles.alldata{d}(:,evalthese),[],1)];
                limitz(1,isnan(limitz(1,:)))=0;
                limitz(2,isnan(limitz(2,:)))=1;
                
                handles.minlimit{d}{a}(evalthese)=limitz(1,:);
                handles.maxlimit{d}{a}(evalthese)=limitz(2,:);
                handles.minfilter{d}{a}(evalthese)=limitz(1,:);
                handles.maxfilter{d}{a}(evalthese)=limitz(2,:);
            end
            
        case 'RANGES'
            for j=1:numel(contents)
                tline=contents{j};
                
                tabz=find(tline==char(9));
                lastindex=find(isstrprop(tline,'alphanum'),1,'last');
                tabz=[tabz(tabz<lastindex) lastindex+1];
                thisvarname=tline(1:tabz(1)-1);
                thistype=lower(tline(tabz(1)+1:tabz(2)-1));
                thisvar=find(strcmp(handles.allnames{d},thisvarname));
                
                if numel(thisvar)>1
                    warning('Rave:Metadata:nonuniquename',['I could not load the variable range information for ',thisvarname,' because you have more than one variable with that name and I don''t know which one you were referring to.'])
                    
                    continue
                end
                
                if isempty(thisvar)
                    clear values
                    warning('Rave:Metadata:noname',['I could not load the variable range information for ',thisvarname,' because there is no variable in your data set with that name.'])
                    continue
                end
                if ~ismember(thistype,{'constant','continuous','discrete','integer','string'})
                    thistype='continuous';
                end
                
                %store type:
                handles.datatype{d}{thisvar}=thistype;
                
                %store values:
                if ~strcmp(thistype,'string')
                    for t=2:numel(tabz)-1
                        values(t-1)=str2double(tline(tabz(t)+1:tabz(t+1)-1)); %#ok<AGROW>
                    end
                    values=unique(values);
                else
                    for t=2:numel(tabz)-1
                        values{t-1}=tline(tabz(t)+1:tabz(t+1)-1);
                    end
                    values=unique(values);
                end
                
                a=sectiona(s);
                
                switch lower(thistype)
                    case 'constant'
                        handles.minlimit{d}{a}(thisvar)=values(1);
                        handles.maxlimit{d}{a}(thisvar)=values(1);
                    case {'continuous','integer','logical'}
                        handles.minlimit{d}{a}(thisvar)=min(values);
                        handles.maxlimit{d}{a}(thisvar)=max(values);
                    case 'discrete'
                        handles.minlimit{d}{a}(thisvar)=min(values);
                        handles.maxlimit{d}{a}(thisvar)=max(values);
                        handles.discretevalues{d}{a}{thisvar}=values';
                    case 'string'
                        handles.minlimit{d}{a}(thisvar)=1;
                        handles.maxlimit{d}{a}(thisvar)=numel(values);
                        handles.discretevalues{d}{a}{thisvar}=values';
                end
                clear values
            end
            
        case 'CONSTRAINTS'
            for c=1:numel(contents)
                thisstr=contents{c};
                thisstr=thisstr(thisstr~=char(9));
                %determine color:
                brack=find(thisstr=='[' | thisstr==']');
                if numel(brack==2)
                    color=eval(thisstr(brack(1):brack(2)));
                    thisstr=thisstr(1:(brack(1)-1));
                else
                    color=handles.colors.constraint;
                end
                
                %determine type of constraint:
                if ~isempty(strfind(thisstr,'<='))
                    thisequality='<=';
                    k=strfind(thisstr,'<=');
                    constraintstring=thisstr(1:k-1);
                    constraintvalue=eval(thisstr(k+2:end));
                elseif ~isempty(strfind(thisstr,'>='))
                    thisequality='>=';
                    k=strfind(thisstr,'>=');
                    constraintstring=thisstr(1:k-1);
                    constraintvalue=eval(thisstr(k+2:end));
                    
                elseif ~isempty(strfind(thisstr,'='))
                    thisequality='=';
                    k=strfind(thisstr,'=');
                    constraintstring=thisstr(1:k-1);
                    constraintvalue=eval(thisstr(k+1:end));
                    
                elseif ~isempty(strfind(thisstr,'<'))
                    thisequality='<';
                    k=strfind(thisstr,'<');
                    constraintstring=thisstr(1:k-1);
                    constraintvalue=eval(thisstr(k+1:end));
                    
                elseif ~isempty(strfind(thisstr,'>'))
                    thisequality='>';
                    k=strfind(thisstr,'>');
                    constraintstring=thisstr(1:k-1);
                    constraintvalue=eval(thisstr(k+1:end));
                    
                else
                    continue %invalid constraint
                end
                
                [constraintfunctionstring,constrainttype,constraintvars]=raveprocessconstraint(handles,d,constraintstring,thisequality,constraintvalue);
                
                %store this function annonymously in handles.
                try
                    eval(['newhandle=@(data) ',constraintfunctionstring,';']);
                catch %#ok<CTCH>
                    errordlg('Something went wrong while creating a constraint. Check to see if you spelled everything right and used correct MATLAB syntax.')
                end
                
                for a=1:numel(handles.analysisnames)
                    handles.constrainthandles{d}{a}{end+1}=newhandle;
                    handles.constrainttext{d}{a}{end+1}=strtrim([constraintstring,thisequality,' ',num2str(constraintvalue)]);
                    handles.constrainttype{d}{a}{end+1}=constrainttype;
                    handles.constraintvars{d}{a}{end+1}=constraintvars;
                    handles.constraintcolors{d}{a}{end+1}=color;
                end
                
                %evaluate this constraint on current data set, and store results in handles.constraintvalues:
                thisindex=numel(handles.constrainthandles{d}{1});
                guidata(handles.fig,handles);
                
                %evaluate constraint:
                a=sectiona(s);
                handles.constraintvalues{d}{a}{end+1}=evaluateconstraint(handles,d,thisindex,handles.alldata{d});
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                %evaluate constraint at end points to get normalization parameters:
                mins=handles.minlimit{d}{a};
                maxs=handles.maxlimit{d}{a};
                
                minvalue=evaluateconstraint(handles,d,thisindex,mins,0);
                maxvalue=evaluateconstraint(handles,d,thisindex,maxs,0);
                meanvalue=evaluateconstraint(handles,d,thisindex,(mins+maxs)./2,0);
                
                sample=[minvalue maxvalue meanvalue];
                
                sample(isnan(sample))=1; %not really sure what to do if nan pops up (most likely from 0/0 expression)
                
                %offset=mean(sample);
                range=max(sample)-min(sample);
                
                if range==0
                    range=1;
                end
                
                %handles.constraintadders{udata.view.dataset}{udata.view.analysis}{end+1}=offset;
                handles.constraintscalers{d}{a}(end+1)=range;
                guidata(handles.fig,handles);
            end
            
            
        case 'CURRENTPOINT'
            % contents=importdata(path,'\t');
            
            nameline=contents{1};
            varnames=eval(['{''',strrep(nameline,char(9),''';'''),'''}']);
            
            values=contents{2};
            values=eval(['[',strrep(values,char(9),','),']']);
            
            a=sectiona(s);
            
            for c=1:numel(varnames);
                thisindex=find(strcmp(handles.allnames{d},varnames{c}));
                handles.xdials{d}{a}(thisindex)=values(c); %#ok<FNDSB>
            end
            
        case 'TARGETSANDPREFERENCES'
            nameline=contents{1};
            varnames=eval(['{''',strrep(nameline,char(9),''';'''),'''}']);
            
            targs=contents{2};
            targs=eval(['[',strrep(targs,char(9),','),']']);
            
            prefs=contents{3};
            prefs=eval(['[',strrep(prefs,char(9),','),']']);
            
            for c=1:numel(varnames);
                thisindex=find(strcmp(handles.allnames{d},varnames{c}));
                handles.preferences{d}(thisindex)=prefs(c);
                handles.targetvalues{d}(thisindex)=targs(c);
            end
            
            
        case 'COLORS'
            varnames=contents{1};
            colors=contents{2};
            
            nametabz=find(varnames==char(9));
            lastindex=find(isstrprop(varnames,'alphanum'),1,'last');
            nametabz=[0 nametabz(nametabz<lastindex) lastindex+1];
            colortabz=find(colors==char(9));
            lastindex=find(colors==']',1,'last');
            colortabz=[0 colortabz(colortabz<lastindex) lastindex+1];
            
            for c=1:min(numel(nametabz,colortabz))-1
                thisindex=find(strcmp(handles.allnames{d},varnames(nametabz(c)+1:nametabz(c+1)-1)));
                
                if numel(thisindex)>1
                    warning('Rave:Metadata:nonuniquename',['I could not load the variable color information for ',varnames(nametabz(c)+1:nametabz(c+1)-1),' because you have more than one variable with that name and I don''t know which one you were referring to.'])
                    
                    continue
                end
                
                if isempty(thisindex)
                    clear values
                    warning('Rave:Metadata:noname',['I could not load the variable color information for ',varnames(nametabz(c)+1:nametabz(c+1)-1),' because there is no variable in your data set with that name.'])
                    continue
                end
                
                if ~isempty(thisindex)
                    handles.variablecolors{d}(thisindex,:)=eval(colors(colortabz(c)+1:colortabz(c+1)-1));
                end
            end
            
        case 'ROWSTATE'
            for c=1:numel(contents)
                contents{c}=eval(['[',contents{c},']']);
            end
            contents=cell2mat(contents(:));
            
            if size(contents,1)~=size(handles.alldata{d},1)
                warning('Rave:Metadata:inconsistentrows',['I could not load the row state metadata because the metadata contains a different number of rows than the data set.'])
            else
                a=sectiona(s);
                handles.rowcolors{d}{a}(1:size(contents,1))=contents(:,1);
                handles.isvisible{d}{a}(1:size(contents,1))=contents(:,2);
                handles.isselected{d}{a}(1:size(contents,1))=contents(:,3);
            end
            
        case 'POINTSOFINTEREST'
            %not yet implemented
    end
end


function g=evaluateconstraint(handles,dataset,thisconstraint,indata,ignore)
%evaluates the regression with index stored in thisconstraint using the data stored in data.
%thisconstraint is an index into:
%handles.constrainthandles{udata.view.dataset}{udata.view.analysis}{thisconstraint}
%ignore is a logical vector of length=size(indata,2) (numvars) where 1 means do not evaluate this
%regression (if that varaible isfunction). use this to do things like sweeps through regressed
%variables. or just make ignore==0, to  evaluate all constraints.


%{

%% determine which variables appear in this constraint:
thisstring=char(handles.constrainthandles{dataset}{a}{thisconstraint});
thisstring=thisstring(8:end);

varnumbers=handles.constraintvars{dataset}{a}{thisconstraint};

varnumbers=setdiff(varnumbers,find(ignore)); %dont evaluate regressions of ignored vars

if ~isempty(varnumbers)
    %evaluate all regressions necessary for this constraint:
    respdata=raveevaluatefunction(handles.fig,udata.view.dataset,varnumbers,indata);
    indata(:,varnumbers)=respdata;
end
%}
%solve evaluate this constraint for y variable:

thishandle=handles.constrainthandles{dataset}{1}{thisconstraint};
g=thishandle(indata);


