function f=ravemakedoe(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
handles.temp=[];
guidata(handles.fig,handles);

%make figure:
f=figure('units','pixels','position',[0 0 651 700],'name',...
    'Design of Experiments','numbertitle','off','menubar','none','resize','off',...
    'color',handles.colors.frame,'windowstyle','normal');
origin=get(handles.fig,'position');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

%make tabs:
titleaxes=axes('units','pixels','position',[11 699 485 41],'parent',f,'visible','off','tag','titleaxes');

handles.cancelbutton=uicontrol('style','pushbutton','units','pixels','position',[465 10 180 30],'tag','cancelbutton',...
    'fontsize',16,'string','Close','callback',@cancelbutton_Callback);

% %make titles
% text(0,.2,'Re-Sample Data Set','parent',titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
%     'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');

u=uipanel('parent',f,'units','pixels','position',[50 50 550 640],'backgroundcolor',handles.colors.tabbg);

%%
a1=axes('parent',u,'units','pixels','position',[5 580+24 400 40],'visible','off');
text(0,0.5,'1. Choose Method','parent',a1,'fontname',handles.prefs.fonts.tab,...
    'color',handles.colors.text,'fontsize',16*handles.prefs.fonts.tabscale);

uicontrol('parent',u,'position',[35 558+26 200 22],'style','text',...
    'string','Data Set:','backgroundcolor',[1 1 1],'fontweight','bold','fontsize',10,'hor','left');
uicontrol('parent',u,'position',[35 528+26 200 22],'style','text',...
    'string','Sample by:','backgroundcolor',[1 1 1],'fontweight','bold','fontsize',10,'hor','left');

uicontrol('parent',u,'position',[115 560+26 200 22],'style','popupmenu',...
    'string',handles.datasetnames,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu');
uicontrol('parent',u,'position',[325 560+26 200 22],'style','popupmenu',...
    'string',handles.analysisnames,...
    'backgroundcolor',[1 1 1],'tag','analysismenu');
methodmenu=uicontrol('parent',u,'position',[115 530+26 200 22],'style','popupmenu',...
    'string',{'Full Factorial DoE';'Latin Hypercube DoE';'Monte Carlo (Random)'},...
    'backgroundcolor',[1 1 1],'tag','methodmenu');
uicontrol('parent',u,'position',[35 465+26 500 55],'style','text',...
    'string','','backgroundcolor',handles.colors.tabbg,'tag','methodtext','hor','left');

%%
a2=axes('parent',u,'units','pixels','position',[5 435+26 400 40],'visible','off');
text(0,0.5,'2. Choose Settings','parent',a2,'fontname',handles.prefs.fonts.tab,...
    'color',handles.colors.text,'fontsize',16*handles.prefs.fonts.tabscale);
uu=uipanel('parent',f,'units','pixels','position',[56 220 540 300],'backgroundcolor',handles.colors.tabbg,'bordertype','none','tag','methodspanel');

%%
a3=axes('parent',u,'units','pixels','position',[5 105+46-20 400 40],'visible','off');
text(0,0.5,'3. Choose Destination','parent',a3,'fontname',handles.prefs.fonts.tab,...
    'color',handles.colors.text,'fontsize',16*handles.prefs.fonts.tabscale);
destmenu=uicontrol('parent',u,'position',[35 85+50-20 250 22],'style','popupmenu',...
    'string',{'Create as a new data set';'Export to a file';'Append to current data set';'Replace current data set'},...
    'tag','destinationmenu','backgroundcolor',[1 1 1],'callback',@destmenu_callback);

handles.donebutton=uicontrol('style','pushbutton','units','pixels','position',[200 10 180 30],'tag','donebutton',...
    'fontsize',16,'string','Create DoE','callback',@donebutton_Callback,'tag','donebutton','parent',u);

%%
handles.temp.tabobjects=[];
handles.temp.destobjects=[];
handles.temp.u=u;
handles.temp.f=f;

guidata(f,handles);
guidata(handles.fig,handles);

%load first method:
methodmenu_callback(methodmenu,[]);
destmenu_callback(destmenu,[]);

function cancelbutton_Callback(src,ev)
close(gcf);

function donebutton_Callback(src,ev)
close(gcf);

function destmenu_callback(src,ev)

handles=guidata(src);
delete(handles.temp.destobjects);
handles.temp.destobjects=[];
u=handles.temp.u;

val=get(src,'value');

if val==1 % new data set
    txt=uicontrol('position',[35 60+50-20 400 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Data set name:',...
        'hor','left','fontsize',10,'fontweight','bold');
    editbox=uicontrol('position',[50 40+50-20 200 22],'parent',handles.temp.u,...
        'style','edit','backgroundcolor',[1 1 1],'hor','left','tag','namebox');
    andfunctions=uicontrol('position',[50 14+50-20 400 22],'parent',handles.temp.u,...
        'style','checkbox','string','New data set inherits function variables from current data set',...
        'tag','andfunctions','backgroundcolor',[1 1 1],'value',1);
    handles.temp.destobjects=[txt editbox andfunctions];
    
elseif val==2 % export to file
    txt=uicontrol('position',[35 60+50-20 400 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Destination file path: (*.txt)',...
        'hor','left','fontsize',10,'fontweight','bold');
    editbox=uicontrol('position',[50 40+50-20 400 22],'parent',handles.temp.u,...
        'style','edit','backgroundcolor',[1 1 1],'tag','browsebox','hor','left');
    browsebutton=uicontrol('position',[460 40+50-20 80 22],'parent',handles.temp.u,...
        'style','push','callback',@browsebutton_callback,'string','Browse...');
    andfunctions=uicontrol('position',[50 14+50-20 400 22],'parent',handles.temp.u,...
        'style','checkbox','string','Evaluate function variables and include them as columns in exported file',...
        'tag','andfunctions','backgroundcolor',[1 1 1]);
    
    handles.temp.destobjects=[txt editbox browsebutton andfunctions];
    
else %append or replace
    colortestbutton=zeros(1,10);
    for c=1:10
        colortestbutton(c)=uicontrol('position',[35+22*c 35+50-20 18 18],'parent',u,...
            'style','togglebutton','string','','value',0,'backgroundcolor',handles.prefs.rowcolors(c,:),...
            'fontname','arial narrow','fontsize',6,'horizontalalignment','left',...
            'tag','rowcolorblob','userdata',c,'callback',@colorbutton_callback);
    end
    set(colortestbutton(handles.prefs.defaultrowcolor),'value',1);
    colortext=uicontrol('units','pixel','position',[35 55+50-20 300 22],'parent',u,'style','text',...
        'string','Row color of newly added rows:','backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');
    selectexisting=uicontrol('units','pixel','position',[35 10+50-20 200 22],'parent',u,'style','checkbox',...
        'string','Select new rows','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','selectexisting');
    handles.temp.destobjects=[colortestbutton colortext selectexisting];
    
end

guidata(handles.fig,handles)
guidata(handles.temp.f,handles)


function methodmenu_callback(src,ev)

value=get(src,'value');

if value==1;
    ff_callback(src,ev);
elseif value==2;
    lhs_callback(src,ev);
elseif value==3
    mc_callback(src,ev);
end


function ff_callback(src,ev)
handles=guidata(src);

delete(handles.temp.tabobjects);
u=handles.temp.u;
f=handles.temp.f;
uu=findobj(handles.temp.f,'tag','methodspanel');

methodtext=findobj(u,'tag','methodtext');
set(methodtext,'string',{'Full Factorial DoE assigns a (small) number of levels to each independent variable, and then generates a data table that includes all possible combinations of those levels. Thus if there are n variables and each variable has m levels, the data set will have m^n rows. Usually m=2 or 3.'})

fftext=uicontrol('parent',uu,'position',[30 240 500 60],'style','text',...
    'string',{'Use the generate button to populate the table and/or edit the table manually. Each variable must have at least one level. Variables do not all have to have the same number of levels. Empty and duplicate cells will be ignored. For text type variables, enter integer string indices.'},'backgroundcolor',handles.colors.tabbg,'hor','left');

datasetmenu=findobj(u,'tag','datasetmenu');
d=get(datasetmenu,'value');

nvars=sum(~handles.isfunction{d});
isvars=find(~handles.isfunction{d});
dblArray=javaArray('java.lang.Double',nvars,10);

colnames={'Level 1' 'Level 2' 'Level 3' 'Level 4' 'Level 5' 'Level 6' 'Level 7' 'Level 8' 'Level 9' 'Level 10'};

dmodel=javax.swing.table.DefaultTableModel(dblArray,colnames);
jtable=javax.swing.JTable(dmodel);
jtable.setAutoResizeMode(jtable.AUTO_RESIZE_OFF);
jscroll=javax.swing.JScrollPane(jtable);

dblRows=javaArray('java.lang.Object',nvars,1); %row header numbers
for g=1:nvars
    dblRows(g,1)=java.lang.String(handles.allnames{d}{isvars(g)});
end
rh=javax.swing.table.DefaultTableModel(dblRows,{'Variable Name'});
rowHeader=javax.swing.JTable(rh);

[jhandle mhandle]=javacomponent(jscroll,[30 0 500 220],uu);

handles.temp.jtable=jtable;
handles.temp.jscroll=jhandle;
guidata(handles.fig,handles);
guidata(f,handles);

%make the header the appropriate width:
headerWidth=150;
rhHeight=rowHeader.getPreferredSize.height;
rowHeader.setPreferredSize(java.awt.Dimension(headerWidth,rhHeight))
rowHeader.setSize(java.awt.Dimension(headerWidth,rhHeight))

%put the row header in a viewport.
jv=javax.swing.JViewport;
jv.setView(rowHeader);
jv.setPreferredSize(java.awt.Dimension(headerWidth,rhHeight));

jscroll.setRowHeader(jv);
jscroll.setCorner(javax.swing.ScrollPaneConstants.UPPER_LEFT_CORNER, rowHeader.getTableHeader());

u1=uicontrol('position',[30 230 100 24],'parent',uu,'string','Generate Levels',...
    'callback',@generatefflevels,'style','pushbutton');
u2=uicontrol('position',[135 230 300 20],'parent',uu,'string','for a Full-Factorial DoE with up to                     levels.',...
    'callback','','style','text','backgroundcolor',handles.colors.tabbg,'hor','left');
u3=uicontrol('position',[395 230 140 20],'parent',uu,'string','Total Rows = 0',...
    'callback','','style','text','backgroundcolor',handles.colors.tabbg,'hor','left','fontweight','bold',...
    'tag','totalrows');
u4=uicontrol('position',[302 230 50 20],'parent',uu,'string','2',...
    'callback','','style','edit','backgroundcolor',handles.colors.tabbg,'hor','center','fontweight','bold',...
    'tag','leveledit');

donebutton=findobj(f,'tag','donebutton');
set(donebutton,'callback',@makefullfact);

generatefflevels(src,ev)


handles.temp.tabobjects=[u1 u2 u3 u4 donebutton mhandle fftext];

guidata(handles.temp.f,handles);

function lhs_callback(src,ev)

function mc_callback(src,ev)

function roundout=roundto(roundees,rounders)
%rounds roundees to nearest value in rounders inefficiently.

roundout=roundees;
for g=1:numel(roundees)
    newval=rounders(abs(rounders-roundees(g))==min(abs(rounders-roundees(g))));
    roundout(g)=newval(1);
end

function generatefflevels(src,ev)

handles=guidata(src);

%delete(handles.temp.tabobjects);
u=handles.temp.u;
uu=findobj(handles.temp.f,'tag','methodspanel');

methodtext=findobj(u,'tag','methodtext');
set(methodtext,'string',{'Full Factorial DoE assigns a (small) number of levels to each independent variable, and then generates a data table that includes all possible combinations of those levels. Thus if there are n variables and each variable has m levels, the data set will have m^n rows. Usually m=2 or 3.'})

datasetmenu=findobj(u,'tag','datasetmenu');
d=get(datasetmenu,'value');
analysismenu=findobj(u,'tag','analysismenu');
a=get(analysismenu,'value');

nvars=sum(~handles.isfunction{d});
isvars=find(~handles.isfunction{d});
dblArray=javaArray('java.lang.Double',nvars,10);

colnames={'Level 1' 'Level 2' 'Level 3' 'Level 4' 'Level 5' 'Level 6' 'Level 7' 'Level 8' 'Level 9' 'Level 10'};

%make default data:
leveledit=findobj(uu,'tag','leveledit');
nlevels=str2double(get(leveledit,'string'));
if isnan(nlevels)
    errordlg('You must enter a number between 1 and 10 in the "levels" box.','Invalid Entry')
    return
end

nlevels=round(nlevels);
if nlevels>10 || nlevels<1
    errordlg('You must enter a number between 1 and 10 in the "levels" box.','Invalid Entry')
    return
end

trows=1;

for r=1:nvars
    switch handles.datatype{d}{isvars(r)}
        case 'continuous'
            vals=linspace(handles.minlimit{d}{a}(isvars(r)),handles.maxlimit{d}{a}(isvars(r)),nlevels);
        case 'constant'
            vals=handles.minlimit{d}{a}(isvars(r));
        case {'discrete','string'}
            vals=linspace(handles.minlimit{d}{a}(isvars(r)),handles.maxlimit{d}{a}(isvars(r)),nlevels);
            vals=roundto(vals,handles.discretevalues{d}{a}{isvars(r)});
        case {'logical'}
            vals=[handles.minlimit{d}{a}(isvars(r)) handles.maxlimit{d}{a}(isvars(r))];
        case {'integer'}
            vals=handles.minlimit{d}{a}(isvars(r)):handles.maxlimit{d}{a}(isvars(r));
            if numel(vals)>nlevels;
                vals=roundto(linspace(handles.minlimit{d}{a}(isvars(r)),handles.maxlimit{d}{a}(isvars(r)),nlevels),vals);
            end
    end
    for c=1:numel(vals)
        dblArray(r,c)=java.lang.Double(vals(c));
    end
    trows=trows*numel(vals);
end

dmodel=javax.swing.table.DefaultTableModel(dblArray,colnames);
handles.temp.jtable.setModel(dmodel);
set(handle(dmodel,'callbackproperties'),'TableChangedCallback',{@fftablechanged,handles.fig})

totalrows=findobj(uu,'tag','totalrows');
set(totalrows,'string',['Total Rows = ',num2str(trows)]);

function fftablechanged(jsrc,ev,src)%#ok

handles=guidata(src);

R=handles.temp.jtable.getRowCount;
C=handles.temp.jtable.getColumnCount;

trows=1;
for r=1:R
    nlevels=0;
    for c=1:C
        thisvalue=handles.temp.jtable.getValueAt(r-1,c-1);
        if ~isempty(thisvalue)
            if ischar(thisvalue);thisvalue=str2double(thisvalue);end
            if ~isnan(thisvalue)
                nlevels=nlevels+1;
            end
        end
    end
    trows=trows*nlevels;
end

totalrows=findobj(handles.temp.u,'tag','totalrows');
set(totalrows,'string',['Total Rows = ',num2str(trows)]);



function makefullfact(src,ev)

%check that destination names are valid...
result=processdest(src,ev);
if ~result;return;end

handles=guidata(src);
%get levels from table:

C=handles.temp.jtable.getColumnCount;
R=handles.temp.jtable.getRowCount;

alllevels=cell(1,R);

for r=1:R
    for c=1:C
        thislevel=handles.temp.jtable.getValueAt(r-1,c-1);
        if isempty(thislevel);continue;end
        if ischar(thislevel);thislevel=str2double(thislevel);end
        if ~isnan(thislevel)
            alllevels{r}=union(alllevels{r},thislevel);
        end
    end
end

levels=cellfun(@numel,alllevels);

points=fliplr(fullfactbuild(fliplr(levels)));

for v=1:R
    points(:,v)=alllevels{v}(points(:,v));
end

processDoE(handles.temp.f,[],points);


function colorbutton_callback(src,ev)
handles=guidata(src);

cbuttons=findobj(handles.temp.f,'tag','rowcolorblob');
set(cbuttons,'value',0);
set(src,'value',1);

function browsebutton_callback(src,ev)
handles=guidata(src);

[name,path]=uiputfile('*.txt','Save file name');

if ~ischar(name);return;end

fullpath=fullfile(path,name);
browsebox=findobj(handles.temp.u,'tag','browsebox');
set(browsebox,'string',fullpath);


function result=processdest(src,ev)
result=true;

handles=guidata(src);

destmenu=findobj(handles.temp.f,'tag','destinationmenu');

if get(destmenu,'value')==1
    namebox=findobj('tag','namebox');
    namestr=get(namebox,'string');
    if isempty(namestr)
        result=false;
        errordlg('You must enter a name for the new data set.');
    end
elseif get(destmenu,'value')==2
    browsebox=findobj('tag','browsebox');
    browsestr=get(browsebox,'string');
    if isempty(browsestr)
        result=false;
        errordlg('You must enter a file name.');
    end
end


function processDoE(src,ev,points)
%sends the DoE defined by "points" to the proper destination:

handles=guidata(src);
destmenu=findobj(handles.temp.f,'tag','destinationmenu');
datasetmenu=findobj(handles.temp.f,'tag','datasetmenu');
analysismenu=findobj(handles.temp.f,'tag','analysismenu');

val=get(destmenu,'value');

d=get(datasetmenu,'value');
a=get(analysismenu,'value');

allnames=handles.allnames{d};
indeps=~handles.isfunction{d};

nrows=size(points,1);

newdata=zeros(nrows,length(indeps));
newdata(:,indeps)=points;

if val==1; %data set
    andfunctions=findobj(handles.temp.f,'tag','andfunctions');
    andfunctions=get(andfunctions,'value');
    
    namebox=findobj(handles.temp.f,'tag','namebox');
    dest=get(namebox,'string');
    
    %first, copy existing data set:
    raveduplicatedataset(handles.fig,[],d,dest)
    handles=guidata(handles.fig);
    
    handles.ispointofinterest{end}=false(nrows,1);
    
    for a=1:numel(handles.analysisnames)
        handles.rowcolors{end}{a}=handles.prefs.defaultrowcolor*ones(nrows,1);
        handles.isvisible{end}{a}=true(nrows,1);
        handles.isselected{end}{a}=false(nrows,1);
        handles.isfiltered{end}{a}=false(nrows,1);
    end
    
    guidata(handles.fig,handles);
    
    newd=numel(handles.datasetnames);
    
    if ~andfunctions
        %now remove extra columns if not keeping functions:
        ravedeletevars(handles.fig,[],newd,handles.isfunction{newd});
        handles=guidata(handles.fig);
        newdata=points;
    else
        %evaluate functions
        index=find(handles.isfunction{d});
        outs=raveevaluatefunction(handles.fig,d,index,newdata);
        newdata(:,index)=outs;
    end
    
    %now replace data with DoE and call raveupdatedataset.
    handles.alldata{end}=newdata;
    handles.alltext{end}=cell(size(newdata));
    handles.isvalid{end}=true(size(newdata));
    
    guidata(handles.fig,handles);
    raveupdatedataset(handles.fig,[],'replaceall',[],numel(handles.datasetnames));
    
elseif val==2; %file
    andfunctions=findobj(handles.temp.f,'tag','andfunctions');
    andfunctions=get(andfunctions,'value');
    
    if ~andfunctions;
        allnames=allnames(indeps);
        newdata=points;
    else
        %evaluate functions
        index=find(handles.isfunction{d});
        outs=raveevaluatefunction(handles.fig,d,index,newdata);
        newdata(:,index)=outs;
    end
    
    browsebox=findobj(handles.temp.f,'tag','browsebox');
    dest=get(browsebox,'string');
    
    fid=fopen(dest,'wt');
    for t=1:numel(allnames)-1
        fprintf(fid,'%s\t',allnames{t});
    end
    fprintf(fid,'%s\n',allnames{end});
    fclose(fid);
    dlmwrite(dest,newdata,'-append','delimiter','\t','newline','pc','precision',8);
    
elseif val==3; %append
    %evaluate functions
    index=find(handles.isfunction{d});
    outs=raveevaluatefunction(handles.fig,d,index,newdata);
    newdata(:,index)=outs;
    
    cblob=findobj(handles.temp.f,'tag','rowcolorblob','value',1);
    rowcolor=get(cblob,'userdata');
    
    selectexisting=findobj(handles.temp.f,'tag','selectexisting');
    doselect=get(selectexisting,'value');
    
    raveappendrows(handles.fig,[],d,newdata,rowcolor,doselect,true,false)
%     
%     
%     handles.alldata{d}=[handles.alldata{d};newdata];
%     handles.alltext{d}=[handles.alltext{d};cell(size(newdata))];
%     handles.isvalid{d}=[handles.isvalid{d};true(size(newdata))];
%     
%     handles.ispointofinterest{d}=[handles.ispointofinterest{d};false(nrows,1)];
%     
%     for a=1:numel(handles.analysisnames)
%         handles.rowcolors{d}{a}=[handles.rowcolors{d}{a};rowcolor*ones(nrows,1)];
%         handles.isvisible{d}{a}=[handles.isvisible{d}{a};true(nrows,1)];
%         handles.isfiltered{d}{a}=[handles.isfiltered{d}{a};false(nrows,1)];
% 
%         oldselect=handles.isselected{d}{a};
%         
%         if doselect
%             newselect=true(size(newdata,1),1);
%         else
%             newselect=false(size(newdata,1),1);
%         end
%         
%         handles.isselected{d}{a}=[oldselect;newselect];
%     end
%     
%     
%     guidata(handles.fig,handles);
%     raveupdatedataset(handles.fig,[],'addrows',numel(handles.datasetnames));
    
    
elseif val==4; %replace
        %evaluate functions
    index=find(handles.isfunction{d});
    outs=raveevaluatefunction(handles.fig,d,index,newdata);
    newdata(:,index)=outs;
    
    cblob=findobj(handles.temp.f,'tag','rowcolorblob','value',1);
    rowcolor=get(cblob,'userdata');
    
    selectexisting=findobj(handles.temp.f,'tag','selectexisting');
    doselect=get(selectexisting,'value');
    
    
    handles.alldata{d}=newdata;
    handles.alltext{d}=cell(size(newdata));
    handles.isvalid{d}=true(size(newdata));
    
    handles.ispointofinterest{d}=false(nrows,1);
    
    for a=1:numel(handles.analysisnames)
        handles.rowcolors{d}{a}=rowcolor*ones(nrows,1);
        handles.isvisible{d}{a}=true(nrows,1);
        handles.isfiltered{d}{a}=false(nrows,1);

        if doselect
            newselect=true(size(newdata,1),1);
        else
            newselect=false(size(newdata,1),1);
        end
        
        handles.isselected{d}{a}=[newselect];
    end
    
    
    guidata(handles.fig,handles);
    raveupdatedataset(handles.fig,[],'replaceall',[],numel(handles.datasetnames));

end

ravefinish
set(handles.infobar,'string','DoE created successfully.');






