function ravemaketextticks(src,ev,xvar,yvar,zvar,cvar,xstr,ystr,zstr,cstr) %#ok<INUSL>
%src must be handle of axes for which to apply text labels.
%only handles one variable for each of the 4 inputs... if any of the 4 inputs is a vector, error will occur.
%for matrix graphs must call this for each of the subgraphs.
% all inputs except src are optional.
% if xvar/yvar/zvar=0, you must also supply xstr/ystr/zstr cell arrays of
% strings. In this case you must have set the xtick/ytick/ztick numerical
% values to be where you want these strings to appear (strings are center
% aligned) before calling this function.
% inputs cvar/cstr currently unused.

% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
d=udata.view.dataset;
a=udata.view.analysis;

%set axis label colors:
if isfield(udata.format,'bg')
    xlab=get(src,'xlabel');
    ylab=get(src,'ylabel');
    zlab=get(src,'zlabel');
    set([xlab ylab zlab],'color',handles.colors.axes{udata.format.bg}(5,:))
end

if ~isempty(xvar)
    xticks=findall(src,'tag','xticklabel');
    delete(xticks)
    if xvar>0
        set(src,'xticklabelmode','manual','xticklabel','');
        switch handles.datatype{d}{xvar}
            case {'constant','continuous','integer'}
                set(src,'xtickmode','auto');
                tickvalues=get(src,'xtick');
                tickvalues(abs(tickvalues)<10^-15)=0;
                thesetexts=cellfun(@num2str,num2cell(tickvalues),'uniformoutput',false);
                %thesetexts=cellfun(@raveformatnumber,num2cell(get(src,'xtick')),'uniformoutput',false);
                if ~iscell(thesetexts);
                    thesetexts=mat2cell(thesetexts,ones(size(thesetexts,1),1),size(thesetexts,2));
                    thesetexts=cellfun(@strtrim,thesetexts,'uniformoutput',false);
                end
                ticknumbers=get(src,'xtick');
                ravemakecustomticks(src,'x',ticknumbers,thesetexts);
            case 'discrete'
                discretez=handles.discretevalues{d}{a}{xvar};
                if numel(discretez)<=30;
                    thesetexts=cellfun(@num2str,num2cell(handles.discretevalues{d}{a}{xvar}),'uniformoutput',false);
                    ticknumbers=handles.discretevalues{d}{a}{xvar};
                    set(src,'xtick',ticknumbers);
                    ravemakecustomticks(src,'x',ticknumbers,thesetexts);
                else %too many, just treat as continuous
                    set(src,'xtickmode','auto');
                    thesetexts=cellfun(@num2str,num2cell(get(src,'xtick')),'uniformoutput',false);
                    if ~iscell(thesetexts);
                        thesetexts=mat2cell(thesetexts,ones(size(thesetexts,1),1),size(thesetexts,2));
                        thesetexts=cellfun(@strtrim,thesetexts,'uniformoutput',false);
                    end
                    ticknumbers=get(src,'xtick');
                    ravemakecustomticks(src,'x',ticknumbers,thesetexts);
                end
            case 'string'
                thesetexts=handles.discretevalues{d}{a}{xvar};
                ticknumbers=1:numel(thesetexts);
                set(src,'xtick',ticknumbers,'xtickmode','manual');
                ravemakecustomticks(src,'x',ticknumbers,thesetexts);
            case 'logical'
                thesetexts={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                ticknumbers=[0 1];
                ravemakecustomticks(src,'x',ticknumbers,thesetexts);
        end
        
        %allow auto updates:
        switch handles.datatype{d}{xvar}
            case {'constant','continuous','integer'}
                set(src,'xtickmode','auto')
        end
        
    else
        set(src,'xticklabelmode','manual','xticklabel','');
        thesetexts=xstr;
        ticknumbers=get(src,'xtick');
        ravemakecustomticks(src,'x',ticknumbers,thesetexts);
    end
end

if ~isempty(yvar)
    yticks=findall(src,'tag','yticklabel');
    delete(yticks)
    if yvar>0
        set(src,'yticklabelmode','manual','yticklabel','');
        switch handles.datatype{d}{yvar}
            case {'constant','continuous','integer'}
                set(src,'ytickmode','auto');
                tickvalues=get(src,'ytick');
                tickvalues(abs(tickvalues)<10^-15)=0;
                thesetexts=cellfun(@num2str,num2cell(tickvalues),'uniformoutput',false);
                if ~iscell(thesetexts);
                    thesetexts=mat2cell(thesetexts,ones(size(thesetexts,1),1),size(thesetexts,2));
                    thesetexts=cellfun(@strtrim,thesetexts,'uniformoutput',false);
                end
                ticknumbers=get(src,'ytick');
                ravemakecustomticks(src,'y',ticknumbers,thesetexts);
            case 'discrete'
                discretez=handles.discretevalues{d}{a}{yvar};
                if numel(discretez)<=30;
                    thesetexts=cellfun(@num2str,num2cell(handles.discretevalues{d}{a}{yvar}),'uniformoutput',false);
                    ticknumbers=handles.discretevalues{d}{a}{yvar};
                    ravemakecustomticks(src,'y',ticknumbers,thesetexts);
                else %too many, just treat as continuous
                    set(src,'ytickmode','auto');
                    thesetexts=cellfun(@num2str,num2cell(get(src,'ytick')),'uniformoutput',false);
                    if ~iscell(thesetexts);
                        thesetexts=mat2cell(thesetexts,ones(size(thesetexts,1),1),size(thesetexts,2));
                        thesetexts=cellfun(@strtrim,thesetexts,'uniformoutput',false);
                    end
                    ticknumbers=get(src,'ytick');
                    ravemakecustomticks(src,'y',ticknumbers,thesetexts);
                end
                
            case 'string'
                thesetexts=handles.discretevalues{d}{a}{yvar};
                ticknumbers=1:numel(thesetexts);
                set(src,'ytick',ticknumbers,'ytickmode','manual');
                ravemakecustomticks(src,'y',ticknumbers,thesetexts);
            case 'logical'
                thesetexts={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                ticknumbers=[0 1];
                ravemakecustomticks(src,'y',ticknumbers,thesetexts);
        end
        
        %allow auto updates:
        switch handles.datatype{d}{yvar}
            case {'constant','continuous','integer'}
                set(src,'ytickmode','auto')
        end
        
    else
        set(src,'yticklabelmode','manual','yticklabel','');
        thesetexts=ystr;
        ticknumbers=get(src,'ytick');
        ravemakecustomticks(src,'y',ticknumbers,thesetexts);
    end
end

if ~isempty(zvar)
    zticks=findall(src,'tag','zticklabel');
    delete(zticks)
    if zvar>0
        set(src,'zticklabelmode','manual','zticklabel','');
        switch handles.datatype{d}{zvar}
            case {'constant','continuous','integer'}
                set(src,'ztickmode','auto');
                thesetexts=cellfun(@num2str,num2cell(get(src,'ztick')),'uniformoutput',false);
                if ~iscell(thesetexts);
                    thesetexts=mat2cell(thesetexts,ones(size(thesetexts,1),1),size(thesetexts,2));
                    thesetexts=cellfun(@strtrim,thesetexts,'uniformoutput',false);
                end
                ticknumbers=get(src,'ztick');
                ravemakecustomticks(src,'z',ticknumbers,thesetexts);
            case 'discrete'
                discretez=handles.discretevalues{d}{a}{zvar};
                if numel(discretez)<=30;
                    thesetexts=cellfun(@num2str,num2cell(handles.discretevalues{d}{a}{zvar}),'uniformoutput',false);
                    ticknumbers=handles.discretevalues{d}{a}{zvar};
                    ravemakecustomticks(src,'z',ticknumbers,thesetexts);
                else %too many, just treat as continuous
                    set(src,'ztickmode','auto');
                    thesetexts=cellfun(@num2str,num2cell(get(src,'ztick')),'uniformoutput',false);
                    if ~iscell(thesetexts);
                        thesetexts=mat2cell(thesetexts,ones(size(thesetexts,1),1),size(thesetexts,2));
                        thesetexts=cellfun(@strtrim,thesetexts,'uniformoutput',false);
                    end
                    ticknumbers=get(src,'ztick');
                    ravemakecustomticks(src,'z',ticknumbers,thesetexts);
                end
            case 'string'
                thesetexts=handles.discretevalues{d}{a}{zvar};
                ticknumbers=1:numel(thesetexts);
                set(src,'ztick',ticknumbers,'ztickmode','manual');
                ravemakecustomticks(src,'z',ticknumbers,thesetexts);
            case 'logical'
                thesetexts={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                ticknumbers=[0 1];
                ravemakecustomticks(src,'z',ticknumbers,thesetexts);
        end
    else
        set(src,'zticklabelmode','manual','zticklabel','');
        thesetexts=zstr;
        ticknumbers=get(src,'ztick');
        ravemakecustomticks(src,'z',ticknumbers,thesetexts);
    end
end




