function raverasterize(varargin)

%optional 3rd input is index of graph to rasterize.
%optional 4th input is true to override handles.prefs.alwaysrasterize
%optional 5th input is an alpha value for the image
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


src=varargin{1};
handles=guidata(src);

if nargin>=3
    ai=varargin{3}; %index of graph to be vectorized;
else
    ai=handles.activeindex;
end

if numel(varargin)<4 && ~handles.prefs.alwaysrasterize
    return
end

% udata=get(handles.fakeax(ai),'userdata');
legdata=get(handles.fakerax(ai),'userdata');

position=get(handles.allblockers(ai),'position'); %cell vector of 1x4 position arrays. (if multiple graphs selected)

if ~isempty(handles.subax{ai})
    numsubgraphs=numel(handles.subax{ai});
else
    numsubgraphs=0;
end

% if ~isempty(handles.statsax{ai})
%     numstatsgraphs=numel(handles.statsax{ai});
% else
%     numstatsgraphs=0;
% end

%make a window for each graph:
savewindow=figure('name','Saving...','units','pixels','position',[50 50 position(3) position(4)],...
    'color',[1 1 1],'colormap',get(handles.fig,'colormap'),'visible','off','renderermode','manual',...
    'renderer','painters','inverthardcopy','off');

%get position of the graph and compare to the blocker position:

graphpos=get(handles.allgraphs(ai),'position');

offset=graphpos-position;


%copy graphs to new window:
SfakerAX=copyobj(handles.fakerax(ai),savewindow);
SfakeAX=copyobj(handles.fakeax(ai),savewindow);
SAX=copyobj(handles.allgraphs(ai),savewindow);
SsubfakerAX=copyobj(handles.subfakerax{ai},savewindow);
SsubfakeAX=copyobj(handles.subfakeax{ai},savewindow);
SsubAX=copyobj(handles.subax{ai},savewindow);
%SstatsAX=copyobj(handles.statsax{ai},savewindow);
Slegends=copyobj(legdata(ishandle(legdata)),savewindow);

f=1;
set(SfakerAX,'position',[offset(1) offset(2) graphpos(3) graphpos(4)]);
set(SfakeAX,'position',[offset(1) offset(2) graphpos(3) graphpos(4)]);
set(SAX,'position',[offset(1) offset(2) graphpos(3) graphpos(4)]);

for y=1:numel(Slegends) %for each legend:
    legpos=get(legdata(y),'position');
    legoffset=legpos-position;
    set(Slegends,'position',[legoffset(1) legoffset(2) legpos(3) legpos(4)])
end

if numsubgraphs(f)~=0
    subgraphpos=get(handles.subax{ai(f)},'position');
    if iscell(subgraphpos);subgraphpos=cell2mat(subgraphpos);end;
    %subgraphpos=cell2mat(get(handles.subax{ai(f)},'position'));
    suboffset=bsxfun(@minus,subgraphpos,position);
    newpos=[suboffset(:,1) suboffset(:,2) subgraphpos(:,3) subgraphpos(:,4)];
    
    for g=1:numsubgraphs(f)
        try %this gives an error for tables, which use subax but not subfakeax/subfakerax
            set(SsubfakerAX(g),'position',newpos(g,:));
            set(SsubfakeAX(g),'position',newpos(g,:));
            set(SsubAX(g),'position',newpos(g,:));
        end
    end
end

%adjust painters object sizes for zbuffer:
%targets=get(get(SAX,'children'),'children');
resolution=get(0,'ScreenPixelsPerInch');
set(savewindow,'papersize',position(3:4)/resolution,'paperposition',[0 0 position(3:4)/resolution]);
filename='raverasterTEMP.tif';
idata = hardcopy(savewindow,'-dzbuffer','-r96');
%imwrite(imdata,'.\raverasterTEMP.tif','tiff','Resolution',96);
close(savewindow)


%get cdata of this image:
%idata=imread(filename);

if numel(varargin)==5
    alpha=varargin{5};
    idata=alpha.*idata + (1-alpha).*ones(size(idata));
end



%send graph to the boneyard:
udata=get(handles.fakeax(ai),'userdata'); %#ok this line may be used by the eval statements!
%hide main axes:
allparents=[handles.allgraphs(ai);handles.fakeax(ai);handles.fakerax(ai);...
    handles.subax{ai}(:);handles.subfakeax{ai}(:);handles.subfakerax{ai}(:)];
%move other graph-specific objects:
graphtype=get(handles.allgraphs(ai),'userdata');
if isfield(handles.graphinfo.(graphtype),'deletehandles')
    deletehandles=handles.graphinfo.(graphtype).deletehandles;
    for f=1:numel(deletehandles)
        thesehandles=eval([deletehandles{f},';']);
        allparents=[allparents;thesehandles(:)];  %#ok<AGROW>
    end
end

allparents=findobj(allparents,'parent',handles.wspanel,'-depth',1);

lastkids=get(handles.boneyard,'children');
set(allparents,'parent',handles.boneyard);
set(handles.boneyard,'children',[lastkids;allparents]);
set(handles.allblockers(ai),'visible','off')
%set(handles.expblockers(ai),'visible','off')
set(handles.grabbers{ai},'visible','off')


% %hide main axes:
% allparents=[handles.allgraphs(ai);handles.fakeax(ai);handles.fakerax(ai);...
%     handles.subax{ai}(:);handles.subfakeax{ai}(:);handles.subfakerax{ai}(:);...
%     handles.statsax{ai}(:)];
% allkidz=get(allparents,'children');
% if iscell(allkidz);
%     allkidz=cell2mat(allkidz);
% end
%
% visibiliteez=get([allparents;allkidz],'visible');
% hidedeez=[allparents;allkidz];
% set(hidedeez,'visible','off');

%show image in fakeax:

%targetpos=get(handles.activegraph,'position');
targetpos=get(handles.allblockers(ai),'position');
targetpos(3:4)=[size(idata,2) size(idata,1)];
targetpos=targetpos + [2 2 -2 -2];

%handles.rasters are the axes. the raster itself has an unstored handle.
if isempty(handles.rasters{ai})
    handles.rasters{ai}=axes('parent',handles.wspanel,'units','pixels','position',targetpos,'visible','off','hittest','off');
    image(idata(2:end-1,2:end-1,:),'parent',handles.rasters{ai},'hittest','on','buttondownfcn',{@raveselectgraph},'userdata',ai);
else %raster already exists, modify it...
    iii=get(handles.rasters{ai},'children');
    set(iii,'cdata',idata(2:end-1,2:end-1,:))
end
    

set(handles.rasters{ai},'visible','off')
set(handles.expblockers(ai),'facecolor',[.5 .5 .5]);

guidata(src,handles);


