function ravereplacedataset(src,d,newdata,isselected,isvisible,rowcolors,keepifsamerownum,evaluatefunctions)
%Replaces the data set in index d with newdata.
% isselected is an rx1 to indicate whether to select the new data, or is
%     empty to unselect all.
% isvisible is an rx1 vector indicating that new data is visible or not, or
%     is empty to show all
% rowcolors is a rx1 vector of row color indices, or is empty to use the
%     default color for all rows.
% minfilter is a 1xc vector of minfilter values, or is empty to use the min
%     limits
% maxfilter is the same thing for max filter.
% discrete filter is a cell array of which discrete values should be
% filtered, or is empty to unfilter all discrete values
% keepifsamerownum is 0 or 1. If 1, and newdata has the same number of rows
%     as the data it is replacing, all other inputs will be ignored and none of
%     these settings will be changed from their previous values. The only
%     change that will be made is that newdata replaces the old data.
% evaluatefunctions is 0 or 1, indicating whether newdata already includes
%     the correct values of all function variables (=0) or whether they should
%     be evaluated by this function (=1);
%
% IMPORTANT: This function does not call raveupdatedataset. You should do
% that immediately after you call this function.
%evaluate functions

handles=guidata(src);

if isempty(newdata)
    handles.alldata{d}=[];
    for a=1:numel(handles.analysisnames)
        handles.isselected{d}{a}=[];
        handles.isvisible{d}{a}=[];
        handles.isfiltered{d}{a}=[];
    end
elseif size(newdata,2)==size(handles.alldata{d},2) %data set is valid

    %evaluate functions?
    if evaluatefunctions
        index=find(handles.isfunction{d});
        outs=raveevaluatefunction(handles.fig,d,index,newdata);
        newdata(:,index)=outs;
    end
    
    %replace data:
    noldrows=size(handles.alldata{d},1);
    nrows=size(newdata,1);
    handles.alldata{d}=newdata;
    handles.alltext{d}=cell(size(newdata));
    handles.isvalid{d}=true(size(newdata));
    
    %ignore all?
    if keepifsamerownum && noldrows==nrows;
        guidata
        return
    end

    %Update Analyses:
    for a=1:numel(handles.analysisnames)
        
        %Row colors?
        if ~isempty(rowcolors)
            handles.rowcolors{d}{a}=rowcolors;
        else
            handles.rowcolors{d}{a}=handles.prefs.defaultrowcolor*ones(nrows,1);
        end
        
        %visibility?
        if ~isempty(isvisible)
            handles.isvisible{d}{a}=isvisible;
        else
            handles.isvisible{d}{a}=true(nrows,1);
        end
        
        %selection?
        if ~isempty(isselected)
            handles.isselected{d}{a}=isselected;
        else
            handles.isselected{d}{a}=false(nrows,1);
        end
        
        %check filters:
        guidata(handles.fig,handles);
        handles.isfiltered{d}{a}=ravecalculatefilteredstate(src,[],d,a);

        %Reset points of interest:
        handles.ispointofinterest{d}=false(nrows,1);
    end

else
    error('ravereplacedataset attempted to replace a data set with a new data set that has the wrong number of columns')
end

guidata(handles.fig,handles);

