function raverestoregraph(src,ev)
%restore a graph that has been put away.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);

set(0,'userdata',[]);

screensize=get(handles.fig,'position');

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-250 screensize(2)+screensize(4)/2-75 500 150],...
    'numbertitle','off','menubar','none','name','Restore Graph','color',handles.colors.frame,...
    'resize','off','windowstyle','modal');
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[20 80 225 40],...
    'string','Choose the graph you would like to restore: ','horizontalalignment','left')
uicontrol('style','popupmenu','string',handles.graphnames(handles.putaway),'backgroundcolor',[1 1 1],'units','pixels',...
    'position',[30 70 440 22],'tag','namebox','horizontalalignment','left')
uicontrol('style','pushbutton','string','Restore','parent',askbox,'units','pixels',...
    'position',[270 10 100 30],'callback',@restorepopup_callback,'fontsize',10);
uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
    'position',[380 10 100 30],'callback',@restorepopup_callback,'fontsize',10);

uiwait(askbox)

index=get(0,'userdata');


if isempty(index)
    return %user hit cancel
end

ai=handles.putaway(index);
handles.putaway=setdiff(handles.putaway,ai);

handles.temp.ai=ai;

%else prompt for placment of this graph:
set(handles.infobar,'string','Click and drag to place this graph.')
set(handles.mainaxes,'buttondownfcn',{@raveplacegraph,handles})
set(handles.explorer,'buttondownfcn',{@raveplacegraph,handles})

guidata(src,handles)



function raveplacegraph(src,eventdata,handles) %#ok<INUSL>
set(handles.fig,'windowbuttondownfcn',[]);
buttons=findobj(handles.tabobjects,'type','uicontrol');
set(buttons,'enable','on');

%check that they clicked on mainaxes:
if ~any([gco==handles.mainaxes])
    return
end

clicked=get(handles.mainaxes,'currentpoint');

xclick=clicked(1,1,1);
yclick=clicked(1,2,1);
handles.temp.startx=xclick;
handles.temp.starty=yclick;

xclick=handles.snapsize*floor(xclick/handles.snapsize);
yclick=handles.snapsize*floor(yclick/handles.snapsize);

handles.temp.putgraphhere(1)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',[1 .5 0],...
    'parent',handles.mainaxes,'hittest','on','buttondownfcn','','facecolor',handles.colors.wsbg);
handles.temp.putgraphhere(2)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',[1 .5 0],...
    'parent',handles.explorer,'hittest','on','buttondownfcn','');

%allow dragging
set(handles.fig,'WindowButtonMotionFcn',@ravesizegraph);
set(handles.fig,'WindowButtonUpFcn',@ravesizegraphdone);
set(handles.fig,'windowbuttondownfcn',[]);

guidata(src,handles);

function ravesizegraph(src,eventdata) %#ok<INUSD>

handles=guidata(src);

finish=get(handles.mainaxes,'currentpoint');
starty=handles.temp.starty;
startx=handles.temp.startx;
endx=finish(1,1,1);
endy=finish(1,2,1);

pos1=handles.snapsize*floor(min(startx,endx)/handles.snapsize);
pos3=handles.snapsize*ceil(max(startx,endx)/handles.snapsize)-pos1;
pos2=handles.snapsize*floor(min(starty,endy)/handles.snapsize);
pos4=handles.snapsize*ceil(max(starty,endy)/handles.snapsize)-pos2;

widthinches=sprintf('%6.2f',round(pos3/96*4)/4);
heightinches=sprintf('%6.2f',round(pos4/96*4)/4);

set(handles.temp.putgraphhere,'position',[pos1 pos2 pos3 pos4]);

set(handles.infobar,'string',['Width=',widthinches,'" Height=',heightinches,'"  at 96 dpi'])

function ravesizegraphdone(src,eventdata) %#ok<INUSD>
handles=guidata(src);
%stop dragging

set(handles.fig,'WindowButtonMotionFcn',@ravedefaultwindowbuttonmotionfcn);
set(handles.fig,'WindowButtonUpFcn',[]);
set(handles.fig,'WindowButtondownFcn',[]);

%make graph in this position
newpos=get(handles.temp.putgraphhere(1),'position');
delete(handles.temp.putgraphhere);

set(handles.allblockers(handles.temp.ai),'position',newpos,'visible','on')
set(handles.expblockers(handles.temp.ai),'position',newpos,'visible','on')
set(handles.rasters{handles.temp.ai},'position',newpos+[20 20 -40 -40],'visible','on')

%grabber order: [SW NW NE SE]
set(handles.grabbers{handles.temp.ai},'visible','on')
set(handles.grabbers{handles.temp.ai}(1),'xdata',newpos(1)+2,'ydata',newpos(2)+3)
set(handles.grabbers{handles.temp.ai}(2),'xdata',newpos(1)+2,'ydata',newpos(2)+newpos(4)-2)
set(handles.grabbers{handles.temp.ai}(3),'xdata',newpos(1)+newpos(3)-3,'ydata',newpos(2)+newpos(4)-2)
set(handles.grabbers{handles.temp.ai}(4),'xdata',newpos(1)+newpos(3)-3,'ydata',newpos(2)+3)

%bring back the graph:
ai=handles.temp.ai;
udata=get(handles.fakeax(ai),'userdata'); %#ok this line may be used by the eval statements!
%hide main axes:
allparents=[handles.allgraphs(ai);handles.fakeax(ai);handles.fakerax(ai);...
    handles.subax{ai}(:);handles.subfakeax{ai}(:);handles.subfakerax{ai}(:)];
%move other graph-specific objects:
graphtype=get(handles.allgraphs(ai),'userdata');
if isfield(handles.graphinfo.(graphtype),'deletehandles')
    deletehandles=handles.graphinfo.(graphtype).deletehandles;
    for f=1:numel(deletehandles)
        thesehandles=eval([deletehandles{f},';']);
        allparents=[allparents;thesehandles(:)];  %#ok<AGROW>
    end
end

allparents=findobj(allparents,'parent',handles.boneyard,'-depth',1);

lastkids=get(handles.wspanel,'children');
set(allparents,'parent',handles.wspanel);
set(handles.wspanel,'children',[allparents;lastkids]);

set(handles.allblockers(ai),'visible','on')
set(handles.expblockers(ai),'visible','on')

handles.sheet(ai)=handles.activesheet;    
guidata(handles.fig,handles);

%% select this graph:

if ~strcmp(handles.graphinfo.(graphtype).class,'table')
    raveresizeaxes(handles.fig,[],handles.temp.ai);
else %tables
    raveresizetable(handles.fig,[],handles.temp.ai);
end


raveselectgraph(handles.allgraphs(handles.temp.ai),[]);
raveresizeaxes(handles.fig,[],handles.temp.ai);
handles=guidata(handles.fig);
clear ravearrangeobjects

% 
% %turn tick labels back on:
% switch handles.activetype
%     case 'scattermatrix'
%         set(get(handles.allgraphs(handles.activeindex),'children'),'visible','on');
% end

%ditch the temp stuff
handles.temp=[];

%clear the infobar
set(handles.infobar,'string','');

%undo creation function
set(handles.mainaxes,'buttondownfcn',@ravedragview);
set(handles.explorer,'buttondownfcn',@ravemoveview);


function restorepopup_callback(src,ev)

popup=get(src,'parent');
if strcmp(get(src,'string'),'Cancel');
    delete(popup);
    return
end

namebox=findobj(popup,'tag','namebox');
index=get(namebox,'value');

set(0,'userdata',index);
delete(popup);






