function ravesavestate(src,eventdata)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);

set(handles.infobar,'string','')

%ask user for a name for this project.

if handles.tablemode
    msgbox('You cannot save a project while viewing the data table. Switch back to the workspace view, and then save the project.',...
        'Sorry about that...','error');
    return
end

h=ravesavegui([],handles.projectname);
uiwait(h);
name=get(0,'userdata');

%did they cancel?
if isempty(name);return;end

load raveprefs
path=raveprefs.defaultdir;

set(0,'userdata','')

blanktab(handles.fig,[]); %clear tabs so tabobjects don't get saved.
handles=guidata(handles.fig);

%delete tray icon:

pause(0.1) %to avoid java crash
sysTray = java.awt.SystemTray.getSystemTray;
if (sysTray.isSupported)
    java.awt.SystemTray.getSystemTray.remove(handles.trayIcon);
end
pause(0.1)
handles.trayIcon=[];
set(handles.fig,'windowbuttonmotionfcn','')
handles.projectname=name;

%delete handles.optimizetabobjects... easier than trying to find them in
%the structure later:
delete(handles.optimizetabobjects(ishandle(handles.optimizetabobjects)));
guidata(handles.fig,handles);

if ~isdir(fullfile(path,name))
    mkdir(fullfile(path,name));
end

filename=fullfile(path,name,[name '.rve']);

hgS = handle2struct(handles.fig);
SaveVer = '070000'; %if error, change to '080000'
SaveVars.(sprintf('hgS_%s', SaveVer)) = hgS;

%Remove java objects from handles.fig:
%dodelete=false(numel(SaveVars.hgS_070000.children),1);
for p=numel(SaveVars.hgS_070000.children):-1:1
    thistype=SaveVars.hgS_070000.children(p).type;
    if strcmp(thistype,'hgjavacomponent')
        %dodelete(p)=true;
        SaveVars.hgS_070000.children(p)=[];
    end
end


%Remove java objects from handles.wspanel:
figchildren=[SaveVars.hgS_070000.children(:).handle];
mainpanelidx=figchildren==handles.wspanel;

dodelete=false(numel(SaveVars.hgS_070000.children(mainpanelidx).children),1);
for p=1:numel(SaveVars.hgS_070000.children(mainpanelidx).children)
    thistype=SaveVars.hgS_070000.children(mainpanelidx).children(p).type;
    if strcmp(thistype,'hgjavacomponent')
        dodelete(p)=true;
    end
end
SaveVars.hgS_070000.children(dodelete)=[];

%Remove references to standard java objects:
SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.maintable.jtable=[];
SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.maintable.defaultRowHeaderRenderer=[];
SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.maintable.rhmodels=[];
SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.maintable.models=[];
SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.maintable.jscroll=[];
SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.mousetimer=[]; %timer objects cant be saved
SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.sounds=[]; %audioplayer objects cant be saved
SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.optimizetabobjects=[]; %clear saved optimize tab


%Remove references to unsaveable objects:
figchildren=[SaveVars.hgS_070000.children(:).handle];
mainpanelidx=figchildren==handles.wspanel;
wsobjects=[SaveVars.hgS_070000.children(mainpanelidx).children(:).handle];

for ai=1:numel(handles.fakeax)
    if ~ishandle(handles.fakeax);continue;end
    udata=get(handles.fakeax(ai),'userdata');
    if ismember('unsaveable',udata.attributes) %remove this graph from the save structure
        %Check handles.allgraph, handles.fakeax, handles.fakerax for java objects
        checkhandles=[handles.allgraphs(ai),handles.fakeax(ai),handles.fakerax(ai)];
        for j=1:numel(checkhandles)
            if strcmp(get(handles.allgraphs(ai),'type'),'uipanel')
                idx = wsobjects==checkhandles(j);
                kids= SaveVars.hgS_070000.children(mainpanelidx).children(idx).children;
                
                dodelete=false(numel(kids),1);
                for p=1:numel(kids)
                    thistype=kids(p).type;
                    if strcmp(thistype,'hgjavacomponent')
                        dodelete(p)=true;
                    end
                end
                SaveVars.hgS_070000.children(mainpanelidx).children(idx).children(dodelete)=[];
            end
        end
    end

    %Remove any java references in unsaveablehandles:
    thistype=get(handles.allgraphs(ai),'userdata');
    if isfield(handles.graphinfo.(thistype),'unsaveablehandles')
        idx = wsobjects==handles.fakeax(ai); %#ok %THIS IS USED IN THE EVAL BELOW
        for g=1:numel(handles.graphinfo.(thistype).unsaveablehandles)
            thisfield=handles.graphinfo.(thistype).unsaveablehandles{g};
            thisfield=thisfield(7:end); %peel of initial "udata." part
            eval(['SaveVars.hgS_070000.children(mainpanelidx).children(idx).properties.UserData.',thisfield,'=[];']);
        end
    end
end

%For reasons I don't understand, new versions can't save annonymous
%function handles. (Well, they can, but they somehow carry along references to all the java objects we just got rid of)
%Remove these and recreate them later:
SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.constrainthandles=[];



drawnow; %cant hurt
% 
% A=SaveVars;
% A.hgS_070000.properties.ApplicationData.UsedByGUIData_m.constrainthandles=[];
% save(filename,'-struct','A');
% 
% 
% 
% fff=fieldnames(A.hgS_070000.properties.ApplicationData.UsedByGUIData_m)
% 
% for f=numel(fff):-1:1
%     fff{f}
%     A.hgS_070000.properties.ApplicationData.UsedByGUIData_m.(fff{f})=[];
% save(filename,'-struct','A');
% pause(1);
% end




save(filename, '-struct', 'SaveVars');

handles.activedir=path;

%restore tray icon:
if (sysTray.isSupported)
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);
    myIcon = fullfile(raveroot,'images\tinyrave.png');
    iconImage = java.awt.Toolkit.getDefaultToolkit.createImage(myIcon);
    handles.trayIcon = java.awt.TrayIcon(iconImage, 'initial tooltip');
    handles.trayIcon.setToolTip('RAVE is running.');
    sysTray.add(handles.trayIcon)
else
    handles.trayIcon=[];
end

%restore mouse motion function:
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)

ravefinish

guidata(src,handles);
managetab(handles.fig,[]);
handles=guidata(handles.fig);
set(handles.infobar,'string','Project saved successfully.')


