function raveselectmodelgui(src,ev)
%gui to seelct surrogate models for use in rave or export to another
%program.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

if isempty(handles.models);
    warndlg('You have not yet created any models. First, use the "Create New Model" button. Then come back here to add the models to your data set or export them for use in another program.','No models found');
    return
end

load raveprefs

origin=get(handles.fig,'position');

hhh=figure('menubar','none','units','pixels','position',[origin(1)+origin(3)/2-600 origin(2)+origin(4)/2-350 1200 700],'color',raveprefs.colors.frame,...
    'name','Select Models','numbertitle','off','resize','off','windowstyle','normal','renderer','painters');

%get context menu:
%get context menu:
handles.temp.contextmenu=copyobj(handles.contextmenus.modelselectiontable,hhh);



titleaxes=axes('units','pixels','position',[20 650 400 40],'parent',hhh,'visible','off');
text(0,.4,'Compare, select, and export models','parent',titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
    'color',raveprefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');

makehelpblob(hhh,540,765,'surrogateModeling_Selection.html',raveprefs.colors.frame);

u=uipanel('units','pixels','position',[20 300+40 1100 310],'title','','bordertype','none','backgroundcolor',raveprefs.colors.frame,'visible','on');

uicontrol('style','text','parent',u,'units','pixels','position',[10 375-100 200 22],'string','View models for data set:','backgroundcolor',get(u,'backgroundcolor'),...
    'fontsize',10,'fontname','arial','fontweight','normal','horizontalalign','left')

uicontrol('style','popup','parent',u,'units','pixels','position',[166 382-100 200 20],'string',handles.datasetnames,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','callback',@datasetvariablemenu_callback)

uicontrol('style','text','parent',u,'units','pixels','position',[410 375-100 200 22],'string','Show models for variable:','backgroundcolor',get(u,'backgroundcolor'),...
    'fontsize',10,'fontname','arial','fontweight','normal','horizontalalign','left')

uicontrol('style','popup','parent',u,'units','pixels','position',[566 382-100 400 20],'string',[{'ALL VARIABLES'},handles.allnames{1}],...
    'backgroundcolor',[1 1 1],'tag','variablemenu','callback',@datasetvariablemenu_callback)

uicontrol('style','text','parent',u,'units','pixels','position',[10 245 1000 22],'string','Select a model to view its information. You may optionally enter a name for each model. Blue rows have been added to the data set as variables.','backgroundcolor',get(u,'backgroundcolor'),...
    'fontsize',10,'fontname','arial','fontweight','normal','horizontalalign','left')

uicontrol('style','pushbutton','parent',u,'units','pixels','position',[1020 200 150 48],'string','<html><center>Add Selected Model<br>to Data Set</html>',...
    'hor','center','fontsize',10,'fontweight','bold','callback',@selectmodel_callback);

uicontrol('style','pushbutton','parent',u,'units','pixels','position',[1020 200-30-30 150 48],'string','<html><center>Export Selected<br>Model to .m File</html>',...
    'hor','center','fontsize',10,'fontweight','bold','callback',@exportmodel_callback);

uicontrol('style','pushbutton','parent',u,'units','pixels','position',[1020 200-90-30 150 48],'string','<html><center>Compare Selected<br>Models</html>',...
    'hor','center','fontsize',10,'fontweight','bold','visible','off');

uicontrol('style','pushbutton','parent',u,'units','pixels','position',[1020 20 150 48],'string','<html><center>Export Model<br>Report</html>',...
    'hor','center','fontsize',10,'fontweight','bold','callback',@exportmodelfitreport_callback);

%make index of visible models:
handles.temp.modeld=zeros(numel(handles.models),1);
handles.temp.modely=zeros(numel(handles.models),1);
for z=1:numel(handles.models)
    handles.temp.modeld(z)=handles.models{z}.dataset;
    handles.temp.modely(z)=handles.models{z}.output;
end

handles.temp.visiblemodels=find(ismember(handles.temp.modeld,1));
handles.temp.selectedmodel=[];

guidata(gcf,handles);
%make table:
dmod=makemodelstablemodel(gcf,[]);
set(handle(dmod,'callbackproperties'),'TableChangedCallback',{@tableEdited_callback,gcf});

jtable=javax.swing.JTable(dmod);
set(handle(jtable,'callbackproperties'),'MousePressedCallback',{@tableClicked_callback,hhh})
set(handle(jtable,'callbackproperties'),'MouseReleasedCallback',{@tableSelectionChanged_callback,gcf});
set(handle(jtable,'callbackproperties'),'KeyPressedCallback',{@tableSelectionChanged_callback,gcf});

j=javax.swing.JScrollPane(jtable);
%jtable.setFillsViewportHeight(true);
[jscrollpane mscrollpane]=javacomponent(j,[30 360 1000 230],hhh);

% Set column widths:
jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
%jtable.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION); %only allow one row to be selected at a time.
jtable.setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION); %only allow one row to be selected at a time.

colwidths=[150 200 200 430]; %some column widths
for c=1:4 %set the widths:
    jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
end

%renderer:
rend=RaveRowStatusRenderer();
obj=java.lang.Object;
class=obj.getClass();
jtable.setDefaultRenderer(class,rend);

handles.temp.jscroll=jscrollpane;
handles.temp.mscroll=mscrollpane;
handles.temp.jtable=jtable;

uicontrol('style','pushbutton','string','Done','fontsize',16,'fontname','arial','fontweight','normal','parent',gcf,'position',[1080 6 100 28],...
    'callback',@donebutton_callback)


%% fit statistics:
u2=uipanel('units','pixels','position',[0 00 1200 400],'title','','bordertype','none','backgroundcolor',raveprefs.colors.frame,'visible','on');

a=axes('parent',u2,'units','pixels','position',[5 50 250 300],'xlim',[0 250],'ylim',[0 300],'visible','off','xlimmode','manual','ylimmode','manual');

text(20,298,'Fit Statistics','parent',a,'fontname','Arial','fontsize',12,'fontweight','bold');

S(1)=text(25,270,'R^2 (fit)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
S(2)=text(25,250,'R^2 (val)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
S(3)=text(25,230,'R^2 (test)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');

S(4)=text(25,210-5,'RMSE (fit)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
S(5)=text(25,190-5,'RMSE (val)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
S(6)=text(25,170-5,'RMSE (test)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');

S(7)=text(25,150-10,'Mean Error (fit)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
S(8)=text(25,130-10,'Mean Error (val)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
S(9)=text(25,110-10,'Mean Error (test)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');

S(10)=text(25,90-15,'Std Dev Error (fit)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
S(11)=text(25,70-15,'Std Dev Error (val)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
S(12)=text(25,50-15,'Std Dev Error (test)','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');

%line([235 235],[0 300],'color',[.5 .5 .5],'linewidth',1,'parent',a);

%% fit statistic values:
s(1)=text(150,270,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(2)=text(150,250,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(3)=text(150,230,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(4)=text(150,210-5,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(5)=text(150,190-5,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(6)=text(150,170-5,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(7)=text(150,150-10,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(8)=text(150,130-10,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(9)=text(150,110-10,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(10)=text(150,90-15,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(11)=text(150,70-15,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');
s(12)=text(150,50-15,'=','parent',a,'fontname','Arial','fontsize',10,'fontweight','normal','interpreter','tex');

handles.temp.statheadings=S;
handles.temp.stattexts=s;

j=axes('parent',u2,'units','pixels','position',[300-25 325+18 250 25],'xlim',[0 250],'ylim',[0 25],'visible','off','xlimmode','manual','ylimmode','manual');
text(0,5,'Actual vs. Predicted','parent',j,'fontname','arial','fontsize',12,'fontweight','bold');
handles.temp.avpax=axes('parent',u2,'units','pixels','position',[300 70 250 250],'tag','avpax','fontsize',8);

j=axes('parent',u2,'units','pixels','position',[615-25 325+18 250 25],'xlim',[0 250],'ylim',[0 25],'visible','off','xlimmode','manual','ylimmode','manual');
text(0,5,'Residual vs. Predicted','parent',j,'fontname','arial','fontsize',12,'fontweight','bold');
handles.temp.rvpax=axes('parent',u2,'units','pixels','position',[615 70 250 250],'tag','rvpax','fontsize',8);

j=axes('parent',u2,'units','pixels','position',[930-25 325+18 250 25],'xlim',[0 250],'ylim',[0 25],'visible','off','xlimmode','manual','ylimmode','manual');
text(0,5,'Error Histogram','parent',j,'fontname','arial','fontsize',12,'fontweight','bold');
handles.temp.histax=axes('parent',u2,'units','pixels','position',[930 70 250 250],'tag','histax','fontsize',8);

guidata(hhh,handles);


function dmod=makemodelstablemodel(src,ev)
handles=guidata(src);

nmodels=numel(handles.temp.visiblemodels);
z=handles.temp.visiblemodels;

if nmodels<1
    javarray = {'','','',''};
    rowstatus=[];
else
    javarray=javaArray('java.lang.String',nmodels,4);
    rowstatus=javaArray('java.lang.Integer',nmodels);
    for r=1:nmodels
        d=handles.models{r}.dataset;
        javarray(r,1)=java.lang.String(handles.models{z(r)}.name);
        javarray(r,2)=java.lang.String(handles.models{z(r)}.longname);
        if handles.models{z(r)}.output>0
            javarray(r,3)=java.lang.String(handles.allnames{d}{handles.models{z(r)}.output});
        else
            javarray(r,3)=java.lang.String('<No Response Variable>');
        end
        javarray(r,4)=java.lang.String(handles.models{z(r)}.notes);
        switch handles.models{z(r)}.status
            case 'Waiting...'
                rowstatus(r)=java.lang.Integer(2);
            case 'Error'
                rowstatus(r)=java.lang.Integer(1);
            case 'Finished'
                rowstatus(r)=java.lang.Integer(0);
        end
        
        if ~isempty(handles.models{z(r)}.modeledcolumn)
            rowstatus(r)=java.lang.Integer(4);
        end
    end
end
cnames = {'Name','Model Type','Variable','Notes'};

%dmod=javax.swing.table.DefaultTableModel(javarray,cnames);
dmod=RaveDataTypeTableModel(javarray,cnames);
if ~isempty(rowstatus)
    dmod.setRowStatus(rowstatus);
end

function donebutton_callback(src,ev)
close(gcf);

function datasetvariablemenu_callback(src,ev)
handles=guidata(src);

datasetmenu=findobj(gcf,'tag','datasetmenu');
variablemenu=findobj(gcf,'tag','variablemenu');

d=get(datasetmenu,'value');
y=get(variablemenu,'value')-1;

if y==0;
    handles.temp.visiblemodels=find(ismember(handles.temp.modeld,d));
else
    handles.temp.visiblemodels=find(ismember(handles.temp.modeld,d) & ismember(handles.temp.modely,y));
end

guidata(gcf,handles);

%refresh table:
dmod=makemodelstablemodel(gcf,ev);
drawnow;
handles.temp.jtable.setModel(dmod);

colwidths=[150 200 200 446]; %some column widths
for c=1:4 %set the widths:
    handles.temp.jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
end
drawnow;

%reset selection:
if isempty(handles.temp.visiblemodels)
    handles.temp.jtable.clearSelection();
    clearselection(gcf,[]);
else
    handles.temp.jtable.setRowSelectionInterval(0,0);
    tableSelectionChanged_callback(handles.temp.jtable,[],gcf)
end


function tableClicked_callback(jsrc,jev,src)
handles=guidata(src);
if isempty(handles.models);return;end

MouseEvent=jev;

if MouseEvent.getClickCount == 2 %double click
    return  
elseif MouseEvent.getButton > 1    %right click
    handles=guidata(src);
    
    %determine where to put context menu:  
    xpos1=get(handles.temp.mscroll,'position');
    xpos2=MouseEvent.getX;
    
    ypos1=get(handles.temp.mscroll,'position');
    ypos3=MouseEvent.getY;
    ypos4=handles.temp.jscroll.getComponent(0).getComponent(0).getTableHeader.getHeight();

    finalpos=[xpos1(1)+xpos2(1) ypos1(2)+ypos1(4)-ypos3-ypos4];
    set(handles.temp.contextmenu,'position',finalpos,'visible','on');
    set(0,'userdata',MouseEvent);
    
else %presumably single left click
    return
end


function tableSelectionChanged_callback(jsrc,jev,src)
%display statistics for this model.
handles=guidata(src);
jtable=jsrc;
r=jtable.getSelectedRows+1;


delete(get(handles.temp.avpax,'children'));
delete(get(handles.temp.rvpax,'children'));
delete(get(handles.temp.histax,'children'));


if numel(r)==1 && r==0
    clearselection(gcf,[]);
    return
end

if isempty(handles.temp.visiblemodels);
    clearselection(gcf,[]);
    return
end


r=handles.temp.visiblemodels(r);

handles.temp.selectedmodel=r;

if numel(r)>1; %if more than 1 selected, do not display graphs
    clearselection(gcf,[]);
    handles.temp.selectedmodel=r;
    guidata(gcf,handles);
    guidata(handles.fig,handles); 
    return
end

if ~handles.models{r}.usesresponse %is selected model does not use a response, do not display graphs
    clearselection(gcf,[]);
    handles.temp.selectedmodel=r;
    guidata(gcf,handles);
    guidata(handles.fig,handles);
    return
end

    

%if statistics havent been calculated, calculate them. Otherwise, just
%display them:
settings=handles.models{r};
actual_fit=settings.actualvalues(settings.fitrows);
actual_val=settings.actualvalues(settings.validaterows);
actual_test=settings.actualvalues(settings.testrows);

pred_fit=settings.predictedvalues(settings.fitrows);
pred_val=settings.predictedvalues(settings.validaterows);
pred_test=settings.predictedvalues(settings.testrows);


%remove nans:
i1=~isnan(actual_fit)&~isnan(pred_fit);
i2=~isnan(actual_val)&~isnan(pred_val);
i3=~isnan(actual_test)&~isnan(pred_test);

if ~any(i1)
    set(handles.temp.rvpax,'xlim',[0 1],'ylim',[0 1]);
    text(0.5,0.5,{'The selected model is predicting';'NaN for all data points.';'Something must have gone wrong';'when creating the model.'},'parent',handles.temp.rvpax,...
        'hor','center','ver','middle')
    return
end


actual_fit=actual_fit(i1);
actual_val=actual_val(i2);
actual_test=actual_test(i3);

pred_fit=pred_fit(i1);
pred_val=pred_val(i2);
pred_test=pred_test(i3);




if isempty(settings.r2)
    %correlation coefficients:
    r2act=corrcoef(actual_fit,pred_fit);
    r2act=r2act(1,2);
    r2val=corrcoef(actual_val,pred_val);
    r2val=r2val(1,2);
    r2test=corrcoef(actual_test,pred_test);
    if numel(r2test)>1
    r2test=r2test(1,2);
    end
    settings.r2=[r2act r2val r2test];
    %rmse:
    rmse_fit=sqrt(mean((actual_fit-pred_fit).^2));
    rmse_val=sqrt(mean((actual_val-pred_val).^2));
    rmse_test=sqrt(mean((actual_test-pred_test).^2));
    settings.rmse=[rmse_fit rmse_val rmse_test];
    %mean error:
    mu_fit=mean(actual_fit-pred_fit);
    mu_val=mean(actual_val-pred_val);
    mu_test=mean(actual_test-pred_test);
    settings.meanerror=[mu_fit mu_val mu_test];
    %sigma error:
    std_fit=std(actual_fit-pred_fit);
    std_val=std(actual_val-pred_val);
    std_test=std(actual_test-pred_test);
    settings.sigmaerror=[std_fit std_val std_test];
    
    handles.models{r}=settings;
    guidata(gcf,handles);
    guidata(handles.fig,handles);
end

%display in table:
S=handles.temp.statheadings;
s=handles.temp.stattexts;

if settings.usesvalidation
    set(S(2),'string','R^2 (val)');
    set(S(3),'string','R^2 (test)');
    set(S(5),'string','RMSE (val)');
    set(S(6),'string','RMSE (test)');
    set(S(8),'string','Mean Error (val)');
    set(S(9),'string','Mean Error (test)');
    set(S(11),'string','Std Dev Error (val)');
    set(S(12),'string','Std Dev Error (test)');
    
    alltexts=[settings.r2(:);settings.rmse(:);settings.meanerror(:);settings.sigmaerror(:)];
    for i=1:8
        if isnan(alltexts(i))
            set(s(i),'string','= NaN')
        else
            set(s(i),'string',sprintf('%6.6g',alltexts(i)))
        end
    end
    
else
    set(S(2),'string','R^2 (test)');
    set(S(3),'string','');
    set(S(5),'string','RMSE (test)');
    set(S(6),'string','');
    set(S(8),'string','Mean Error (test)');
    set(S(9),'string','');
    set(S(11),'string','Std Dev Error (test)');
    set(S(12),'string','');
    
    alltexts=[settings.r2(:);settings.rmse(:);settings.meanerror(:);settings.sigmaerror(:)];
    idx=[1 3 4 6 7 9 10 12];
    udx=[1 2 4 5 7 8 10 11];
    for i=1:8
        if isnan(alltexts(idx(i)))
            set(s(udx(i)),'string','')
        else
            set(s(udx(i)),'string',strtrim(sprintf('%6.6g',alltexts(idx(i)))))
        end
    end
    set(s([3 6 9 12]),'string','')
end

%actual vs predicted:
delete(get(handles.temp.avpax,'children'));
line([min([actual_fit;actual_test;actual_val;pred_fit;pred_test;pred_val]),max([actual_fit;actual_test;actual_val;pred_fit;pred_test;pred_val])],...
    [min([actual_fit;actual_test;actual_val;pred_fit;pred_test;pred_val]),max([actual_fit;actual_test;actual_val;pred_fit;pred_test;pred_val])],...
    'linestyle','-','color',[.7 .7 .7],'parent',handles.temp.avpax);

line(pred_test,actual_test,'parent',handles.temp.avpax,'linestyle','none',handles.prefs.fitreport.testmarker{:});
line(pred_val,actual_val,'parent',handles.temp.avpax,'linestyle','none',handles.prefs.fitreport.valmarker{:});
line(pred_fit,actual_fit,'parent',handles.temp.avpax,'linestyle','none',handles.prefs.fitreport.fitmarker{:});

axis(handles.temp.avpax,'equal')
xlabel(handles.temp.avpax,'Model Predicted Value');
ylabel(handles.temp.avpax,'Actual Value');


%residual vs predicted:
delete(get(handles.temp.rvpax,'children'));
line([min([pred_fit;pred_test;pred_val]),max([pred_fit;pred_test;pred_val])],...
    [0,0],...
    'linestyle','-','color',[.7 .7 .7],'parent',handles.temp.rvpax);

line(pred_test,actual_test-pred_test,'parent',handles.temp.rvpax,'linestyle','none',handles.prefs.fitreport.testmarker{:});
line(pred_val,actual_val-pred_val,'parent',handles.temp.rvpax,'linestyle','none',handles.prefs.fitreport.valmarker{:});
line(pred_fit,actual_fit-pred_fit,'parent',handles.temp.rvpax,'linestyle','none',handles.prefs.fitreport.fitmarker{:});

if min([pred_fit;pred_test;pred_val]) < max([pred_fit;pred_test;pred_val])
    set(handles.temp.rvpax,'xlim',[min([pred_fit;pred_test;pred_val]),max([pred_fit;pred_test;pred_val])],'ylimmode','auto');
elseif ~all(isnan(pred_fit))
    set(handles.temp.rvpax,'xlim',[min([pred_fit;pred_test;pred_val]),min([pred_fit;pred_test;pred_val])+1],'ylimmode','auto');
else
    set(handles.temp.rvpax,'xlim',[0 1],'ylim',[0 1]);
    text(0.5,0.5,{'The selected model is predicting';'NaN for all data points.';'Something must have gone wrong';'when creating the model.'},'parent',handles.temp.rvpax,...
        'hor','center','ver','middle')
end


xlabel(handles.temp.rvpax,'Model Predicted Value');
ylabel(handles.temp.rvpax,'Residual');


%error histogram:
allerror=settings.actualvalues-settings.predictedvalues;
binz=linspace(min(allerror),max(allerror),40);

histc(actual_fit-pred_fit,binz);

countz=zeros(numel(binz),3);
countz(:,1)=histc(actual_fit-pred_fit,binz);
countz(:,2)=histc(actual_val-pred_val,binz);
countz(:,3)=histc(actual_test-pred_test,binz);

bar(binz,countz,'stacked','parent',handles.temp.histax,'barwidth',1,'edgecolor','none');
colormap([handles.prefs.fitreport.fitcolor;handles.prefs.fitreport.valcolor;handles.prefs.fitreport.testcolor])
xlabel(handles.temp.histax,'Residual');
ylabel(handles.temp.histax,'Count');

guidata(gcf,handles);
guidata(handles.fig,handles);

function tableEdited_callback(jsrc,jev,src)
handles=guidata(src);

if jev.getColumn==0 %update name
    r=jev.getFirstRow;
    R=handles.temp.visiblemodels(r+1);
    handles.models{R}.name=jsrc.getValueAt(r,0);
elseif jev.getColumn==3 %update notes
    r=jev.getFirstRow;
    R=handles.temp.visiblemodels(r+1);
    handles.models{R}.notes=jsrc.getValueAt(r,3);
end

guidata(gcf,handles);
guidata(handles.fig,handles);


function clearselection(src,ev)
%clear graphs so that no model is displayed.
handles=guidata(src);

s=handles.temp.stattexts;
for i=1:12
    set(s(i),'string','=')
end

delete(get(handles.temp.avpax,'children'));
delete(get(handles.temp.rvpax,'children'));
delete(get(handles.temp.histax,'children'));

handles.temp.selectedmodel=[];
guidata(gcf,handles);

function selectmodel_callback(src,ev)
%add model to data set.

handles=guidata(src);
if isempty(handles.temp.selectedmodel);return;end

r=handles.temp.selectedmodel;

if ~handles.models{r}.usesresponse
    warndlg('The selected model cannot be added to the dataset because it does not calculate values of an output variable. This is not an error, rather it is a property of the selected model. Generally models with this property are instead accessed directly by special types of workspace objects, optimizers, or other models.','Model cannot be added to data set')
    return 
end

if numel(r)>1
    for j=1:numel(r)
        names{j}=handles.models{r(j)}.name;
    end
    
   if any(cellfun(@isempty,names))
       errordlg('Multiple models cannot be added to the data set unless each selected model has been named. Enter a name for each model in the first column of the table.')
       return       
   end
end

for j=1:numel(r)
settings=handles.models{r(j)};

if numel(r)==1;
    if isempty(settings.modeledcolumn) %if model has been named but hasnt been made into a column, suggest using that name
        if ~isempty(settings.name)
        answer=inputdlg('Enter a name for the new variable','Add column',1,{settings.name});
        else
            answer=inputdlg('Enter a name for the new variable','Add column',1,{[handles.datasetnames{settings.dataset},'_',handles.allnames{settings.dataset}{settings.output}]});
        end
    else %dont suggest any name
        answer=inputdlg('Enter a name for the new variable','Add column');
    end
else
    answer=names(j);
end


if isempty(answer);return;end
answer=answer{1};
funname=strrep(answer,' ','');
if isempty(funname);return;end
answer2=genvarname(funname);
if ~strcmp(answer2,funname);
    warndlg('You must enter a valid variable name. It cannot begin with a number or contain symbols.');
    return
end

%make temporary file:
modelindex=handles.temp.selectedmodel;
filename=fullfile(handles.prefs.defaultdir,'modelfun.m');
VarNameFormatted='y';
makemodelmfile(gcf,[],modelindex,filename,VarNameFormatted);

%copy to real file:
filename=fullfile(handles.prefs.defaultdir,[funname,'FUN.m']);
fid=fopen(filename,'w+'); %note, this line gives an error on new matlab installs, maybe because old defaultdir isnt on new path.

fprintf(fid,'function %s=%sFUN(inputs,prefs,targets)\n',funname,funname); %error here on new install, see fid=fopen... line above.
fprintf(fid,'%s=modelfun(inputs,prefs,targets);\n\n',funname); %error here on new install, see fid=fopen... line above.

fid2=fopen(fullfile(handles.prefs.defaultdir,'modelfun.m'),'r');
tline = fgetl(fid2);
while ischar(tline)
    fprintf(fid,'%s\n',tline);
    tline = fgetl(fid2);
end

fclose all;

%check for a function with the same name higher on the path:
fcn=which([funname,'FUN']);
[dir,junk,junk2]=fileparts(fcn);

if ~strcmpi(fcn,fullfile(handles.prefs.defaultdir,[funname,'FUN.m']))
    warndlg({['The variable name you selected could not be used because there is another function ',...
        'with the same name higher on your MATLAB search path. Rename the variable or remove other ',...
        'functions with the same name and try again.']; '(To see all functions on your path with this name, type:';['"which ',...
        funname,'FUN -all" at the MATLAB command prompt.)']},'I can''t let you do that');
    return
end



delete(fullfile(handles.prefs.defaultdir,'modelfun.m'));
rehash;

if isempty(settings.name)
    handles.models{r(j)}.name=answer;
end

% add column to handles:
d=settings.dataset;

%evaluate function:
funhandle=str2func([funname,'FUN']);
allvalues=cell(1,nargout(funhandle));
prefs=handles.preferences{d};
targets=handles.targetvalues{d};
if nargin(funhandle)==1
    [allvalues{:}]=funhandle(handles.alldata{d});
else %should have 3 inputs
    [allvalues{:}]=funhandle(handles.alldata{d},prefs,targets);
end

%update fields:
newvalues=allvalues{1}(:,1); %just take first column, this protects against array outputs, which rave cant handle.

handles.alldata{d}(:,end+1)=newvalues;
handles.isvalid{d}(:,end+1)=true;
handles.alltext{d}=[handles.alltext{d},cell(length(newvalues),1)];
handles.allnames{d}{end+1}=answer;
handles.datasetlinks{d}{end+1}=[];
handles.datasetlinkinfo{d}{end+1}={};
handles.variablecolors{d}(end+1,:)=rand(1,3);
handles.isresponse{d}(end+1)=1;
handles.allunits{d}{end+1}=''; %havent implemented this yet
handles.isincluded{d}(end+1)=1;
handles.isdecided{d}(end+1)=0;
handles.isconstant{d}(end+1)=0; %note, should really use an if statement here to check if it's constant.
handles.isfunction{d}(end+1)=1;
handles.funclass{d}{end+1}='matlab_generic';
handles.funnames{d}{end+1}=[funname,'FUN'];
handles.funvars{d}{end+1}=sort(settings.inputs);
handles.nargout{d}(end+1)=1;
handles.datatype{d}{end+1}='continuous';
handles.isnoise{d}(end+1)=false;
handles.distname{d}{end+1}='uniform';
handles.distparams{d}{end+1}=[min(newvalues) max(newvalues)];
handles.preferences{d}(end+1)=0;
handles.targetvalues{d}(end+1)=-inf;
handles.isobjective{d}(end+1)=false;
handles.maintable.colwidths{d}(end+1)=handles.prefs.colwidth;
for a=1:numel(handles.analysisnames)
    handles.xdials{d}{a}(end+1)=0;
    handles.minlimit{d}{a}(end+1)=min(newvalues);
    handles.maxlimit{d}{a}(end+1)=max(newvalues);
    handles.minfilter{d}{a}(end+1)=min(newvalues);
    handles.maxfilter{d}{a}(end+1)=max(newvalues);
    handles.discretevalues{d}{a}{end+1}=[];
end

handles.models{r(j)}.modeledcolumn=union(size(handles.alldata{d},2),handles.models{r(j)}.modeledcolumn);


guidata(gcf,handles);
guidata(handles.fig,handles);
set(handles.infobar,'string','Function loaded successfully.');

%update maintable udata to display new columns:
handles.maintable.x{d}=union(handles.maintable.x{d},size(handles.alldata{d},2));
raveupdatedataset(handles.fig,[],'addcolumns',[],d);
ravefinish; %make a done sound.


%update tablemodel to show that this model has been added to the data set:
dmod=makemodelstablemodel(handles.fig,[]);
drawnow;
handles.temp.jtable.setModel(dmod);
drawnow;
colwidths=[150 200 200 446]; %some column widths
for c=1:4 %set the widths:
    handles.temp.jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
end
drawnow;

end


function exportmodel_callback(src,ev)
handles=guidata(src);

if isempty(handles.temp.selectedmodel);return;end



if handles.models{handles.temp.selectedmodel}.output==0
    warndlg('Rave cannot (yet) export .m files for models that do not calculate values of an output variable.','File cannot be saved')
    return 
end




[name,path] = uiputfile('*.m','Save as','');

if ~ischar(name);return;end
filename=fullfile(path,name);
modelindex=handles.temp.selectedmodel;
VarNameFormatted='y';
makemodelmfile(gcf,[],modelindex,filename,VarNameFormatted);


function exportmodelfitreport_callback(src,ev)
handles=guidata(src);

if isempty(handles.temp.selectedmodel);return;end

%makefolder:
direc=['Fit Report ',datestr(now)];
direc=strrep(direc,':','.');

mkdir(handles.prefs.defaultdir,direc);

%save graphs:
savewindow=figure('name','Saving... DO NOT CLOSE THIS WINDOW','units','inches','position',[1 1 3 3],...
    'color',[1 1 1],'colormap',get(handles.fig,'colormap'),'visible','off',...
    'renderer','painters','inverthardcopy','off','numbertitle','off','menubar','none','toolbar','none');
set(savewindow,'colormap',[handles.prefs.fitreport.fitcolor;handles.prefs.fitreport.valcolor;handles.prefs.fitreport.testcolor])
figpos=getpixelposition(savewindow);
resolution=get(0,'ScreenPixelsPerInch');
set(savewindow,'paperunits','inches','papersize',figpos(3:4)/resolution,'paperposition',[0 0 figpos(3:4)/resolution]);
path=fullfile(handles.prefs.defaultdir,direc);

%avp:
h=copyobj(handles.temp.avpax,savewindow);
set(h,'units','inches','position',[0.5 0.4 2.4 2.4],handles.prefs.fitreport.font{:});
set([get(h,'xlabel') get(h,'ylabel')],handles.prefs.fitreport.font{:});

print(savewindow,fullfile(path,'avp.png'),'-dpng','-r600','-opengl')
delete(h);

%rvp:
h=copyobj(handles.temp.rvpax,savewindow);
set(h,'units','inches','position',[0.5 0.4 2.4 2.4],handles.prefs.fitreport.font{:});
set([get(h,'xlabel') get(h,'ylabel')],handles.prefs.fitreport.font{:});
print(savewindow,fullfile(path,'rvp.png'),'-dpng','-r600','-opengl')
delete(h);

%error histogram:
h=copyobj(handles.temp.histax,savewindow);
set(h,'units','inches','position',[0.5 0.4 2.4 2.4],handles.prefs.fitreport.font{:});
set([get(h,'xlabel') get(h,'ylabel')],handles.prefs.fitreport.font{:});
print(savewindow,fullfile(path,'hist.png'),'-dpng','-r600','-opengl')
delete(h);

delete(savewindow);

%text info:
mod=handles.models{handles.temp.selectedmodel};

if mod.output==0
    warndlg('Rave cannot (yet) export model reports for models that do not calculate values of an output variable.','Model report cannot be saved')
    return 
end


d=mod.dataset;

fid=fopen(fullfile(handles.prefs.defaultdir,direc,'model_info.txt'),'w+'); 
fprintf(fid,'Variable to model:\t%s\r\n',handles.allnames{d}{mod.output});

inputs=handles.allnames{d}(mod.inputs);
inputstr='';
for i=1:numel(inputs)
   inputstr=[inputstr,inputs{i},', ']; %#ok<AGROW>
end
inputstr=inputstr(1:end-2);

fprintf(fid,'Modeled as a function of :\t%s\r\n',inputstr);
fprintf(fid,'Data set:\t%s\r\n',handles.datasetnames{d});
fprintf(fid,'Model type:\t%s\r\n',mod.longname);
fprintf(fid,'Model name:\t%s\r\n',mod.name);
fprintf(fid,'Notes:\t%s\r\n',mod.notes);
fprintf(fid,'R2 (fit):\t%s\r\n',num2str(mod.r2(1)));
fprintf(fid,'R2 (val):\t%s\r\n',num2str(mod.r2(2)));
fprintf(fid,'R2 (test):\t%s\r\n',num2str(mod.r2(3)));
fprintf(fid,'RMSE (fit):\t%s\r\n',num2str(mod.rmse(1)));
fprintf(fid,'RMSE (val):\t%s\r\n',num2str(mod.rmse(2)));
fprintf(fid,'RMSE (test):\t%s\r\n',num2str(mod.rmse(3)));
fprintf(fid,'Mean Error (fit):\t%s\r\n',num2str(mod.meanerror(1)));
fprintf(fid,'Mean Error (val):\t%s\r\n',num2str(mod.meanerror(2)));
fprintf(fid,'Mean Error (test):\t%s\r\n',num2str(mod.meanerror(3)));
fprintf(fid,'Std Dev Error (fit):\t%s\r\n',num2str(mod.sigmaerror(1)));
fprintf(fid,'Std Dev Error (val):\t%s\r\n',num2str(mod.sigmaerror(2)));
fprintf(fid,'Std Dev Error (test):\t%s\r\n',num2str(mod.sigmaerror(3)));
fprintf(fid,'Training Input Data (x):\t%s\r\n',mat2str(mod.allactualvalues));
fprintf(fid,'Training Response Data (y):\t%s\r\n',mat2str(mod.actualvalues));
fprintf(fid,'Model Predicted Response (y):\t%s\r\n',mat2str(mod.predictedvalues));
fprintf(fid,'Fit to Rows:\t%s\r\n',mat2str(double(mod.fitrows)));
fprintf(fid,'Validation Rows:\t%s\r\n',mat2str(double(mod.validaterows)));
fprintf(fid,'Test Rows:\t%s\r\n',mat2str(double(mod.testrows)));
fprintf(fid,'Source Code:\r\n');
for i=1:numel(mod.modelstring)
    fprintf(fid,'%s\r\n',mod.modelstring{i});
end
fclose all;

ravefinish;


function makemodelmfile(src,ev,modelindex,filename,VarNameFormatted)
handles=guidata(src);
settings=handles.models{handles.temp.selectedmodel};

%makes a function m file from the model at modelindex with the specified
%filename.

fid=fopen(filename,'w+'); %note, this line gives an error on new matlab installs, maybe because old defaultdir isnt on new path.
[path,functionname]=fileparts(filename);

d=settings.dataset;
xcol=settings.inputs;
%ycol=settings.output;

varorder=xcol;
varnames=handles.allnames{d}(xcol);


%remove illegal characters from varnames:
varnames=ravemakevariablenames(varnames);

nvars=numel(xcol);

fprintf(fid,'function %s=%s(inputs,prefs,targets)\n',VarNameFormatted,functionname); %error here on new install, see fid=fopen... line above.
%end

fprintf(fid,'%%This is a function for the variable: %s.\n',functionname);
fprintf(fid,'%%This file was auto-generated by the "Import txt Function" button in RAVE.\n');
fprintf(fid,'%%This function is not meant to be called from the command line.\n');
fprintf(fid,'%%I don''t even know why you''re reading this help file.\n');
fprintf(fid,'%%This function was created with RAVETXT2M version 1.0\n');
fprintf(fid,'\n\n');
%Print the function:
fprintf(fid,'%%expand input array out to its component vectors:\n');
for jj=1:nvars
    varnames{jj}=strrep(varnames{jj},' ',''); %remove spaces from variable names.
    if varorder(jj)~=0
        fprintf(fid,'%s=inputs(:,%1.0f);\n',varnames{jj},varorder(jj));
    end
end
fprintf(fid,'\n');
fprintf(fid,'nruns=size(inputs,1);\n');

for g=1:numel(settings.modelstring)
    fprintf(fid,'%s\n',settings.modelstring{g});
end

fclose(fid);













