function exploretab(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

ravechangetab(src,[]);
handles=guidata(src);
set(handles.infobar,'string','');
handles.activetab='explore';
guidata(src,handles);

%move front tabs to front if necessary
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y);


set(handles.infobar,'string','');

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,4,1);  %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% clear existing tab
delete(handles.tabobjects)

%%  get methods:
if handles.tablemode
    handles.tabobjects=[];
    guidata(handles.fig,handles);
    return
end

%check for graphs:
if isempty(handles.activetype)
    handles.tabobjects=[];
    guidata(handles.fig,handles);
    return
end

methodkeywords=fieldnames(handles.exploreinfo);

%get long names:
for m=numel(methodkeywords):-1:1
   methodnames{m}=handles.exploreinfo.(methodkeywords{m}).longname;
end

finalmethods=[];
%compare to allowable methods for selected graph type:
if ~isempty(handles.activeindex) && isfield(handles.graphinfo.(handles.activetype),'exploremethods')
    allowedmethods=handles.graphinfo.(handles.activetype).exploremethods;
    [finalmethods,ia,ib]=intersect(lower(allowedmethods),lower(methodkeywords));
else
   ib=[];
end

if isempty(finalmethods)
    handles.tabobjects=[];
    guidata(handles.fig,handles);
    return
end

%% text headers
methodtext=text(.15-.10317,0.97,'Method','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% pulldown menu
methodmenu=uicontrol('position',[origin(1)+25 origin(2)+570 220 30],...
    'style','popupmenu','string',[cellstr('<Select a method from list>') methodnames(ib)],...
    'callback',@methodmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','methodmenu','userdata',[cellstr('');methodkeywords(ib)]);

%% finish up
handles.tabobjects=[methodtext methodmenu];
guidata(src,handles);


function methodmenu_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);

%origin=get(handles.tabaxes,'position');
methods=get(src,'string');
selected=methods{get(src,'value')};

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.explore.option=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

%% get input info from graph:
%delete method specific uicontrols:
if length(handles.tabobjects)>2 %only two method-indepenedent uicontrols
    delete(handles.tabobjects(3:end));
    handles.tabobjects=handles.tabobjects(1:2);
    guidata(src,handles);
end


%do selected method:
methodfunctions=get(src,'userdata');
thismethod=get(src,'value');
thisfunction=methodfunctions{thismethod};
controls=[];

if thismethod>1 %1 = none selected
    if ~isempty(which(['makeuicontrols',thisfunction,handles.activetype]))
        f=str2func(['makeuicontrols',thisfunction,handles.activetype]);
        controls=f(src,[]);
        
        %eval(['makeuicontrols',thisfunction,handles.activetype,'(src,[]);']);
    else %run the generic flavor of this function:
        %eval(['makeuicontrols',thisfunction,'(src,[]);']);
        f=str2func(['makeuicontrols',thisfunction]);
        controls=f(src,[]);
        
    end
end
handles=guidata(src); %reload in case the makeuicontrols fcn changed it
handles.tabobjects=[handles.tabobjects controls];

guidata(handles.fig,handles);
